<?php
/**
 * Smart by NiceThemes.
 *
 * This file loads functions that may not be available if the user is running
 * an updated version of our framework.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.3
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_doing_ajax' ) ) :
/**
 * Check if we're in an AJAX context.
 *
 * @since  1.0.3
 *
 * @return bool
 */
function nice_doing_ajax() {
	/**
	 * @hook nice_doing_ajax
	 *
	 * Hook in here to change the AJAX detection result.
	 *
	 * @since 1.0.3
	 */
	return apply_filters( 'nice_doing_ajax', ( defined( 'DOING_AJAX' ) && DOING_AJAX ) );
}
endif;

if ( ! function_exists( 'nice_int_option' ) ) :
/**
 * Obtain a theme setting as an integer value.
 *
 * @since 1.1
 *
 * @param string       $option_name  Name of the option.
 * @param string|null  $default      (Optional)
 *
 * @return int
 */
function nice_int_option( $option_name, $default = null ) {
	return intval( nice_get_option( $option_name, $default ) );
}
endif;

if ( ! function_exists( 'nice_option_equals' ) ) :
/**
 * Check if the stored value for an option matches another value.
 *
 * @since 1.1
 *
 * @param string $option_name Name of the option.
 * @param mixed  $match       The value that needs to be compared.
 * @param null   $type        Data type for matched value (optional).
 * @param null   $default     Default value of the option (optional).
 *
 * @return bool
 */
function nice_option_equals( $option_name, $match, $type = null, $default = null ) {
	if ( $type ) {
		settype( $match, $type );
	}

	return nice_get_option( $option_name, $default ) === $match;
}
endif;

if ( ! function_exists( 'nice_get_theme_file_path' ) ) :
/**
 * Obtain the path to a file inside the current theme or child theme.
 *
 * @since  1.1
 *
 * @param  string $path            Relative path to a file inside the theme.
 * @param  string $stylesheet_path The current stylesheet path. Optional.
 *
 * @return string
 */
function nice_get_theme_file_path( $path, $stylesheet_path = null ) {
	static $use_fallback;

	if ( is_null( $use_fallback ) ) {
		$use_fallback = ! function_exists( 'get_theme_file_path' );
	}

	if ( $use_fallback ) {
		if ( ! $stylesheet_path ) {
			$stylesheet_path = get_stylesheet_directory();
		}

		return trailingslashit( $stylesheet_path ) . $path;
	}

	return get_theme_file_path( $path );
}
endif;

if ( ! function_exists( 'nice_is_mixed_content' ) ) :
/**
 * Useful if some libraries need to be loaded in both admin and public sides.
 *
 * @since 1.1
 */
function nice_is_mixed_content() {
	/**
	 * @hook nice_is_mixed_content
	 *
	 * Hook in here to change default mixed content value.
	 */
	return apply_filters( 'nice_is_mixed_content', false );
}
endif;

if ( ! function_exists( 'nice_default_setting' ) ) :
/**
 * Return the default value of a theme setting before it's been saved to the
 * database.
 *
 * @since  1.1
 *
 * @param  string $option_name
 *
 * @return mixed|null
 */
function nice_default_setting( $option_name ) {
	static $default_settings = null;

	if ( is_null( $default_settings ) ) {
		/**
		 * @hook nice_default_settings
		 *
		 * Hook in here to define your theme's default settings.
		 *
		 * This hook is supposed to run only once to set all default values
		 * when the function is called for the first time.
		 *
		 * @since 1.1
		 */
		$default_settings = apply_filters( 'nice_default_settings', array() );
	}

	return isset( $default_settings[ $option_name ] ) ? $default_settings[ $option_name ] : null;
}
endif;
