/**
 * Smart by NiceThemes.
 *
 * NiceAnimations
 *
 * Handle theme animations.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceAnimations = ( function( $ ) {
	'use strict';

	/**
	 * Check if the module has been fired for the first time.
	 *
	 * @type {boolean}
	 */
	var fired = false,

	/**
	 * Setup animations.
	 */
	setupAnimations = function() {
		$( document ).ready( function() {
			var $body       = $( 'body' ),
				useOverlay  = ( 'full-page' === $body.data( 'page-loader' ) );

			if ( useOverlay ) {
				$body.on( 'overlayHidden', function() {
					$body.trigger( 'NiceAnimations' );
				} );
			} else {
				Pace.on( 'done', function() {
					$body.trigger( 'NiceAnimations' );
				} );
			}

			$( document ).on( 'NiceDOMChanged', function() {
				$body.trigger( 'NiceAnimationsDOMChanged' );
				$body.trigger( 'NiceAnimations' );
			} );

			$body.on( 'NiceAnimations', function() {
				var $animations = $( '.nice-animation:not(.nice-animation-start)' );
				$animations.NiceScheduleAnimation();
			} );
		} );
	},

	/**
	 * Setup object data to handle cascading effect in grids.
	 *
	 * @since 1.0.0
	 */
	setupGridCascading = function() {
		/**
		 * Obtain timeout of a grid element.
		 *
		 * @param $el
		 * @param $prev
		 * @param cascadePosition
		 * @param maxCascadeCols
		 * @param colDelay
		 */
		var _getTimeout = function ( $el, $prev, cascadePosition, maxCascadeCols, colDelay ) {
			if ( cascadePosition > maxCascadeCols ) {
				return 0;
			}

			if ( 1 === cascadePosition ) {
				return 0;
			}

			if ( 'undefined' === typeof $prev.data( 'animationTimeout' ) ) {
				return 0;
			}

			return ( parseInt( $prev.data( 'animationTimeout' ) ) + parseInt( colDelay ) );
		},

		/**
		 * Obtain position in cascade for a grid element.
		 *
		 * @param $prev
		 * @param maxCascadeCols
		 */
		_getCascadePosition = function( $prev, maxCascadeCols ) {
			if ( 'undefined' === typeof $prev.data( 'cascadePosition' ) ) {
				return 1;
			}

			if ( $prev.data( 'cascadePosition' ) >= maxCascadeCols ) {
				return 1;
			}

			return ( parseInt( $prev.data( 'cascadePosition' ) ) + 1 );
		},

		/**
		 * Setup cascade for grid elements.
		 *
		 * @since 1.0.0
		 */
		_setup = function() {
			var $grids = $( '.grid[data-animated="true"][data-cascade="true"]' );

			$grids.each( function() {
				var $items         = $( this ).find( '.item' ),
					colDelay       = $( this ).data( 'cascade-delay' ),
					maxCascadeCols = $( this ).data( 'columns' );

				$items.each( function() {
					var $el             = $( this ),
						$prev           = $el.prev( '.item' ),
						cascadePosition = _getCascadePosition( $prev, maxCascadeCols ),
						timeout         = _getTimeout( $el, $prev, cascadePosition, maxCascadeCols, colDelay );

					if ( $el.hasClass( 'nice-animation-cascading' ) ) {
						return;
					}

					$el.data( 'cascadePosition', cascadePosition );
					$el.data( 'animationTimeout', timeout );
					$el.addClass( 'nice-animation-cascading' );
				} );
			} );
		};

		$( document ).ready( _setup );
		$( 'body' ).on( 'NiceAnimationsDOMChanged', _setup );
	},

	/**
	 * Extend jQuery library.
	 */
	extendLib = function() {
		/**
		 * Check if the Waypoints script is being loaded.
		 *
		 * @type {boolean}
		 */
		var waitForScript = false,

		/**
		 * List of elements to be setup once the Waypoints library is available.
		 *
		 * @type {Array}
		 */
		delayed = [];

		/**
		 * Animate an element based on given animation style, delay and speed.
		 *
		 * @since 1.0.0
		 *
		 * @constructor
		 */
		$.fn.NiceAnimate = function() {
			this.each( function() {
				var $el            = $( this ),
					animationDelay = $el.data( 'delay' ) ? parseInt( $el.data( 'delay' ) ) : 0,
					animationSpeed = $el.data( 'speed' ) ? parseInt( $el.data( 'speed' ) ) : 1000,
					animation      = $el.data( 'animation' ) ? $el.data( 'animation' ) : '',
					useCascade     = ( 1024 < $( window ).width() ) && ( 'undefined' !== typeof $el.data( 'animationTimeout' ) ),
					cascadeTimeout = useCascade ? parseInt( $el.data( 'animationTimeout' ) ) : 0,
					timeout        = Number( animationDelay + cascadeTimeout );

				if ( animationSpeed ) {
					var animationSpeedClass = 'animation-duration-' + animationSpeed;
					$el.addClass( animationSpeedClass );
				}

				setTimeout( function () {
					$el.addClass( 'animated' );
					$el.addClass( 'nice-animation-start' );
					$el.addClass( animation );
				}, timeout );
			} );
		};

		/**
		 * Remove animation classes from an element.
		 *
		 * @since 1.0.0
		 *
		 * @constructor
		 */
		$.fn.NiceRemoveAnimation = function() {
			$( this ).removeClass( 'animated' );
			$( this ).removeClass( 'nice-animation-start' );
		};

		/**
		 * Schedule a waypoint if the library is available.
		 *
		 * @since 1.0.0
		 *
		 * @param callback
		 * @param args
		 * @constructor
		 */
		$.fn.NiceWaypoint = function( callback, args ) {
			var waypointURL     = $.NiceLazyScript( 'waypoints' ),
				$currentElement = this,

			/**
			 * Setup waypoint for the current element.
			 *
			 * @param el
			 * @param _callback
			 * @param _args
			 */
			setupWaypoint = function( el, _callback, _args ) {
				_args = ( 'undefined' === typeof _args ) ? {} : _args;

				_args.offset = ( 'undefined' === typeof _args.offset ) ? '95%' : _args.offset;
				_args.triggerOnce = ( 'undefined' === typeof _args.triggerOnce ) ? true : _args.triggerOnce;

				//noinspection JSUnresolvedFunction
				el.waypoint( function( direction ) {
					_callback( direction );

					if ( _args.triggerOnce && 'function' === typeof this.disable ) {
						//noinspection JSUnresolvedFunction
						this.disable();
					}
				}, _args );
			};

			/**
			 * Try to setup waypoint for the current element depending on the
			 * loading status of the Waypoints library.
			 *
			 * If the library still hasn't been loaded, setting up the current
			 * element will be delayed to be executed once the library is
			 * available.
			 */
			if ( 'function' !== typeof $.fn.waypoint && waypointURL && waitForScript ) {
				// If the Waypoints script is still loading, set the callback
				// to be executed as soon as possible in the future.
				delayed.push( { el: $currentElement, callback: callback, args: args } );

			} else if ( 'function' !== typeof $.fn.waypoint && waypointURL && ! waitForScript ) {
				// Set flag to true to indicate the Waypoints script is still loading.
				waitForScript = true;

				$.getScript( waypointURL, function() {
					// Set flag to false once the script is loaded.
					waitForScript = false;

					// Setup waypoint for the current element.
					setupWaypoint( $currentElement, callback, args );

					// Setup waypoints for all the delayed elements.
					for ( var i = 0; i < delayed.length; i++ ) {
						setupWaypoint( delayed[i].el, delayed[i].callback, delayed[i].args );
					}
				} );

			} else if ( 'function' === typeof $.fn.waypoint ) {
				// Setup waypoint if the library has already been loaded.
				setupWaypoint( $currentElement, callback, args );

			} else if ( 'function' !== typeof $.fn.waypoint ) {
				$.NiceDev.log( 'Warning: Waypoints is not initialized.' );
			}
		};

		/**
		 * Schedule an animation using waypoint arguments.
		 *
		 * @since 1.0.0
		 *
		 * @param args
		 * @constructor
		 */
		$.fn.NiceScheduleAnimation = function( args ) {
			//noinspection JSUnresolvedFunction
			this.each( function() {
				var $el = jQuery( this );

				$el.NiceWaypoint( function() {
					if ( $el.hasClass( 'nice-animation-start' ) ) {
						return;
					}

					$el.NiceAnimate();
				}, args );
			} );
		};
	},

	/**
	 * Initialize animations.
	 */
	init = function() {
		if ( fired ) {
			return;
		}

		extendLib();

		setupAnimations();

		setupGridCascading();

		fired = true;
	};

	return {
		init: init
	};
} )( jQuery );

NiceAnimations.init();