/**
 * Smart by NiceThemes.
 *
 * NiceCssRules
 *
 * Setup dynamic creation of CSS rules.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceCssRules = ( function( $ ) {
	'use strict';

	/**
	 * Check if the module has been loaded for the first time.
	 *
	 * @type {boolean}
	 */
	var fired = false,

	/**
	 * List of CSS rules to be added to the DOM.
	 *
	 * @type {Array}
	 */
	cssRules = [],

	/**
	 * List of CSS rules printed to the DOM.
	 *
	 * @type {Array}
	 */
	printedRules = [],

	/**
	 * Setup CSS rules.
	 */
	setup = function() {
		// Fire custom jQuery extensions.
		extendLib();

		// Fire CSS rules for the first time.
		$.NiceCssRules();

		// Setup future triggers for CSS rules.
		$( document ).on( 'NiceDOMChanged', function() {
			// Only fire CSS rules if they were already fired.
			// We're doing this to make sure rules are added for the first time
			// before Pace finished loading, and when Pace is reloaded in the
			// future.
			if ( fired ) {
				$.NiceCssRules();
			}
		} );
	},

	/**
	 * Extend jQuery functionality.
	 */
	extendLib = function() {
		/**
		 * NiceCssRules
		 *
		 * Add CSS rules asynchronously and on demand.
		 *
		 * @since 1.0.0
		 *
		 * @constructor
		 */
		$.NiceCssRules = function() {
			setRules();
			printRules();
		};
	},

	/**
	 * Set CSS rules to be printed based on HTML data.
	 *
	 * @since 1.0.0
	 */
	setRules = function() {
		var $opacity = $( '[data-opacity]' );

		/**
		 * Opacity rules.
		 *
		 * @since 1.0.0
		 */
		if ( $opacity.length ) {
			$opacity.each( function () {
				var value = $( this ).data( 'opacity' ),
					className = 'opacity-' + value;

				$( this ).addClass( className );

				if ( 'undefined' === typeof cssRules[ className ] ) {
					cssRules[ className ] = '.' + className + ' { opacity: ' + ( ( 1 / 100 ) * value ) + '; }';
				}
			} );
		}
	},

	/**
	 * Add CSS rules to the DOM inside the <head> tag.
	 *
	 * @since 1.0.0
	 */
	printRules = function() {
		if ( ! Object.keys( cssRules ).length ) {
			return;
		}

		var css = '';

		for ( var rule in cssRules ) {
			if ( cssRules.hasOwnProperty( rule ) && ! printedRules.hasOwnProperty( rule ) ) {
				css += cssRules[ rule ] + "\n";
				printedRules[ rule ] = true;
			}
		}

		if ( css ) {
			$( 'html > head' ).append( '<style>' + css + '</style>' );
		}

		$.NiceDev.log( 'CSS rules were printed.' );
	},

	/**
	 * Initialize CSS rules.
	 */
	init = function() {
		if ( fired ) {
			return;
		}

		$( setup );

		fired = true;
	};

	return {
		init: init
	};
} )( jQuery );

NiceCssRules.init();
