/**
 * Smart by NiceThemes
 *
 * NiceSlider
 *
 * Manage implementation of Flexslider.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceSlider = ( function( $ ) {
	// Tell browsers we're not doing anything silly.
	'use strict';

	/**
	 * Data model for the current module.
	 */
	var data = {
		// Container variable for all slider objects.
		sliders: null,

		// Container variable for all Vimeo player object.
		players: {},

		// Current slider object being processed.
		currentSlider: null,

		// Current slide being processed.
		currentSlide: null,

		// Next slide to be displayed.
		nextSlide: null,

		// List of callbacks to be called for each slider object.
		callbacks: null,

		// Amount of time (in miliseconds) that the overlay will be visible.
		overlayTimeout: 800,

		/**
		 * Set reference to all slider objects.
		 *
		 * @private
		 */
		setSliders: function( sliders ) {
			// Obtain all slider objects.
			data.sliders = $( sliders );
		},
		
		/**
		 * Set reference to the current slider object.
		 *
		 * @param {obj} slider
		 */
		setCurrentSlider: function( slider ) {
			data.currentSlider = $( slider );
		},

		/**
		 * Get the currently active slide.
		 *
		 * @param   {jQuery} $slider
		 *
		 * @returns {jQuery}
		 */
		getCurrentSlide: function( $slider ) {
			// Return early if we don't have any slides to display.
			if ( 'undefined' === typeof $slider.slides ) {
				return;
			}

			// Set current slide from given slider object.
			var $currentSlide = $( $slider.slides.eq( $slider.currentSlide ) ),
				iframe = $currentSlide.find( 'iframe' );

			// Add our own properties to current slide.
			return $.extend( $currentSlide, {
				'iframe':       iframe,
				'directionNav': $( $slider.directionNav ),
				'controlNav':   $( $slider.controlNav ),
				'colorScheme':  $currentSlide.attr( 'data-color-scheme' )
			} );
		},

		/**
		 * Set the next slide to be active.
		 *
		 * @param {jQuery} $slider
		 */
		getNextSlide: function( $slider ) {
			return $( $slider.slides.eq( $slider.animatingTo ) );
		},

		/**
		 * Set list of callbacks.
		 *
		 * @param {obj} callbacks
		 */
		setCallbacks: function( callbacks ) {
			data.callbacks = callbacks;
		},

		/**
		 * Bind callbacks to be executed for every single triggered action.
		 */
		setupCallbacks: function() {
			if ( 'undefined' === typeof data.callbacks ) {
				return;
			}

			for ( var action in data.callbacks ) {
				if ( ! data.callbacks.hasOwnProperty( action ) ) {
					continue;
				}

				for ( var i = 0; i < action.length; i ++ ) {
					data.currentSlider.on( action, data.callbacks[ action ][ i ] );
				}
			}
		},

		createVimeoElementID: function( element ) {
			var elementID = 'player_' + Math.floor( 1000 + Math.random() * 9000 );
			$( element ).attr( 'id', elementID );
			return elementID;
		},
		getVimeoElementID: function( element ) {
			var IDAttr = $( element ).attr( 'id' );
			return IDAttr ? IDAttr : this.createVimeoElementID( element );
		},
		getVimeoPlayer: function( element ) {
			var elementID = this.getVimeoElementID( element );

			if ( 'undefined' !== typeof data.players[ elementID ] ) {
				return data.players[ elementID ];
			}

			data.players[ elementID ] = new window.Vimeo.Player( elementID );

			return data.players[ elementID ];
		}
	};

	/**
	 * Controller for the current module.
	 */
	var controller = {
		/**
		 * Process properties and schedule all slider-related actions.
		 *
		 * @private
		 */
		init: function() {
			var url = $.NiceLazyScript( 'flexslider' ),
			doInit = function() {
				data.sliders.each( controller.setupSlider );
			};

			data.setCallbacks( controller.getCallbacks() );
			data.setSliders( $( '.flexslider:not(.loaded)' ) );

			if ( data.sliders.length ) {
				if ( typeof $.fn.flexslider !== 'function' && url ) {
					$.getScript( url, doInit );
				} else if ( typeof $.fn.flexslider === 'function' ) {
					doInit();
				} else {
					$.NiceDev.log( 'Warning: FlexSlider is not initialized.' );
				}
			}
		},

		/**
		 * Setup a single slider object.
		 *
		 * @param {int} index
		 * @param {obj} slider
		 */
		setupSlider: function( index, slider ) {
			// Set reference to the current slider object.
			data.setCurrentSlider( slider );

			// Setup callbacks for the current slider.
			data.setupCallbacks();

			// Fire processes for the current slider.
			controller.registerSlider();
		},

		/**
		 * Fire processes for the current slider.
		 */
		registerSlider: function() {
			data.currentSlider.trigger( 'register', [ data.currentSlider ] );
		},

		/**
		 * Obtain lists of callbacks to be executed for every single triggered
		 * action.
		 *
		 * @returns {obj}
		 */
		getCallbacks: function() {
			return {
				'register': [
					view.setSliderHeight,
					view.handleVimeo,
					view.fireSlider,
					view.addLoadingClass
				],
				'init': [
					view.resizeSlider,
					view.hideVideoForMobile
				],
				'start': [
				    view.setStatus,
				    view.removeOverlay,
					view.setInitialStatus,
				    view.playSlideVideo,
				    view.setDirectionClasses
				],
				'before': [
					view.applyNewHeight,
					view.removeColorScheme,
					view.pauseSlideVideo
				],
				'after': [
					view.setActiveSlide,
					view.applyColorScheme,
					view.playSlideVideo
				]
			};
		},
		
		/**
		 * Obtain variables for the current slider.
		 *
		 * @returns {obj}
		 * @private
		 */
		getFlexsliderVars: function() {
			return {
				startAt:        0,
				animation:      'fade',
				reverse:        false,
				pauseOnHover:   true,
				randomize:      false,
				animationLoop:  true,
				animationSpeed: 600,
				slideshow:      false,
				slideshowSpeed: 7000,
				controlNav:     true,
				directionNav:   true,
				smoothHeight:   true
			};
		},

		/**
		 * Obtain methods to run for the current slider.
		 *
		 * @returns {obj}
		 */
		getFlexsliderMethods: function() {
			return {
				init:   controller.flexsliderInit,
				start:  controller.flexsliderStart,
				before: controller.flexsliderBefore,
				after:  controller.flexsliderAfter
			};
		},

		/**
		 * Obtain settings from a specific variable name.
		 *
		 * @param   {jQuery} $slider
		 *
		 * @returns {obj}
		 */
		getFlexsliderCallbackSettings: function( $slider ) {
			var varName  = $slider.data( 'settings' ),
			    settings = ( 'undefined' !== typeof window.generalData[ varName ] ) ? window.generalData[ varName ] : {};

			return settings;
		},

		/**
		 * Obtain settings for current slider, from a mixed object consisting on
		 * vars and methods.
		 *
		 * @param   {jQuery} $slider
		 *
		 * @returns {obj}
		 */
		getFlexsliderSettings: function( $slider ) {
			var defaultSettings    = $.extend( controller.getFlexsliderVars(), controller.getFlexsliderMethods() ),
				sliderSettings     = $slider.data(),
				callbackSettings   = controller.getFlexsliderCallbackSettings( $slider );

			return $.extend( defaultSettings, sliderSettings, callbackSettings );
		},

		/**
		 * Process all slider-related actions.
		 */
		handleSliders: function() {
			$( document ).ready( data.setup( $( '.flexslider' ) ) );
		},

		/**
		 * Fire actions after the slider is initially setup.
		 *
		 * @callback
		 *
		 * @param {jQuery} $slider
		 */
		flexsliderInit: function( $slider ) {
			$slider.trigger( 'init', [ $slider ] );
		},

		/**
		 * Fire actions when the slider loads the first slide.
		 *
		 * @callback
		 *
		 * @param {jQuery} $slider
		 */
		flexsliderStart: function( $slider ) {
			$slider.trigger( 'start', [ $slider ] );
		},

		/**
		 * Fire actions when a slide animation begins.
		 *
		 * @callback
		 *
		 * @param {jQuery} $slider
		 */
		flexsliderBefore: function( $slider ) {
			$slider.trigger( 'before', [ $slider ] );
		},

		/**
		 * Fire actions after a slide animation ends.
		 *
		 * @callback
		 *
		 * @param {jQuery} $slider
		 */
		flexsliderAfter: function( $slider ) {
			$slider.trigger( 'after', [ $slider ] );
		}
	};

	/**
	 * Set of methods for direct DOM manipulation.
	 */
	var view = {
		/**
		 * Set the initial height of the slider.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		setSliderHeight: function( event, $slider ) {
			$slider.css( { height: Math.floor( $slider.height() ) } );
		},
		/**
		 * Fire Flexslider using our custom settings.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		fireSlider: function( event, $slider ) {
			var _fire = function() {
				if ( $slider.data( 'fired' ) ) {
					return;
				}

				$slider.flexslider( controller.getFlexsliderSettings( $slider ) );
				$slider.data( 'fired', true );
			};

			// Fire now for the first time.
			 _fire();

			// Fire when DOM changes.
			$( document ).on( 'NiceDOMChanged', _fire );
		},

		/**
		 * Set "resized" status and auto height to current slider.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		resizeSlider: function( event, $slider ) {
			var _resize = function() {
				if ( ! $slider.hasClass( 'resized' ) ) {
					$slider.addClass( 'resized' ).css( 'height', 'auto' );
				}
			};

			_resize();

			$( window ).resize( _resize );
		},

		/**
		 * Show video image instead of embed for matching user agents.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		hideVideoForMobile: function( event, $slider ) {
			if ( navigator.userAgent.match(/(Android|iPod|iPhone|iPad|IEMobile|Opera Mini)/) ) {
				$slider.find( '.nice-slide-video-container' ).hide();
				$slider.find( '.nice-slide-video-image' ).addClass( 'mobile' );
			}
		},

		/**
		 * Setup compatibility for Vimeo players.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		handleVimeo: function( event, $slider ) {
			var vimeoPlayers = $slider.find( 'iframe' ),
			url = $.NiceLazyScript( 'vimeoPlayer' ),

			/**
			 * Setup Vimeo players.
			 *
			 * @param player
			 * @private
			 */
			_setup = function( player ) {
				var settings = controller.getFlexsliderSettings( $slider );

				// Play slider when video is paused.
				player.on( 'pause', function() {
					$.NiceDev.log( 'Pausing video, playing slider.' );
					// There's no need to call `$slider.flexslider( 'play' )` here - the slider will resume play by itself.
				} );

				// Pause slider when video is played.
				player.on( 'play', function() {
					$.NiceDev.log( 'Playing video, pausing slider' );

					if ( settings.slideshow ) {
						$slider.flexslider( 'pause' );
					}
				} );
			},

			/**
			 * Initialize Vimeo players.
			 *
			 * @private
			 */
			_initVimeoPlayer = function() {
				for ( var i = 0, length = vimeoPlayers.length; i < length; i++ ) {
					var player = data.getVimeoPlayer( vimeoPlayers[ i ] );
					_setup( player );
				}
			};

			// Initialize video players in case the Vimeo API is available.
			if ( vimeoPlayers.length && typeof window.Vimeo !== 'object' && url ) {
				$.getScript( url, _initVimeoPlayer );
			} else if ( vimeoPlayers.length && typeof window.Vimeo === 'object' ) {
				_initVimeoPlayer();
			} else if ( vimeoPlayers.length && typeof window.Vimeo !== 'object' ) {
				$.NiceDev.log( 'Warning: Vimeo is not initialized' );
			}
		},

		/**
		 * Apply styles and classes to the current slider before the overlay
		 * disappears.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		setStatus: function( event, $slider ) {
			// The timeout needs to be the lower than the overlay's, so we can
			// make sure everything is displayed correctly when the slider is
			// revealed.
			setTimeout( function() {
				// Remove "loading" class and add "loaded", since the slider has finished loading by now.
				$slider.removeClass( 'loading' );
				$slider.addClass( 'loaded' );
			}, ( data.overlayTimeout * 0.66 ) );
		},

		/**
		 * Remove overlay element from the current slider.
		 *
		 * We need to use a timeout to make sure the slider is fully loaded
		 * before showing it.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		removeOverlay: function( event, $slider ) {
			setTimeout( function() {
				$slider.children( '.overlay' ).fadeOut();
			}, data.overlayTimeout );
		},

		/**
		 * Add "loading" class to current slider.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		addLoadingClass: function( event, $slider ) {
			// Set "loading" status.
			$slider.addClass( 'loading' );
		},

		/**
		 * Add helper classes to current slider and its slides.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		setInitialStatus: function( event, $slider ) {
			var $currentSlide = data.getCurrentSlide( $slider );

			// Return early if we couldn't get the current slider.
			if ( 'undefined' === typeof $currentSlide ) {
				return;
			}

			// Fire potential DOM changes.
			$( document ).trigger( 'NiceDOMChanged' );

			// Add helper class to active slide.
			$currentSlide.addClass( 'active' );

			// Add helper class to all hidden slides.
			$slider.find( '.slides li:not(.active):not(.clone)' ).addClass( 'hidden' );

			// Apply color schemes.
			view.applyColorScheme( null, $slider );
		},

		/**
		 * Apply color scheme to navigation elements.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		applyColorScheme: function( event, $slider ) {
			var $currentSlide = data.getCurrentSlide( $slider );

			$currentSlide.directionNav.closest( 'ul' ).addClass( $currentSlide.colorScheme );
			$currentSlide.controlNav.closest( 'ol' ).addClass( $currentSlide.colorScheme );
		},

		/**
		 * Remove color scheme from navigation elements.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		removeColorScheme: function( event, $slider ) {
			var $currentSlide = data.getCurrentSlide( $slider );

			$currentSlide.directionNav.closest( 'ul' ).removeClass( $currentSlide.colorScheme );
			$currentSlide.controlNav.closest( 'ol' ).removeClass( $currentSlide.colorScheme );
		},

		/**
		 * Auto-play hosted video inside a slide.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		playSlideVideo: function( event, $slider ) {
			var $iframe = data.getCurrentSlide( $slider ).iframe,
				player = $iframe.length ? data.getVimeoPlayer( $iframe ) : null;

			if ( ! player ) {
				return;
			}

			player.play().catch( function() {
				$.NiceDev( 'Error playing video.' );
			} );
		},

		/**
		 * Set additional classes for direction controls.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		setDirectionClasses: function( event, $slider ) {
			var settings    = controller.getFlexsliderSettings( $slider ),
				$navButtons = $slider.find( '.flex-direction-nav a' );

			if ( ! $navButtons.length ) {
				return;
			}

			if ( 'undefined' !== typeof settings.directionNavStyle ) {
				$navButtons.addClass( 'btn btn-' + settings.directionNavStyle );
			}

			if ( 'undefined' !== typeof settings.directionNavShape ) {
				$navButtons.addClass( 'btn-' + settings.directionNavShape );
			}

			if ( 'undefined' !== typeof settings.directionNavColor ) {
				$navButtons.addClass( 'btn-' + settings.directionNavColor );
			}
		},

		/**
		 * Pause video inside the current slide.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		pauseSlideVideo: function( event, $slider ) {
			var $iframe = data.getCurrentSlide( $slider ).iframe,
				player = $iframe.length ? data.getVimeoPlayer( $iframe ) : null;

			if ( ! player ) {
				return;
			}

			player.pause().catch( function() {
				$.NiceDev( 'Error pausing video.' );
			} );
		},

		/**
		 * Apply new height (using next slide) to current slider while
		 * sliding.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		applyNewHeight: function( event, $slider ) {
			var settings       = controller.getFlexsliderSettings( $slider ),
				$currentSlide  = data.getCurrentSlide( $slider ),
			    $nextSlide     = data.getNextSlide( $slider ),
				currentHeight  = Math.floor( $currentSlide.height() );

			// Hide current slide.
			$currentSlide.removeClass( 'active' ).addClass( 'hiding' );

			// Remove helper class from the slide being loaded.
			$nextSlide.removeClass( 'hidden' ).addClass( 'sliding' );

			var newHeight = Math.floor( $nextSlide.height() );

			if ( currentHeight !== newHeight ) {
				$slider.animate( { 'height': newHeight }, settings.animationSpeed );
			} else {
				$slider.css( 'height', currentHeight );
			}
		},

		/**
		 * Apply "active" class to current slide, "hidden" class to every other
		 * slide, and remove "hiding" and "sliding" classes.
		 *
		 * @callback
		 *
		 * @param {Event}  event
		 * @param {jQuery} $slider
		 */
		setActiveSlide: function( event, $slider ) {
			var $currentSlide = data.getCurrentSlide( $slider );

			$slider.find( '.hiding' ).removeClass( 'hiding' ).addClass( 'hidden' );

			// Remove auto height.
			$slider.removeClass( 'resized' );
			$currentSlide.addClass( 'active' ).removeClass( 'sliding' );
		}
	};

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	var ready = function() {
		window.Pace.on( 'done', controller.init );
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};
} )( jQuery );

jQuery( NiceSlider.ready );

/**
 * Smart by NiceThemes
 *
 * NiceSliderParallax
 *
 * Manage Parallax implementation on Homepage slider.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceSliderParallax = ( function( $ ) {
	'use strict';

	/**
	 * Reference to the homepage slider object.
	 *
	 * @var {jQuery}
	 */
	var $slider;

	/**
	 * Current scroll position in pixels.
	 *
	 * @var {int}
	 */
	var scrollPosition;

	/**
	 * Percentage of current scroll position. This value should be relative to
	 * the height of the slider element.
	 *
	 * @var {int}
	 */
	var scrolledPercent;

	/**
	 * Initialize Parallax functionality if required.
	 */
	var init = function() {
		// Obtain slider object.
		$slider = $( '#homepage-slider.flexslider.parallax-scroll.position-1' );

		// Return early if the queried object doesn't exist.
		if ( ! $slider ) {
			return;
		}

		// Fire for the first time.
		fireParallax();

		// Setup scrolling functionality.
		$( window ).scroll( fireParallax );
	};

	/**
	 * Reset slider to its original status.
	 *
	 * @returns {boolean}
	 */
	var reset = function() {
		$slider.find( '.slide-info' ).css( {
			'opacity': 1
		} );

		$slider.find( 'img' ).css( {
			'-webkit-transform': 'scale(1.01)',
			   '-moz-transform': 'scale(1.01)',
			    '-ms-transform': 'scale(1.01)',
			     '-o-transform': 'scale(1.01)',
			        'transform': 'scale(1.01)',
			              'top': 0
		} );

		$slider.find( '.slide-overlay' ).css( {
			'opacity': 0
		} );

		return true;
	};

	/**
	 * Apply parallax styles to slider.
	 *
	 * @returns {boolean}
	 */
	var applyParallax = function() {
		var scrolledScale = 1 + ( scrolledPercent / 1000 );

		$slider.find( '.slide-info' ).css( {
			'opacity': 1 - ( ( scrolledPercent / 100 ) * 1.54 )
		} );

		$slider.find( 'img' ).css( {
			'-webkit-transform': 'scale(' + scrolledScale + ')',
			   '-moz-transform': 'scale(' + scrolledScale + ')',
			    '-ms-transform': 'scale(' + scrolledScale + ')',
			     '-o-transform': 'scale(' + scrolledScale + ')',
			        'transform': 'scale(' + scrolledScale + ')',
			              'top': ( scrolledPercent / 2 ) + '%'
		} );

		$slider.find( '.slide-overlay' ).css( {
			'opacity': ( scrolledPercent / 100 )
		} );

		return true;
	};

	/**
	 * Obtain current scroll position in pixels.
	 *
	 * @returns {int}
	 */
	var getScrollPosition = function() {
		return $( window ).scrollTop();
	};

	/**
	 * Obtain percentage of current scrolled position
	 *
	 * @returns {number}
	 */
	var getScrolledPercent = function() {
		if ( 'undefined' === typeof scrollPosition ) {
			scrollPosition = $( window ).scrollTop();
		}

		return ( ( scrollPosition / 100 ) / ( $slider.height () / 100 ) ) * 100;
	};

	/**
	 * Apply parallax styles based on scroll position.
	 *
	 * @returns {boolean}
	 */
	var fireParallax = function() {
		if ( $( window ).width() <= 970 ) {
			reset();

			return;
		}

		scrollPosition  = getScrollPosition();
		scrolledPercent = getScrolledPercent();

		if ( scrolledPercent <= 100 ) {
			applyParallax();
		}
	};

	/**
	 * Fire events on document ready, and bind other events.
	 */
	var ready = function() {
		// Initialize parallax once the page is fully loaded.
		Pace.on( 'done', init );

		$( document ).on( 'NiceDOMChanged', init );
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};
} )( jQuery );

NiceSliderParallax.ready();
