/**
 * Smart by NiceThemes.
 *
 * NiceGeneral
 *
 * General functionality for the theme.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceGeneral = ( function( $ ) {
	'use strict';

	/**
	 * Check if this module has already been loaded.
	 *
	 * @type {boolean}
	 */
	var fired = false,

    /**
     * Breakpoint (in px) for responsive behavior.
     *
     * @type {number}
     */
    breakpoint = 1024,

	/**
	 * Set state to the document's body.
	 *
	 * @since 1.0.3
	 */
	setupBodyState = function() {
        var _setupMobileState = function() {
        	$( 'body' ).data( 'is-mobile-viewport', ( $( window ).width() < breakpoint ) );
        	return true;
        };

		_setupMobileState() && $( window ).resize( _setupMobileState );
	},

    /**
     * Manage auto-hide functionality for fixed header.
     *
     * @since 1.0.0
     */
    autoHideHeader = function() {
	    // Return early for mobile devices.
	    if ( navigator.userAgent.match(/(Android|iPod|iPhone|iPad|IEMobile|Opera Mini)/) ) {
			return;
	    }

        var $header            = null,
	        lastScrollPosition = 0,
	        headerHeight       = 0,

        /**
         * Handle show/hide functionality depending on viewport and scroll
         * direction.
         *
         * @private
         */
        _setAutoHideHeader = function() {
	        // Return early if we're not allowed to auto-hide the header.
	        if ( ! $header.is( '.horizontal.fixed.autohide' ) ) {
		        return;
	        }

	        // Reset header state and return early if we're in a small viewport.
	        if ( $( window ).width() < breakpoint ) {
		        return _resetHeader();
	        }

	        // Obtain current scroll position.
	        var scrollPosition = $( window ).scrollTop();

	        // Reset header state and return early if we're at the top of the page.
	        if ( scrollPosition === 0 ) {
		        return _resetHeader();
	        }

	        // Show or hide header depending on scroll direction.
	        if ( scrollPosition < lastScrollPosition || scrollPosition < _vBreakpoint() ) {
		        // Try to show header when scrolling up.
		        _showHeaderMaybe();
	        } else {
		        // Try to hide header when scrolling down.
		        _hideHeaderMaybe();
	        }

	        // Update last scroll position.
	        lastScrollPosition = scrollPosition;

	        return lastScrollPosition;
        },

        /**
         * Obtain vertical breakpoint where the header can be hidden.
         *
         * @returns {number}
         * @private
         */
        _vBreakpoint = function() {
	        return $( window ).height() / 2;
        },

        /**
         * Reset header to its initial state.
         *
         * @private
         */
        _resetHeader = function() {
	        $header.data( 'hidden', false );
	        $header.data( 'locked', false );

	        $header.removeClass( 'auto-hidden' );
        },

        /**
         * Try to hide header.
         *
         * @private
         */
        _hideHeaderMaybe = function() {
	        // Return early if the header is already hidden or locked.
	        if ( $header.data( 'hidden' ) || $header.data( 'locked' ) ) {
		        return;
	        }

	        // Lock header, so we can't show it again until we hide it completely.
	        $header.data( 'locked', true );

	        if ( ! $header.hasClass( 'auto-hidden' ) ) {
		        $header.addClass( 'auto-hidden' );
		        $header.data( 'hidden', true );
		        $header.data( 'locked', false );
	        }
        },

        /**
         * Try to show header.
         *
         * @private
         */
        _showHeaderMaybe = function() {
	        // Return early if the header is not hidden or it's locked.
	        if ( ! $header.data( 'hidden' ) || $header.data( 'locked' ) ) {
		        return;
	        }

	        // Lock header, so we can't hide it again until we show it completely.
	        $header.data( 'locked', true );

	        if ( $header.hasClass( 'auto-hidden' ) ) {
		        $header.removeClass( 'auto-hidden' );
		        $header.data( 'hidden', false );
		        $header.data( 'locked', false );
	        }
        };

        $( document ).ready( function() {
        	$header = $( '#header' );

	        // Fire hide/show functionality once the initial height of the desktop
	        // header is set.
	        $header.on( 'setDesktopHeight', function( event, height ) {

		        // Set our internal height indicator.
		        headerHeight = height;

		        _setAutoHideHeader();

		        // Schedule to run functionality on resize and scroll events.
		        $( window ).resize( _setAutoHideHeader );
		        $( window ).scroll( $.NiceThrottle( 250, _setAutoHideHeader ) );
	        } );
        } );
    },

    /**
     * Manage hiding header until the use scrolls to a specific amount of pixels.
     *
     * @since 1.0.0
     */
    hideHeaderUntilScroll = function() {
	    // Return early for mobile devices.
	    if ( navigator.userAgent.match(/(Android|iPod|iPhone|iPad|IEMobile|Opera Mini)/) ) {
		    return;
	    }

	    var $header            = null,
		    lastScrollPosition = 0,
		    headerHeight       = 0,

	    /**
	     * Handle show/hide functionality depending on viewport and scroll
	     * direction.
	     *
	     * @private
	     */
	    _setAutoHideHeader = function() {
		    // Return early if we're not allowed to auto-show the header.
		    if ( ! $header.is( '.horizontal.fixed.autoshow' ) ) {
			    return;
		    }

		    // Reset header state and return early if we're in a small viewport.
		    if ( $( window ).width() < breakpoint ) {
			    return _resetHeader();
		    }

		    // Obtain current scroll position.
		    var scrollPosition = $( window ).scrollTop();

		    // Reset header state and return early if we're at the top of the page.
		    if ( scrollPosition === 0 ) {
			    return _resetHeader();
		    }

		    // Show or hide header depending on scroll direction.
		    if ( scrollPosition <= _vBreakpoint() ) {
			    // Try to hide header when scrolling up.
			    _hideHeaderMaybe();
		    } else {
			    // Try to show header when scrolling down.
			    _showHeaderMaybe();
		    }

		    // Update last scroll position.
		    lastScrollPosition = scrollPosition;

		    return lastScrollPosition;
	    },

	    /**
	     * Obtain vertical breakpoint where the header can be hidden.
	     *
	     * @returns {number}
	     * @private
	     */
	    _vBreakpoint = function() {
	        var breakpoint;

		    if ( $header.data( 'autoshow-breakpoint' ) ) {
		        breakpoint = parseInt( $header.data( 'autoshow-breakpoint' ) );
		    } else {
		        breakpoint = $( window ).height() / 2;
		    }

		    return breakpoint;
	    },

	    /**
	     * Reset header to its initial state.
	     *
	     * @private
	     */
	    _resetHeader = function() {
		    $header.data( 'hidden', true );
		    $header.data( 'locked', false );

		    $header.addClass( 'hide-until-scroll' );
	    },

	    _hideHeaderMaybe = function() {
		    // Return early if the header is already hidden or locked.
		    if ( $header.data( 'hidden' ) || $header.data( 'locked' ) ) {
		        return;
		    }

		    // Lock header, so we can't show it again until we hide it completely.
		    $header.data( 'locked', true );

		    if ( ! $header.hasClass( 'hide-until-scroll' ) ) {
			    $header.addClass( 'hide-until-scroll' );
			    $header.data( 'hidden', true );
			    $header.data( 'locked', false );
		    }
	    },

	    /**
	     * Try to show header.
	     *
	     * @private
	     */
	    _showHeaderMaybe = function() {
	        // Return early if the header is not hidden or it's locked.
		    if ( ! $header.data( 'hidden' ) || $header.data( 'locked' ) ) {
			    return;
		    }

		    // Lock header, so we can't hide it again until we show it completely.
		    $header.data( 'locked', true );

		    if ( $header.hasClass( 'hide-until-scroll' ) ) {
			    $header.removeClass( 'hide-until-scroll' );
			    $header.data( 'hidden', false );
			    $header.data( 'locked', false );
		    }
	    };

	    $( document ).ready( function() {
	    	$header = $( '#header' );

		    // Fire hide/show functionality once the initial height of the desktop
		    // header is set.
		    $header.on( 'setDesktopHeight', function( event, height ) {

			    // Set our internal height indicator.
			    headerHeight = height;

			    _setAutoHideHeader();

			    // Schedule to run functionality on resize and scroll events.
			    $( window ).resize( _setAutoHideHeader );
			    $( window ).scroll( $.NiceThrottle( 250, _setAutoHideHeader ) );
		    } );
	    } );
    },

    /**
     * Setup size of header for both mobile and desktop viewports. Also
     * padding of page headings.
     *
     * @since 1.0.0
     */
    setupHeaderHeight = function() {
	    var $header             = null,
		    $heading            = null,
		    headerDesktopHeight = 0,
		    headerMobileHeight  = 0,

	    /**
	     * Set height of desktop header.
	     *
	     * @private
	     */
	    _setDesktopHeight = function() {
		    if ( ! headerDesktopHeight && $( window ).width() >= breakpoint ) {
			    headerDesktopHeight = parseInt( $header.height() );
			    $header.trigger( 'setDesktopHeight', [ headerDesktopHeight ] );
		    }
	    },

	    /**
	     * Set height of mobile header.
	     *
	     * @private
	     */
	    _setMobileHeight = function() {
		    if ( ! headerMobileHeight && $( window ).width() < breakpoint ) {
			    headerMobileHeight = parseInt( $header.height() );
			    $header.trigger( 'setMobileHeight', [ headerMobileHeight ] );
		    }
	    },

	    /**
	     * Set heading for the current page.
	     *
	     * @private
	     */
	    _setHeading = function() {
	    	$heading = $( '#heading' );

	    	if ( ! $heading.length ) {
			    var $container = $( '#content' ).find( '.position-1' );

			    if ( ! $container.length ) {
				    $heading = $header.next();
				    return;
			    }

			    if ( $container.hasClass( 'slider' ) ) {
				    $heading = $container.find( '.slide-info' );
			    } else {
				    $heading = $container.find( '.inner' );
			    }
		    }
	    };

	    $( document ).ready( function() {
		    $header = $( '#header' );

		    if ( ! $header.is( '.horizontal.fixed, .transparent' ) ) {
			    return;
		    }

		    _setHeading();
		    _setDesktopHeight();
		    _setMobileHeight();

		    // Set functions to run on window resize.
		    $( window ).resize( _setDesktopHeight );
		    $( window ).resize( _setMobileHeight );
	    } );
    },

    /**
     * For vertical header, show and hide menu when initially hidden.
     *
     * @since 1.0.0
     */
    toggleVerticalHiddenNavigation = function() {
	    /**
	     * Toggle show/hide menu.
	     *
	     * @private
	     */
	    var _toggleMenu = function() {
		    // Return early for small viewports.
			if ( $( window ).width() < breakpoint ) {
				return;
			}

		    $( '#header' ).toggleClass( 'initially-hidden' );
	    };

	    $( document ).ready( function() {
	    	var $container = $( '#toggle-nav-container' );

		    // Return early if the header is not vertical and initially hidden.
		    if ( ! $( 'body' ).is( '.vertical-navigation.navigation-initially-hidden' ) ) {
			    return;
		    }

		    if ( 'desktop' === $( window ).data( 'viewport-type' ) ) {
		    	$( '#header-inner' ).prepend( $container );
		    }

		    $( window ).on( 'mobile2desktop', function() {
			    $( '#header-inner' ).prepend( $container );
		    } );

		    $( window ).on( 'desktop2mobile', function() {
			    $( '#navigation-extras-inner' ).append( $container );
		    } );

		    // Toggle on click.
		    $container.find( '.toggle-nav' ).click( _toggleMenu );
	    } );
    },

    /**
     * Schedule shrinking header to be initialized once the size for
     * desktop header is defined.
     *
     * @since 1.0.0
     */
	setupShrinkHeader = function() {
		$( document ).ready( function() {
			$( '#header' ).bind( 'setDesktopHeight', shrinkHeader );
		} );
	},

	/**
	 * Process header shrinking.
	 *
	 * @since 1.0.0
	 */
    shrinkHeader = function() {
		var $header     = null,
			hBreakpoint = breakpoint,
			vBreakpoint = 0,

		/**
		 * Adjust the vertical breakpoint.
		 *
		 * @since 1.0.3
		 *
		 * @private
		 */
		_setVBreakpoint = function() {
			vBreakpoint = ( function() {
				var $topBar = $( '#top-bar' ),
					breakpoint = 0;

				if ( $topBar.length && ! $topBar.is( '.fixed' ) ) {
					breakpoint += $topBar.outerHeight();
				}

				return breakpoint;
			} )();
		},

		/**
	     * Define shrinking functionality.
	     *
	     * @since 1.0.0
		 */
		shrink = function() {
			var scrollPosition = $( window ).scrollTop();

			if ( scrollPosition > vBreakpoint && $( window ).width() > hBreakpoint ) {
				$header.addClass( 'shrunk' );
			} else {
				$header.removeClass( 'shrunk' );
			}
		};

		// Redefine general data if necessary.
		if ( 'undefined' === typeof window.generalData ) {
			window.generalData = {
				headerFixed:  false,
				headerShrink: false
			};
		}

		// Return early if the header cannot be fixed.
		if ( ! window.generalData.headerFixed ) {
			return;
		}
		if ( ! window.generalData.headerShrink ) {
			// Return early if the header cannot be shrunk.
			return;
		}

		$( document ).ready( function() {
			$header = $( '#header' );

			// Return early for vertical navigation.
			if ( $header.hasClass( 'vertical' ) ) {
				return;
			}

			// Mark header as shrinkable.
			$header.addClass( 'shrinkable' );

			// Try to shrink once on first load.
			_setVBreakpoint();
			shrink();

			// Schedule shrinking for scroll and resize.
			$( window ).scroll( $.NiceThrottle( 250, shrink ) );
			$( window ).resize( function() {
				_setVBreakpoint();
				shrink();
			} );
		} );
	},

    /**
     * Handle search overlay functionality.
     *
     * @since 1.0.0
     */
    handleSearch = function() {
        var $searchHolder,
            $searchField,
            $searchButton,
            $closeButton,
	        $searchContainer,

        /**
         * Open search container.
         *
         * @since   1.0.0
         * @private
         */
        _openSearch = function( e ) {
	        if ( _isSearchOpen() ) {
		        return;
	        }

	        if ( 'undefined' !== typeof e ) {
		        e.preventDefault();
	        }

	        $searchContainer.fadeIn( 400, function() {
		        $searchContainer.addClass( 'visible' );
		        $searchField.focus();
		        $searchField.data( 'open', true );
	        } );
        },

        /**
         * Close search container.
         *
         * @since   1.0.0
         *
         * @private
         */
        _closeSearch = function( e ) {
	        if ( ! _isSearchOpen() ) {
		        return;
	        }

	        if ( 'undefined' !== typeof e ) {
		        e.preventDefault();
	        }

	        $searchContainer.removeClass( 'visible' );

	        setTimeout( function() {
		        $searchContainer.fadeOut( 400 );
		        $searchField.data( 'open', false );
	        }, 600 );
        },

        /**
         * Move search container to the end of body tag, so it doesn't get
         * hidden when resizing the window.
         *
         * @since   1.0.0
         *
         * @private
         */
        _moveSearchContainer = function() {
	        $searchContainer.appendTo( 'body' ).addClass( 'moved' );
        },

        /**
         * Check if search is open.
         *
         * @since   1.0.0
         *
         * @returns {boolean}
         * @private
         */
        _isSearchOpen = function() {
	        return $searchField.data( 'open' );
        };

        $( document ).ready( function() {
	        $searchHolder    = $( '#header-search' );
		    $searchField     = $searchHolder.find( '.input-text' );
		    $searchButton    = $searchHolder.find( 'a.search' );
		    $closeButton     = $searchHolder.find( 'a.close' );
		    $searchContainer = $( '#header-search-container' );

	        // Setup container relocation.
	        _moveSearchContainer();

	        // Open search when clicking "search" link.
	        $searchButton.click( _openSearch );

	        // Close search when clicking "close" link.
	        $closeButton.click( _closeSearch );

	        // Close search when pressing "esc" key.
	        $( document ).keyup( function( e ) {
		        if ( e.keyCode === 27 ) {
			        _closeSearch();
		        }
	        } );
        } );
    },

	/**
	 * Handle "back to top" functionality.
	 *
	 * @since 1.0.0
	 */
	backToTop = function() {
		var isBacktotopDisplayed = false,
			scrollPosition       = 0,
			breakpoint           = 100,
			$backButton          = null,

		/**
	     * Handle displaying and hiding "back to top" button.
	     *
	     * @since 1.0.0
		 */
		_buttonDisplay = function() {
			scrollPosition = $( window ).scrollTop();

			if ( scrollPosition < breakpoint && isBacktotopDisplayed ) {
				$backButton.animate( { bottom: '-48px', opacity: 0 }, 100 );

				isBacktotopDisplayed = false;
			} else if ( scrollPosition > breakpoint && ! isBacktotopDisplayed ) {
				$backButton.animate( { bottom: '17px', opacity: 1 }, 100 );
				isBacktotopDisplayed = true;
			}
		};

		// Setup displaying the button.
		$( document ).ready( function() {
			$backButton = $( '#back-to-top' );
			$( window ).scroll( $.NiceThrottle( 250, _buttonDisplay ) );
		} );
	},

    /**
     * Prevent elastic scrolling in Webkit.
     *
     * @since 1.0.0
     *
     * @param e
     *
     * @returns {boolean}
     */
	scrollHandler = function ( e ) {
		$( document ).ready( function() {
			var stopScrollX = false,
				stopScrollY = false,
				deltaX      = e.wheelDeltaX,
				deltaY      = e.wheelDeltaY,
				scrollToX   = window.scrollX - deltaX,
				scrollToY   = window.scrollY - deltaY,
				scrollMaxX  = document.body.scrollWidth - window.innerWidth,
				scrollMaxY  = document.body.scrollHeight - window.innerHeight,
				scrollX     = window.scrollX,
				scrollY     = window.scrollY;

			if ( deltaX > 0 && scrollX <= 0 ) {
				stopScrollX = true;
				scrollToX = 0;
			}

			if ( deltaY > 0 && scrollY <= 0 ) {
				stopScrollY = true;
				scrollToY = 0;
			}

			if ( deltaX < 0 && scrollX >= scrollMaxX ) {
				stopScrollX = true;
				scrollToX = scrollMaxX;
			}

			if ( deltaY < 0 && scrollY >= scrollMaxY ) {
				stopScrollY = true;
				scrollToY = scrollMaxY;
			}

			if ( stopScrollX || stopScrollY ) {
				e.preventDefault();
				e.stopPropagation();
				window.scroll( scrollToX, scrollToY );
			}

			return (stopScrollX || stopScrollY);
		} );
    },

    /**
     * Fire scroll handler functionality.
     *
     * @since 1.0.0
     */
    fireScrollHandler = function() {
    	var scrollBreakpoint = 0,

	    UA = new UserAgent(),

	    /**
	     * Adjust scrolling breakpoint.
	     *
	     * @since 1.0.3
	     *
	     * @private
	     */
	    _setScrollBreakpoint = function() {
		    scrollBreakpoint = ( function() {
			    var $topBar = $( '#top-bar' ),
				    breakpoint = 0;

			    if ( $topBar.length && ( ! $topBar.is( '.fixed' ) || $( 'body' ).data( 'is-mobile-viewport' ) ) ) {
				    breakpoint += $topBar.outerHeight();
			    }

			    return breakpoint;
		    } )();
	    },

        /**
         * Set scroll state for header.
         *
         * @since   1.0.0
         * @private
         */
	    _scrollState = function() {
	    	var $header = $( '#header' ),
			    passedBreakpoint = $( window ).width() >= breakpoint,
		        isFixedHeader = $header.is( '.fixed' ) || $header.is( '.fixed-mobile' );

		    if ( ( passedBreakpoint || $header.is( '.fixed-mobile' ) ) && $( window ).scrollTop() > scrollBreakpoint && isFixedHeader ) {
			    $header.addClass( 'scrolled' );
		    } else {
			    $header.removeClass( 'scrolled' );
		    }
	    };

	    // Setup scroll state management.
	    $( document ).ready( function() {
		    _setScrollBreakpoint();
		    _scrollState();

		    $( window ).resize( function() {
			    _setScrollBreakpoint();
		    	_scrollState();
		    } );

		    $( window ).scroll( _scrollState );
	    }  );


	    // Add an event listener for elastic scroll.
        UA.isSafari() && document.addEventListener( 'mousewheel', function ( e ) {
	        scrollHandler( e );
        }, false );
    },

    /**
     * Handle Local scroll.
     *
     * @since 1.0.0
     */
    setupScroll = function() {
	    var $header = null,
		    $links  = null,
		    url     = null,

	    _cleanupLinks = function() {
		    $links = $links.not('[href="#"]:not([data-target])');
	    },

        _setup = function() {
	        $links.each( function( i, link ) {
	        	$( link ).click( _scroll );
	        } );
        },

		/**
	     * Obtain gap (in px) produced by WP Admin bar.
	     *
	     * @since   1.0.0
	     *
		 * @returns {int}
	     *
	     * @private
	     */
		_getAdminBarGap = function() {
			var $adminBar = $( '#wpadminbar' );

			if ( $adminBar.length && 'fixed' === $adminBar.css( 'position' ) ) {
				return $adminBar.outerHeight();
			}

			return 0;
		},

		/**
	     * Obtain gap (in px) produced by fixed header.
	     *
	     * @since   1.0.0
	     *
		 * @returns {int}
	     *
	     * @private
	     */
		_getHeaderGap = function() {
			var headerFixed            = $header.hasClass( 'fixed' ) && ! $header.hasClass( 'vertical' ) && ( 'fixed' === $header.css( 'position' ) ),
				headerGap              = _getAdminBarGap(),
				headerShrinkable       = headerFixed && $header.hasClass( 'shrinkable' ),
				$logo                  = $( '#logo' ),
				headerShrinkableHeight = 0;

			if ( headerShrinkable && ! $header.hasClass( 'shrunk' ) && $( window ).width() >= breakpoint ) {
				headerShrinkableHeight = ( parseInt( $logo.css( 'padding-top' ) ) + parseInt( $logo.css( 'padding-bottom' ) ) ) / 2;
				headerShrinkableHeight += $logo.hasClass( 'has-image' ) ? $logo.height() * 0.9 : $logo.height();
			} else if ( headerShrinkable ) {
				headerShrinkableHeight = $header.outerHeight();
				headerShrinkableHeight -= $logo.hasClass( 'has-image' ) ? $logo.height() : 0;
				headerShrinkableHeight += $logo.hasClass( 'has-image' ) ? $logo.height() * 0.9 : 0;
			}

			headerGap += headerShrinkable ? headerShrinkableHeight : ( headerFixed ? $header.outerHeight() : 0 );

			return headerGap;
		},

		/**
	     * Scroll to local target.
	     *
	     * @since   1.0.0
	     *
		 * @param   {event} e
	     *
	     * @private
	     */
		_scroll = function( e ) {
			e.preventDefault();
			e.stopPropagation();

		    var target = $( this ).data( 'target' ) || $( this ).attr( 'href' );

			if ( ! $( target ).length ) {
				return;
			}

			var targetPosition = $( target ).offset().top - _getHeaderGap();

			if ( targetPosition < 0 ) {
				targetPosition = 0;
			}

			$( window ).scrollTo( targetPosition, 300 );
		};

		$( document ).ready( function() {
			$header = $( '#header' );
			$links  = $( 'body' ).find( 'a[data-target], #header a[href^="#"], #top-bar a[href^="#"]' );
			url     = $.NiceLazyScript( 'scrollTo' );

			_cleanupLinks();

			// Return early if we don't have any targets.
			if ( ! $links.length ) {
				return;
			}

			if ( typeof $.fn.scrollTo !== 'function' && url ) {
				$.getScript( url, _setup );
			} else if ( typeof $.fn.scrollTo === 'function' ) {
				_setup();
			} else if ( typeof $.fn.scrollTo !== 'function' ) {
				$.NiceDev.log( 'Warning: LocalScroll is not initialized.' );
			}
		} );
    },

    /**
     * Handle browser compatibility.
     *
     * @since 1.0.0
     */
    browserCompat = function() {
	    // Support object-fit properties for IE.
	    $( document ).ready( function() {
		    if ( 'function' !== typeof objectFitImages ) {
			    return;
		    }
		    objectFitImages();
	    } );
    },

    /**
     * Load required scripts on demand.
     */
    loadScripts = function() {
	    /**
	     * Load Nice Tabs Widget script if required.
	     *
	     * @private
	     */
        var _loadTabsWidgetMaybe = function() {
	        var url = $.NiceLazyScript( 'tabsWidget' ),
		        $tabs = $( 'ul.nice-tabs' );

	        if ( ! $tabs.length ) {
		        return;
	        }

	        if ( 'object' !== typeof window.NiceTabsWidget && url ) {
		        $.getScript( url );
	        }
        };

	    // Load Nice Tabs Widget script if required.
        $( document ).ready( _loadTabsWidgetMaybe );
    },

	/**
	 * Obtain information about the current user agent.
	 *
	 * @since 1.0.3
	 */
	UserAgent = function() {
		this.isLynx = function() {
		    return navigator.userAgent.match( /(Lynx)/ );
	    };
		this.isGecko = function() {
		    return /gecko\/\d/i.test( navigator.userAgent );
	    };
		this.isOpera = function( strict ) {
			var isOpera = /Opera\//.test( navigator.userAgent );

			if ( Boolean( strict ) ) {
				isOpera = isOpera || /OPR\//.test( navigator.userAgent );
			}

		    return isOpera;
	    };
	    this.isNS4 = function() {
		    return navigator.userAgent.match( /(Mozilla\/4)/ );
	    };
	    this.isSafari = function() {
		    return /Apple Computer/.test( navigator.vendor );
	    };
	    this.isChrome = function() {
	    	return /Chrome\//.test( navigator.userAgent ) && ! this.isIE();
	    };
	    this.isIE = function() {
	    	var n = {
	    		isMSIE: function() {
	    			return /MSIE \d/.test( navigator.userAgent );
			    },
			    isTrident: function() {
	    			return /Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec( navigator.userAgent );
			    },
			    isEdge: function() {
				    return /Edge\//.test( navigator.userAgent );
			    }
		    };

		    return ( n.isMSIE() || n.isTrident() || n.isEdge() );
	    };
	},

	/**
	 * Restore user agent class to deal with possible caching issues.
	 *
	 * @since 1.0.0
	 */
	setUserAgent = function() {
    	var supportedUserAgents = [
    		'lynx',
	        'gecko',
	        'opera',
	        'ns4',
	        'safari',
	        'chrome',
		    'ie'
	    ],
        currentUserAgent = 'unknown',
    	UA = new UserAgent();

		if ( UA.isLynx() ) {
    		currentUserAgent = 'lynx';
	    } else if ( UA.isOpera() ) {
    		currentUserAgent = 'opera';
	    } else if ( UA.isNS4() ) {
    		currentUserAgent = 'ns4';
	    } else if ( UA.isGecko() ) {
		    currentUserAgent = 'gecko';
	    } else if ( UA.isSafari() ) {
    		currentUserAgent = 'safari';
	    } else if ( UA.isChrome() ) {
    		currentUserAgent = 'chrome';
	    } else if ( UA.isIE() ) {
    		currentUserAgent = 'ie';
	    }

	    $( document ).ready( function() {
		    var $body = $( 'body' ).removeClass( supportedUserAgents.join( ' ' ) ).addClass( currentUserAgent );

		    if ( UA.isIE() && parseInt( $.browser.version ) <= 11 ) {
			    $body.addClass( 'ie' + parseInt( $.browser.version ) );
		    }
	    } );
	},

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	init = function() {
		if ( fired ) {
			return;
		}

		setupBodyState();
		autoHideHeader();
		hideHeaderUntilScroll();
		setupShrinkHeader();
		setupHeaderHeight();
		setupScroll();
		toggleVerticalHiddenNavigation();
		handleSearch();
		backToTop();
		fireScrollHandler();
		setUserAgent();
		browserCompat();
		loadScripts();

		fired = true;
	};

	// Expose the ready function to the world.
	return {
		init: init
	};

} )( jQuery );

NiceGeneral.init();
