/**
 * Smart by NiceThemes.
 *
 * NiceHeadingParallax
 *
 * Manage Parallax implementation on the Heading Section slider.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceHeadingParallax = ( function( $ ) {
	'use strict';

	/**
	 * Reference to the heading object.
	 *
	 * @var {jQuery}
	 */
	var $heading,

	/**
	 * Current scroll position in pixels.
	 *
	 * @var {int}
	 */
	scrollPosition,

	/**
	 * Percentage of current scroll position. This value should be relative to
	 * the height of the slider element.
	 *
	 * @var {int}
	 */
	scrolledPercent,

	/**
	 * Check if the module has already been initialized.
	 *
	 * @type {boolean}
	 */
	fired = false,

	/**
	 * Initialize Parallax functionality if required.
	 */
	setup = function() {
		// Obtain heading object.
		$heading = $( '#heading.parallax' );

		// Return early if the queried object doesn't exist.
		if ( ! $heading ) {
			return;
		}

		// Fire for the first time.
		fireParallax();

		// Setup scrolling functionality.
		$( window ).scroll( fireParallax );
		$( window ).resize( fireParallax );
	},

	/**
	 * Reset slider to its original status.
	 *
	 * @returns {boolean}
	 */
	reset = function() {
		$heading.css( 'background-position', 'center top' );

		$heading.find( '.overlay' ).css( {
			'opacity': 0
		} );

		return true;
	},

	/**
	 * Apply parallax styles to slider.
	 *
	 * @returns {boolean}
	 */
	applyParallax = function() {
		if ( ! $heading.data( 'original-position-x' ) && ! $heading.data( 'original-position-y' ) ) {
			var background = $heading.css( 'background-position' ).split( ' ' );

			$heading.data( 'original-position-x', background[0] );
			$heading.data( 'original-position-y', background[1] );
		}

		var positionX = $heading.data( 'original-position-x' );

		$heading.css( 'background-position', positionX + ' calc(' + positionX + ' - ' + ( scrollPosition * 0.5 ) + 'px)' );

		$heading.find( '.overlay' ).css( {
			'opacity': ( scrolledPercent / 100 )
		} );

		return true;
	},

	/**
	 * Obtain current scroll position in pixels.
	 *
	 * @returns {int}
	 */
	getScrollPosition = function() {
		return $( window ).scrollTop();
	},

	/**
	 * Obtain percentage of current scrolled position
	 *
	 * @returns {number}
	 */
	getScrolledPercent = function() {
		if ( 'undefined' === typeof scrollPosition ) {
			scrollPosition = $( window ).scrollTop();
		}

		return ( ( scrollPosition / 100 ) / ( $heading.height () / 100 ) ) * 100;
	},

	/**
	 * Apply parallax styles based on scroll position.
	 */
	fireParallax = function() {
		if ( $( window ).width() <= 768 ) {
			reset();

			return;
		}

		scrollPosition  = getScrollPosition();
		scrolledPercent = getScrolledPercent();

		if ( scrolledPercent <= 100 ) {
			applyParallax();
		}
	},

	/**
	 * Initialize heading parallax module.
	 */
	init = function() {
		if ( fired ) {
			return;
		}

		setup();

		Pace.on( 'done', function() {
			if ( fired ) {
				setup();
			}
		} );

		fired = true;
	};

	// Expose the ready function to the world.
	return {
		init: init
	};
} )( jQuery );

NiceHeadingParallax.init();
