/**
 * Smart by NiceThemes.
 *
 * NiceOverlayNavigation
 *
 * Handle overlay navigation.
 *
 * @since 1.0.0
 *
 * @package Smart
 */
var NiceOverlayNavigation = ( function( $ ) {
	'use strict';

	var fired = false,

    /**
     * Manage menu overlay functionality.
     *
     * @since 1.0.0
     */
    handleMenu = function() {
	    // Return early if we don't have an overlay menu.
	    if ( ! $( 'body' ).hasClass( 'has-overlay-menu' ) ) {
		    return;
	    }

        var $menu            = $( '#main-nav.overlay' ),
            $navigation      = $( '#navigation' ),
            $subMenuHolders  = $menu.find( '.menu-item-has-children, #header-search' ),
            $toggleButton    = $( '#toggle-nav-container' ).find( '.toggle-nav' ),
            $closeButton     = $( '#close-nav-container' ),

        /**
         * Open a given submenu.
         *
         * @since   1.0.0
         *
         * @param   $subMenuHolder
         * @returns {boolean}
         * @private
         */
        _openSubMenu = function( $subMenuHolder ) {
	        if ( $subMenuHolder.data( 'open' ) ) {
		        return false;
	        }

	        $subMenuHolder.find( '.sub-menu:first' ).slideToggle( 400 ).addClass( 'visible' );
	        $subMenuHolder.data( 'open', true );

	        return true;
        },

        /**
         * Close a given submenu.
         *
         * @since   1.0.0
         *
         * @param   $subMenuHolder
         * @returns {boolean}
         * @private
         */
        _closeSubMenu = function( $subMenuHolder ) {
	        if ( ! $subMenuHolder.data( 'open' ) ) {
		        return false;
	        }

	        $subMenuHolder.find( '.sub-menu:first' ).slideToggle( 400 ).removeClass( 'visible' );
	        $subMenuHolder.data( 'open', false );

	        return true;
        },

        /**
         * Close all submenus except the one passed as parameter and its
         * ancestors.
         *
         * @since   1.0.0
         *
         * @param   $subMenuHolder
         * @returns {boolean}
         * @private
         */
        _closeAllExcept = function( $subMenuHolder ) {
	        // Initialize list of exclusions.
	        var $exclude = [ $subMenuHolder ],

            // Obtain all parent submenus for the current one.
	        $parents = $( $subMenuHolder ).parents( '.menu-item-has-children' );

            // Add parents to list of exclusions.
	        if ( $parents.length ) {
		        $parents.each( function( $i, $obj ) {
			        $exclude.push( $obj );
		        } );
	        }

	        $subMenuHolders.each( function( $i, $obj ) {
		        // Only close submenus that are not present in the list.
				if ( $.inArray( $obj, $exclude ) < 0 ) {
					_closeSubMenu( $( $obj ) );
				}
	        } );
        },

        /**
         * Toggle state of submenus given one as reference.
         *
         * @since   1.0.0
         *
         * @param   $subMenuHolder
         * @private
         */
        _toggleSubmenus = function( $subMenuHolder ) {
	        _closeAllExcept( $subMenuHolder );

	        if ( ! _openSubMenu( $( $subMenuHolder ) ) ) {
		        _closeSubMenu( $( $subMenuHolder ) );
	        }
        },

        /**
         * Relocate the navigation toggle button.
         *
         * @since   1.0.0
         *
         * @private
         */
        _moveCloseButton = function() {
	        $closeButton.prependTo( $navigation );
        },

        /**
         * Add clickable icons to menus containing children.
         *
         * @since   1.0.0
         *
         * @private
         */
        _addIcons = function() {
			if ( ! $menu.hasClass( 'no-arrows' ) ) {
				$subMenuHolders.find( 'a:first' ).after( '<i class="fa fa-angle-down"></li>');
			}
        },

        /**
         * Move navigation at the end of body.
         *
         * @since   1.0.0
         * @private
         */
        _moveNavigation = function() {
	        $navigation.appendTo( 'body' );
	        $navigation.addClass( 'moved' );
        },

        /**
         * Show navigation menu.
         *
         * @since   1.0.0
         *
         * @param   e Possible event.
         * @private
         */
        _showNavigationMenu = function( e ) {
	        if ( 'undefined' !== typeof e ) {
		        e.preventDefault();
	        }

	        $navigation.fadeIn( 400, function() {
		        $navigation.addClass( 'visible' );
		        $menu.addClass( 'visible' );
	        } );
        },

        /**
         * Hide navigation menu.
         *
         * @since   1.0.0
         *
         * @param   e Possible event.
         * @private
         */
        _hideNavigationMenu = function( e ) {
	        if ( 'undefined' !== typeof e ) {
		        e.preventDefault();
	        }

	        $navigation.removeClass( 'visible' );
	        $menu.removeClass( 'visible' );

	        setTimeout( function() {
		        $navigation.fadeOut( 400 );
	        }, 600 );
        };

        // Fire actions on first load.
	    _moveNavigation();
	    _moveCloseButton();
	    _addIcons();

        // Setup actions to show and hide the navigation menu.
        $toggleButton.click( _showNavigationMenu );
        $closeButton.click( _hideNavigationMenu );

	    // Close navigation when pressing "esc" key.
	    $( document ).keyup( function( e ) {
		    if ( e.keyCode === 27 ) {
			    _hideNavigationMenu();
		    }
	    } );

        // Setup actions to toggle the state of submenus when clicking its parents.
        $subMenuHolders.each( function( $index, $subMenuHolder ) {
	        $( $subMenuHolder ).find( 'i:first' ).click( function( e ) {
		        e.preventDefault();
		        _toggleSubmenus( $subMenuHolder );
	        } );
        } );
    },

    init = function() {
    	if ( fired ) {
    		return;
	    }

        handleMenu();

        fired = true;
    };

    return {
    	init: init
    };
} )( jQuery );

NiceOverlayNavigation.init();
