/**
 * Smart by NiceThemes.
 *
 * Manage posts load via AJAX.
 *
 * @since   1.0.0
 * @package Smart
 */
var NicePostsLoader = ( function( $ ) {
	// Tell browsers we're not doing anything silly.
	'use strict';

	/**
	 * Define some basic variables.
	 *
	 * @since 1.0.0
	 */
	var masonry_container   = $( '#masonry-grid' ),
		page                = 1,
		loading             = false,

	/**
	 * Initialize Masonry after all images have loaded.
	 *
	 * @since 1.0.0
	 */
	initialize = function() {
		Pace.on( 'done', function() {
			load();
			$( '#posts-ajax-loader-button' ).css( 'display', 'block' );
		} );

		// Bind slider elements to future events.
		setupSliders();
		setupPlaylists();
	},

	setupPlaylists = function() {
		var $playlists = $( '.wp-playlist' );

		$playlists.each( function () {
			$( this ).addClass( 'active' );
		} );
	},

	reloadPlaylists = function() {
		var $playlists = $( '.wp-playlist' );

		$playlists.each( function () {
			if ( ! $( this ).hasClass( 'active' ) ) {
				$( this ).addClass( 'active' );

				return new WPPlaylistView( { el : this } );
			}
		} );
	},

    /**
     * Handle slider-related functionality.
     *
     * @since 1.0.0
     */
    setupSliders = function() {
	    var $previousSliders = ( 'undefined' === typeof $previousSliders ) ? null : $previousSliders,
	        $sliders         = $( '.slider.flexslider' );

	    // Unbind from previous events, so the same actions don't get executed more than once
	    // on the same element.
	    if ( null !== $previousSliders ) {
		    $previousSliders.each( function() {
			    $( this ).bind( 'start', reloadMasonry );
			    $( this ).bind( 'after', reloadMasonry );
		    } );
	    }

	    // Bind elements to slider events.
	    $sliders.each( function() {
		    $( this ).bind( 'start', reloadMasonry );
		    $( this ).bind( 'after', reloadMasonry );
	    } );

	    // Mark current sliders as old.
	    $previousSliders = $sliders;
    },

    /**
     * Handle Masonry reloading functionality.
     *
     * @since 1.0.0
     */
    reloadMasonry = function() {
	    masonry_container.isotope();
    },

	/**
	 * Load posts conditionally.
	 *
	 * @since 1.0.0
	 */
	load = function() {
		if ( postsLoader.onScroll ) {
			$( window ).scroll( function() {
				var content_offset = masonry_container.offset();
				if ( ( $( window ).scrollTop() + $( window ).height() ) > ( masonry_container.scrollTop() + masonry_container.height() + content_offset.top ) ) {
					loadMasonryBlogPosts();
				}
			} );
		} else {
			$( '#posts-ajax-loader-button' ).click( function() {
				loadMasonryBlogPosts();
				return false;
			} );
		}
	},

	/**
	 * Fire posts loader.
	 *
	 * @since 1.0.0
	 */
	loadMasonryBlogPosts = function() {
		if ( ! loading ) {
			loading = true;
			page++;
			loadPosts();
		}
	},

	/**
	 * Load posts via AJAX.
	 *
	 * @since 1.0.0
	 */
	loadPosts = function() {
		$.ajax( {
			type      : 'POST',
			url       : generalData.adminAjaxURL,
			data      : 'action=nice_more_posts_loader&singular=0&nonce=' + generalData.playNiceNonce + '&pageNumber=' + page,
			beforeSend: function () {
				if ( 1 != page ) {
					$( '#posts-ajax-loader-button' ).hide();
					$( '#posts-ajax-loader' ).show();
				}
			},
			success   : function( data ) {
				var $data = $( data );

				if ( ! $data.hasClass( 'no-more-posts' ) ) {
					masonry_container.append( $data )
						.trigger( 'gridReload' )
						.isotope( { transitionDuration: '400ms' } )
						.isotope( 'appended', $data );

					reloadPlaylists();

					setTimeout( function() {
						$( '#posts-ajax-loader' ).hide();
						$( '#posts-ajax-loader-button' ).show();

						loading = false;
					}, 400 );

					/**
					 * Apply Flexslider to galleries if the object exists.
					 */
					if ( typeof NiceSlider !== 'undefined' ) {
						// Setup events for new sliders.
						setupSliders();

						// Set a timeout, so we make sure all the images inside the slider
						// use all the available height.
						setTimeout( function() {
							// Fire slider functionality.
							NiceSlider.ready();

							// Notify about DOM changes.
							$( document ).trigger( 'NiceDOMChanged' );
						}, 400 );
					}

				} else {
					$( '#posts-ajax-loader' ).hide();
					$( '#posts-ajax-loader-button' ).hide();
					$( '#content' ).append( data );
				}
			},
			error     : function ( jqXHR, textStatus, errorThrown ) {
				$( '#posts-ajax-loader' ).hide();
				$( '#posts-ajax-loader-button' ).css( 'visibility', 'visible' );
				console.log( jqXHR + ' :: ' + textStatus + ' :: ' + errorThrown );
			}
		} );
	},

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	ready = function() {
		initialize();
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};

} )( jQuery );

jQuery( NicePostsLoader.ready );
