<?php
/**
 * Smart by NiceThemes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Custom_Navigation_Menu' ) ) :
/**
 * Setup custom navigation menu options in admin-side.
 *
 * @since 1.0.0
 */
class Nice_Custom_Navigation_Menu {
	/**
	 * Name of the walker class.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $walker = 'Nice_Walker_Nav_Menu_Edit';

	/**
	 * IDs of the menus where this object should have an impact.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $menu_ids = array();

	/**
	 * Duration of internal transients to manage menu item information.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected $transient_duration = 3600; // 1 hour.

	/**
	 * Nice_Custom_Navigation_Menu_Admin constructor.
	 *
	 * @param array $args
	 */
	public function __construct( $args = array() ) {
		if ( ! empty( $args ) ) {
			foreach ( $args as $key => $value ) {
				if ( property_exists( $this, $key ) ) {
					$this->{$key} = $value;
				}
			}
		}
	}

	/**
	 * Setup object hooks.
	 *
	 * @since 1.0.0
	 */
	public function init() {
		add_filter( 'wp_edit_nav_menu_walker', array( $this, 'edit_walker' ) );
		add_filter( 'wp_setup_nav_menu_item', array( $this, 'add_custom_nav_fields' ) );
		add_action( 'wp_update_nav_menu_item', array( $this, 'update_custom_nav_fields' ), 10, 2 );
		add_filter( 'nav_menu_css_class', array( $this, 'nav_menu_css_class' ), 10, 4 );
		add_filter( 'walker_nav_menu_start_el', array( $this, 'walker_nav_menu_start_el' ), 10, 4 );
	}

	/**
	 * Modify the name of the walker class for the edit page of navigation menus.
	 *
	 * @since  1.0.0
	 *
	 * @see    Nice_Walker_Nav_Menu_Edit
	 *
	 * @param  string $class
	 *
	 * @return string
	 */
	public function edit_walker( $class ) {
		return class_exists( $this->walker ) ? $this->walker : $class;
	}

	/**
	 * Add custom properties to WP_Post object in menu item context.
	 *
	 * This is a pretty shitty way to do this, but we don't really have another choice right now :(
	 *
	 * @since  1.0.0
	 *
	 * @param  mixed|stdClass|WP_Post $menu_item
	 *
	 * @return WP_Post
	 */
	public function add_custom_nav_fields( $menu_item ) {
		$menu_item->{'megamenu'}     = get_post_meta( $menu_item->ID, '_menu_item_megamenu', true );
		$menu_item->{'image_column'} = get_post_meta( $menu_item->ID, '_menu_item_image_column', true );

		return $menu_item;
	}

	/**
	 * Update custom navigation fields.
	 *
	 * @since 1.0.0
	 *
	 * @param int $menu_id
	 * @param int $menu_item_db_id
	 */
	public function update_custom_nav_fields( $menu_id, $menu_item_db_id ) {
		// Return early if we don't have a valid menu ID.
		if ( ! is_int( $menu_id ) ) {
			return;
		}

		// Obtain megamenu value if possible.
		if ( isset( $_POST['menu-item-megamenu'][ $menu_item_db_id ] ) ) {
			$megamenu_value = (string) wp_unslash( $_POST['menu-item-megamenu'][ $menu_item_db_id ] );
		}

		// Save megamenu value.
		update_post_meta( $menu_item_db_id, '_menu_item_megamenu', ! empty( $megamenu_value ) ? $megamenu_value : '' );

		// Obtain image column value if possible.
		if ( isset( $_POST['menu-item-image-column'][ $menu_item_db_id ] ) ) {
			$image_value = esc_url( wp_unslash( $_POST['menu-item-image-column'][ $menu_item_db_id ] ) );
		}

		// Save image column value.
		update_post_meta( $menu_item_db_id, '_menu_item_image_column', ! empty( $image_value ) ? $image_value : '' );

		self::reset_children( $menu_item_db_id );
	}

	/**
	 * Modify CSS classes of a menu item.
	 *
	 * @since  1.0.0
	 *
	 * @param  array    $classes
	 * @param  WP_Post  $item
	 * @param  stdClass $args
	 * @param  int      $depth
	 *
	 * @return array
	 */
	public function nav_menu_css_class( array $classes, $item, stdClass $args, $depth ) {
		if ( empty( $args->menu_id ) || 'main-nav' !== $args->menu_id ) {
			return $classes;
		}

		if ( ! in_array( $args->menu_id, $this->menu_ids, true ) ) {
			return $classes;
		}

		$in_megamenu = $this->in_megamenu( $item, $args, $depth );

		if ( ! ( $children = self::get_children( $item->ID ) ) ) {
			$children = self::reset_children( $item->ID );
		}

		if ( 0 === (int) $depth && ! empty( $item->megamenu ) && 'on' === $item->megamenu && $children > 0 && $children <= 6 ) {
			array_push( $classes, 'megamenu', 'columns-' . $children );
		}

		if ( 1 === (int) $depth && $in_megamenu && ! empty( $item->image_column ) && ( $url = esc_url( $item->image_column ) ) ) {
			$classes[] = 'image-column';
		}

		$classes = array_unique( $classes );

		return $classes;
	}

	/**
	 * Handle starting walker element.
	 *
	 * @since  1.0.0
	 *
	 * @param  string   $output
	 * @param  WP_Post  $item
	 * @param  int      $depth
	 * @param  stdClass $args
	 *
	 * @return string
	 */
	public function walker_nav_menu_start_el( $output = '', $item, $depth = 0, stdClass $args ) {
		// Return early if we're not in the main navigation menu.
		if ( 'main-nav' !== $args->menu_id ) {
			return (string) $output;
		}

		// Add extra markup for text labels.
		if ( in_array( 'label-popular', $item->classes, true ) ) {
			$output = str_replace( '</a>', '<span class="menu-item-label text-label text-label-popular">' . esc_html__( 'Popular', 'smart' ) . '</span></a>', $output );
		} elseif ( in_array( 'label-new', $item->classes, true ) ) {
			$output = str_replace( '</a>', '<span class="menu-item-label text-label text-label-new">' . esc_html__( 'New', 'smart' ) . '</span></a>', $output );
		} elseif ( in_array( 'label-hot', $item->classes, true ) ) {
			$output = str_replace( '</a>', '<span class="menu-item-label text-label text-label-hot">' . esc_html__( 'Hot', 'smart' ) . '</span></a>', $output );
		} elseif ( in_array( 'label-sale', $item->classes, true ) ) {
			$output = str_replace( '</a>', '<span class="menu-item-label text-label text-label-sale">' . esc_html__( 'Sale', 'smart' ) . '</span></a>', $output );
		}

		$in_megamenu = $this->in_megamenu( $item, $args, $depth );

		// Use image instead of text if the URL of an image is set for this field at depth 1 of a megamenu.
		if ( 1 === (int) $depth && $in_megamenu && ! empty( $item->image_column ) && ( $url = esc_url( $item->image_column ) ) ) {
			// Replicate original image attributes.
			$atts = array();
			$atts['title']  = ! empty( $item->attr_title ) ? $item->attr_title : '';
			$atts['target'] = ! empty( $item->target )     ? $item->target     : '';
			$atts['rel']    = ! empty( $item->xfn )        ? $item->xfn        : '';
			$atts['href']   = ! empty( $item->url )        ? $item->url        : '';

			/**
			 * From WP Core:
			 *
			 * Filters the HTML attributes applied to a menu item's anchor element.
			 *
			 * @since 3.6.0
			 * @since 4.1.0 The `$depth` parameter was added.
			 *
			 * @param array $atts {
			 *     The HTML attributes applied to the menu item's `<a>` element, empty strings are ignored.
			 *
			 *     @type string $title  Title attribute.
			 *     @type string $target Target attribute.
			 *     @type string $rel    The rel attribute.
			 *     @type string $href   The href attribute.
			 * }
			 * @param object $item  The current menu item.
			 * @param array  $args  An array of wp_nav_menu() arguments.
			 * @param int    $depth Depth of menu item. Used for padding.
			 */
			$atts = apply_filters( 'nav_menu_link_attributes', $atts, $item, $args, $depth );

			$attributes = '';
			foreach ( $atts as $attr => $value ) {
				if ( ! empty( $value ) ) {
					$value = ( 'href' === $attr ) ? esc_url( $value ) : esc_attr( $value );
					$attributes .= ' ' . $attr . '="' . $value . '"';
				}
			}

			$output = $args->before;
			$output .= '<a ' . $attributes . '>';
			$output .= $args->link_before . '<img src="' . $url . '" />' . $args->link_after;
			$output .= '</a>';
			$output .= $args->after;
		}

		return (string) $output;
	}

	/**
	 * Check if we're inside a megamenu from a menu item context.
	 *
	 * @since  1.0.0
	 *
	 * @param  WP_Post  $item
	 * @param  stdClass $args
	 * @param  int      $depth
	 *
	 * @return bool
	 */
	protected function in_megamenu( $item, stdClass $args, $depth = 0 ) {
		static $in_megamenu = false;

		if ( ! in_array( $args->menu_id, $this->menu_ids, true ) ) {
			return false;
		}

		if ( 0 === $depth ) {
			$in_megamenu = ( ! empty( $item->megamenu ) && 'on' === $item->megamenu );
		}

		return $in_megamenu;
	}

	/**
	 * Set children of a menu item.
	 *
	 * @since 1.0.0
	 *
	 * @param int $menu_item_id
	 */
	protected function set_children( $menu_item_id = 0 ) {
		global $wpdb;

		$children = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT COUNT(*) FROM $wpdb->postmeta WHERE meta_key = %s AND meta_value = %d", '_menu_item_menu_item_parent',
				$menu_item_id
			)
		);

		// Save new transients.
		set_transient( md5( 'nav_menu_item_children_' . $menu_item_id ), $children, $this->transient_duration );
	}

	/**
	 * Remove children of a menu item.
	 *
	 * @since 1.0.0
	 *
	 * @param int $menu_item_id
	 */
	protected function remove_children( $menu_item_id = 0 ) {
		// Delete saved transients.
		if ( $this->get_children( $menu_item_id ) ) {
			delete_transient( md5( 'nav_menu_item_children_' . $menu_item_id ) );
		}
	}

	/**
	 * Remove, set and obtain the number of children for a menu item.
	 *
	 * @since  1.0.0
	 *
	 * @param  int $menu_item_id
	 *
	 * @return int
	 */
	protected function reset_children( $menu_item_id = 0 ) {
		$this->remove_children( $menu_item_id );
		$this->set_children( $menu_item_id );

		return $this->get_children( $menu_item_id );
	}

	/**
	 * Obtain number of children for a menu item.
	 *
	 * @since  1.0.0
	 *
	 * @param  int $menu_item_id
	 *
	 * @return int
	 */
	protected function get_children( $menu_item_id = 0 ) {
		return get_transient( md5( 'nav_menu_item_children_' . $menu_item_id ) );
	}
}
endif;
