<?php
/**
 * Smart by NiceThemes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Walker_Nav_Menu' ) ) :
/**
 * Class Nice_Walker_Nav_Menu
 *
 * Extend default walker functionality to show the site logo as a menu item.
 *
 * @package Smart
 * @since   1.0.0
 * @uses    Walker_Nav_Menu
 */
class Nice_Walker_Nav_Menu extends Walker_Nav_Menu {
	/**
	 * Used to check if we're inside a megamenu at a given time.
	 *
	 * @since 1.0.0
	 *
	 * @var bool
	 */
	protected $is_megamenu = false;

	/**
	 * Current position in menu using top parent level as reference.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected $current_parent_position = 0;

	/**
	 * Number of items before the logo gets printed if using the centered inline layout.
	 *
	 * @since 1.0.0
	 *
	 * @var null
	 */
	protected $logo_breakpoint = null;

	/**
	 * Duration (in miliseconds) of transients where internal data is stored.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	public $transient_duration = 3600; // One hour.

	/**
	 * Setup class hooks.
	 *
	 * @since 1.0.0
	 */
	public function init() {
		$navigation_style     = nice_get_option( '_header_navigation' );
		$navigation_alignment = nice_get_option( '_header_top_navigation_alignment' );

		// Modify location of site logo.
		if ( 'default' === $navigation_style && 'logo-centered-inline' === $navigation_alignment ) {
			remove_action( 'nice_logo', 'nice_logo_display', 10 );
			add_filter( 'nice_logo_nav_menu', array( $this, 'logo_nav_menu' ), 10 );
		}

		add_action( 'wp_update_nav_menu_item', array( $this, 'remove_breakpoint' ) );
	}

	/**
	 * Extend parent starting element.
	 *
	 * @see   Walker_Nav_Menu::start_el()
	 * @since 1.0.0
	 *
	 * @param string           $output
	 * @param WP_Post|stdClass $item
	 * @param int              $depth
	 * @param array|stdClass   $args
	 * @param int              $id
	 */
	public function start_el( &$output, $item, $depth = 0, $args = array(), $id = 0 ) {
		$this->set_breakpoint_maybe( (object) $args );

		if ( 0 === (int) $depth ) {
			$this->current_parent_position++;
			$this->is_megamenu = ( 'on' === $item->megamenu ); // Reset megamenu indicator.
		}

		if ( $this->logo_breakpoint === $this->current_parent_position && 0 === (int) $depth ) {
			/**
			 * @hook nice_logo_nav_menu
			 *
			 * The output of the logo should be returned here.
			 *
			 * @since 1.0.0
			 *
			 * Hooked here:
			 * @see nice_logo_nav_menu() - 10
			 */
			$output .= apply_filters( 'nice_logo_nav_menu', '' );
		}

		$args_before = $args->{'before'};

		// Wrap megamenu elements.
		if ( ( 1 === (int) $depth ) && $this->is_megamenu ) {
			if ( in_array( 'background-image', $item->classes ) ) {
				$classes    = array();
				$searchword = 'background-';
				$matches    = array_filter( $item->classes, function ( $var ) use ( $searchword ) {
					return preg_match( "/\b$searchword\b/i", $var );
				} );

				if ( ! empty( $matches ) ) {
					foreach ( $matches as $match ) {
						if ( 'background-image' !== $match ) {
							unset( $item->classes[ array_search( $match, $item->classes ) ] );
						}
						
						$classes[] = $match;
					}
				}

				$classes[] = 'megamenu-column';

				$args->{'before'} = '<div ' . nice_css_classes( $classes, false ) . ' style="background-image: url(\'' . $item->description . '\');">';
			} else {
				$args->{'before'} = '<div class="megamenu-column">';
			}
		}

		parent::start_el( $output, $item, $depth, $args, $id );

		$args->{'before'} = $args_before;
	}

	/**
	 * Extend parent closing element.
	 *
	 * @see   Walker_Nav_Menu::end_el()
	 * @since 1.0.0
	 *
	 * @param string           $output
	 * @param WP_Post|stdClass $item
	 * @param int              $depth
	 * @param array|stdClass   $args
	 */
	public function end_el( &$output, $item, $depth = 0, $args = array() ) {
		$args_after = $args->{'after'};

		// Close wrap for megamenu elements.
		if ( ( 1 === (int) $depth ) && $this->is_megamenu ) {
			$args->{'after'} = '</div>';
		}

		parent::end_el( $output, $item, $depth, $args );

		$args->{'after'} = $args_after;
	}

	/**
	 * Replace the last occurrence of a string inside a string.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $search
	 * @param  string $replace
	 * @param  string $subject
	 *
	 * @return string
	 */
	protected static function replace_last( $search, $replace, $subject ) {
		$pos = strrpos( $subject, $search );

		if ( $pos !== false ) {
			$subject = substr_replace( $subject, $replace, $pos, strlen( $search ) );
		}

		return $subject;
	}

	/**
	 * Display logo using framework functionality.
	 *
	 * @uses  nice_logo()
	 *
	 * @since 1.0.0
	 */
	public function logo_nav_menu() {
		$container_classes = array(
			'menu-item',
		);

		if ( ! nice_header_vertical() ) {
			if ( nice_bool_option( '_hide_logo' ) ) {
				$container_classes[] = 'hide';
			}

			if ( nice_bool_option( '_header_fixed' ) && nice_bool_option( '_header_fixed_hide_logo' ) ) {
				$container_classes[] = 'hide-fixed';
			}
		}

		/**
		 * @hook nice_logo_images
		 *
		 * Hook in here to modify the list of images.
		 *
		 * @since 1.0.0
		 */
		$logo_images = apply_filters( 'nice_logo_images', array() );

		return nice_logo( array(
			'before'        => '<li id="logo-container" ' . nice_css_classes( $container_classes, false ) . '><div id="logo" ' . nice_logo_class( false ) . '>',
			'after'         => '</div></li>',
			'images'        => $logo_images,
			'default_image' => nice_logo_default_image(),
			'height'        => nice_get_option( '_logo_height' ),
			'echo'          => false,
		) );
	}

	/**
	 * Remove breakpoint transient.
	 *
	 * @since 1.0.0
	 */
	public function remove_breakpoint() {
		delete_transient( md5( 'nice_nav_menu_logo_breakpoint' ) );
	}

	/**
	 * Create transient for logo breakpoint as needed.
	 *
	 * @since 1.0.0
	 *
	 * @param stdClass $args
	 */
	protected function set_breakpoint_maybe( stdClass $args ) {
		if ( is_null( $this->logo_breakpoint ) ) {
			$logo_breakpoint = get_transient( md5( 'nice_nav_menu_logo_breakpoint' ) );

			if ( false === $logo_breakpoint ) {
				$menu_elements = wp_get_nav_menu_items( $args->menu );
				$parent_elements = 0;

				foreach( $menu_elements as $element ) {
					if( 0 === (int) $element->menu_item_parent ) {
						$parent_elements++;
					}
				}

				$logo_breakpoint = ceil( $parent_elements / 2 ) + 1;

				set_transient( md5( 'nice_nav_menu_logo_breakpoint' ), $logo_breakpoint, $this->transient_duration );
			}

			$this->logo_breakpoint = (int) $logo_breakpoint;
		}
	}

	/**
	 * Get current value of logo breakpoint transient.
	 *
	 * @since  1.0.0
	 *
	 * @return mixed|bool|int
	 */
	protected function get_breakpoint() {
		return get_transient( md5( 'nice_nav_menu_logo_breakpoint' ) );
	}
}
endif;
