<?php
/**
 * Smart by NiceThemes.
 *
 * Load integrations and dependencies.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_remove_likes_filters' ) ) :
add_action( 'after_setup_theme', 'nice_likes_integration_remove_filters' );
/**
 * Remove filters for likes in content, so they can be added later to post meta.
 *
 * @since 1.0.0
 */
function nice_likes_integration_remove_filters() {
	remove_filter( 'the_content', 'nice_likes_content_filter', 999 );
	remove_filter( 'the_excerpt', 'nice_likes_content_filter', 999 );
}
endif;

if ( ! function_exists( 'nice_likes_integration_remove_actions' ) ) :
add_action( 'after_setup_theme', 'nice_likes_integration_remove_actions' );
/**
 * Remove actions for Likes.
 *
 * @global Nice_Likes_Public $nice_likes_public Current instance of public Likes object.
 *
 * @since 1.0.0
 */
function nice_likes_integration_remove_actions() {
	/**
	 * Remove plugin styles, so we can use our own theme's.
	 *
	 * @since 1.0.0
	 */
	remove_action( 'nice_likes_plugin_enqueue_styles', 'nice_likes_enqueue_styles' );

	/**
	 * Also remove scripts if we're in the home page.
	 *
	 * @since 1.0.0
	 */
	if ( nice_is_homepage() ) {
		remove_action( 'nice_likes_plugin_enqueue_scripts', 'nice_likes_enqueue_scripts' );
	}
}
endif;

if ( ! function_exists( 'nice_likes_post_meta' ) ) :
add_filter( 'nice_post_footer_meta', 'nice_likes_integration_post_meta' );
/**
 * Add Likes to post header meta.
 *
 * @since  1.0.0
 *
 * @param  string $post_meta Unfiltered HTML.
 * @return mixed             Filtered HTML.
 */
function nice_likes_integration_post_meta( $post_meta ) {
	$post_likes = nice_likes( array(
			'before' => '<section class="likes">',
			'after'  => '</section>',
		)
	);

	$post_meta = str_replace( '[post_likes]', $post_likes, $post_meta );

	return $post_meta;
}
endif;

if ( ! function_exists( 'nice_likes_integration_single_footer_meta_template' ) ) :
add_filter( 'nice_post_footer_meta_template', 'nice_likes_integration_single_footer_meta_template', 10 );
/**
 * Add Likes functionality to footer meta in single posts.
 *
 * @since  1.0.0
 *
 * @param  string $template Unfiltered footer meta template.
 *
 * @return string
 */
function nice_likes_integration_single_footer_meta_template( $template = '' ) {
	$location = is_single() ? 'posts' : ( is_page() ? 'pages' : 'other' );

	if ( nice_likes_apply_to( $location ) ) {
		$template = str_replace( '[post_comments]', '[post_comments] [post_likes]', $template );
	}

	return $template;
}
endif;

if ( ! function_exists( 'nice_likes_integration_args' ) ) :
add_filter( 'nice_likes_default_args', 'nice_likes_integration_args' );
/**
 * Modify default arguments for nice_likes() function.
 *
 * @see    nice_likes()
 *
 * @since  1.0.0
 *
 * @param  array $args Default arguments.
 *
 * @return array
 */
function nice_likes_integration_args( array $args = array() ) {
	return array_merge( $args, array(
			'icon'         => '<i class="fa fa-heart-o"></i>', // Use default FontAwesome icon from theme.
			'loading_text' => '<i class="fa fa-circle-o-notch fa-spin"></i>',
		)
	);
}
endif;

/**
 * Remove un-like text.
 *
 * @since 1.0.0
 */
add_filter( 'nice_likes_unlike_template', '__return_empty_string' );

if ( ! function_exists( 'nice_likes_integration_modify_postfix' ) ) :
add_filter( 'nice_likes_count_text_postfix', 'nice_likes_integration_modify_postfix', 10, 2 );
/**
 * Modify postfix texts.
 *
 * @since 1.0.0
 *
 * @param  string $text    Current postfix text (not used).
 * @param  int    $count   Current number of likes.
 *
 * @return string
 */
function nice_likes_integration_modify_postfix( $text, $count ) {
	static $template_name = null;
	static $post          = null;

	/**
	 * Try to obtain current post and current template name.
	 */
	if ( ! empty( $_REQUEST['fromurl'] ) ) {
		$post = $post ? $post : get_post( url_to_postid( wp_unslash( $_REQUEST['fromurl'] ) ) );

		if ( $post ) {
			$template_name = $template_name ? $template_name : get_post_meta( $post->ID, '_wp_page_template', true );
		}
	}

	/**
	 * Return original postfix text in case we're not in a list of posts, or
	 * this action hasn't been requested from one.
	 */
	if ( ( ( nice_likes_doing_ajax() && isset( $_REQUEST['singular'] ) && wp_unslash( $_REQUEST['singular'] ) ) || is_singular() )
	     && 'page-templates/blog.php' !== $template_name
	) {
		return $text;
	}

	/**
	 * Construct HTML for a simplified count of likes.
	 */
	$text = '<span class="likes-count">' . $count . '</span>';

	return $text;
}
endif;

if ( ! function_exists( 'nice_likes_integration_vars' ) ) :
add_filter( 'nice_likes_vars', 'nice_likes_integration_vars' );
/**
 * Modify JavaScript variables to fit our integration.
 *
 * @since  1.0.0
 *
 * @param  array $args
 *
 * @return array
 */
function nice_likes_integration_vars( array $args = array() ) {
	if ( is_page_template( 'page-templates/blog-masonry.php' ) ) {
		$args['singular'] = false;
	}

	return $args;
}
endif;

if ( ! function_exists( 'nice_likes_integration_apply_to_other' ) ) :
add_filter( 'nice_likes_apply_to_other', 'nice_likes_integration_apply_to_other', 10, 2 );
/**
 * Show Likes in blog and Masonry blog templates when needed.
 *
 * @since 1.0.0
 *
 * @param  bool  $apply
 * @param  array $settings
 *
 * @return bool
 */
function nice_likes_integration_apply_to_other( $apply = false, array $settings = array() ) {
	/**
	 * Apply custom rule only when:
	 *
	 * - We're seeing a page with a Masonry blog template
	 * - or we're seeing a page with a Blog template
	 * - or we're in AJAX context (needed to reload posts in Masonry blog).
	 */
	if ( ! $apply && (
			   is_page_template( 'page-templates/blog-masonry.php' )
			|| is_page_template( 'page-templates/blog.php' )
		    || nice_likes_doing_ajax()
	     )
	) {
		$apply = ! empty( $settings['add_to']['other'] ) && ! empty( $settings['add_to']['posts'] );
	}

	return $apply;
}
endif;

if ( ! function_exists( 'nice_portfolio_integration_likes_apply_to_current' ) ) :
add_action( 'the_post', 'nice_portfolio_integration_likes_apply_to_current' );
/**
 * Hook into `nice_portfolio_likes_apply_to_current()` when we're seeing a
 * blog template.
 *
 * @see   nice_portfolio_likes_apply_to_current()
 *
 * @since 1.0.0
 */
function nice_portfolio_integration_likes_apply_to_current() {
	if ( is_page_template( 'page-templates/blog.php' ) ) {
		add_filter( 'nice_portfolio_likes_apply_to_current', '__return_true' );
	}

	/**
	 * Make sure this action runs only once.
	 */
	remove_action( current_action(), __FUNCTION__ );
}
endif;

if ( ! function_exists( 'nice_likes_integration_masonry_post_footer_meta_likes' ) ) :
add_action( 'nice_masonry_post_footer_meta', 'nice_likes_integration_masonry_post_footer_meta_likes', 15 );
/**
 * Print post comments inside footer meta.
 *
 * @since 1.0.0
 */
function nice_likes_integration_masonry_post_footer_meta_likes() {
	nice_likes( array(
			'before' => '<section class="entry-likes entry-meta inline-block inherit-skin alignright">',
			'after'  => '</section>',
			'echo'   => true,
		)
	);
}
endif;
