<?php
/**
 * Smart by NiceThemes.
 *
 * Manage interactions between integrations of both Portfolio and Likes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_portfolio_integration_likes_setup' ) ) :
add_action( 'nice_portfolio_setup_page', 'nice_portfolio_integration_likes_setup' );
/**
 * Setup Likes functionality everywhere within the portfolio related pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Nice_Portfolio_PublicDisplayResponder::setup_page()
 */
function nice_portfolio_integration_likes_setup() {
	// Return early if Likes is not installed.
	if ( ! class_exists( 'Nice_Likes_Plugin' ) ) {
		return;
	}

	/**
	 * Allow Likes in homepage (blog only).
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_home', '__return_true' );

	/**
	 * Allow Likes in archive pages.
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_archive', '__return_true' );

	/**
	 * Allow Likes in category pages.
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_category', '__return_true' );

	/**
	 * Allow Likes in tag pages.
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_tag', '__return_true' );

	/**
	 * Allow Likes in selected portfolio pages.
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_page', '__return_true' );

	/**
	 * Allow Likes in single pages (posts, pages, etc).
	 *
	 * @since 1.0.0
	 */
	add_filter( 'nice_portfolio_likes_apply_to_singular', '__return_true' );

	/**
	 * Remove default Likes location before single projects.
	 *
	 * @since 1.0.0
	 */
	remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_likes_before_single_project_content', 0 );

	/**
	 * Remove default Likes location after single projects.
	 *
	 * @since 1.0.0
	 */
	remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_likes_after_single_project_content', 999 );
}
endif;

if ( ! function_exists( 'nice_portfolio_integration_modify_likes_postfix' ) ) :
add_filter( 'nice_likes_count_text_postfix', 'nice_portfolio_integration_modify_likes_postfix', 10, 3 );
/**
 * Modify postfix texts.
 *
 * @since 1.0.0
 *
 * @param  string $text    Current postfix text (not used).
 * @param  int    $count   Current number of likes.
 * @param  int    $id      ID of current post.
 *
 * @return string
 */
function nice_portfolio_integration_modify_likes_postfix( $text, $count, $id ) {
	/**
	 * Obtain post ID and post object from our context (doing AJAX or not)
	 */
	$post_id = ( nice_likes_doing_ajax() && isset( $_POST['fromurl'] ) ) ? url_to_postid( $_POST['fromurl'] ) : get_the_ID();
	$post    = $post_id ? get_post( $post_id ) : get_post( $id );

	/**
	 * If the obtained post ID is the same as received one (not a child item),
	 * then return default text.
	 */
	if ( intval( $id ) === intval( $post_id ) ) {
		return $text;
	}

	/**
	 * If we're not in the selected portfolio page or looking at the contents
	 * of a shortcode, then return default text.
	 */
	if ( ! nice_portfolio_is_page( $post_id ) && ! has_shortcode( $post->post_content, 'portfolio' ) ) {
		return $text;
	}

	/**
	 * Create simplified text for portfolio page and shortcode.
	 */
	$text = '<span class="likes-count">' . $count . '</span>';

	return $text;
}
endif;

if ( ! function_exists( 'nice_portfolio_integration_loop_likes' ) ) :
add_action( 'nice_portfolio_integration_loop_project_content', 'nice_portfolio_integration_loop_likes', 30 );
/**
 * Display Likes for looped projects.
 *
 * @since 1.0.0
 *
 * @param null|int $project_id
 */
function nice_portfolio_integration_loop_likes( $project_id = null ) {
	// Return early if Likes is not installed.
	if ( ! class_exists( 'Nice_Likes_Plugin' ) ) {
		return;
	}

	if ( nice_likes_apply_to( nice_portfolio_post_type_name() ) ) {
		nice_likes( array(
				'echo'    => true,
				'post_id' => $project_id = nice_portfolio_get_valid_ID( $project_id ),
				'before' => '<section class="entry-likes entry-meta inline-block inherit-skin">',
				'after' => '</section>',
				'show_postfix' => false,
			)
		);
	}
}
endif;

if ( ! function_exists( 'nice_portfolio_integration_single_project_likes' ) ) :
add_action( 'nice_portfolio_integration_single_project_details', 'nice_portfolio_integration_single_project_likes', 0 );
/**
 * Display Likes for a single project.
 *
 * @since 1.0.0
 *
 * @param int $project_id
 */
function nice_portfolio_integration_single_project_likes( $project_id = null ) {
	// Return early if Likes is not installed.
	if ( ! class_exists( 'Nice_Likes_Plugin' ) ) {
		return;
	}

	if ( nice_likes_apply_to( nice_portfolio_post_type_name() ) ) {
		nice_likes( array(
				'echo'    => true,
				'post_id' => nice_portfolio_get_valid_ID( $project_id ),
				'before'  => '<section class="portfolio-project-likes">',
				'after'   => '</section>',
			)
		);
	}
}
endif;
