<?php
/**
 * Smart by NiceThemes.
 *
 * This file adds actions and filters to hooks that are fired within templates,
 * setting up the visibility of single projects, portfolio pages and other
 * plugin elements within the theme.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/** ==========================================================================
 *  HTML wrappers.
 *  ======================================================================= */
/**
 * Remove default content wrappers. We're using our own wrappers inside templates.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see All portfolio page templates.
 */
remove_action( 'nice_portfolio_before_main_content', 'nice_portfolio_wrapper_start', 10 );
remove_action( 'nice_portfolio_after_main_content', 'nice_portfolio_wrapper_end', 10 );

/** ==========================================================================
 *  Navigation.
 *  ======================================================================= */

/**
 * Remove default page navigation. We're replacing it with the custom theme
 * functionality inside templates.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see All portfolio page templates.
 */
remove_action( 'nice_portfolio_after_main_content', 'nice_portfolio_loop_projects_page_navigation', 0 );

/** ==========================================================================
 *  Content.
 *  ======================================================================= */

/**
 * Remove default thumbnail view from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_thumbnail', 10 );

/**
 * Remove default embed view from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_embed', 20 );

/**
 * Relocate gallery view for single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_gallery', 40 );
add_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_gallery', 10 );

/**
 * Remove default list of project attributes from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_meta', 50 );

/**
 * Remove default list of project categories from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_categories', 60 );

/**
 * Remove default list of project tags view from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_tags', 70 );

/**
 * Remove default list of related projects from single project pages.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see Single project content template.
 */
remove_action( 'nice_portfolio_single_project_content', 'nice_portfolio_single_project_related', 80 );

/**
 * Display project attributes in single project details area.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see template-parts/single-project/details.php
 *
 * @uses nice_portfolio_single_project_meta()
 */
add_action( 'nice_portfolio_integration_single_project_details', 'nice_portfolio_single_project_meta', 10 );

/**
 * Display project categories in single project sidebar.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see includes/plugin-integrations/nice-portfolio/single-project/categories.php
 *
 * @uses nice_portfolio_single_project_categories()
 */
add_action( 'nice_portfolio_integration_single_project_details', 'nice_portfolio_single_project_categories', 20 );

/**
 * Display project tags in single project sidebar.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see template-parts/single-project/project-tags.php
 *
 * @uses nice_portfolio_single_project_tags()
 */
add_action( 'nice_portfolio_integration_single_project_details', 'nice_portfolio_single_project_tags', 30 );

/**
 * Add additional classes to container element depending on the current portfolio page.
 *
 * @since  1.0.0
 *
 * Hook origin:
 * @see nice_get_container_class()
 *
 * @uses nice_portfolio_integration_get_container_class()
 */
add_filter( 'nice_container_class', 'nice_portfolio_integration_get_container_class' );

/**
 * Remove default project title from looped projects.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see nice_portfolio_loop_projects()
 */
remove_action( 'nice_portfolio_loop_project', 'nice_portfolio_loop_project_title', 30 );

/**
 * Relocate project title inside looped projects.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see includes/plugin-integrations/nice-portfolio/templates/loop/project/content.php
 *
 * @uses nice_portfolio_loop_project_title()
 */
add_action( 'nice_portfolio_integration_loop_project_content', 'nice_portfolio_loop_project_title', 10 );

/**
 * Show project comments inside looped projects.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see includes/plugin-integrations/nice-portfolio/templates/loop/project/content.php
 *
 * @uses nice_portfolio_integration_loop_comments()
 */
add_action( 'nice_portfolio_integration_loop_project_content', 'nice_portfolio_integration_loop_comments', 20 );

/**
 * Load template for permalink markup inside looped projects.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see nice_portfolio_loop_projects()
 *
 * @uses nice_portfolio_integration_loop_project_permalink()
 */
add_action( 'nice_portfolio_loop_project', 'nice_portfolio_integration_loop_project_permalink', 15 );

/**
 * Load template for project content inside looped projects.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see nice_portfolio_loop_projects()
 *
 * @uses nice_portfolio_integration_loop_project_content()
 */
add_action( 'nice_portfolio_loop_project', 'nice_portfolio_integration_loop_project_content', 30 );

/** ==========================================================================
 *  Widget.
 *  ======================================================================= */
/**
 * Modify markup for widget title.
 *
 * Hook origin:
 * @see Nice_Portfolio_WP_Widget::widget_title()
 */
add_filter( 'nice_portfolio_widget_title', 'nice_portfolio_integration_widget_title', 10, 2 );

/**
 * Add classes to the current widget's container.
 *
 * Hook origin:
 * @see nice_portfolio_get_widget_class()
 */
add_filter( 'nice_portfolio_widget_class', 'nice_portfolio_integration_widget_class', 10 );

/** ==========================================================================
 *  Other.
 *  ======================================================================= */

/**
 * Add classes to portfolio class.
 *
 * @since 1.0.0
 *
 * Hook origin:
 * @see nice_portfolio_get_class()
 *
 * @uses  nice_portfolio_integration_get_class()
 */
add_filter( 'nice_portfolio_class', 'nice_portfolio_integration_get_class' );
