/**
 * Smart by NiceThemes.
 *
 * NiceVCPostsPagination
 *
 * Manage posts pagination for VC.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceVCPostsPagination = ( function( $ ) {
	// Tell browsers we're not doing anything silly.
	'use strict';

    /**
     * Current pages for every Isotope grid, listed by container ID.
     *
     * @since 1.0.0
     *
     * @type {Array}
     */
    var containerCurrentPages = [],

    /**
     * Current maximum number of pages for every Isotope grid, listed by
     * container ID
     *
     * @since 1.0.0
     *
     * @type {Array}
     */
    containerMaxPages = [],

    /**
     * Current queried category for every Isotope grid, listed by container ID.
     *
     * @since 1.0.0
     *
     * @type {Array}
     */
    containerCurrentCategory = [],

    /**
     * AJAX responses for every Isotope grid, listed by container ID.
     *
     * @since 1.0.0
     *
     * @type {Array}
     */
    containerResponses = [],

    /**
     * List of pages that we should wait to load until they're available.
     *
     * @since 1.0.0
     *
     * @type {Array}
     */
    containerDelayedPages = [],

    /**
     * Set current page for a given container.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    page
     */
    setContainerCurrentPage = function( $container, page ) {
	    if ( ! $container.attr( 'id' ) || page < 1 ) {
		    return;
	    }

        containerCurrentPages[ $container.attr( 'id' ) ] = page;
    },

    /**
     * Obtain current page for a given container.
     *
     * @since   1.0.0
     *
     * @param   {jQuery} $container
     *
     * @returns {int}
     */
    getContainerCurrentPage = function( $container ) {
	    if ( ! $container.attr( 'id' ) ) {
		    return null;
	    }

	    return containerCurrentPages[ $container.attr( 'id' ) ];
    },

    /**
     * Set current category for a given container.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {string} category
     */
    setContainerCurrentCategory = function( $container, category ) {
	    if ( ! $container.attr( 'id' ) ) {
		    return;
	    }

        containerCurrentCategory[ $container.attr( 'id' ) ] = category;
    },

    /**
     * Obtain current category for a given container.
     *
     * @since   1.0.0
     *
     * @param   {jQuery} $container
     *
     * @returns {string}
     */
    getContainerCurrentCategory = function( $container ) {
	    if ( ! $container.attr( 'id' ) ) {
		    return null;
	    }

	    return containerCurrentCategory[ $container.attr( 'id' ) ];
    },

    /**
     * Set current maximum number of pages for a given container.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    pages
     */
    setContainerMaxPages = function( $container, pages ) {
	    if ( ! $container.attr( 'id' ) ) {
		    return;
	    }

	    containerMaxPages[ $container.attr( 'id' ) ] = pages;
    },

    /**
     * Obtain current maximum number of pages for a given container.
     *
     * @since   1.0.0
     *
     * @param   {jQuery} $container
     *
     * @returns {int}
     */
    getContainerMaxPages = function( $container ) {
	    if ( ! $container.attr( 'id' ) ) {
		    return null;
	    }

	    return containerMaxPages[ $container.attr( 'id' ) ];
    },

    /**
     * Set response for a page of a given container.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    page
     * @param {string} category
     * @param {JSON}   response
     */
    setContainerResponse = function( $container, page, category, response ) {
	    if ( ! $container.attr( 'id' ) || page < 1 ) {
		    return;
	    }

	    var id = $container.attr( 'id' );

	    if ( 'undefined' === typeof containerResponses[ id ] ) {
		    containerResponses[ id ] = [];
	    }

	    if ( 'undefined' === typeof containerResponses[ id ][ category ] ) {
		    containerResponses[ id ][ category ] = [];
	    }

	    if ( 'undefined' === typeof containerResponses[ $container.attr( 'id' ) ][ category ][ page ] ) {
		    containerResponses[ $container.attr( 'id' ) ][ category ][ page ] = response;
	    }
    },

    /**
     * Get response for a page of a given container.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    page
     * @param {string} category
     *
     * @returns {JSON}
     */
    getContainerResponse = function( $container, page, category ) {
	    if ( ! $container.attr( 'id' ) || page < 1 ) {
		    return;
	    }

	    var response;

	    var id = $container.attr( 'id' );

	    try {
		    response = containerResponses[ id ][ category ][ page ];
	    } catch ( error ) {
		    $.NiceDev.log( error.message );
	    }

	    return response;
    },

    /**
     * Set the status of a page in order to know whether its loading process
     * should be delayed.
     *
     * @since 1.0.0
     *
     * @param {jQuery}  $container
     * @param {int}     page
     * @param {string}  category
 	 * @param {boolean} status
     */
    setContainerDelayedPage = function( $container, page, category, status ) {
        if ( ! $container.attr( 'id' ) || page < 1 ) {
	        return;
        }

	    var id = $container.attr( 'id' );

	    if ( 'undefined' === typeof containerDelayedPages[ id ] ) {
		    containerDelayedPages[ id ] = [];
	    }

	    if ( 'undefined' === typeof containerDelayedPages[ id ][ category ] ) {
		    containerDelayedPages[ id ][ category ] = [];
	    }

        containerDelayedPages[ id ][ category ][ page ] = status;
    },

    /**
     * Get the status of a page in order to know whether its loading process
     * should be delayed.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    page
     * @param {string} category
	 *
 	 * @returns {JSON}
     */
    getContainerDelayedPage = function( $container, page, category ) {
        if ( ! $container.attr( 'id' ) || page < 1 ) {
	        return;
        }

	    var id = $container.attr( 'id' );

		return containerDelayedPages[ id ][ category ][ page ];
    },

    /**
     * Handle how posts are loaded via AJAX for infinite scroll and
     * pagination.
     *
     * @since 1.0.0
     */
    handlePostLoading = function() {
	    var $containers = $( '.nice-vc-posts-element' );

	    $containers.each( function() {
		    if ( ! $( this ).attr( 'id' ) ) {
			    return;
		    }

		    var $container = $( this ),
			    $content   = $container.find( '.nice-vc-posts-element-content' ),
			    $loader    = $container.find( '.posts-loader' );

		    // Initialize responses for the current container.
		    containerResponses[ $( this ).attr( 'id' ) ] = [];

			if ( $content.data( 'max-pages' ) <= 1 ) {
				return;
			}

		    // Return early if we don't need to process pagination.
		    if ( ! $content.data( 'pagination' ) || 'disabled' === $content.data( 'pagination' ) ) {
			    return;
		    }

		    // Set the current page for first load.
		    setContainerCurrentPage( $container, 1 );

		    // Set the maximum number of pages for first load.
		    setContainerMaxPages( $container, $content.data( 'max-pages' ) );

		    // Set current category.
		    setContainerCurrentCategory( $container, window.NiceVCShortcodeCollection[ $container.attr( 'id' ) ].category );
			var category = getContainerCurrentCategory( $container );

		    // Fetch page contents for the first two pages before loading them.
		    for ( var page = 1; page <= 2; page++ ) {
			    if ( page <= $content.data( 'max-pages' ) ) {
				    preLoad( $container, page, $content.data( 'pagination' ), category );
				    setContainerDelayedPage( $container, page, category, false );
			    }
		    }

		    // Schedule asynchronous fetching for the next page once a page is
		    // added to the DOM.
		    $container.bind( 'pageLoaded', function( event, response, page, category ) {
			    var nextPage = page + 1;

			    if ( nextPage <= response.max_pages ) {
				    preLoad( $container, nextPage, $content.data( 'pagination' ), category );
				    setContainerDelayedPage( $container, nextPage, category, false );
			    }
		    } );

		    // Fire the required pagination type.
		    switch ( $content.data( 'pagination' ) ) {
			    case 'infinite-scroll':
				    processInfiniteScroll( $container, $content, $loader );
				    break;
			    case 'click-to-load':
				    processClickToLoad( $container, $content, $loader );
				    break;
			    case 'standard':
				    processStandardPagination( $container, $content, $loader );
				    break;
			    default:
				    break;
		    }

		    // Fire filter handler on first load.
		    processFiltering( $container, $content, $content.data( 'pagination' ) );
	    } );
    },

    /**
     * Process how posts are loaded via AJAX for automatic infinite scroll pagination.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {jQuery} $content
     * @param {jQuery} $loader
     */
	processInfiniteScroll = function( $container, $content, $loader ) {
	    var lock = false;

	    /**
	     * Process page loading.
	     *
	     * @since 1.0.0
		 *
		 * @param {string} category
	     */
	    var loadNext = function( category ) {
		    var page = getContainerCurrentPage( $container );

		    // Return early if the required page exceeds the maximum number of pages in the query.
		    if ( page >= getContainerMaxPages( $container ) ) {
			    return;
		    }

		    // Remove "hidden" class from loader element.
		    // This class may have been added in a previous iteration of this same function.
		    $loader.removeClass( 'hidden' );

		    // Add "loading" class to loader element.
		    $loader.addClass( 'loading' );

		    // Rise page number to request the next page.
		    page++;

		    // Obtain response for required page.
		    var response = getContainerResponse( $container, page, category );

		    // Delay loading if we still haven't received a response.
		    if ( ! response || 'undefined' === typeof response ) {
			    setContainerDelayedPage( $container, page, category, true );
			    return;
		    }

		    // Remove "loading" class from loader.
		    $loader.removeClass( 'loading' );

		    // Add response contents to Isotope grid.
		    // We're assuming here that Isotope is already applied to the $content element.
		    var $items = $( response.content );

		    $content.append( $items );

		    if ( $content.is( '.isotope-grid' ) ) {
			    $content.isotope( { transitionDuration: 0 } );
		    }

		    $content.trigger( 'gridReload' );

		    if ( $content.is( '.isotope-grid' ) ) {
			    $content.isotope( 'appended', $items );
		    }

		    // Setup animations if needed.
		    if ( $content.data( 'animated' ) ) {
			    $items.NiceScheduleAnimation();
		    }

		    // Notify about DOM changes.
		    $( document ).trigger( 'NiceDOMChanged' );

		    // Add "hidden" class to loader if we reach the maximum number of pages.
		    if ( page >= getContainerMaxPages( $container ) ) {
			    $loader.addClass( 'hidden' );
		    }

		    // Reset current page number.
		    setContainerCurrentPage( $container, page );

		    // Unlock loading via scroll.
		    lock = false;

		    // Allow hooking here.
		    $container.trigger( 'pageLoaded', [ response, page, category ] );
	    };

	    var scroll = function() {
		    // Return early if loading via scroll is locked.
		    if ( lock ) {
			    return;
		    }

		    var bottomPosition = ( $( window ).scrollTop() + $( window ).height() ),
			    bottomBreakpoint = ( ( $container.height() + $container.offset().top ) / 100 ) * 85;

		    // Return early if scroll is not close enough to the bottom of the container.
		    if ( bottomPosition <= bottomBreakpoint ) {
		        return;
		    }

		    /**
		     * Load a given page asynchronously.
		     *
		     * @since 1.0.0
		     *
		     * @param {Event}  event
		     * @param {Array}  response
		     * @param {int}    page
		     * @param {string} category
		     */
		    var handleAsyncLoading = function( event, response, page, category ) {
			    if ( getContainerDelayedPage( $( this ), page, category ) ) {
				    loadNext( category );
			    }
		    };

		    // Lock loading via scroll.
		    lock = true;

		    // Load required page.
		    loadNext( getContainerCurrentCategory( $container ) );

		    // Setup asynchronous loading.
		    $container.bind( 'preLoaded', handleAsyncLoading );
	    };

	    // Setup scrolling functionality.
		$( window ).scroll( scroll );

	    // Setup actions after a filter is processed.
	    $container.bind( 'filtered', function( event, response ) {
		    lock = false;

		    // Add response contents to Isotope grid.
		    var $previousItems = $content.find( '.item' );

		    $content.css( 'opacity', 0 );

		    setTimeout( function() {
		    	$content.isotope( 'remove', $previousItems )
				    .html( response.content )
					.trigger( 'gridReload' )
					.isotope( 'reloadItems' )
					.isotope( { transitionDuration: 0 } );

			    // Setup animations if needed.
			    if ( $content.data( 'animated' ) ) {
			    	$content.find( '.item' ).NiceScheduleAnimation();
			    }

			    $content.animate( { 'opacity': 1 }, 500 );

			    // Notify about DOM changes.
			    $( document ).trigger( 'NiceDOMChanged' );
		    }, 500 );
	    } );
	},

    /**
     * Process how posts are loaded via AJAX for click-to-load infinite scroll.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {jQuery} $content
     * @param {jQuery} $loader
     */
	processClickToLoad = function( $container, $content, $loader ) {
	    /**
	     * Process page loading.
	     *
	     * @since 1.0.0
		 *
		 * @param {string} category
	     */
	    var loadNext = function( category ) {
		    var page = getContainerCurrentPage( $container );

		    // Return early if the required page exceeds the maximum number of pages in the query.
		    if ( page >= getContainerMaxPages( $container ) ) {
			    return;
		    }

		    // Remove "hidden" class from loader element.
		    // This class may have been added in a previous iteration of this same function.
		    $loader.removeClass( 'hidden' );

		    // Add "loading" class to loader element.
		    $loader.addClass( 'loading' );

		    // Rise page number to request the next page.
		    page++;

		    // Obtain response for required page.
		    var response = getContainerResponse( $container, page, category );

		    // Delay loading if we still haven't received a response.
		    if ( ! response || 'undefined' === typeof response ) {
			    $.NiceDev.log( 'Delaying page ' + page + ' for click-to-load.' );
			    setContainerDelayedPage( $container, page, category, true );
			    return;
		    }

			$loader.removeClass( 'loading' );

		    // Add response contents to Isotope grid.
		    // We're assuming here that Isotope is already applied to the $content element.
		    var $items = $( response.content );
		    $content.append( $items );

		    if ( $content.is( '.isotope-grid' ) ) {
		    	$content.isotope( { transitionDuration: 0 } );
		    }

		    $content.trigger( 'gridReload' );

		    if ( $content.is( '.isotope-grid' ) ) {
		        $content.isotope( 'appended', $items );
		    }

		    // Setup animations if needed.
		    if ( $content.data( 'animated' ) ) {
		    	$items.NiceScheduleAnimation();
		    }

		    // Notify about DOM changes.
		    $( document ).trigger( 'NiceDOMChanged' );

		    // Add "hidden" class to loader if we reach the maximum number of pages.
		    if ( page >= getContainerMaxPages( $container ) ) {
			    $loader.addClass( 'hidden' );
		    }

		    // Reset current page number.
		    setContainerCurrentPage( $container, page );

		    // Allow hooking here.
		    $container.trigger( 'pageLoaded', [ response, page, category ] );
	    },

	    /**
	     * Add a preloaded page to the DOM.
	     *
	     * @since 1.0.0
	     *
	     * @param {Event}  event
	     * @param {Array}  response
	     * @param {int}    page
	     * @param {string} category
	     */
	    handlePreloadedPage = function( event, response, page, category ) {
		    if ( getContainerDelayedPage( $( this ), page, category ) ) {
			    loadNext( category );
		    }
	    };

	    // Setup actions when button is clicked.
	    $loader.find( '.load' ).click( function( event ) {
			event.preventDefault();

		    // Setup asynchronous loading.
		    $container.one( 'preLoaded', handlePreloadedPage );

		    // Load required page.
		    loadNext( getContainerCurrentCategory( $container ) );
		} );

	    // Setup actions when a filter is clicked.
	    $container.on( 'filterClicked', function() {
		    $loader.addClass( 'hidden' );
	    } );

	    // Setup actions after filter is processed.
	    $container.bind( 'filtered', function( event, response ) {
		    var $previousItems = $content.find( '.item' );

		    $content.css( 'opacity', 0 );

		    // Re-draw Isotope grid.
		    setTimeout( function() {
			    $content.isotope( { transitionDuration: 0 } )
				    .isotope( 'remove', $previousItems );

			    $content.html( response.content )
				    .trigger( 'gridReload' )
				    .isotope( 'reloadItems' )
				    .isotope( { transitionDuration: 0 } );

			    // Setup animations if needed.
			    if ( $content.data( 'animated' ) ) {
				    $content.find( '.item' ).NiceScheduleAnimation();
			    }

			    $content.animate( { 'opacity': 1 }, 500 );

			    // Notify about DOM changes.
			    $( document ).trigger( 'NiceDOMChanged' );

			    // Style loader properly.
			    if ( getContainerMaxPages( $container ) > 1 ) {
				    $loader.removeClass( 'hidden' );
			    } else {
				    $loader.addClass( 'hidden' );
			    }
		    }, 500 );
	    } );

	    // Remove "loading" class from loader.
	    $container.bind( 'preLoaded', function() {
		    $loader.removeClass( 'loading' );
	    } );
	},

    /**
     * Process how posts are loaded via AJAX for standard pagination.
     *
     * @since 1.0.0
     *
     * @param {jQuery}  $container
     * @param {jQuery}  $content
     * @param {jQuery}  $loader
     */
	processStandardPagination = function( $container, $content, $loader ) {
	    /**
	     * Process page loading.
	     *
	     * @since  1.0.0
	     *
	     * @param {jQuery} $link
	     * @param {string} category
	     */
	    var load = function( $link, category ) {
		    var page = $link.data( 'page' );

		    // Return early if the requested page is the same as the current one.
		    if ( page === getContainerCurrentPage( $container ) ) {
			    return;
		    }

		    // Remove "hidden" class from loader element.
		    // This class may have been added in a previous iteration of this same function.
		    $loader.removeClass( 'hidden' );

		    // Add "loading" class to loader element.
		    $loader.addClass( 'loading' );

		    // Obtain response for required page.
		    var response = getContainerResponse( $container, page, category );

		    // Delay loading if we still haven't received a response.
		    if ( ! response || 'undefined' === typeof response ) {
			    // Force pre-load in case we're switching to a non-consecutive page.
			    preLoad( $container, page, $content.data( 'pagination' ), category );
			    setContainerDelayedPage( $container, page, category, true );

			    return;
		    }

			var $previousItems = $content.find( '.item' );

			if ( $content.data( 'animated' ) ) {
				$previousItems.NiceRemoveAnimation();
			}

		    // Re-draw Isotope grid.
		    // We're assuming here that Isotope is already applied to the $content element.
		    setTimeout( function() {
		        $content.prepend( response.content );
			    $content.trigger( 'gridReload' );

		        if ( $content.is( '.isotope-grid' ) ) {
				    $content.isotope( 'reloadItems' )
					    .isotope( 'remove', $previousItems )
					    .isotope( { transitionDuration: 0 } );
			    } else {
		            $previousItems.remove();
			    }

			    // Notify about DOM changes.
			    $( document ).trigger( 'NiceDOMChanged' );

			    // Setup animations if needed.
			    if ( $content.data( 'animated' ) ) {
				    $content.find( '.item' ).NiceAnimate();
			    }
		    }, 1000 );

		    // Reset current page number.
		    setContainerCurrentPage( $container, page );

		    // Refresh pagination links, so they can be clickable again.
		    refreshPagination( response.pagination );

		    // Allow hooking here.
		    $container.trigger( 'pageLoaded', [ response, page, category ] );
	    },

        /**
         * Refresh pagination for requested page.
         *
         * @since 1.0.0
         *
         * @param {string} html
         */
        refreshPagination = function( html ) {
	        // Delay refreshing pagination.
	        setTimeout( function() {
		        // Replace loader contents with current pagination HTML.
		        $loader.html( html );

		        // Remove "loading" class from loader.
		        $loader.removeClass( 'loading' );

		        // Re-process pagination for new inserted elements.
		        setupClick();
	        }, 1000 );
        },

	    /**
	     * Handle clicks on navigation links.
	     *
	     * @since 1.0.0
	     */
	    setupClick = function () {
		    // Fire when clicking a page link.
		    $loader.find( 'a' ).click( function ( event ) {
			    event.preventDefault();

			    // Save current object to use it inside a callback.
			    var $link = $( this ),

			    /**
			     * Load a given page asynchronously.
			     *
			     * @since 1.0.0
			     *
			     * @param {Event}  event
			     * @param {Array}  response
			     * @param {int}    page
			     * @param {string} category
			     */
			    handleDelayedLoading = function ( event, response, page, category ) {
				    if ( getContainerDelayedPage( $( this ), page, category ) ) {
					    load( $link, category );
				    }

				    // Prevent recursion.
				    $container.unbind( 'preLoaded', handleDelayedLoading );
			    };

			    // Load required page.
			    load( $link, getContainerCurrentCategory( $container ) );

			    // Setup asynchronous loading.
			    $container.bind( 'preLoaded', handleDelayedLoading );
		    } );
	    };

	    // Setup actions when a filter is clicked.
	    $container.on( 'filterClicked', function() {
		    $loader.addClass( 'hidden' );
	    } );

	    // Setup actions after a filter is processed.
	    $container.bind( 'filtered', function( event, response ) {
		    var $previousItems = $content.find( '.item' );

		    $content.css( 'opacity', 0 );

			// Re-draw Isotope grid.
		    setTimeout( function() {
			    $content.isotope( 'remove', $previousItems );
			    $content.html( response.content )
				    .isotope( 'remove', $previousItems )
				    .trigger( 'gridReload' )
				    .isotope( 'reloadItems' )
				    .isotope( { transitionDuration: 0 } );

			    // Setup animations if needed.
			    if ( $content.data( 'animated' ) ) {
				    $content.find( '.item' ).NiceAnimate();
			    }

		        $content.animate( { 'opacity': 1 }, 500 );

			    // Notify about DOM changes.
			    $( document ).trigger( 'NiceDOMChanged' );

			    // Style loader properly.
			    if ( getContainerMaxPages( $container ) > 1 ) {
				    $loader.removeClass( 'hidden' );
			    } else {
				    $loader.addClass( 'hidden' );
			    }
		    }, 500 );

		    // Refresh pagination links, so they can be clickable again.
		    refreshPagination( response.pagination );
	    } );

	    // Setup clicks on first load.
	    setupClick();
	},

    /**
     * Process AJAX filtering when using pagination.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {jQuery} $content
     * @param {string} pagination
     */
    processFiltering = function( $container, $content, pagination ) {
	    var $filters = $container.find( 'a.filter' ),
		    $filterContainer = $container.find( 'section.filter' );

	    // Return early if we don't have any filters.
	    if ( ! $filters.length ) {
		    return;
	    }

	    /**
	     * Load first page for the current filter.
	     *
	     * @since 1.0.0
		 *
		 * @param {string} category
	     */
	    var load = function( category ) {
			// Obtain response for required page.
			var response = getContainerResponse( $container, 1, category );

		    // Delay loading if we still haven't received a response.
		    if ( ! response || 'undefined' === typeof response ) {
			    setContainerDelayedPage( $container, 1, category, true );
			    return;
		    }

			// Set current category.
			setContainerCurrentCategory( $container, category );

		    // Reset current page.
		    setContainerCurrentPage( $container, 1 );

		    // Reset maximum number of pages.
		    setContainerMaxPages( $container, response.max_pages );

		    // Allow hooking here.
		    // The filters handler doesn't load the response HTML on its own.
		    // That needs to be managed by every pagination module using it.
		    $container.trigger( 'filtered', [ response, $container, $content ] );

		    $.NiceDev.log( 'Filter was fired.' );
	    };

	    /**
	     * Pre-load pages for the current filter via AJAX.
	     *
	     * @since 1.0.0
	     */
	    var processAJAX = function( category ) {
		    $.ajax( {
			    type : 'POST',
			    url : window.generalData.adminAjaxURL,
			    data : {
				    action: 'nice_vc_integration_posts_shortcode_ajax',
				    shortcode_atts: window.NiceVCShortcodeCollection[ $container.attr( 'id' ) ],
				    shortcode_page: 1,
				    shortcode_pagination: pagination,
				    shortcode_category: category,
				    nonce: window.generalData.playNiceNonce
			    },
			    success : function( response ) {
				    response = JSON.parse( response );

				    setContainerMaxPages( $container, response.max_pages );

					setContainerResponse( $container, 1, category, response );

				    // Pre-load the first two pages.
				    for ( var page = 1; page <= 2; page++ ) {
					    if ( page <= response.max_pages ) {
						    preLoad( $container, page, pagination, category );
						    setContainerDelayedPage( $container, page, category, false );
					    }
				    }

				    $container.trigger( 'AJAXProcessed' );
			    }
		    } );
	    };

	    /**
	     * Load the first page asynchronously.
	     *
	     * @since 1.0.0
	     *
	     * @param {Event}  event
	     * @param {Array}  response
	     * @param {int}    page
	     * @param {string} category
	     */
	    var handleDelayedLoading = function( event, response, page, category ) {
		    // Make sure we only load contents if the first page is requested.
		    if ( getContainerDelayedPage( $container, page, category ) && page === 1 ) {
				// Prevent recursion.
				$container.unbind( 'preLoaded', handleDelayedLoading );

				load( category );
		    }
	    };

	    $( $filters ).each( function() {
		    var preProcessed = false,
			    locked = false;

		    $( this ).click( function() {
			    // Prevent filter from fire if the requested category is the same as the current
				// one.
				var category = $( this ).data( 'term' );

			    if ( category === getContainerCurrentCategory( $container ) ) {
					return;
			    }

				if ( locked ) {
				    return;
			    }

			    /**
			     * AJAX handler. This function only needs to run once by clicked link.
			     *
			     * @since 1.0.0
			     */
			    var AJAXProcessed = function() {
				    // Setup flags.
				    preProcessed = true;
				    locked = false;

				    // Prevent recursion.
				    $container.unbind( 'AJAXProcessed', AJAXProcessed );

				    // Process first load.
				    load( category );
			    };

				// Lock current process to prevent recursion.
				locked = true;

			    $container.trigger( 'filterClicked' );

			    if ( ! preProcessed ) {
					// Setup AJAX process.
					$container.bind( 'AJAXProcessed', AJAXProcessed );

					// Setup asynchronous loading.
					$container.bind( 'preLoaded', handleDelayedLoading );

					processAJAX( category );
				} else {
					// Load cached contents.
					load( category );

				    locked = false;
			    }
		    } );
		} );

	    $container.on( 'filterClicked', function() {
		    $filterContainer.addClass( 'filtering' );
	    } );

	    $container.on( 'filtered', function() {
		    $filterContainer.removeClass( 'filtering' );
	    } );
    },

    /**
     * Pre-load a page via AJAX.
     *
     * @since 1.0.0
     *
     * @param {jQuery} $container
     * @param {int}    page
     * @param {string} paginationMode
     * @param {string} category
     */
    preLoad = function( $container, page, paginationMode, category ) {
	    var response = getContainerResponse( $container, page, category );

	    if ( response ) {
			$.NiceDev.log( 'Page content already cached.' );
		    return;
	    }

	    $.ajax( {
		    type : 'POST',
		    url : window.generalData.adminAjaxURL,
		    data : {
			    action: 'nice_vc_integration_posts_shortcode_ajax',
			    shortcode_atts: window.NiceVCShortcodeCollection[ $container.attr( 'id' ) ],
			    shortcode_page: page,
			    shortcode_pagination: paginationMode,
			    shortcode_category: category,
			    nonce: window.generalData.playNiceNonce
		    },
		    success : function( response ) {
			    var pageResponse = JSON.parse( response );

			    setContainerResponse( $container, page, category, pageResponse );

			    $container.trigger( 'preLoaded', [ pageResponse, page, category, paginationMode ] );

				$.NiceDev.log( 'loaded page ' + page + ' with ' + paginationMode );
		    }
	    } );
    },

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	ready = function() {
		handlePostLoading();
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};

} )( jQuery );

jQuery( NiceVCPostsPagination.ready );
