/**
 * Smart by NiceThemes.
 *
 * NiceVCGeneral
 *
 * Manage general implementation processes for VC.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceVCGeneral = ( function( $ ) {
	// Tell browsers we're not doing anything silly.
	'use strict';

	var videoBackgroundInit = function() {
		$( document ).bind( 'ready', videoBackgroundResize );
		$( window ).bind( 'resize', videoBackgroundResize );
		$( window ).bind( 'DOMSubtreeModified', videoBackgroundResize );

		if ( navigator.userAgent.indexOf( 'Chrome' ) > 0 && ! /Edge\/12./i.test( navigator.userAgent ) ) {
			$( '.nice-bg-video-container' ).each( function () {
				if ( $( this ).find( 'video source[type="video/webm"]' ).length > 0 ) {
					var webmSource = $( this ).find( 'video source[type="video/webm"]' ).attr( 'src' ) + "?id=" + Math.ceil( Math.random() * 10000 );
					var firstVideo = $( this ).find( 'video' ).get( 0 );
					firstVideo.src = webmSource;
					firstVideo.load();
				}
			} );
		}
	},

	videoBackgroundResize = function () {
		var min_width = 1200, // minimum video width allowed
			video_original_width,
			video_original_height; // original dimensions

		video_original_width  = 1280;
		video_original_height = 720;

		$( '.nice-bg-video-container' ).each( function () {
			var $containerHeight = $( this ).parents( '.vc_row' ).outerHeight();
			var $containerWidth  = $( this ).parents( '.vc_row' ).outerWidth();

			// set the video viewport to the window size
			$( this ).width( $containerWidth );
			$( this ).height( $containerHeight );

			// use largest scale factor of horizontal/vertical
			var scale_horizontal = $containerWidth / video_original_width;
			var scale_vertical   = ($containerHeight - $containerHeight) / video_original_height;
			var scale            = scale_horizontal > scale_vertical ? scale_horizontal : scale_vertical;

			// update minimum width to never allow excess space
			min_width = 1280 / 720 * ( $containerHeight + 40 );

			// don't allow scaled width < minimum video width
			if ( scale * video_original_width < min_width ) {
				scale = min_width / video_original_width;
			}

			// now scale the video
			$( this ).find( 'video' ).width( Math.ceil( scale * video_original_width ) );
			$( this ).find( 'video' ).height( Math.ceil( scale * video_original_height ) );

			// and center it by scrolling the video viewport
			$( this ).scrollLeft( ($( this ).find( 'video' ).width() - $containerWidth) / 2 );
			$( this ).scrollTop( ($( this ).find( 'video' ).height() - ($containerHeight)) / 2 );
		} );
	},

	initCounter = function () {
		var $counters = $( '.nice-counter .counter' ),
			url       = $.NiceLazyScript( 'counterup' ),

		/**
		 * Initialize counters.
		 */
		init = function() {
			$counters.NiceCounterUp( {
				delay : 10,
				time : 1500
			} );
		};

		if ( ! $counters.length ) {
			return;
		}

		if ( 'function' !== typeof $.fn.NiceCounterUp && url ) {
			$.getScript( url, init );
		} else if ( 'function' === typeof $.fn.NiceCounterUp ) {
			init();
		} else if ( 'function' !== typeof $.fn.NiceCounterUp ) {
			$.NiceDev.log( 'Warning: Counter is not initialized' );
		}
	},

	/**
	 * Unset default VC animations based on waypoints - we're not gonna need them.
	 *
	 * @since 1.0.0
	 */
	unsetVCAnimations = function () {
		delete window.vc_waypoints;
		window.vc_waypoints = function() {};
	},

	initCountdown = function () {
		var $counters = $( '[data-countdown]' ),
			url       = $.NiceLazyScript( 'countdown' ),

		/**
		 * Setup countdowns.
		 */
		setup = function() {
			var _doCountdown = function () {
				$counters = $( '[data-countdown]:not(.countdown-started)' );

				$counters.each( function () {
					var $this     = $( this ),
						finalDate = $( this ).data( 'countdown' );

					$this.countdown( finalDate, function ( event ) {
						$this.html( event.strftime( '%D <small>' + window.VCData.days + '</small> %H <small>' + window.VCData.hours + '</small> %M <small>' + window.VCData.minutes + '</small> %S <small>' + window.VCData.seconds + '</small>' ) );
					} );
					$this.addClass( 'countdown-started' );
				} );
			};

			_doCountdown();

			$( window ).on( 'vc_reload', _doCountdown );
		};

		if ( ! $counters.length ) {
			return;
		}

		if ( 'function' !== typeof $.fn.countdown && url ) {
			$.getScript( url, setup );
		} else if ( 'function' === typeof $.fn.countdown ) {
			setup();
		} else if ( 'function' !== typeof $.fn.countdown ) {
			$.NiceDev.log( 'Warning: Countdown is not initialized.' );
		}
	},

	/**
	 * Setup progress bars.
	 *
	 * @since 1.0.0
	 */
	initProgressBar = function() {
		var $bars = $( '.vc_single_progress_bar .vc_bar' );

		Pace.on( 'done', function() {
			$bars.each( function() {
				var $bar = $( this );

				$bar.NiceWaypoint( function() {
					$bar.css( { 'width': $bar.data( 'value' ) + '%' } );
				} );
			} );
		} );
	},

	/**
	 * Override VC Pie Chart functionality if we're using a full-page loader.
	 *
	 * @since 1.0.0
	 */
	initPieChart = function() {
		var useOverlay = ( 'full-page' === $( 'body' ).data( 'page-loader' ) );

		if ( ! useOverlay ) {
			return;
		}

		var triggerPieChart = window.vc_pieChart;

		// Start firing up pie charts when Pace is done and not later, since the
		// elements could seem a little jumpy.
		window.vc_pieChart = function() {
			Pace.on( 'done', triggerPieChart );
			$( window ).on( 'vc_reload', triggerPieChart );
		};
	},

	/**
	 * Resize title font for custom heading.
	 *
	 * @since 1.0.0
	 */
	resizeCustomHeading = function() {
		var sensitivity           = 2,
			minimumFontSizeFactor = 3,
			bodyFontSize          = parseInt( $( 'body' ).css( 'font-size' ) ),
			minimumFontSize       = bodyFontSize * minimumFontSizeFactor,
			mobileBreakpoint      = 800,
			$resizeableElements   = $( '.vc_custom_heading > .nice-heading-title' ),
			siteWidth;

		// Get the site width for responsive type
		if ( $( window ).width() >= mobileBreakpoint ) {
			siteWidth = parseInt( 1080 );
			// If we are below $mobile_break_point of viewport width, set $mobile_break_point as
			// site width
		} else {
			siteWidth = mobileBreakpoint;
		}

		// The resizing factor can be fine-tuned through a custom sensitivity; values below 1
		// decrease resizing speed
		var windowSiteWidthRatio = $( window ).width() / siteWidth,
			resizeFactor         = 1 - ( ( 1 - windowSiteWidthRatio ) * sensitivity );

		if ( $( window ).width() <= siteWidth ) {
			// Loop over all heading tags
			$resizeableElements.each( function() {
				var factorizedFontSize = $( this ).data( 'font-size' ) * resizeFactor;

				// Return early if we don't have a font-size.
				if ( isNaN( factorizedFontSize ) ) {
					return;
				}

				// Only decrease font-size if the we stay above $minimum_font_size
				if ( factorizedFontSize > minimumFontSize ) {
					$( this ).css( {
						'font-size': factorizedFontSize
					} );
				} else if ( $( this ).data( 'font-size' ) > minimumFontSize ) {
					// If decreased font size would become too small, natural font size is above
					// $minimum_font_size, set font size to $minimum_font_size

					$( this ).css( {
						'font-size': minimumFontSize
					} );
				}
			} );
		} else {
			// If window width is larger than site width, delete any resizing styles
			$resizeableElements.each(
				function() {
					// If initially an inline font size was set, restore it
					if ( $( this ).data( 'font-size' ) ) {
						$( this ).css( 'font-size', $( this ).data( 'font-size' ) );
						// Otherwise remove inline font size
					} else {
						$( this ).css( 'font-size', '' );
					}
				}
			);
		}
	},

	/**
	 * Setup resizing process for custom heading.
	 *
	 * @since 1.0.0
	 */
	initCustomHeading = function() {
		resizeCustomHeading();

		$( window ).on( 'resize', resizeCustomHeading );
		$( window ).on( 'orientationchange', resizeCustomHeading );
	},

	/**
	 * Setup cards gallery functionality
	 *
	 * @since 1.0.0
	 */
	initCardsGallery = function() {
		var $wrapper = $( '.nice-cards-gallery-wrapper' );

		if ( $wrapper.length ) {
			$wrapper.each( function () {
				var gallery = $( this ),
					cards   = gallery.find( '.card' ),
					side    = gallery.data( 'side' ),
					marginValue;

				switch ( side ) {
					case 'left':
						marginValue = '0 0 0 20%';
						break;
					case 'right':
						marginValue = '0 20% 0 0';
						break;
					case 'top':
						marginValue = '20% 0 0 0';
						break;
					case 'bottom':
						marginValue = '0 0 20% 0';
						break;
				}

				cards.each( function () {
					var card = $( this );
					card.click( function () {
						if ( ! cards.last().is( card ) ) {
							card.addClass( 'out' ).animate( {
								opacity : 0,
								margin : marginValue
							}, 200, function () {
								card.detach();
								card.insertAfter( cards.last() ).animate( {
									opacity : 1,
									margin : '0'
								}, 500, function () {
									card.removeClass( 'out' );
								} );
								cards = gallery.find( '.card' );
							} );
							return false;
						}
					} );
				} );
			} );
		}
	},

	/**
	 * Animate SVG images.
	 *
	 * @since 1.0.0
	 */
	initAnimatedSVG = function() {
		var $animatedSVG = $( 'svg.svg-animated' ),
			url          = $.NiceLazyScript( 'vivus' ),

		/**
		 * Setup animated SVG images.
		 */
		setupVivus = function() {
			$animatedSVG.each( function() {
				var id = $( this ).attr( 'id' ),
					animationData = { start: 'inViewport' },
					svg = null;

				if ( $( this ).data( 'delay' ) ) {
					animationData.delayStart = $( this ).data( 'delay' );
				}

				if ( $( this ).data( 'duration' ) ) {
					animationData.duration = $( this ).data( 'duration' );
				}

				svg = new Vivus( id, animationData, null );
			} );
		};

		if ( ! $animatedSVG.length ) {
			return;
		}

		if ( 'function' !== typeof Vivus && url ) {
			$.getScript( url, setupVivus );
		} else if ( 'function' === typeof Vivus ) {
			setupVivus();
		} else if ( 'function' !== typeof Vivus ) {
			$.NiceDev.log( 'Warning: Vivus is not initialized.' );
		}
	},

	/**
	 * Make sure parallax rows are correctly positioned once they enter the
	 * viewport.
	 *
	 * @since 1.0.3
	 */
	controlParallax = function() {
		$( '.vc_row.vc_parallax' ).NiceWaypoint( window.vc_rowBehaviour, { triggerOnce: false, offset: '100%' } );
	},

	/**
	 * Make sure rows are reloaded after the DOM changes. This can be specially
	 * useful for parallax backgrounds that appear after a row with variable
	 * height (such as a VC Posts module with infinite scroll).
	 *
	 * @since 1.0.3
	 */
	resetRows = function() {
		if ( 'function' !== typeof window.vc_rowBehaviour ) {
			return;
		}

		// Set a flag to check if the iframe is waiting to be reloaded.
		var waitForReload = false;

		$( document ).on( 'NiceDOMChanged', function() {
			if ( waitForReload ) {
				return;
			}

			waitForReload = true;

			// Avoid reloading rows too frequently.
			setTimeout( function() {
				window.vc_rowBehaviour();
				waitForReload = false;
			}, 10 );
		} );
	},

	/**
	 * Make sure the DOM is updated after VC Front-End editor is loaded.
	 *
	 * @since 1.0.3
	 */
	setupFrontEndEditorDOM = function() {
		// Wait for the DOM to be loaded for the first time.
		window.Pace.on( 'done', function() {
			// Return early if we're not in the front-end editor.
			if ( ! $( 'body' ).is( '.compose-mode' ) ) {
				return;
			}

			/**
			 * Setup an observer, so we can watch DOM changes without
			 * ruining performance.
			 *
			 * @type {MutationObserver}
			 */
			var observer = new MutationObserver( function( mutations ) {
				mutations.forEach( function( mutation ) {
					if ( ! mutation.removedNodes.length || waitForIframeReload ) {
						return;
					}

					waitForIframeReload = true;

					// Avoid reloading the iframe too frequently.
					setTimeout( function() {
						window.vc_iframe.reload();
						waitForIframeReload = false;
					}, 10 );
				} );
			} ),
			// Observer configuration.
			observerConfig = {
				childList: true
			},
			// Target node.
			targetNode = document.body,
			// Set a flag to check if the iframe is waiting to be reloaded.
			waitForIframeReload = false;

			// Watch for DOM changes.
			observer.observe( targetNode, observerConfig );

			// Trigger event for initial DOM changes.
			$.NiceDev.log( 'Trigger NiceDOMChanged' );
			$( document ).trigger( 'NiceDOMChanged' );
		} );

		// Trigger DOM changes when an element is reloaded.
		$( window ).on( 'vc_reload', function() {
			$.NiceDev.log( 'window.vc_iframe.reload' ) && $( document ).trigger( 'NiceDOMChanged' );
		} );
	},

	loadScripts = function() {
		var _loadPostsPaginationMaybe = function() {
			var url         = $.NiceLazyScript( 'postsPagination' ),
				$containers = $( '.nice-vc-posts-element' );

			if ( ! $containers.length ) {
				return;
			}

			if ( 'object' !== typeof window.NiceVCPostsPagination && url ) {
				$.getScript( url );
			}
		};

		_loadPostsPaginationMaybe();
	},

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	ready = function() {
		videoBackgroundInit();
		initCounter();
		initCountdown();
		initProgressBar();
		initPieChart();
		unsetVCAnimations();
		initCustomHeading();
		initCardsGallery();
		initAnimatedSVG();
		controlParallax();
		resetRows();
		setupFrontEndEditorDOM();
		loadScripts();
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};

} )( jQuery );

jQuery( NiceVCGeneral.ready );
