<?php
/**
 * Smart by NiceThemes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_VCIntegration_Post_Item_HTML_Builder' ) ) :
/**
 * Class Nice_VCIntegration_Post_Item_HTML_Builder
 *
 * Build HTML for a post looped through VC Posts shortcode.
 *
 * @see   Nice_VCIntegration_Posts_Shortcode
 * @see   Nice_VCIntegration_Post_Item
 *
 * @since 1.0.0
 */
class Nice_VCIntegration_Post_Item_HTML_Builder {
	/**
	 * Current looped post item.
	 *
	 * @var Nice_VCIntegration_Post_Item
	 */
	protected $item;

	/**
	 * Generated HTML segments.
	 *
	 * @var array
	 */
	protected $segments = array();

	/**
	 * Nice_VCIntegration_Post_Item_HTML_Builder constructor.
	 *
	 * @param Nice_VCIntegration_Post_Item $item
	 */
	public function __construct( Nice_VCIntegration_Post_Item $item ) {
		$this->item = $item;
	}

	/**
	 * Add an HTML segment to the list.
	 *
	 * @param string $segment
	 */
	protected function add_segment( $segment = '' ) {
		if ( ! is_string( $segment ) || ! $segment ) {
			return;
		}

		array_push( $this->segments, $segment );
	}

	/**
	 * Build a specific segment using an available method.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key
	 * @param null|mixed  $value
	 */
	public function build( $key, $value = null ) {
		if ( ! isset( $this->item->layout[ $key ] ) ) {
			return;
		}

		$value = $value ? $value : $this->item->layout[ $key ];

		if ( method_exists( $this, 'build_' . $key ) ) {
			call_user_func( array( $this, 'build_' . $key ), $value );
		} else {
			$this->build_extra( $key, $value );
		}
	}

	/**
	 * Build all HTML segments using available methods.
	 *
	 * @since 1.0.0
	 */
	public function build_all() {
		foreach ( $this->item->layout as $key => $value ) {
			$this->build( $key, $value );
		}
	}

	/**
	 * HTML additions for non-default properties are fired here through a
	 * dynamic filter.
	 *
	 * @since 1.0.0
	 *
	 * @param string $field
	 * @param array  $args
	 */
	protected function build_extra( $field, array $args = array() ) {
		$this->add_segment( apply_filters( 'nice_vc_integration_posts_html_' . $field, '', $args ) );
	}

	/**
	 * Build HTML for an item's icon.
	 *
	 * @since  1.0.0
	 */
	public function build_icon() {
		$icon_type = esc_attr( $this->item->shortcode->attr( 'icon_type', 'fontawesome' ) );
		$icon      = $this->block_has_overlay() ? $this->item->shortcode->attr( 'icon_' . $icon_type ) : '';

		if ( ! empty( $icon ) ) {
			$icon = '<i class="' . esc_attr( $icon ) . ' entry-overlay-icon"></i>';
		}

		$this->add_segment( $icon );
	}

	/**
	 * Build the title of a VC block item.
	 *
	 * @since  1.0.0
	 */
	public function build_title() {
		if ( 'quote' === $this->item->post_format && $quote = $this->get_quote() ) {
			$this->add_segment( $quote );

			return;
		}

		if ( $title = $this->item->post_title ) {
			$classes = array(
				'entry-title',
			);

			$post_type = get_post_type_object( $this->item->post_type );

			if ( $transform = $this->item->shortcode->attr( 'single_title_transform' ) ) {
				array_push( $classes, 'text-' . esc_attr( $transform ) );
			}

			if ( $size = $this->item->shortcode->attr( 'single_title_font_size' ) ) {
				array_push( $classes, esc_attr( $size ) );
			}

			$styles = $this->item->shortcode->get_post_title_styles();

			// Wrap with permalink.
			if ( $post_type->publicly_queryable && $post_type->public ) {
				$title = '<a href="' . $this->item->permalink . '">' . $title . '</a>';
			}

			// Wrap with title tag.
			$title = '<h3 class="' . join( ' ', $classes ) . '" ' . $styles . '>' . $title . '</h3>';
		}

		$this->add_segment( $title );
	}

	/**
	 * Obtain quote HTML as title replacement for quote posts.
	 *
	 * @since  1.0.0
	 *
	 * @return string
	 */
	protected function get_quote() {
		$quote = '';

		$quote_text   = get_post_meta( $this->item->post_id, 'quote', true );
		$quote_author = get_post_meta( $this->item->post_id, 'quote-author', true );

		if ( $quote_text ) {
			$classes = array(
				'entry-title',
				'entry-quote',
			);

			if ( $transform = $this->item->shortcode->attr( 'single_title_transform' ) ) {
				array_push( $classes, 'text-' . esc_attr( $transform ) );
			}

			$quote .= '<div class="' . join( ' ', $classes ) . '">';
			$quote .= '<a href="' . $this->item->permalink . '" title="' . $this->item->post_title . '">';
			$quote .= '<blockquote class="quote">' . esc_attr( $quote_text ) . '</blockquote>';
			$quote .= $quote_author ? '<span class="quote-author">' . esc_attr( $quote_author ) . '</span>' : '';
			$quote .= '</a>';
			$quote .= '</div>';
		}

		return $quote;
	}

	/**
	 * Build the HTML for a block item's type meta.
	 *
	 * @since  1.0.0
	 *
	 * @param array $args
	 */
	public function build_type( array $args = null ) {
		$display_type = empty( $args[0] ) ? 'block' : $args[0];
		$skin_class   = empty( $args[2] ) ? 'inherit-skin' : $args[2];
		$align_class  = empty( $args[3] ) ? 'align-default' : 'align' . $args[3];

		$post_type = get_post_type_object( $this->item->post_type );
		$icon_class = '';

		if ( 'post' !== $this->item->post_type ) {
			$type_name = $post_type->labels->singular_name;
		} else {
			$type_name = $this->item->post_format ? get_post_format_string( $this->item->post_format ) : $post_type->labels->singular_name;
			$icon_class = $this->get_post_icon_class();
		}

		if ( empty( $icon_class ) ) {
			$icon_class = 'fa fa-file-o';
		}

		/**
		 * @hook nice_vc_integration_posts_item_type_icon_class
		 *
		 * Hook in here to modify the HTML class for the current post type icon.
		 *
		 * @since 1.0.0
		 */
		$icon_class = apply_filters( 'nice_vc_integration_posts_item_type_icon_class', $icon_class );

		$icon = ( ! isset( $args[1] ) || 'text' !== $args[1] ) ? '<i class="' . esc_attr( $icon_class ) . '" title="' . esc_attr( $type_name ) . '"></i>' : '';
		$text = ( ! isset( $args[1] ) || 'icon' !== $args[1] ) ? '<span class="entry-type-name">' . esc_attr( $type_name ) . '</span>' : '';

		$type = '<div class="entry-type entry-meta ' . esc_attr( $display_type ) . ' ' . esc_attr( $skin_class ) . ' ' . esc_attr( $align_class ) . '">' . $icon . $text . '</div>';

		$this->add_segment( $type );
	}

	/**
	 * Obtain the HTML class for the current post type icon.
	 *
	 * @since  1.0.0
	 *
	 * @return string
	 */
	protected function get_post_icon_class() {
		$icon_class = '';

		switch ( $this->item->post_format ) {
			case 'gallery' :
				$icon_class = 'fa fa-image';
				break;
			case 'image' :
				$icon_class = 'fa fa-image';
				break;
			case 'audio' :
				$icon_class = 'fa fa-volume-up';
				break;
			case 'video' :
				$icon_class = 'fa fa-film';
				break;
			case 'quote' :
				$icon_class = 'fa fa-quote-right';
				break;
			case 'link' :
				$icon_class = 'fa fa-link';
				break;
			default;
				break;
		}

		return $icon_class;
	}

	/**
	 * Build the HTML for a block item's date.
	 *
	 * @since  1.0.0
	 *
	 * @param array $args
	 */
	public function build_date( array $args = null ) {
		$display_type = empty( $args[0] ) ? 'block' : $args[0];
		$skin_class   = empty( $args[2] ) ? 'inherit-skin' : $args[2];

		$icon = ( ! isset( $args[1] ) || 'text' !== $args[1] ) ? '<i class="fa fa-clock-o"></i>' : '';

		$date = '<div class="entry-meta ' . esc_attr( $display_type ) . ' ' . esc_attr( $skin_class ) . '"><span class="entry-date entry-meta">' . $icon . get_the_date( '', $this->item->post_id ) . '</span></div>';

		$this->add_segment( $date );
	}

	/**
	 * Obtain the HTML for a block item's text.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	public function build_text( array $args = null ) {
		$length = ( isset( $args[0] ) ) ? $args[0] : 'excerpt';

		$text = ( 'excerpt' === $length ) ? get_the_excerpt() : get_the_content();

		if ( $this->block_has_overlay() && ! $this->block_is_clickable() ) {
			$text = preg_replace( '/<\/?a(.|\s)*?>/', '', $text );
		}

		$text = '<div class="entry-excerpt">' . $text . '</div>';

		$this->add_segment( $text );
	}

	/**
	 * Build HTML for a block item's link.
	 *
	 * @since  1.0.0
	 *
	 * @param  array $args
	 */
	public function build_link( array $args = null ) {
		$button_class = array();

		$shape = isset( $args[0] ) ? $args[0] : 'default';
		$button_shape = '';

		if ( 'default' !== $shape ) {
			$button_shape = nice_theme_shape_button_class( $shape );
		}

		if ( 'btn-link' !== $button_shape ) {
			array_push( $button_class, 'btn' );
			array_push( $button_class, 'btn-default' );
			array_push( $button_class, nice_theme_shape_button_class( 'outline' ) );
		} else {
			array_push( $button_class, 'inherit-skin' );
		}

		if ( $button_shape ) {
			array_push( $button_class, $button_shape );
		}

		$text_transform = isset( $args[1] ) ? $args[1] : 'text-no-transform';
		array_push( $button_class, $text_transform );

		$button_width = isset( $args[2] ) ? $args[2] : 'default-width';
		array_push( $button_class, $button_width );

		if ( 'link' === $this->item->post_format ) {
			$link = get_post_meta( $this->item->post_id, 'link', true );
			$target = '_blank';
		}

		/**
		 * Try to obtain "read more" text from shortcode.
		 *
		 * @since 1.0.3
		 */
		$button_text = $this->item->shortcode->attr( 'read_more_text', '' );

		if ( ! $button_text ) {
			$button_text = isset( $link ) ? esc_html__( 'Visit link', 'smart' ) : esc_html__( 'Read More', 'smart' );
		}

		if ( ! isset( $link ) ) {
			$link = $this->item->permalink;
			$target = '';
		}

		/**
		 * @hook nice_vc_integration_posts_item_button_class
		 *
		 * Hook here to modify the HTML classes assigned to item "Read More" buttons.
		 *
		 * @since 1.0.0
		 */
		$button_class = apply_filters( 'nice_vc_integration_posts_item_button_class', $button_class );

		$link = '<div class="entry-readmore"><a href="' . esc_url( $link ) . '" class="' . esc_attr( join( ' ', $button_class ) ) . '" target="' . $target . '">' . esc_attr( $button_text ) . '</a></div>';

		$this->add_segment( $link );
	}

	/**
	 * Build the HTML for a block item's author.
	 *
	 * @since  1.0.0
	 *
	 * @param array $args
	 */
	public function build_author( array $args = null ) {
		$display_type = empty( $args[0] ) ? 'block' : $args[0];
		$skin_class   = empty( $args[1] ) ? 'inherit-skin' : $args[1];
		$author_name  = get_the_author_meta( 'display_name', $this->item->post_author );
		$author_link  = get_author_posts_url( $this->item->post_author );

		$author = '<div class="entry-author entry-meta ' . esc_attr( $display_type ) . ' ' . esc_attr( $skin_class ) . '">';
		$author .= '<a href="' . $author_link . '">' . get_avatar( $this->item->post_author, 24 ) . '<span>' . esc_html__( 'by ', 'smart' ) . $author_name . '</span></a>';
		$author .= '</div>';

		$this->add_segment( $author );
	}

	/**
	 * Build the HTML for a block item's categories.
	 *
	 * @since  1.0.0
	 *
	 * @param array $args
	 */
	public function build_category( array $args = null ) {
		$categories_array = wp_get_post_terms( $this->item->post_id, 'category' );

		/**
		 * @hook nice_vc_integration_posts_item_categories
		 *
		 * Process categories for the current item.
		 *
		 * @since 1.0.0
		 */
		$categories_array = apply_filters( 'nice_vc_integration_posts_item_categories', $categories_array, get_post( $this->item->post_id ) );

		if ( empty( $categories_array ) ) {
			return;
		}

		$display_type = empty( $args[0] ) ? 'block' : $args[0];
		$text_style   = empty( $args[1] ) ? 'text' : $args[1];
		$skin_class   = empty( $args[2] ) ? 'inherit-skin' : $args[2];
		$position     = empty( $args[3] ) ? 'position-default' : 'position-' . $args[3];

		switch ( $text_style ) {
			case 'text-icon':
				$text_class = 'text-icon';
				break;
			case 'icon':
				$text_class = 'icon-only';
				break;
			case 'pills';
				$text_class = 'pills';
				break;
			default:
				$text_class = 'text-only';
		}

		$categories = '';
		$category_links = array();

		$category_classes = array(
			'entry-categories',
			'entry-meta',
			$display_type,
			$skin_class,
			$text_class,
			$position,
		);

		if ( 'pills' === $text_class ) {
			$nice_colors  = nice_theme_colors();
			$color        = $this->item->shortcode->attr( 'single_category_pills_color', 'default' );
			$color_css_id = isset( $nice_colors[ $color ]['css_id'] ) ? $nice_colors[ $color ]['css_id'] : 'default';

			$category_classes[] = 'pills-' . $color_css_id;

			if ( $pills_shape = $this->item->shortcode->attr( 'single_category_pills_shape', '' ) ) {
				$category_classes[] = 'pills-' . $pills_shape;
			}

			if ( $pills_size = $this->item->shortcode->attr( 'single_category_pills_size', '' ) ) {
				$category_classes[] = 'pills-' . $pills_size;
			}

			if ( 'yes' === $this->item->shortcode->attr( 'single_category_pills_outlined', '' ) ) {
				$category_classes[] = 'pills-outline';
			}
		}

		foreach ( $categories_array as $category ) {
			$category_links[] = '<a href="' . esc_url( get_term_link( $category->term_id, $category->taxonomy ) ) . '">' . esc_attr( $category->name ). '</a>';
		}

		$join_str = '';

		/**
		 * @hook nice_vc_integration_posts_item_category_icon_class
		 *
		 * Hook here to modify the category list's icon.
		 *
		 * @since 1.0.0
		 */
		$icon_class = apply_filters( 'nice_vc_integration_posts_item_category_icon_class', 'fa fa-folder-open-o' );

		$icon = ( isset( $args[1] ) && false !== stripos( $args[1], 'icon' ) ) ? '<i class="' . esc_attr( $icon_class ) . '"></i>' : '';

		if ( 'pills' !== $text_class ) {
			$join_str = ', ';
		}

		$categories .= '<div class="' . esc_attr( join( ' ', $category_classes ) ) . '">';
		$categories .= $icon . '<span>' . join( $join_str, $category_links ) . '</span>';
		$categories .= '</div>';

		$this->add_segment( $categories );
	}

	/**
	 * Build the HTML for a block item's comments.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	public function build_comments( array $args = null ) {
		if ( ! comments_open( $this->item->post_id ) ) {
			return;
		}

		$display_type    = empty( $args[0] ) ? 'block' : $args[0];
		$skin_class      = empty( $args[2] ) ? 'inherit-skin' : $args[2];
		$align_class     = empty( $args[3] ) ? 'align-default' : 'align' . $args[3];
		$comments_number = get_comments_number( $this->item->post_id );

		$icon = ( ! isset( $args[1] ) || 'text' !== $args[1] ) ? '<i class="fa fa-comment-o"></i>' : '';
		$text = ( ! isset( $args[1] ) || 'icon' !== $args[1] ) ? ' ' . _nx( 'Comment', 'Comments', $comments_number, 'comments', 'smart' ) : '';

		$comments       = '<div class="entry-comments entry-meta ' . esc_attr( $display_type ) . ' ' . esc_attr( $skin_class ) . ' ' . esc_attr( $align_class ) . '">';
		$entry_comments = $icon . '<span>' . $comments_number . $text . '</span>';
		$comments      .= '<a href="' . get_comments_link( $this->item->post_id ) . '">' . $entry_comments . '</a>';
		$comments      .= '</div>';

		$this->add_segment( $comments );
	}

	/**
	 * Build HTML for block element separator.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	public function build_separator( array $args = null ) {
		$length = ( isset( $args[0] ) && 'reduced' === esc_attr( $args[0] ) ) ? 'reduced' : 'full';
		$separator = '<hr class="entry-separator ' . $length . '" />';

		$this->add_segment( $separator );
	}

	/**
	 * Print out segments' HTML.
	 *
	 * @since 1.0.0
	 */
	public function render() {
		if ( empty( $this->segments ) ) {
			return;
		}

		echo join( "\n", $this->segments );
	}

	/**
	 * Reset list of segments.
	 *
	 * @since 1.0.0
	 */
	public function reset() {
		$this->segments = array();
	}

	/**
	 * Check if the current item's block is supposed to have an overlay effect.
	 *
	 * @return bool
	 */
	protected function block_has_overlay() {
		return ( 'overlay' === $this->item->shortcode->attr( 'single_text' ) );
	}

	/**
	 * Check if the current item's block is supposed to be fully clickable.
	 *
	 * @return bool
	 */
	protected function block_is_clickable() {
		return ( 'yes' === $this->item->shortcode->attr( 'single_elements_click' ) );
	}
}
endif;
