<?php
/**
 * Smart by NiceThemes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_VCIntegration_Post_Item') ) :
/**
 * Class Nice_VCIntegration_Post_Item
 *
 * Manage internal data of post items looped through the VC Posts shortcode.
 *
 * @see   Nice_VCIntegration_Posts_Shortcode
 *
 * @since 1.0.0
 *
 * @property-read Nice_VCIntegration_Posts_Shortcode $shortcode
 * @property-read int $post_id
 * @property-read string $post_title
 * @property-read string $post_content
 * @property-read string $post_excerpt
 * @property-read string $post_author
 * @property-read string $post_type
 * @property-read string $post_format
 * @property-read string $permalink
 * @property-read array $layout
 * @property-read Nice_VCIntegration_Post_Item_HTML_Builder $layout_builder
 * @property-read int $thumbnail_id
 */
class Nice_VCIntegration_Post_Item {
	/**
	 * Current post object.
	 *
	 * @var WP_Post
	 */
	protected $post;

	/**
	 * Object containing current shortcode's data.
	 *
	 * @var Nice_VCIntegration_Posts_Shortcode
	 */
	protected $shortcode;

	/**
	 * ID of current post.
	 *
	 * @var int
	 */
	protected $post_id;

	/**
	 * Title of current post.
	 *
	 * @var string
	 */
	protected $post_title;

	/**
	 * Title of current post.
	 *
	 * @var string
	 */
	protected $post_content;

	/**
	 * Excerpt from current post.
	 *
	 * @var string
	 */
	protected $post_excerpt;

	/**
	 * Current post's author
	 *
	 * @var string
	 */
	protected $post_author;

	/**
	 * Post type of current post.
	 *
	 * @var string
	 */
	protected $post_type;

	/**
	 * Post format of current post.
	 *
	 * @var string
	 */
	protected $post_format;

	/**
	 * ID of current post's featured image.
	 *
	 * @var int
	 */
	protected $thumbnail_id;

	/**
	 * Current post's permalink.
	 *
	 * @var string
	 */
	protected $permalink;

	/**
	 * List of keys describing the kind of information that should be
	 * displayed for the current post.
	 *
	 * @var array
	 */
	protected $layout;

	/**
	 * Layout builder object.
	 *
	 * @var Nice_VCIntegration_Post_Item_HTML_Builder|null
	 */
	protected $layout_builder = null;

	/**
	 * Nice_VCIntegration_Post_Item constructor.
	 *
	 * Initialize object.
	 *
	 * @param WP_Post                            $post
	 * @param Nice_VCIntegration_Posts_Shortcode $shortcode
	 */
	public function __construct( WP_Post $post, Nice_VCIntegration_Posts_Shortcode $shortcode ) {
		$this->post           = $post;
		$this->shortcode      = $shortcode;
		$this->layout_builder = new Nice_VCIntegration_Post_Item_HTML_Builder( $this );
	}

	/**
	 * Magic method to obtain the value of protected and private properties,
	 * and to set them if needed.
	 *
	 * @param $property
	 *
	 * @return mixed
	 */
	public function __get( $property ) {
		// Return value of property if the property exists.
		if ( property_exists( $this, $property ) ) {
			// Try to set the value of the property if not set yet.
			if ( is_null( $this->$property ) && method_exists( $this, 'get_' . $property ) ) {
				$this->$property = call_user_func( array( $this, 'get_' . $property ) );
			}

			return $this->$property;
		}

		return null;
	}

	/**
	 * Obtain ID of current post.
	 *
	 * @return int
	 */
	protected function get_post_id() {
		return $this->post->ID;
	}

	/**
	 * Obtain title of current post.
	 *
	 * @return string
	 */
	protected function get_post_title() {
		return get_the_title( $this->post );
	}

	/**
	 * Obtain post type of current post.
	 *
	 * @return false|string
	 */
	protected function get_post_type() {
		return get_post_type( $this->post );
	}

	/**
	 * Obtain post format of current post.
	 *
	 * @return false|string
	 */
	protected function get_post_format() {
		return get_post_format( $this->post );
	}

	/**
	 * Obtain content of current post.
	 *
	 * @return string
	 */
	protected function get_post_content() {
		return $this->post->post_content;
	}

	/**
	 * Obtain excerpt from current post.
	 *
	 * @return string
	 */
	protected function get_post_excerpt() {
		return $this->post->post_excerpt;
	}

	/**
	 * Obtain current post's author.
	 *
	 * @return string
	 */
	protected function get_post_author() {
		return $this->post->post_author;
	}

	/**
	 * Obtain ID of current post's featured image.
	 *
	 * @return int|string
	 */
	protected function get_thumbnail_id() {
		return get_post_thumbnail_id( $this->post );
	}

	/**
	 * Obtain layout for current post.
	 *
	 * @return array
	 */
	protected function get_layout() {
		$layout = array();

		$key_map = apply_filters( 'nice_vc_integration_posts_layout_key_map', array(
				'post' => 'post_items',
				'page' => 'page_items',
			)
		);

		foreach ( $key_map as $post_type => $key ) {
			if ( ! post_type_exists( $post_type ) ) {
				unset( $key_map[ $post_type ] );
			}
		}

		if ( isset( $key_map[ $this->post->post_type ] ) && $key = $key_map[ $this->post->post_type ] ) {
			$layout = nice_vc_integration_posts_flat_array( vc_sorted_list_parse_value( $this->shortcode->attr( $key, array() ) ) );
		}

		return $layout;
	}

	/**
	 * Obtain current post's permalink.
	 *
	 * @return false|string
	 */
	protected function get_permalink() {
		return get_permalink( $this->post );
	}

	/**
	 * Process HTML for categories segment depending on its position.
	 *
	 * @since 1.0.0
	 */
	public function process_categories() {
		$args = isset( $this->layout['category'] ) ? $this->layout['category'] : null;

		if ( is_null( $args ) || empty( $args[3] ) || 'default' === $args[3] ) {
			return;
		}

		$this->layout_builder->reset();
		$this->layout_builder->build( 'category' );
		$this->layout_builder->render();
		$this->layout_builder->reset();

		unset( $this->layout['category'] );
	}

	/**
	 * Process and print current post's data.
	 *
	 * @uses Nice_VCIntegration_Post_Item_HTML_Builder
	 */
	public function process_layout() {
		$this->layout_builder->build_all();
		$this->layout_builder->render();
	}
}
endif;
