<?php
/**
 * Smart by NiceThemes.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_VCIntegration_Posts_Shortcode' ) ) :
/**
 * Class Nice_VCIntegration_Posts_Shortcode
 *
 * Extend VC for custom posts implementation.
 *
 * @since 1.0.0
 */
class Nice_VCIntegration_Posts_Shortcode extends WPBakeryShortCode {
	/**
	 * @var array
	 *
	 * @since 1.0.0
	 */
	protected $filter_categories = array();

	/**
	 * @var WP_Query
	 *
	 * @since 1.0.0
	 */
	protected $query = null;

	/**
	 * @var array
	 *
	 * @since 1.0.0
	 */
	protected $loop_args = array();

	/**
	 * @var null
	 *
	 * @since 1.0.0
	 */
	protected $taxonomies = null;

	/**
	 * @var bool
	 *
	 * @since 1.0.0
	 */
	protected $is_home = false;

	/**
	 * Nice_VCIntegration_Posts_Shortcode constructor.
	 *
	 * Setup object.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings
	 */
	public function __construct( array $settings = array() ) {
		// Make sure we always have nice_posts as shortcode base.
		// Otherwise, AJAX request will not use this shortcode name.
		$settings['base'] = 'nice_posts';

		parent::__construct( $settings );
	}

	/**
	 * Obtain filename of the shortcode's template.
	 *
	 * @return string
	 *
	 * @since  1.0.0
	 */
	protected function getFileName() {
		$filename = 'nice_posts';

		if ( parent::getFileName() !== $filename ) {
			return $filename;
		}

		return parent::getFileName();
	}

	/**
	 * Shortener for $this->query->have_posts().
	 *
	 * @since  1.0.0
	 *
	 * @return bool
	 */
	public function have_posts() {
		return $this->query->have_posts();
	}

	/**
	 * Process the current post item.
	 *
	 * @since 1.0.0
	 */
	public function the_post() {
		// Process current post object inside the loop.
		$this->query->the_post();

		// Store new post item object.
		$post_item = new Nice_VCIntegration_Post_Item( get_post(), $this );

		// Globalize post item object.
		nice_vc_integration_posts_current_item( $post_item );
	}

	/**
	 * Obtain the value of a shortcode attribute by its name.
	 *
	 * @param mixed       $attr
	 * @param null|mixed  $default
	 *
	 * @return null|mixed
	 */
	public function attr( $attr, $default = null ) {
		if ( ! empty( $this->atts[ $attr ] ) ) {
			return $this->atts[ $attr ];
		}

		return $default;
	}

	/**
	 * Process shortcode output.
	 *
	 * @param  array       $atts
	 * @param  null|string $content
	 * @param  string      $base
	 *
	 * @return string
	 */
	public function output( $atts, $content = null, $base = '' ) {
		if ( empty( $atts['loop'] ) ) {
			$atts['loop'] = 'size:10|order_by:date|post_type:post';
		}

		return parent::output( $atts, $content, $base );
	}


	/**
	 * Load shortcode template.
	 *
	 * @param  array       $atts
	 * @param  null|string $content
	 *
	 * @return string
	 */
	protected function loadTemplate( $atts, $content = null ) {
		// Add first page to queried loop.
		$atts['loop'] .= '|paged:1';

		// Globalize current object and set its attributes.
		$this->process_object( $atts );

		add_action( 'pre_get_posts', array( $this, 'validate_query_post_types' ) );

		add_action( 'pre_get_posts', array( $this, 'save_query_home_value' ) );

		if ( 'yes' === $this->attr( 'ignore_sticky_posts' ) ) {
			add_action( 'pre_get_posts', array( $this, 'set_query_home_false' ) );
		}

		// Load Google fonts
		$this->load_google_fonts();

		// Create new WP_Query object.
		$this->process_loop();

		// Load template file.
		return parent::loadTemplate( $atts, $content );
	}

	/**
	 * Return the maximum number of pages for the current query.
	 *
	 * @since  1.0.0
	 *
	 * @return int
	 */
	public function get_max_num_pages() {
		return $this->query->max_num_pages;
	}

	/**
	 * Process AJAX request for pagination and filtering.
	 *
	 * @since 1.0.0
	 *
	 * @param array $atts
	 * @param int   $page
	 * @param int   $category
	 */
	public function process_ajax( array $atts = array(), $page = 1, $category = 0 ) {
		// Return early if not in AJAX context.
		if ( ! nice_doing_ajax() ) {
			return;
		}

		// Return early if we don't have a loop to process.
		if ( ! isset( $atts['loop'] ) ) {
			return;
		}

		// Globalize current object and set its attributes.
		$this->process_object( $atts );

		// Add required page to queried loop.
		$this->atts['loop'] .= '|paged:' . intval( $page );

		if ( $category ) {
			$this->atts['loop'] .= '|categories:' . esc_attr( $category );
		}

		add_action( 'pre_get_posts', array( $this, 'save_query_home_value' ) );

		if ( 'yes' !== $this->attr( 'ignore_sticky_posts' ) ) {
			add_action( 'pre_get_posts', array( $this, 'set_query_home_true' ) );
		}

		$exclude_id = isset( $atts['origin_id'] ) ? intval( $atts['origin_id'] ) : null;

		// Create new WP_Query object.
		$this->process_loop( $exclude_id );
	}

	/**
	 * Validate queried post types.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Query $query
	 */
	public function validate_query_post_types( WP_Query $query ) {
		if ( 'pre_get_posts' !== current_action() ) {
			return;
		}

		$post_types = $query->get( 'post_type' );

		foreach ( $post_types as $key => $post_type ) {
			if ( ! post_type_exists( $post_type ) ) {
				unset( $post_types[ $key ] );
			}
		}

		$query->set( 'post_type', $post_types );

		// Don't return anything if we have no valid post types.
		if ( empty( $post_types ) ) {
			$query->set( 'post__in', array( 0 ) );
			$query->set( 'ignore_sticky_posts', true );
		}

		$query->parse_query();

		remove_action( 'pre_get_posts', array( $this, 'validate_query_post_types' ) );
	}

	/**
	 * Save original value of $query->is_home.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Query $query
	 */
	public function save_query_home_value( WP_Query $query ) {
		if ( 'pre_get_posts' !== current_action() ) {
			return;
		}

		$this->is_home = $query->is_home;

		remove_action( 'pre_get_posts', array( $this, 'save_query_home_value' ) );
	}

	/**
	 * Restore $query->is_home to its previous value.
	 *
	 * @since 1.0.0
	 */
	public function restore_query_home_value() {
		$this->query->is_home = $this->is_home;
	}

	/**
	 * Set $query->is_home to true.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Query $query
	 */
	public function set_query_home_true( WP_Query $query ) {
		if ( 'pre_get_posts' !== current_action() ) {
			return;
		}

		$query->is_home = true;

		remove_action( 'pre_get_posts', array( $this, 'set_query_home_true' ) );
	}

	/**
	 * Set $query->is_home to false.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Query $query
	 */
	public function set_query_home_false( WP_Query $query ) {
		if ( 'pre_get_posts' !== current_action() ) {
			return;
		}

		$query->is_home = false;

		remove_action( 'pre_get_posts', array( $this, 'set_query_home_false' ) );
	}

	/**
	 * Globalize current object and set its attributes.
	 *
	 * @since 1.0.0
	 *
	 * @param array $atts
	 */
	protected function process_object( array $atts = array() ) {
		// Process shortcode attributes.
		$this->atts = nice_vc_integration_posts_shortcode_atts( $atts );

		// Globalize current shortcode object.
		nice_vc_integration_posts_current_shortcode( $this );
	}

	/**
	 * Obtain categories for current query.
	 *
	 * @return array
	 */
	public function get_categories() {
		$query = $this->query;

		if ( $this->attr( 'pagination_mode' ) && 'disabled' !== $this->attr( 'pagination_mode' ) ) {
			$full_loop_args = $this->loop_args;
			$full_loop_args['posts_per_page'] = -1;
			$full_loop_args['cat'] = isset( $full_loop_args['tax_query'][0]['terms'] ) ? $full_loop_args['tax_query'][0]['terms'] : '';
			unset( $full_loop_args['tax_query'] );

			$query = new WP_Query( $full_loop_args );
		}

		$categories = array();

		if ( $query instanceof WP_Query ) {
			foreach ( $query->posts as $post ) {
				/**
				 * @hook nice_vc_integration_posts_categories
				 *
				 * Modify posts categories for VC Posts component.
				 *
				 * @since 1.0.0
				 */
				$categories = apply_filters( 'nice_vc_integration_posts_categories', $categories, $post );
			}
		}

		$categories = self::clean_categories( $categories );

		return $categories;
	}

	/**
	 * Clean list of categories in order to make sure no duplicated terms are
	 * displayed.
	 *
	 * @param  array $categories
	 *
	 * @return array
	 */
	static protected function clean_categories( array $categories = array() ) {
		$terms  = array();
		$category_ids = array();

		if ( ! empty( $categories ) ) {
			$terms = get_terms( array( 'taxonomy' => current( $categories )->taxonomy ) );

			foreach ( $categories as $key => $category ) {
				$category_ids[] = $category->term_id;
			}

			if ( ! empty( $terms ) ) {
				foreach ( $terms as $key => $term ) {
					if ( ! in_array( $term->term_id, $category_ids, true ) ) {
						unset( $terms[ $key ] );
					}
				}
			}
		}

		return $terms;
	}

	/**
	 * Build and obtain WP_Query object with given data.
	 *
	 * @param  array $query
	 * @param  bool $exclude_id
	 *
	 * @return array
	 */
	public function build_query( $query = array(), $exclude_id = false ) {
		$data          = self::parse_data( $query );
		$query_builder = new Nice_VCIntegration_LoopQueryBuilder( $data );

		if ( $exclude_id ) {
			$query_builder->excludeId( $exclude_id );
		}

		$query = $query_builder->build();

		return $query;
	}

	/**
	 * Build query for the requested loop.
	 *
	 * @param array $query
	 * @param bool  $exclude_id
	 *
	 * @return array
	 */
	public function build_loop( $query = array(), $exclude_id = false ) {
		return self::build_query( $query, $exclude_id );
	}

	/**
	 * Parse posts data.
	 *
	 * @param  string|array $args
	 *
	 * @return array
	 */
	protected function parse_data( $args = array() ) {
		if ( is_array( $args ) ) {
			return $args;
		}

		// Split and parse data if we didn't receive an array.
		$data         = array();
		$values_pairs = preg_split( '/\|/', $args );

		foreach ( $values_pairs as $pair ) {
			if ( ! empty( $pair ) ) {
				list( $key, $args ) = preg_split( '/\:/', $pair );
				$data[ $key ] = $args;
			}
		}

		return $data;
	}

	/**
	 * Format a listed loop of posts.
	 *
	 * @since 1.0.0
	 *
	 * @param mixed|null|int $exclude_id
	 */
	public function process_loop( $exclude_id = null ) {
		$exclude_id = $exclude_id ? $exclude_id : get_the_ID();

		list( $this->loop_args, $this->query ) = $this->build_loop( esc_attr( $this->atts['loop'] ), $exclude_id );

		$this->restore_query_home_value();
	}

	/**
	 * Obtain shortcode attributes.
	 *
	 * @since  1.0.0
	 *
	 * @return mixed
	 */
	public function get_atts() {
		return $this->atts;
	}

	/**
	 * Obtain shortcode's WP_Query
	 *
	 * @since  1.0.0
	 *
	 * @return WP_Query
	 */
	public function get_query() {
		return $this->query;
	}

	/**
	 * Obtain loop arguments for current shortcode.
	 *
	 * @since  1.0.0
	 *
	 * @return array
	 */
	public function get_loop_args() {
		return $this->loop_args;
	}

	/**
	 * Load google fonts if necessary
	 *
	 * @since  1.0.0
	 *
	 */
	public function load_google_fonts() {

		if ( 'custom' === $this->attr( 'title_font_family' ) ) {
			$google_fonts     = $this->attr( 'google_fonts' );
			$google_fonts_obj = new Vc_Google_Fonts();
			$google_fonts_field_settings = isset( $google_fonts_field, $google_fonts_field['settings']['fields'] ) ? $google_fonts_field['settings']['fields'] : array();
			$google_fonts_data = strlen( $google_fonts ) > 0 ? $google_fonts_obj->_vc_google_fonts_parse_attributes( $google_fonts_field_settings, $google_fonts ) : '';

			$google_font = isset( $google_fonts_data['values']['font_family'] ) ? $google_fonts_data['values']['font_family'] : null;

			if ( $google_font && ! nice_web_font_is_registered( $google_font ) ) {
				nice_register_web_font( $google_font );
				wp_enqueue_style( 'vc_google_fonts_' . vc_build_safe_css_class( $google_font ), nice_web_font_uri( $google_font ) );
			}
		}
	}

	/**
	 * Create inline styles for the post title.
	 *
	 * @since  1.0.0
	 *
	 * @return string
	 */

	public function get_post_title_styles() {

		$style = '';

		if ( 'custom' === $this->attr( 'title_font_family' ) ) {
			$google_fonts     = $this->attr( 'google_fonts' );
			$google_fonts_obj = new Vc_Google_Fonts();
			$google_fonts_field_settings = isset( $google_fonts_field, $google_fonts_field['settings']['fields'] ) ? $google_fonts_field['settings']['fields'] : array();
			$google_fonts_data = strlen( $google_fonts ) > 0 ? $google_fonts_obj->_vc_google_fonts_parse_attributes( $google_fonts_field_settings, $google_fonts ) : '';

			if ( ! empty( $google_fonts_data ) && isset( $google_fonts_data['values'], $google_fonts_data['values']['font_family'], $google_fonts_data['values']['font_style'] ) ) {
				$google_fonts_family = explode( ':', $google_fonts_data['values']['font_family'] );
				$styles[] = 'font-family:' . $google_fonts_family[0];
				$google_fonts_styles = explode( ':', $google_fonts_data['values']['font_style'] );
				$styles[] = 'font-weight:' . $google_fonts_styles[1];
				$styles[] = 'font-style:' . $google_fonts_styles[2];
			}

			if ( ! empty( $styles ) ) {

				$style = 'style="';

				foreach ( $styles as $s ) {
					if ( false === strpos( $s, 'text-align' ) ) {
						$style .= esc_attr( $s ) . ';';
					}
				}

				$style .= '"';

			}
		}

		return $style;
	}
}
endif;
