<?php
/**
 * Smart by NiceThemes.
 *
 * General functions for Visual Composer integration.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_vc_is_used' ) ) :
/**
 * Check if the current post object is using Visual Composer.
 *
 * @since  1.0.0
 *
 * @return bool
 */
function nice_vc_is_used() {
	$nice_vc_post_types = nice_vc_get_post_types();

	if ( in_array( get_post_type(), $nice_vc_post_types, true ) ) {
		$content = get_the_content() ? : get_post( get_the_ID() )->post_content;

		return has_shortcode( $content, 'vc_row' );
	}

	return false;
}
endif;

if ( ! function_exists( 'nice_vc_inline' ) ) :
/**
 * Check if we're in the front-end editor main frame.
 *
 * @since 1.0.3
 */
function nice_vc_inline() {
	return isset( $_REQUEST['vc_action'] ) && ( 'vc_inline' === wp_unslash( $_REQUEST['vc_action'] ) );
}
endif;

if ( ! function_exists( 'nice_vc_integration_google_fonts' ) ) :
add_filter( 'vc_google_fonts_get_fonts_filter', 'nice_vc_integration_google_fonts' );
/**
 * Customize Visual Composer's Google Fonts list.
 *
 * @see Vc_Google_Fonts::$fonts_list
 *
 * @since  1.0.0
 *
 * @param array $fonts_list
 *
 * @return array
 */
function nice_vc_integration_google_fonts( $fonts_list ) {
	static $google_fonts = null;

	if ( is_null( $google_fonts ) ) {
		$google_fonts = array();

		// Obtain Nice Google Fonts.
		$nice_google_fonts = nice_get_google_fonts();

		foreach ( $nice_google_fonts as $nice_google_font ) {
			/**
			 * Obtain font family.
			 */
			$font_family = $nice_google_font['name'];

			/**
			 * Obtain font styles.
			 *
			 * They should be one or more of the following strings:
			 * `'100'`
			 * `'100italic'`
			 * `'200'`
			 * `'200italic'`
			 * `'300'`
			 * `'300italic'`
			 * `'regular'`
			 * `'italic'`
			 * `'500'`
			 * `'500italic'`
			 * `'600'`
			 * `'600italic'`
			 * `'700'`
			 * `'700italic'`
			 * `'800'`
			 * `'800italic'`
			 * `'900'`
			 * `'900italic'`
			 */
			$font_styles = explode( ',', str_replace( ':', '', $nice_google_font['variant'] ) );

			/**
			 * Obtain font types.
			 */
			$font_types = array();

			foreach ( $font_styles as $font_style ) {
				/**
				 * Obtain font weight and variant.
				 *
				 * Font weight should result an integer multiple of 100, between 100 and 900 (normal weight is 400).
				 * Font variant should result `'normal'` or `'italic'`.
				 */
				if ( false === strpos( $font_style, 'italic' ) ) {
					$font_weight  = ( 'regular' === $font_style ) ? 400 : intval( $font_style );
					$font_variant = 'normal';
				} else {
					$font_weight  = ( 'italic' === $font_style ) ? 400 : intval( str_replace( 'italic', '', $font_style ) );
					$font_variant = 'italic';
				}

				/**
				 * Obtain font weight name.
				 *
				 * It should result `'light'`, an empty string (normal) or `'bold'`.
				 */
				if ( 400 < $font_weight ) {
					$font_weight_name = 'bold';
				} elseif ( 400 > $font_weight ) {
					$font_weight_name = 'light';
				} else {
					$font_weight_name = '';
				}

				/**
				 * Construct font type for the current style.
				 *
				 * It should result one of the following strings:
				 * `'100 light regular:100:normal'`
				 * `'100 light italic:100:italic'`
				 * `'200 light regular:200:normal'`
				 * `'200 light italic:200:italic'`
				 * `'300 light regular:300:normal'`
				 * `'300 light italic:300:italic'`
				 * `'400 regular:400:normal'`
				 * `'400 italic:400:italic'`
				 * `'500 bold regular:500:normal'`
				 * `'500 bold italic:500:italic'`
				 * `'600 bold regular:600:normal'`
				 * `'600 bold italic:600:italic'`
				 * `'700 bold regular:700:normal'`
				 * `'700 bold italic:700:italic'`
				 * `'800 bold regular:800:normal'`
				 * `'800 bold italic:800:italic'`
				 * `'900 bold regular:900:normal'`
				 * `'900 bold italic:900:italic'`
				 */
				$font_types[] = $font_weight . ( $font_weight_name ? ' ' . $font_weight_name : '' ) . ' ' . ( 'normal' !== $font_variant ? $font_variant : 'regular' ) . ':' . $font_weight . ':' . $font_variant;
			}

			// Add Google Font.
			$google_fonts[] = (object) array(
				'font_family' => $font_family,
				'font_styles' => implode( ',', $font_styles ),
				'font_types'  => implode( ',', $font_types ),
			);
		}
	}

	/**
	 * @hook nice_vc_integration_google_fonts
	 *
	 * Hook here to change Visual Composer integration Google Fonts list.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_vc_integration_google_fonts', $google_fonts, $fonts_list );
}
endif;

if ( ! function_exists( 'nice_vc_integration_colors' ) ) :
/**
 * Return an array of the theme colors to use in the dropdown values.
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_vc_integration_colors() {
	$nice_colors = nice_theme_colors();

	$colors = array();
	$colors[ esc_html__( 'Default', 'smart' ) ] = '';

	foreach ( $nice_colors as $v ) {
		$colors[ $v['name'] ] = $v['id'];
	}

	return apply_filters( 'nice_vc_integration_colors', $colors );
}
endif;

if ( ! function_exists( 'nice_vc_integration_heading_size' ) ) :
/**
 * Return an array of the different Heading Sizes.
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_vc_integration_heading_size() {
	$heading_size = array(
		esc_html__( 'Default CSS', 'smart' ) => '',
		esc_html__( 'h1', 'smart' ) => 'h1',
		esc_html__( 'h2', 'smart' ) => 'h2',
		esc_html__( 'h3', 'smart' ) => 'h3',
		esc_html__( 'h4', 'smart' ) => 'h4',
		esc_html__( 'h5', 'smart' ) => 'h5',
		esc_html__( 'h6', 'smart' ) => 'h6',
	);

	return $heading_size;
}
endif;

if ( ! function_exists( 'nice_vc_integration_font_weight' ) ) :
/**
 * Return an array of the different font weight options
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_vc_integration_font_weight() {
	$font_weight = array(
		esc_html__( 'Default CSS', 'smart' ) => '',
		esc_html__( '100', 'smart' ) => 100,
		esc_html__( '200', 'smart' ) => 200,
		esc_html__( '300', 'smart' ) => 300,
		esc_html__( '400', 'smart' ) => 400,
		esc_html__( '500', 'smart' ) => 500,
		esc_html__( '600', 'smart' ) => 600,
		esc_html__( '700', 'smart' ) => 700,
		esc_html__( '800', 'smart' ) => 800,
		esc_html__( '900', 'smart' ) => 900,
	);

	return $font_weight;
}
endif;

if ( ! function_exists( 'nice_vc_integration_text_transform' ) ) :
/**
 * Return an array of the different text transform options
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_vc_integration_text_transform() {
	$text_transform = array(
		esc_html__( 'Default CSS', 'smart' ) => '',
		esc_html__( 'UPPERCASE', 'smart' ) => 'uppercase',
		esc_html__( 'lowercase', 'smart' ) => 'lowercase',
		esc_html__( 'Capitalize', 'smart' ) => 'capitalize',
	);

	return $text_transform;
}
endif;

if ( ! function_exists( 'nice_vc_integration_letter_spacing' ) ) :
/**
 * Return an array of the different Letter Spacing values
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_vc_integration_letter_spacing() {
	$letter_spacing = array(
		esc_html__( 'Default CSS', 'smart' ) => '',
		esc_html__( '-1.5px', 'smart' ) => 'letter-spacing-m-1-5',
		esc_html__( '-1px', 'smart' ) => 'letter-spacing-m-1',
		esc_html__( '1px', 'smart' ) => 'letter-spacing-p-1',
		esc_html__( '1.5px', 'smart' ) => 'letter-spacing-p-1-5',
		esc_html__( '2px', 'smart' ) => 'letter-spacing-p-2',
		esc_html__( '2em', 'smart' ) => 'letter-spacing-p-em-2',
	);

	return $letter_spacing;
}
endif;

if ( ! function_exists( 'nice_vc_integration_remove_theme_defaults' ) ) :
add_action( 'wp_head', 'nice_vc_integration_remove_theme_defaults' );
/**
 * Remove default actions when VC is being used
 *
 * @since  1.0.0
 */
function nice_vc_integration_remove_theme_defaults() {
	global $nice_vc_post_types, $post;

	if ( is_singular( $nice_vc_post_types ) ) {
		$the_content = $post->post_content;

		if ( has_shortcode( $the_content, 'vc_row' ) ) {
			remove_action( 'nice_before_single_post', 'nice_post_content_title', 10 );
		}
	}
}
endif;

if ( ! function_exists( 'nice_vc_integration_has_sidebar' ) ) :
add_filter( 'nice_has_sidebar', 'nice_vc_integration_has_sidebar' );
/**
 * Hook into `nice_has_sidebar` to redefine if the current page should display
 * a sidebar.
 *
 * @since 1.1
 *
 * @param  bool $has_sidebar
 *
 * @return bool
 */
function nice_vc_integration_has_sidebar( $has_sidebar = false ) {
	if ( $has_sidebar ) {
		$has_sidebar = ! nice_vc_is_used() || nice_vc_integration_override_sidebar();
	}

	return $has_sidebar;
}
endif;

if ( ! function_exists( 'nice_vc_integration_post_custom_fields' ) ) :
add_filter( 'nice_custom_fields', 'nice_vc_integration_post_custom_fields', 99, 2 );
/**
 * Add custom fields to override default VC behavior.
 *
 * @since 1.1
 *
 * @param array  $nice_fields
 * @param string $post_type
 *
 * @return array
 */
function nice_vc_integration_post_custom_fields( array $nice_fields = array(), $post_type ) {
	$vc_sidebar_fields[] = array(
		'name'      => '_post_sidebar_override_vc',
		'std'       => false,
		'label'     => esc_html__( 'Override Visual Composer', 'smart' ),
		'type'      => 'checkbox',
		'desc'      => esc_html__( 'Force sidebar to display when using Visual Composer.', 'smart' ),
		'condition' => '_post_sidebar:is(true)',
		'operator'  => 'and',
	);

	nice_custom_fields_enqueue( $nice_fields, $vc_sidebar_fields, '_post_sidebar_position', false );

	$vc_content_fields[] = array(
		'name'      => '_post_content_width_override_vc',
		'std'       => false,
		'label'     => esc_html__( 'Override Visual Composer', 'smart' ),
		'type'      => 'checkbox',
		'desc'      => esc_html__( 'Apply defined content width when using Visual Composer.', 'smart' ),
	);

	nice_custom_fields_enqueue( $nice_fields, $vc_content_fields, '_post_content_width_value', false );

	return $nice_fields;
}
endif;

if ( ! function_exists( 'nice_vc_integration_override_sidebar' ) ) :
/**
 * Check if default sidebar behavior when VC is used should be overriden.
 *
 * @since  1.1
 *
 * @return bool
 */
function nice_vc_integration_override_sidebar() {
	$override = false;

	if ( is_page() || is_single() ) {
		$override = nice_bool( get_post_meta( get_the_ID(), '_post_sidebar_override_vc', true ) );
	}

	return $override;
}
endif;

if ( ! function_exists( 'nice_vc_integration_override_content_width' ) ) :
/**
 * Check if default content behavior when VC is used should be overriden.
 *
 * @since 1.1
 *
 * @return bool
 */
function nice_vc_integration_override_content_width() {
	$override = false;

	if ( is_page() || is_single() ) {
		$override = nice_bool( get_post_meta( get_the_ID(), '_post_content_width_override_vc', true ) );
	}

	return $override;
}
endif;

if ( ! function_exists( 'nice_vc_integration_remove_sidebar' ) ) :
add_action( 'nice_sidebar', 'nice_vc_integration_remove_sidebar', -9999 );
/**
 * Disable all sidebars when using VC in a post or page.
 *
 * @since 1.0.0
 */
function nice_vc_integration_remove_sidebar() {
	if ( nice_has_sidebar() ) {
		return;
	}

	// Disable anything that can be printed as a sidebar.
	remove_all_actions( 'nice_sidebar' );
}
endif;
