/**
 * Smart by NiceThemes.
 *
 * Manage WooCommerce shopping cart actions.
 *
 * @since   1.0.0
 * @package Smart
 */
var NiceWooCommerce = ( function( $ ) {
	// Tell browsers we're not doing anything silly.
	'use strict';

	/**
	 * Check if the object is already bound to WooCommerce events.
	 *
	 * @since 1.0
	 *
	 * @type  bool
	 */
	var bound = false,

	/**
	 * Allow this class to be re-executed after WooCommerce modifies the DOM.
	 *
	 * @since 1.0.0
	 */
	bind = function() {
		if ( bound ) {
			return;
		}

		var $body = $( 'body' );

		$body.bind( 'added_to_cart', ready );
		$body.bind( 'wc_fragments_refreshed', ready );

		/**
		 * Make sure to notify about DOM changes after the checkout box is updated.
		 *
		 * @since 1.0.3
		 */
		$body.bind( 'updated_checkout', function() {
			$( document ).trigger( 'NiceDOMChanged' );
		} );

		bound = true;
	},

	/**
	 * Handle showing/hiding WooCommerce cart details in header.
	 *
	 * @since 1.0.0
	 */
	display = function() {
		var $cart        = $( '#nice-mini-cart' ),
		    $cartWrapper = $( '#header-shopping-cart-wrapper' );

		if ( ! $cart.length ) {
			return;
		}

		$cartWrapper.hoverIntent( {
			over: function() {
				$cart.fadeIn();
			},
			out: function() {
				if ( ! $cart.is( ':hover' ) && ! $cartWrapper.is( ':hover' ) ) {
					$cart.fadeOut();
				}
			},
			timeout: 400,
			selector: '> a, > div'
		} );
	},

    /**
     * Handle adding elements to the shopping cart.
     *
     * @since 1.0.0
     */
	addToCart = function() {
	    var $addButton = $( '.ajax_add_to_cart.product_type_simple' ),
		    $body = $( 'body' ),

        /**
         * Add product to shopping cart.
         *
         * @since   1.0.0
         *
         * @param   $product
         * @private
         */
        _addProduct = function( $product ) {
	        var $notification = $( $product ).siblings( '.nice-cart-notification' ),
	            buttonUpdated = false,
	            triggers      = [ 'added_to_cart', 'wc_cart_button_updated' ],

	        /**
	         * Manage the "add to cart" button update.
	         *
	         * @since 1.0.3
	         *
	         * @private
	         */
	        __updateButton = function() {
		        var $added = $( $product ).siblings( '.added_to_cart' );

		        if ( ! $added.length || buttonUpdated ) {
			        return;
		        }

		        buttonUpdated = true;

		        $notification.removeClass( 'loading' ).addClass( 'added' );

		        if ( ! $added.hasClass( 'view-cart' ) ) {
			        $added.hide();
		        }

		        $added.addClass( $( $product ).attr( 'class' ) ).addClass( 'view-cart' );
		        $added.removeClass( 'added add_to_cart_button product_type_simple' );

		        setTimeout( function() {
			        $added.css( { display: 'inline-block' } );
			        $( $product ).hide();

			        _productAdded( $notification );

		        }, 1000 );
	        };

	        $notification.addClass( 'bring-forward appear loading' );

	        for ( var i = 0; i < triggers.length; i++ ) {
		        $body.on( triggers[i], __updateButton );
	        }
        },

        /**
         * Process actions after adding a product to the shopping cart.
         *
         * @since   1.0.0
         *
         * @param   $notification
         * @private
         */
        _productAdded = function( $notification ) {
	        $notification.removeClass( 'appear' );

	        setTimeout( function() {
		        $notification.removeClass( 'added bring-forward' );
	        }, 1000 );
        };

	    // Return early if AJAX cart is not set up from WC.
	    if ( ! $body.data( 'wc-ajax-cart' ) ) {
		    return;
	    }

	    // Add product when the button is clicked.
		$addButton.click( function( e ) {
			e.preventDefault();
			_addProduct( this );
		} );
	},

	/**
	 * Watch actions on modal window.
	 *
	 * @since 1.0.0
	 */
	watchModal = function() {
		if ( 'undefined' === typeof window.NiceModalData ) {
			window.NiceModalData = {};
		}

		var $modal     = $( '#modal' ),
			$container = $( '#modal-content-inner' ),
			$modalOpeners = [],

			/**
			 * Open the modal window from an event.
			 *
			 * @param link
			 * @private
			 */
			_modalOpen = function( link ) {
				$modal
					.trigger( 'setModalData', [ link, $modal, $container ] )
					.addClass( 'active' )
					.removeClass( 'inactive' )
					.data( 'open', true )
					.trigger( 'modalOpen', [ link, $modal, $container ] );

				$( 'html' ).addClass( 'modal-open' );
			},
			/**
			 * Close the modal window from an event.
			 *
			 * @param link
			 * @private
			 */
			_modalClose = function( link ) {
				$modal
					.addClass( 'inactive' )
					.removeClass( 'active' )
					.removeClass( 'has-content' )
					.data( 'open', false )
					.trigger( 'modalClose', [ link, $modal, $container ] );

				$( 'html' ).removeClass( 'modal-open' );

				$container.html( '' );
			},
			/**
			 * Set current modal openers.
			 *
			 * @private
			 */
			_setModalOpeners = function() {
				$modalOpeners = $( '.modal-open' );
			},
			/**
			 * Setup actions on modal openers.
			 *
			 * @private
			 */
			_setup = function() {
				if ( $modalOpeners.length ) {
					$modalOpeners.unbind( 'click', _setupClick );
				}

				_setModalOpeners();

				$modalOpeners.click( _setupClick );
			},
			/**
			 * Fire when a modal opener is clicked.
			 *
			 * @param e
			 * @private
			 */
			_setupClick = function( e ) {
				e.preventDefault();
				_modalOpen( $( this ) );
			};

		if ( ! $modal.length ) {
			return;
		}

		// Set the initial open status to false.
		$modal.data( 'open', false );

		Pace.on( 'done', _setup );

		// Close modal when clicking certain elements.
		$( '.modal-close' ).click( function( e ) {
			e.preventDefault();
			_modalClose( $( this ) );
		} );

		// Close modal when clicking anywhere inside the window but the content.
		$modal.click( function() {
			_modalClose( null );
		} );
		$container.click( function( e ) {
			e.stopPropagation();
		} );

		// Close modal when pressing the "escape" key.
		$( document ).keyup( function( e ) {
			if ( e.keyCode === 27 && $modal.data( 'open' ) ) { // "escape" key maps to keycode `27`
				_modalClose( null );
			}
		} );

		// Send AJAX request when the "modalOpen" event happens.
		$modal.bind( 'modalOpen', function( event, link, $modal, $container ) {
			$.ajax( {
				type: 'POST',
				url: window.generalData.adminAjaxURL,
				data: window.NiceModalData,
				beforeSend: function() {
					$.NiceDev.log( 'Sending AJAX request to: ' + window.generalData.adminAjaxURL );
				},
				success: function( response ) {
					$container.html( response );
					$modal.addClass( 'has-content' );
					$modal.trigger( 'modalOpenSuccess', [ link, $modal, $container ] );

					$( document ).trigger( 'NiceDOMChanged' );
				},
				error: function ( jqXHR, textStatus, errorThrown ) {
					$.NiceDev.log( jqXHR, textStatus, errorThrown );
				}
			} );
		} );

		/**
		 * Modify modal data when the "setModalData" event happens.
		 *
		 * @since 1.0.0
		 */
		$modal.bind( 'setModalData', function( event, link ) {
			var $link = $( link );

			if ( ! $link.data( 'product_id' ) ) {
				return;
			}

			window.NiceModalData = {
				action: 'nice_woocommerce_integration_quick_view',
				nonce: window.generalData.playNiceNonce,
				id: $link.data( 'product_id' )
			};
		} );

		// Fire Flexslider for the modal window after the content is generated via AJAX.
		$modal.bind( 'modalOpenSuccess', function() {
			window.NiceWooCommerceOwlCarousel.ready();
		} );
	},

	/**
	 * Watch clicks on wishlist link.
	 *
	 * @since 1.0.0
	 */
	watchWishlist = function() {
		var _trackWishlist = function() {
			$( '.item-wishlist-icon .add_to_wishlist' ).click( function() {
				if ( $( this ).data( 'tracked' ) ) {
					return;
				}

				$( this ).data( 'tracked', true );

				var $icon = $( this ).find( 'i' );

				if ( $icon.hasClass( 'fa-heart' ) ) {
					$icon.removeClass( 'fa-heart' );
				}

				$icon.addClass( 'fa-circle-o-notch' ).addClass( 'fa-spin' );
			} );
		};

		_trackWishlist();

		Pace.on( 'done', _trackWishlist );
	},

	/**
	 * Update wishlist counter each time an item is added to the wishlist.
	 *
	 * @since 1.0.0
	 */
	watchWishlistCounter = function() {
		var $wishlistCounter = $( '#top-bar-wishlist-count' ),
			$body = $( 'body' );

		$body.on( 'added_to_wishlist', function() {
			var count = $wishlistCounter.data( 'count' ) + 1;

			// Update counter.
			$wishlistCounter.data( 'count', count );
			$wishlistCounter.attr( 'data-count', count );
			$wishlistCounter.html( count );
		} );

		$body.on( 'removed_from_wishlist', function() {
			var count = $wishlistCounter.data( 'count' ) - 1;

			// Update counter.
			$wishlistCounter.data( 'count', count );
			$wishlistCounter.attr( 'data-count', count );
			$wishlistCounter.html( count );
		} );
	},

	/**
	 * Fire events on document ready, and bind other events.
	 *
	 * @since 1.0.0
	 */
	ready = function() {
		bind();
		display();
		addToCart();
		watchModal();
		watchWishlist();
		watchWishlistCounter();
	};

	// Expose the ready function to the world.
	return {
		ready: ready
	};

} )( jQuery );

jQuery( NiceWooCommerce.ready );
