/**
 * Smart by NiceThemes.
 *
 * Manage Owl Carousel implementation for WooCommerce.
 *
 * @since   1.0.1
 * @package Smart
 */
var NiceWooCommerceOwlCarousel = ( function( $ ) {
	'use strict';

	var $gallery,

	$thumbnails,

	fired = false,

	/**
	 * Load Owl Carousel implementation.
	 *
	 * @since 1.0.1
	 */
	load = function() {
		var url = $.NiceLazyScript( 'owlCarousel' );

		$gallery    = $( '.owl-carousel:not(.owl-thumbs)' );
		$thumbnails = $( '.owl-carousel.owl-thumbs' );

		if ( 'function' !== typeof $.fn.owlCarousel && url ) {
			$.getScript( url, init );
		} else if ( 'function' === typeof $.fn.owlCarousel ) {
			init();
		} else {
			$.NiceDev.log( 'Warning: Owl Carousel is not initialized.' );
		}
	},

	/**
	 * Initialize Owl Carousel implementation.
	 *
	 * @since 1.0.1
	 */
	init = function() {
		loadGallery();
		loadThumbnails();

		// Notify about DOM changes.
		$( document ).trigger( 'NiceDOMChanged' );
	},

	/**
	 * Load carousel for single product gallery.
	 *
	 * @since 1.0.1
	 */
	loadGallery = function() {
		if ( ! $gallery.length ) {
			return;
		}

		if ( ! $gallery.hasClass( 'owl-carousel' ) ) {
			$( document ).trigger( 'NiceSingleProductGalleryLoaded', [ $gallery ] );

			return;
		}

		// Make sure Fancybox doesn't add cloned slides to its gallery.
		$( document ).on( 'NiceFancyboxSetup', function() {
			$gallery.find( '.cloned [data-fancybox]' ).removeAttr( 'data-fancybox' );
		} );

		$gallery.on( 'initialized.owl.carousel', function() {
			$( document ).trigger( 'NiceSingleProductGalleryLoaded', [ $gallery ] );
		} );

		$( document ).on( 'NiceSingleProductGalleryLoaded', function() {
			fired = true;
		} );

		$gallery.owlCarousel( {
			rtl : 'rtl' === $( 'html' ).attr( 'dir' ),
			loop : true,
			nav : true,
			navText: [ window.NiceWooCommerceData.prev, window.NiceWooCommerceData.next ],
			dots : false,
			URLhashListener : true,
			autoplayHoverPause : true,
			startPosition: 'URLHash',
			responsive : {
				0 : {
					items : 1
				}
			}
		} );
	},

	/**
	 * Load carousel for single product thumbnails.
	 *
	 * @since 1.0.1
	 */
	loadThumbnails = function() {
		var $imgs       = $thumbnails.length ? $thumbnails.find( 'img' ) : null,
			useCarousel = $thumbnails.hasClass( 'owl-carousel' ),
			hash        = window.location.hash.substr( 1 ),
			focused     = false,

			/**
			 * Set focus class on a given thumbnail.
			 *
			 * @param   {jQuery} $img
			 * @private
			 */
			_setFocus   = function( $img ) {
				if ( useCarousel ) {
					$thumbnails.find( '.owl-item' ).removeClass( 'focus' );
					$img.parents( '.owl-item' ).addClass( 'focus' );
				} else {
					$thumbnails.find( 'li' ).removeClass( 'focus' );
					$img.parents( 'li' ).addClass( 'focus' );
				}

				focused = true;
			};

		// Return early if we don't have any images.
		if ( ! $imgs || ! $imgs.length ) {
			return;
		}

		$imgs.each( function() {
			// Set focus for active thumbnail on page load.
			if ( hash && ( ( '#' + hash ) === $( this ).parent().attr( 'href' ) ) ) {
				_setFocus( $( this ) );
			}

			// Set focus for active thumbnail when clicking an image.
			$( this ).on( 'click', function() {
				_setFocus( $( this ) );
			} );
		} );

		// Set focus for active thumbnail in the first image if it hasn't been set already.
		if ( ! focused ) {
			_setFocus( $( $imgs[0] ) );
		}

		// Return early if we don't need to use Owl Carousel.
		if ( ! useCarousel ) {
			return;
		}

		$thumbnails.owlCarousel( {
			loop : false,
			margin : 5,
			nav : true,
			navText: [ window.NiceWooCommerceData.prev, window.NiceWooCommerceData.next ],
			dots : false,
			responsive : {
				0 : {
					items : 4
				}
			}
		} );
	},

	/**
	 * Check if Owl Carousel was already fired for the current gallery.
	 *
	 * @since 1.1
	 *
	 * @returns {boolean}
	 */
	isFired = function () {
		return fired;
	},

	ready = function() {
		load();
	};

	return {
		ready: ready,
		fired: isFired
	};

} )( jQuery );

jQuery( NiceWooCommerceOwlCarousel.ready );