<?php
/**
 * Smart by NiceThemes.
 *
 * This file includes general functions for WooCommerce integration.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_woocommerce_integration_header_display_navigation' ) ) :
/**
 * Return whether to display or not the shopping cart icon in the header.
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_woocommerce_integration_header_display_navigation() {
	$nice_display_woocommerce_navigation = nice_get_option( 'nice_woocommerce_navigation' );

	if ( is_page() || is_single() ) {
		$_post_display_woocommerce_navigation = get_post_meta( get_the_ID(), '_post_header_woocommerce_navigation', true );

		if ( '' !== $_post_display_woocommerce_navigation ) {
			$nice_display_woocommerce_navigation = $_post_display_woocommerce_navigation;
		}
	}

	return nice_bool( apply_filters( 'nice_woocommerce_integration_header_display_navigation', $nice_display_woocommerce_navigation ) );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_loop_use_view_mode' ) ) :
/**
 * Check if "View mode" option should be displayed before the list of products.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_products_loop_use_view_mode() {
	$use = nice_bool_option( '_woocommerce_allow_shop_layout_switch', true );

	/**
	 * @hook nice_woocommerce_integration_products_loop_use_view_mode
	 *
	 * Hook in here to modify the return value of the function used to check if
	 * a view mode should be used.
	 *
	 * @since 1.0.2
	 */
	return apply_filters( 'nice_woocommerce_integration_products_loop_use_view_mode', $use );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_loop_view_mode' ) ) :
/**
 * Obtain the current view mode for a list of products.
 *
 * @since  1.0.2
 *
 * @return mixed|null|string
 */
function nice_woocommerce_integration_products_loop_view_mode() {
	$default_view_mode  = nice_get_option( '_woocommerce_shop_layout', 'grid' );
	$use_view_mode      = nice_woocommerce_integration_products_loop_use_view_mode();
	$allowed_view_modes = $use_view_mode ? array(
		'grid',
		'list',
	) : array();

	if ( isset( $_GET['view_mode'] ) && in_array( wp_unslash( $_GET['view_mode'] ), $allowed_view_modes, true ) ) {
		$view_mode = esc_attr( wp_unslash( $_GET['view_mode'] ) );
	}

	if ( ! isset( $view_mode ) ) {
		$view_mode = $default_view_mode;
	}

	/**
	 * @hook nice_woocommerce_integration_products_loop_view_mode
	 *
	 * Hook in here to modify the product list view mode.
	 *
	 * @since 1.0.2
	 */
	return apply_filters( 'nice_woocommerce_integration_products_loop_view_mode', $view_mode );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_loop_container_get_class' ) ) :
/**
 * Obtain classes for products container.
 *
 * @since 1.0.0
 *
 * @param mixed|array|string $class
 *
 * @return array
 */
function nice_woocommerce_integration_products_loop_container_get_class( $class = null ) {
	$view_mode = nice_woocommerce_integration_products_loop_view_mode();

	$classes = array(
		'grid',
		'style-default',
	);

	if ( 'list' === $view_mode ) {
		$classes[] = 'grid-list';
		$classes[] = 'single-gutter-v';
		$classes[] = 'no-gap-h';
	}

	if ( 'grid' === $view_mode ) {
		$classes[] = 'half-gutter-v';
		$classes[] = 'single-gutter-h';
	}

	// Add classes from function argument.
	if ( ! empty( $class ) ) {
		if ( ! is_array( $class ) ) {
			$class = preg_split( '#\s+#', $class );
		}

		$classes = array_merge( $classes, $class );
	}

	$classes = array_map( 'esc_attr', $classes );

	return apply_filters( 'nice_woocommerce_integration_products_loop_container_class', $classes );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_loop_container_get_data' ) ) :
/**
 * Obtain data for products container.
 *
 * @since 1.0.0
 *
 * @param array|string $custom_data
 *
 * @return array
 */
function nice_woocommerce_integration_products_loop_container_get_data( $custom_data = array() ) {
	global $woocommerce_loop;

	$columns = isset( $woocommerce_loop['columns'] ) ? $woocommerce_loop['columns'] : nice_get_option( 'nice_woocommerce_columns', 4 );

	$data = array(
		'columns' => intval( $columns ),
	);

	if ( ! empty( $custom_data ) ) {
		$data = array_merge( $data, $custom_data );
	}

	$data = array_map( 'esc_attr', $data );

	return apply_filters( 'nice_woocommerce_integration_products_loop_container_data', $data );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_price_class' ) ) :
/**
 * Print classes for product price.
 *
 * @since 1.0.0
 */
function nice_woocommerce_integration_product_price_class() {
	$class = array(
		'entry-price',
		'price',
		'entry-meta',
		'inherit-skin',
		'inline-block',
	);

	nice_css_classes( apply_filters( 'nice_woocommerce_integration_product_price_class', $class ) );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_price' ) ) :
/**
 * Print the price of a given product.
 *
 * @since 1.0.0
 *
 * @param WC_Product $product
 */
function nice_woocommerce_integration_product_price( WC_Product $product ) {
	$output = $product->get_price_html();

	if ( nice_bool_option( '_woocommerce_add_to_cart_text' ) ) {
		$product_type = method_exists( $product, 'get_type' ) ? $product->get_type() : $product->product_type;

		$class = array(
			'product_type_' . $product_type,
			$product->is_purchasable() && $product->is_in_stock() ? 'add_to_cart_button' : '',
			$product->supports( 'ajax_add_to_cart' ) ? 'ajax_add_to_cart' : '',
		);

		$output .= '<span class="permalink"><a class="' . esc_attr( join( ' ', $class ) ) . '" href="' . esc_url( $product->add_to_cart_url() ) . '" data-product_id="' . esc_attr( get_the_ID() ) . '">' . esc_html( $product->add_to_cart_text() ) . '</a></span>';
	}

	echo $output; // WPCS: XSS Ok.
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_gallery_position' ) ) :
/**
 * Obtain the position of the product image gallery.
 *
 * @since 1.1
 *
 * @return string
 */
function nice_woocommerce_integration_product_gallery_position() {
	static $gallery_position = null;

	if ( is_null( $gallery_position ) ) {
		if ( is_product() ) {
			$gallery_position = get_post_meta( get_the_ID(), '_product_gallery_position', true );
		}

		if ( ! $gallery_position ) {
			$gallery_position = nice_get_option( '_woocommerce_gallery_position' );
		}
	}

	return $gallery_position;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_empty_price_html' ) ) :
add_filter( 'woocommerce_empty_price_html', 'nice_woocommerce_integration_empty_price_html' );
add_filter( 'woocommerce_variable_empty_price_html', 'nice_woocommerce_integration_empty_price_html' );
/**
 * Get fallback HTML for empty price.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_empty_price_html() {
	$view_mode = nice_woocommerce_integration_products_loop_view_mode();

	if ( ! is_product() && ! is_cart() && ! is_checkout() && 'grid' === $view_mode ) {
		return '<span class="woocommerce-Price-amount amount">&nbsp;</span>';
	}

	return '';
}
endif;
