<?php
/**
 * Smart by NiceThemes.
 *
 * This file includes functions to manage WooCommerce single products.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_woocommerce_integration_single_product_init' ) ) :
add_action( 'wp', 'nice_woocommerce_integration_single_product_init' );
/**
 * Manage actions needed to correctly display a single product.
 *
 * @since 1.0.0
 */
function nice_woocommerce_integration_single_product_init() {
	if ( ! is_product() ) {
		return;
	}

	// Remove the default Image & Thumbnails.
	remove_action( 'woocommerce_product_thumbnails', 'woocommerce_show_product_thumbnails', 20 );
	add_filter( 'woocommerce_single_product_image_html', '__return_empty_string' );

	// Add our custom function to parse the images :)
	add_action( 'woocommerce_product_thumbnails', 'nice_woocommerce_integration_single_product_images' );

	// remove the default product meta
	remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_meta', 40 );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_single_product_images' ) ) :
/**
 * Display images gallery for single product.
 *
 * The carousel is added only if there is more than one image.
 *
 * @since 1.0.0
 *
 * @param bool $display_thumbnails
 */
function nice_woocommerce_integration_single_product_images( $display_thumbnails = true ) {
	global $product;

	// Make sure $display_thumbnails is a boolean.
	$display_thumbnails = is_bool( $display_thumbnails ) ? $display_thumbnails : true;

	if ( has_post_thumbnail() ) {
		$gallery_layout  = nice_woocommerce_integration_product_gallery_layout();
		$use_thumbnails  = stripos( $gallery_layout, 'slider-thumbs' ) === 0;
		$attachment_ids  = method_exists( $product, 'get_gallery_image_ids' ) ? $product->get_gallery_image_ids() : $product->get_gallery_attachment_ids();
		$image           = wp_get_attachment_image( get_post_thumbnail_id(), apply_filters( 'single_product_large_thumbnail_size', 'shop_single' ) );
		$image_expand    = '<a class="product-expand" data-fancybox="group" rel="group[woocommerce]" href="' . wp_get_attachment_url( get_post_thumbnail_id() ) . '" title="' . get_the_title( get_post_thumbnail_id() ) . '" data-caption="' . get_the_title( get_post_thumbnail_id() ) . '">' . $image . '</a>';
		$image_thumb     = wp_get_attachment_image( get_post_thumbnail_id(), apply_filters( 'single_product_small_thumbnail_size', 'shop_thumbnail' ) );
		$slider_contents = '<li data-hash="slide-0">' . $image_expand . '</li>';
		$carousel_contents = $use_thumbnails ? '<li><a href="#slide-0">' . $image_thumb . '</a></li>' : null;

		$loop    = 0;
		$columns = apply_filters( 'woocommerce_product_thumbnails_columns', 3 );

		if ( count( $attachment_ids ) ) {
			foreach ( $attachment_ids as $attachment_id ) {
				$classes = array( 'zoom' );

				if ( 0 === $loop || 0 === ( $loop % $columns ) ) {
					$classes[] = 'first';
				}
				if ( 0 === ( ( $loop + 1 ) % $columns ) ) {
					$classes[] = 'last';
				}

				$image_link = wp_get_attachment_url( $attachment_id );

				if ( ! $image_link ) {
					continue;
				}

				$image        = wp_get_attachment_image( $attachment_id, apply_filters( 'single_product_small_thumbnail_size', 'shop_thumbnail' ) );
				$image_large  = wp_get_attachment_image( $attachment_id, apply_filters( 'single_product_large_thumbnail_size', 'shop_single' ) );
				$image_expand = '<a class="product-expand" data-fancybox="group" rel="group[woocommerce]" href="' . wp_get_attachment_url( $attachment_id ) . '" title="' . get_the_title( $attachment_id ) . '" data-caption="' . get_the_title( get_post_thumbnail_id() ) . '">' . $image_large . '</a>';

				$slider_contents .= '<li data-hash="slide-' . esc_attr( $loop + 1 ) . '">' . $image_expand . '</li>';

				if ( $use_thumbnails ) {
					$carousel_contents .= '<li><a href="#slide-' . esc_attr( $loop + 1 ) . '">' . $image . '</a></li>';
				}

				$loop ++;
			}
		}

		// Start template.

		/**
		 * @hook nice_woocommerce_integration_before_single_product_gallery
		 *
		 * Print HTLM before the single product gallery.
		 *
		 * @since 1.0.0
		 */
		do_action( 'nice_woocommerce_integration_before_single_product_gallery' );
		?>

		<div id="product-gallery-container" data-layout="gallery-<?php echo esc_attr( $gallery_layout ); ?>">
			<ul id="product-gallery" <?php nice_woocommerce_integration_single_product_gallery_class(); ?>>
				<?php echo $slider_contents; // WPCS: XSS Ok. ?>
			</ul>
		</div>

		<?php if ( $display_thumbnails && count( $attachment_ids ) && $carousel_contents ) : ?>
			<div id="product-thumbnails-container" data-layout="thumbnails-<?php echo esc_attr( $gallery_layout ); ?>">
				<ul id="product-thumbnails" <?php nice_woocommerce_integration_single_product_gallery_thumbnails_class(); ?>>
					<?php echo $carousel_contents; // WPCS: XSS Ok. ?>
				</ul>
			</div>
		<?php endif; ?>

		<?php
		/**
		 * @hook nice_woocommerce_integration_after_single_product_gallery
		 *
		 * Print HTLM before the single product gallery.
		 *
		 * @since 1.0.0
		 */
		do_action( 'nice_woocommerce_integration_after_single_product_gallery' );

		// End template.
	}
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_gallery_layout' ) ) :
/**
 * Obtain gallery layout for the current product.
 *
 * @since  1.0.2
 *
 * @return mixed|bool|string
 */
function nice_woocommerce_integration_product_gallery_layout() {
	static $layout = null;

	if ( is_null( $layout ) ) {
		$layout = get_post_meta( get_the_ID(), '_product_gallery_layout', true );

		if ( ! $layout ) {
			$layout = nice_get_option( '_woocommerce_gallery_layout', 'slider-thumbs-bottom' );
		}

		/**
		 * @hook nice_woocommerce_integration_product_gallery_layout
		 *
		 * Hook in here to modify the product's gallery loayout.
		 */
		$layout = apply_filters( 'nice_woocommerce_integration_product_gallery_layout', $layout );
	}

	return $layout;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_single_product_gallery_class' ) ) :
/**
 * Print classes for single product gallery.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_single_product_gallery_class() {
	$gallery_layout = nice_woocommerce_integration_product_gallery_layout();
	$use_slider     = stripos( $gallery_layout, 'slider' ) === 0;

	$classes = array();

	if ( $use_slider ) {
		$classes = array_merge( $classes, array(
				'slides',
				'owl-carousel',
				'owl-theme',
			)
		);
	}

	nice_css_classes( $classes );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_single_product_gallery_thumbnails_class' ) ) :
/**
 * Print classes for single product gallery thumbnails.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_single_product_gallery_thumbnails_class() {
	$gallery_layout = nice_woocommerce_integration_product_gallery_layout();
	$use_slider     = 'slider-thumbs-bottom' === $gallery_layout;

	$classes = array();

	if ( $use_slider ) {
		$classes = array_merge( $classes, array(
				'slides',
				'owl-carousel',
				'owl-theme',
				'owl-thumbs',
			)
		);
	}

	nice_css_classes( $classes );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_display_related_products' ) ) :
/**
 * Check if related products should be displayed.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_display_related_products() {
	if ( is_product() ) {
		$value = get_post_meta( get_the_ID(), '_product_linked_display_related', true );

		if ( $value ) {
			$display = nice_bool( $value );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_bool_option( '_woocommerce_products_related', true );
	}

	return $display;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_hide_related_products' ) ) :
add_action( 'wp', 'nice_woocommerce_integration_hide_related_products' );
/**
 * Hide related products if set from theme options.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_hide_related_products() {
	if ( nice_woocommerce_integration_display_related_products() ) {
		return;
	}

	remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20 );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_related_products_amount' ) ) :
add_filter( 'woocommerce_output_related_products_args', 'nice_woocommerce_integration_related_products_amount' );
/**
 * Change the amount and columns for the related products functionality
 *
 * @since 1.0.0
 *
 * @param  array $args
 *
 * @return array
 */
function nice_woocommerce_integration_related_products_amount( $args ) {
	$args['posts_per_page'] = nice_woocommerce_integration_products_related_limit();
	$args['columns']        = nice_woocommerce_integration_products_related_columns();

	return $args;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_related_limit' ) ) :
/**
 * Obtain the maximum number of related products to be displayed in the current page.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_products_related_limit() {
	if ( is_product() ) {
		$use_custom = 'custom' === get_post_meta( get_the_ID(), '_product_linked_related_limit', true );

		if ( $use_custom ) {
			$display = get_post_meta( get_the_ID(), '_product_linked_related_limit_custom', true );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_get_option( '_woocommerce_products_related_limit', 4 );
	}

	return $display;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_products_related_columns' ) ) :
/**
 * Obtain the maximum number of columns for related products in the current page.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_products_related_columns() {
	if ( is_product() ) {
		$use_custom = 'custom' === get_post_meta( get_the_ID(), '_product_linked_related_columns', true );

		if ( $use_custom ) {
			$display = get_post_meta( get_the_ID(), '_product_linked_related_columns_custom', true );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_get_option( '_woocommerce_products_related_columns', 4 );
	}

	return $display;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_display_upsells' ) ) :
/**
 * Check if related products should be displayed.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_display_upsells() {
	if ( is_product() ) {
		$value = get_post_meta( get_the_ID(), '_product_linked_display_upsells', true );

		if ( $value ) {
			$display = nice_bool( $value );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_bool_option( '_woocommerce_products_upsells', true );
	}

	return $display;
}
endif;

if ( ! function_exists( 'woocommerce_output_upsells' ) ) :
remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_upsell_display', 15 );
add_action( 'woocommerce_after_single_product_summary', 'woocommerce_output_upsells', 15 );
/**
 * Change the amount and columns for the upsells product functionality
 *
 * @since 1.0.0
 */
function woocommerce_output_upsells() {
	if ( ! nice_woocommerce_integration_display_upsells() ) {
		return;
	}

	$limit   = intval( nice_woocommerce_integration_upsells_limit() );
	$columns = intval( nice_woocommerce_integration_upsells_columns() );

	woocommerce_upsell_display( $limit, $columns );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_upsells_limit' ) ) :
/**
 * Obtain the maximum number of up-sells to be displayed in the current page.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_upsells_limit() {
	if ( is_product() ) {
		$use_custom = 'custom' === get_post_meta( get_the_ID(), '_product_linked_upsells_limit', true );

		if ( $use_custom ) {
			$display = get_post_meta( get_the_ID(), '_product_linked_upsells_limit_custom', true );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_get_option( '_woocommerce_products_upsells_limit', 4 );
	}

	return $display;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_upsells_columns' ) ) :
/**
 * Obtain the maximum number of columns for related products in the current page.
 *
 * @since 1.0.2
 */
function nice_woocommerce_integration_upsells_columns() {
	if ( is_product() ) {
		$use_custom = 'custom' === get_post_meta( get_the_ID(), '_product_linked_upsells_columns', true );

		if ( $use_custom ) {
			$display = get_post_meta( get_the_ID(), '_product_linked_upsells_columns_custom', true );
		}
	}

	if ( ! isset( $display ) ) {
		$display = nice_get_option( '_woocommerce_products_upsells_columns', 4 );
	}

	return $display;
}
endif;

if ( ! function_exists( 'nice_woocommerce_sale_flash' ) ) :
add_filter( 'woocommerce_sale_flash', 'nice_woocommerce_sale_flash' );
/**
 * Change "Sale" text.
 *
 * @since 1.0.0
 */
function nice_woocommerce_sale_flash() {
	$onsale_text = nice_get_option( 'nice_woocommerce_sale_flash_text' );
	$onsale_text = $onsale_text ? $onsale_text : esc_html__( 'Sale', 'smart' );

	$output = '<span class="onsale">' . $onsale_text . '</span>';

	return $output;
}
endif;

if ( ! function_exists( 'nice_woocommerce_product_description_heading' ) ) :
add_filter( 'woocommerce_product_description_heading', 'nice_woocommerce_product_description_heading' );
/**
 * Custom "Product Description" title
 *
 * @since 1.0.0
 */
function nice_woocommerce_product_description_heading() {
	return apply_filters( 'nice_woocommerce_product_description_heading', '' );
}
endif;

if ( ! function_exists( 'nice_woocommerce_product_additional_information_heading' ) ) :
add_filter( 'woocommerce_product_additional_information_heading', 'nice_woocommerce_product_additional_information_heading' );
/**
 * Custom "Additional Information" title
 *
 * @since 1.0.0
 */
function nice_woocommerce_product_additional_information_heading() {
	return apply_filters( 'nice_woocommerce_product_additional_information_heading', '' );
}
endif;

if ( ! function_exists( 'nice_woocommerce_template_single_meta' ) ) :
add_action( 'woocommerce_single_product_summary', 'nice_woocommerce_template_single_meta', 40 );
/**
 * Use our own product meta template
 *
 * @since 1.0.0
 */
function nice_woocommerce_template_single_meta() {
	global $product;

	?>
	<div class="product_meta">

		<?php do_action( 'woocommerce_product_meta_start' ); ?>

		<?php if ( wc_product_sku_enabled() && ( $product->get_sku() || $product->is_type( 'variable' ) ) ) : ?>

			<span class="sku_wrapper"><span class="meta-label"><?php esc_html_e( 'SKU', 'woocommerce' ); ?></span> <span class="sku"><?php echo ( $sku = $product->get_sku() ) ? $sku : esc_html__( 'N/A', 'smart' ); ?></span></span>

		<?php endif; ?>

		<?php nice_woocommerce_integration_single_product_categories(); ?>

		<?php nice_woocommerce_integration_single_product_tags(); ?>

		<?php do_action( 'woocommerce_product_meta_end' ); ?>

	</div>
	<?php
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_tabs' ) ) :
add_filter( 'woocommerce_product_tabs', 'nice_woocommerce_integration_product_tabs', 9999 );
/**
 * Filter single product tabs.
 *
 * @since  1.0.2
 *
 * @param  array $tabs
 *
 * @return array
 */
function nice_woocommerce_integration_product_tabs( array $tabs = array() ) {
	if ( is_product() && isset( $tabs['reviews'] ) ) {

		$display = get_post_meta( get_the_ID(), '_product_reviews_display', true );

		if ( $display !== '' ) {
			if ( 'false' === $display ) {
				unset( $tabs['reviews'] );
			}
		} else {
			$display = nice_bool_option( '_product_reviews_display', true );

			if ( ! $display ) {
				unset( $tabs['reviews'] );
			}
		}
	}

	return $tabs;
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_tabs_position' ) ) :
/**
 * Obtain side for product tabs to be displayed.
 *
 * @since  1.0.2
 *
 * @return mixed|bool|string
 */
function nice_woocommerce_integration_product_tabs_position() {
	$position = get_post_meta( get_the_ID(), '_product_tabs_position', true );

	if ( empty( $position ) ) {
		$position = nice_get_option( '_product_tabs_position', 'top' );
	}

	/**
	 * @hook nice_woocommerce_integration_product_tabs_position
	 *
	 * Hook in here to modify the product's tabs side.
	 */
	return apply_filters( 'nice_woocommerce_integration_product_tabs_position', $position );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_tabs_direction' ) ) :
/**
 * Obtain position for product tabs to be displayed.
 *
 * @since  1.0.2
 *
 * @return string
 */
function nice_woocommerce_integration_product_tabs_direction() {
	switch ( nice_woocommerce_integration_product_tabs_position() ) {
		case 'side':
			$position = nice_woocommerce_integration_product_tabs_position_side();
			break;
		default:
			$position = nice_woocommerce_integration_product_tabs_position_top();
	}

	/**
	 * @hook nice_woocommerce_integration_product_tabs_direction
	 *
	 * Hook in here to modify the product's tabs direction.
	 */
	return apply_filters( 'nice_woocommerce_integration_product_tabs_direction', $position );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_tabs_position_top' ) ) :
/**
 * Obtain side for product tabs to be displayed (Top)
 *
 * @since  1.1
 *
 * @return mixed|bool|string
 */
function nice_woocommerce_integration_product_tabs_position_top() {
	$side = get_post_meta( get_the_ID(), '_product_tabs_position_top', true );

	if ( empty( $side ) ) {
		$side = nice_get_option( '_product_tabs_position_top', 'left' );
	}

	/**
	 * @hook nice_woocommerce_integration_product_tabs_position_top
	 *
	 * Hook in here to modify the product's tabs side.
	 */
	return apply_filters( 'nice_woocommerce_integration_product_tabs_position_top', $side );
}
endif;

if ( ! function_exists( 'nice_woocommerce_integration_product_tabs_position_side' ) ) :
/**
 * Obtain side for product tabs to be displayed (Side)
 *
 * @since  1.1
 *
 * @return mixed|bool|string
 */
function nice_woocommerce_integration_product_tabs_position_side() {
	$side = get_post_meta( get_the_ID(), '_product_tabs_position_side', true );

	if ( empty( $side ) ) {
		$side = nice_get_option( '_product_tabs_position_side', 'left' );
	}

	/**
	 * @hook nice_woocommerce_integration_product_tabs_position_side
	 *
	 * Hook in here to modify the product's tabs side.
	 */
	return apply_filters( 'nice_woocommerce_integration_product_tabs_position_side', $side );
}
endif;
