<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains public functions for "slider" post type.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_slide_get_slides' ) ) :
/**
 * Obtain list of slides as post objects.
 *
 * @since  1.0.0
 *
 * @return WP_Query
 */
function nice_slide_get_slides() {
	// Allow bypass.
	if ( $slides = apply_filters( 'nice_slide_slides', null ) ) {
		return $slides;
	}

	$slides = new WP_Query( array(
		'post_type' => 'slide',
		'showposts' => nice_get_option( 'nice_slider_entries' ),
		'orderby'   => 'menu_order',
		'order'     => 'ASC',
	) );

	return $slides;
}
endif;

if ( ! function_exists( 'nice_slide_overlay_class' ) ) :
/**
 * Print out classes for main slider overlay.
 *
 * @since 1.0.0
 */
function nice_slide_overlay_class() {
	$classes = array(
		'overlay'
	);

	$classes[] = nice_theme_color_background_class( 'black-color' );

	$classes[] = nice_get_option( '_slider_loader_icon', 'beat' );

	/**
	 * @hook nice_slide_overlay_class
	 *
	 * @since 1.0.0
	 *
	 * Modify classes for main slider overlay here.
	 */
	$classes = apply_filters( 'nice_slide_overlay_class', $classes );

	nice_css_classes( $classes );
}
endif;

if ( ! function_exists( 'nice_slide_is_video' ) ) :
/**
 * Check if a slide is a video slide
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get caption from.
 *
 * @return string
 */
function nice_slide_is_video( WP_Post $post = null ) {
	if ( is_null( $post ) ) {
		$post = get_post();
	}

	return nice_bool( get_post_meta( $post->ID, '_slide_video', true ) );
}
endif;

if ( ! function_exists( 'nice_slide_get_embed' ) ) :
/**
 * Get the embedded video for a slide.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get caption from.
 *
 * @return string
 */
function nice_slide_get_embed( WP_Post $post = null ) {

	if ( ! $post ) {
		$post = get_post();
	}

	$video = nice_embed( apply_filters( 'nice_slide_video_embed_args', array(
	        'key'      => 'embed',
	        'id'       => $post->ID,
	        'width'    => 2560,
	        'height'   => 400,
	        'wrap'     => true,
	        'echo'     => false,
	        'embed_id' => 'player_' . $post->ID,
		)
	) );

	return $video;
}
endif;

if ( ! function_exists( 'nice_slide_get_image' ) ) :
/**
 * Obtain image for a slide.
 *
 * @since  1.0.0
 *
 * @param  WP_Post           $post Post object to get image from.
 * @param  array             $args Arbitrary arguments to obtain image.
 *
 * @return mixed|void|string
 */
function nice_slide_get_image( WP_Post $post = null, $args = array() ) {
	// Allow bypass.
	if ( $image = apply_filters( 'nice_slide_image', '' ) ) {
		return $image;
	}

	if ( ! $post ) {
		$post = get_post();
	}

	$defaults = apply_filters( 'nice_slide_image_args', array(
			'size'  => 'nice-slide',
			'key'   => 'slide-image',
			'class' => '',
			'id'    => $post->ID,
			'echo'  => false,
		)
	);

	$args = wp_parse_args( $args, $defaults );

	$image = nice_image( $args );

	return $image;
}
endif;

if ( ! function_exists( 'nice_slide_get_description' ) ) :
/**
 * Obtain the description of a slide.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get description from.
 *
 * @return string
 */
function nice_slide_get_description( WP_Post $post = null ) {
	// Allow bypass.
	if ( $description = apply_filters( 'nice_slide_description', '' ) ) {
		return $description;
	}

	if ( ! $post ) {
		$post = get_post();
	}

	$description = wpautop( get_post_meta( $post->ID, 'slide_description', true ) );

	return $description;
}
endif;

if ( ! function_exists( 'nice_slide_get_buttons' ) ) :
/**
 * Obtain buttons of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_get_buttons( WP_Post $post = null ) {
	// Allow bypass.
	if ( $buttons = apply_filters( 'nice_slide_buttons', '' ) ) {
		return $buttons;
	}

	if ( ! $post ) {
		$post = get_post();
	}

	$button_url = get_post_meta( $post->ID, '_button_url', true );

	if ( nice_bool( $button_url ) ) {

		$button_url_text = get_post_meta( $post->ID, '_button_url_text', true );
		$button_classes = array();

		if ( empty( $button_url_text ) ) {
			$button_url_text = esc_html__( 'Read More', 'smart' );
		}

		$button_classes[] = 'btn';
		$button_classes[] = nice_theme_size_button_class( nice_slide_get_button_size( $post ) );
		$button_classes[] = nice_theme_shape_button_class( nice_slide_get_button_shape( $post ) );
		$button_classes[] = nice_theme_style_button_class( nice_slide_get_button_outlined( $post ) );
		$button_classes[] = nice_slide_get_button_text_transform( $post );
		$button_classes[] = nice_slide_get_button_color( $post );

		$buttons .= '<a ' . nice_css_classes( $button_classes, false ) . ' href="' . esc_url( $button_url ) . '" title="' . esc_attr( $post->post_title ) . '">' . $button_url_text . '</a>';
	}

	$button_2 = get_post_meta( $post->ID, '_slide_button_2', true );

	if ( nice_bool( $button_2 ) ) {

		$button_2_url = get_post_meta( $post->ID, '_button_2_url', true );
		$button_2_url_text = get_post_meta( $post->ID, '_button_2_url_text', true );
		$button_2_classes = array();

		$button_2_classes[] = 'btn';
		$button_2_classes[] = nice_theme_size_button_class( nice_slide_get_button_size( $post, 'button_2' ) );
		$button_2_classes[] = nice_theme_shape_button_class( nice_slide_get_button_shape( $post, 'button_2' ) );
		$button_2_classes[] = nice_theme_style_button_class( nice_slide_get_button_outlined( $post, 'button_2' ) );
		$button_2_classes[] = nice_slide_get_button_text_transform( $post, 'button_2' );
		$button_2_classes[] = nice_slide_get_button_color( $post, 'button_2' );

		if ( empty( $button_2_url_text ) ) {
			$button_2_url_text = esc_html__( 'Read More', 'smart' );
		}

		$buttons .= '<a ' . nice_css_classes( $button_2_classes, false ) . ' href="' . esc_url( $button_2_url ) . '" title="' . esc_attr( $post->post_title ) . '">' . $button_2_url_text . '</a>';
	}

	if ( ! empty( $buttons ) ) {
		$buttons = '<div class="slide-buttons"><div class="slide-buttons-inner">' . $buttons . '</div></div>';
	}

	return $buttons;
}
endif;

if ( ! function_exists( 'nice_slide_get_button_size' ) ) :
/**
 * Obtain the button size class
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post   Post object to get title from.
 * @param  string $button  The button ID
 *
 * @return string
 */
function nice_slide_get_button_size( WP_Post $post = null, $button = 'button' ) {

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$nice_slide_button_size = get_post_meta( $post->ID, "_{$button}_size", true );

	if ( ! empty( $nice_slide_button_size ) ) {
		return $nice_slide_button_size;
	}

	return nice_get_option( 'nice_slider_caption_button_size', apply_filters( 'nice_slider_button_size', 'xl', $post ) );

}
endif;

if ( ! function_exists( 'nice_slide_get_button_shape' ) ) :
/**
 * Obtain the button shape
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post   Post object to get title from.
 * @param  string $button  The button ID
 *
 * @return string
 */
function nice_slide_get_button_shape( WP_Post $post = null, $button = 'button' ) {

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$nice_slide_button_shape = get_post_meta( $post->ID, "_{$button}_shape", true );

	if ( ! empty( $nice_slide_button_shape ) ) {
		return $nice_slide_button_shape;
	}

	return nice_get_option( 'nice_slider_caption_button_shape', apply_filters( 'nice_slider_button_shape', '', $post ) );

}
endif;

if ( ! function_exists( 'nice_slide_get_button_outlined' ) ) :
/**
 * Obtain the class for the button outlined
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post   Post object to get title from.
 * @param  string $button  The button ID
 *
 * @return string
 */
function nice_slide_get_button_outlined( WP_Post $post = null, $button = 'button' ) {

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$nice_slide_button_outlined = get_post_meta( $post->ID, "_{$button}_outlined", true );

	if ( ! empty( $nice_slide_button_outlined ) ) {
		return $nice_slide_button_outlined;
	}

	return nice_get_option( 'nice_slider_caption_button_outlined', apply_filters( 'nice_slider_button_outlined', 'outline', $post ) );

}
endif;

if ( ! function_exists( 'nice_slide_get_button_text_transform' ) ) :
/**
 * Obtain the class for the button text transformation
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post   Post object to get title from.
 * @param  string $button  The button ID
 *
 * @return string
 */
function nice_slide_get_button_text_transform( WP_Post $post = null, $button = 'button' ) {

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$nice_slide_button_text_transform = get_post_meta( $post->ID, "_{$button}_text_transform", true );

	if ( ! empty( $nice_slide_button_text_transform ) ) {
		return $nice_slide_button_text_transform;
	}

	return nice_get_option( 'nice_slider_caption_button_text_transform', apply_filters( 'nice_slider_button_text_transform', '', $post ) );

}
endif;

if ( ! function_exists( 'nice_slide_get_button_color' ) ) :
/**
 * Obtain the button color from the post/option and return the class.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post   Post object to get title from.
 * @param  string $button  The button ID
 *
 * @return string
 */
function nice_slide_get_button_color( WP_Post $post = null, $button = 'button' ) {

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$nice_slide_button_color = get_post_meta( $post->ID, "_{$button}_color", true );

	if ( empty( $nice_slide_button_color ) ) {
		$colors   = nice_theme_colors();
		$color_id = apply_filters( 'nice_slider_button_color', 'nice_accent_color', $post );

		$nice_slide_button_color = nice_get_option( 'nice_slider_caption_button_color', $colors[ $color_id ]['css_id'] );
	}

	if ( ! empty( $nice_slide_button_color ) ) {
		return nice_theme_color_button_class( $nice_slide_button_color );
	}

	return '';
}
endif;

if ( ! function_exists( 'nice_slide_get_title' ) ) :
/**
 * Obtain title of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_get_title( WP_Post $post = null ) {
	// Allow bypass.
	if ( $title = apply_filters( 'nice_slide_title', '', $post ) ) {
		return $title;
	}

	if ( ! $post ) {
		$post = get_post();
	}

	$heading_tag = get_post_meta( $post->ID, '_slide_caption_heading_tag', true );

	$heading_tag = ( ! empty( $heading_tag ) ) ? $heading_tag : nice_get_option( 'nice_slider_caption_heading_tag', 'h2' );

	$title = "<{$heading_tag} class=\"slide-title\">" . $post->post_title . "</{$heading_tag}>";

	return $title;
}
endif;

if ( ! function_exists( 'nice_slide_get_skin' ) ) :
/**
 * Obtain scheme of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_get_skin( WP_Post $post = null ) {
	// Allow bypass.
	if ( $skin = apply_filters( 'nice_slide_skin', '' ) ) {
		return $skin;
	}

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$skin = get_post_meta( $post->ID, '_slide_skin', true );

	return $skin;
}
endif;

if ( ! function_exists( 'nice_slide_caption_get_position_x' ) ) :
/**
 * Obtain horizontal alignment of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_caption_get_position_x( WP_Post $post = null ) {
	// Allow bypass.
	if ( $position_x = apply_filters( 'nice_slide_caption_position_x', '' ) ) {
		return $position_x;
	}

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$position_x = get_post_meta( $post->ID, '_slide_caption_position_x', true );

	return $position_x;
}
endif;

if ( ! function_exists( 'nice_slide_caption_get_position_y' ) ) :
/**
 * Obtain vertical alignment of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_caption_get_position_y( WP_Post $post = null ) {
	// Allow bypass.
	if ( $position_y = apply_filters( 'nice_slide_caption_position_y', '' ) ) {
		return $position_y;
	}

	if ( is_null( $post ) ) {
		$post = get_post();
	}

	$position_y = get_post_meta( $post->ID, '_slide_caption_position_y', true );

	return $position_y;
}
endif;

if ( ! function_exists( 'nice_slide_get_caption_background' ) ) :
/**
 * Obtain the background caption property of a slide element.
 *
 * @since  1.0.0
 *
 * @param  WP_Post $post Post object to get title from.
 *
 * @return string
 */
function nice_slide_get_caption_background( WP_Post $post = null ) {
	// Allow bypass.
	if ( $caption_background = apply_filters( 'nice_slide_caption_background', '' ) ) {
		return $caption_background;
	}

	if ( ! $post ) {
		$post = get_post();
	}

	$caption_background = get_post_meta( $post->ID, '_slide_caption_background', true );

	if ( nice_bool( $caption_background ) ) {
		$class = 'caption-background';
		return $class;
	}

	return '';
}
endif;

if ( ! function_exists( 'nice_slide_metabox_title' ) ) :
add_filter( 'nice_metabox_title', 'nice_slide_metabox_title', 10, 2 );
/**
 * Set the metabox title for the slide custom post type
 *
 * @since  1.0.0
 *
 * @param  string $title     Default Title
 * @param  string $post_type The current Post Type
 *
 * @return string
 */
function nice_slide_metabox_title( $title, $post_type ) {
	if ( 'slide' === $post_type ) {
		$title = esc_html__( 'Slide Settings', 'smart' );
	}

	return $title;
}
endif;

if ( ! function_exists( 'nice_get_slider_class' ) ) :
/**
 * Obtain array of HTML classes for slider element.
 *
 * @since  1.2.0
 *
 * @param  string|array $class
 * @param  int          $count
 *
 * @return array
 */
function nice_get_slider_class( $class = '', $count = 0 ) {
	// Set initial classes.
	$classes = array(
		'flexslider',
		'slider',
	);

	// Add classes for homepage slider.
	if ( nice_doing_homepage_slider() ) {

		$classes = array_merge( $classes, array(
			'position-' . nice_get_homepage_position(),
			nice_get_option( 'nice_slider_scroll' ) . '-scroll',
		    nice_get_option( 'nice_slider_animation' ) . '-effect',
		) );

		if ( $direction_nav = ( nice_bool( nice_get_option( 'nice_slider_direction_nav' ) ) && $count > 1 ) ) {
			$classes[] = 'has-direction-nav';
		}

		$direction_nav_style = nice_get_option( 'nice_slider_direction_nav_style' );

		if ( isset( $direction_nav_style ) && ( '' !== trim( $direction_nav_style ) ) ) {
			$classes = array_merge( $classes, array(
				'direction-nav-' . $direction_nav_style
				)
			);

			// if the navigation style is solid background then add the background color class
			if ( 'solid' === $direction_nav_style ) {

				$direction_nav_color = nice_get_option( 'nice_slider_direction_nav_color' );
				if ( isset( $direction_nav_color ) && ( '' !== trim( $direction_nav_color ) ) ) {
					$classes = array_merge( $classes, array(
						'direction-nav-' . str_replace( '_', '-', str_replace( 'nice_', '', $direction_nav_color ) )
						)
					);
				}

			}

			$direction_nav_shape = nice_get_option( 'nice_slider_direction_nav_shape' );
			if ( isset( $direction_nav_shape ) && ( '' !== trim( $direction_nav_shape ) ) ) {
				$classes = array_merge( $classes, array(
					'direction-nav-' . $direction_nav_shape
					)
				);
			}

		}

	}

	// Add custom classes.
	if ( ! empty( $class ) ) {
		if ( ! is_array( $class ) ) {
			$class = preg_split( '#\s+#', $class );
		}

		$classes = array_merge( $classes, $class );
	} else {
		// Ensure that we always coerce class to being an array.
		$class = array();
	}

	return apply_filters( 'nice_slider_class', $classes, $class );
}
endif;

if ( ! function_exists( 'nice_get_slide_caption_class' ) ) :
/**
 * Obtain array of HTML classes for a slide's caption.
 *
 * @since  1.2.0
 *
 * @param  string|array $class
 *
 * @return array
 */
function nice_get_slide_caption_class( $class = '' ) {
	// Set initial classes.
	$classes = array(
		'flex-caption',
	);

	// Add classes for homepage slider.
	if ( nice_doing_homepage_slider() ) {
		$classes = array_merge( $classes, array(
			nice_slide_get_caption_background(),
			nice_slide_get_skin(),
		) );
	}

	// Add custom classes.
	if ( ! empty( $class ) ) {
		if ( ! is_array( $class ) ) {
			$class = preg_split( '#\s+#', $class );
		}

		$classes = array_merge( $classes, $class );
	} else {
		// Ensure that we always coerce class to being an array.
		$class = array();
	}

	return apply_filters( 'nice_slide_caption_class', $classes, $class );
}
endif;
