<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains functions for the public-facing side of the theme.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.6
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_is_homepage' ) ) :
/**
 * Check if the current page has the Home page template.
 *
 * @since  1.0.0
 *
 * @return bool
 */
function nice_is_homepage() {
	/**
	 * @hook nice_is_homepage
	 *
	 * Hook in here to redefine if we're seeing the Homepage template.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_is_homepage', is_page_template( 'page-templates/home.php' ) );
}
endif;

if ( ! function_exists( 'nice_is_blog' ) ) :
/**
 * Check if the current page is a default blog view or a blog page template.
 *
 * @since  1.0.3
 *
 * @return bool
 */
function nice_is_blog() {
	$is_blog = false;

	$conditions = array(
		is_home(),
		is_page_template( 'page-templates/blog.php' ),
		is_page_template( 'page-templates/blog-masonry.php' ),
	);

	foreach ( $conditions as $condition ) {
		if ( $condition && ( $is_blog = $condition ) ) {
			break;
		}
	}

	/**
	 * @hook nice_is_blog
	 *
	 * Hook in here to redefine if we're seeing a blog view.
	 *
	 * @since 1.0.3
	 */
	return apply_filters( 'nice_is_blog', $is_blog );
}
endif;

if ( ! function_exists( 'nice_get_archive_title' ) ) :
/**
 * Obtain the title of an archive page.
 *
 * @since  1.0.0
 *
 * @return string
 */
function nice_get_archive_title() {
	if ( function_exists( 'get_the_archive_title' ) ) { // Try to use `get_the_archive_title()`, available since WordPress 4.1.
		$title = get_the_archive_title();
	} else {
		if ( is_day() ) {
			$title = sprintf( esc_html__( 'Daily Archives: %s', 'smart' ), get_the_date() );
		} elseif ( is_month() ) {
			$title = sprintf( esc_html__( 'Monthly Archives: %s', 'smart' ), get_the_date( _x( 'F Y', 'monthly archives date format', 'smart' ) ) );
		} elseif ( is_year() ) {
			$title = sprintf( esc_html__( 'Yearly Archives: %s', 'smart' ), get_the_date( _x( 'Y', 'yearly archives date format', 'smart' ) ) );
		} else {
			$title = esc_html__( 'Archives', 'smart' );
		}
	}

	/**
	 * @hook nice_archive_title
	 *
	 * Hook in here to modify the current archie title.
	 *
	 * @since 1.0.0
	 */
	$title = apply_filters( 'nice_archive_title', $title );

	return $title;
}
endif;

if ( ! function_exists( 'nice_post_categories' ) ) :
/**
 * Display the list of categories for a post.
 *
 * @since 1.0.0
 *
 * @param  WP_Post $post      Post object to retrieve categories from.
 * @param  bool    $echo      Whether to print the generated HTML or not.
 * @param  string  $separator Single character to separate categories.
 *
 * @return string
 */
function nice_post_categories( $post, $echo = true, $separator = '' ) {
	// Return transient if we have one.
	if ( $categories = get_transient( md5( 'nice_post_categories_' . $post->ID ) ) ) {
		/**
		 * @hook nice_post_categories
		 *
		 * Hook in here to modify the HTML output for this post's categories.
		 *
		 * @since 1.0.0
		 */
		$categories = apply_filters( 'nice_post_categories', $categories );

		return $categories;
	}

	// Obtain the list of category terms for the current post.
	$post_terms = wp_get_object_terms( $post->ID, 'category', array( 'fields' => 'ids' ) );

	// Obtain a non-formatted list of categories.
	$categories = wp_list_categories( array(
		'echo'    => false,
		'style'   => 'none',
		'include' => implode( ',', $post_terms ),
	) );

	// Format list of categories.
	$categories = str_replace( '<br />', $separator . ' ', $categories );
	$categories = rtrim( $categories );
	$categories = rtrim( $categories, $separator );

	// Set transient.
	/**
	 * @hook nice_post_categories_transient_timeout
	 *
	 * Hook in here to modify the timeout for the transient of current post tags.
	 *
	 * @since 1.0.0
	 */
	$timeout = apply_filters( 'nice_post_categories_transient_timeout', 3600 );

	set_transient( md5( 'nice_post_categories_' . $post->ID ), $categories, $timeout );

	/**
	 * @hook nice_post_categories
	 *
	 * Hook in here to modify the HTML output for this post's categories.
	 *
	 * @since 1.0.0
	 */
	$categories = apply_filters( 'nice_post_categories', $categories );

	if ( $echo ) {
		echo $categories;
	}

	return $categories;
}
endif;

if ( ! function_exists( 'nice_post_tags' ) ) :
/**
 * Display the list of tags for a post.
 *
 * @since 1.0.0
 *
 * @param  WP_Post $post      Post object to retrieve categories from.
 * @param  string  $separator Single character to separate categories.
 * @return string
 */
function nice_post_tags( $post, $separator = '' ) {
	// Return transient if we have one.
	if ( $tags = get_transient( md5( 'nice_post_tags_' . $post->ID ) ) ) {
		/**
		 * @hook nice_post_tags
		 *
		 * Hook in here to modify the HTML output for this post's tags.
		 *
		 * @since 1.0.0
		 */
		$tags = apply_filters( 'nice_post_tags', $tags );

		return $tags;
	}

	// Obtain the list of category terms for the current post.
	$post_terms = wp_get_object_terms( $post->ID, 'post_tag', array( 'fields' => 'ids' ) );

	if ( ! empty( $post_terms ) ) {

		if ( false ) {
			the_tags();
		}

		// Obtain a non-formatted list of tags.
		$tags = wp_list_categories( array(
			'taxonomy'         => 'post_tag',
			'echo'             => false,
			'style'            => 'none',
			'include'          => implode( ',', $post_terms ),
			'show_option_none' => esc_html__( 'No tags', 'smart' ),
		) );

		// Format list of categories.
		$tags = str_replace( '<br />', $separator . ' ', $tags );
		$tags = rtrim( $tags );
		$tags = rtrim( $tags, $separator );
	} else {
		$tags = esc_html__( 'No tags', 'smart' );
	}

	// Set transient.
	/**
	 * @hook nice_post_tags_transient_timeout
	 *
	 * Hook in here to modify the timeout for the transient of current post tags.
	 *
	 * @since 1.0.0
	 */
	$timeout = apply_filters( 'nice_post_tags_transient_timeout', 3600, $post->ID );

	set_transient( md5( 'nice_post_tags_' . $post->ID ), $tags, $timeout );

	/**
	 * @hook nice_post_tags
	 *
	 * Hook in here to modify the HTML output for this post's tags.
	 *
	 * @since 1.0.0
	 */
	$tags = apply_filters( 'nice_post_tags', $tags );

	return $tags;
}
endif;

if ( ! function_exists( 'nice_top_bar_height' ) ) :
/**
 * Obtain the height of the top bar.
 *
 * @since 1.0.0
 */
function nice_top_bar_height() {
	$nice_top_bar_height = 13; // This should be the same as the value set for $size__top-bar-height in _header.scss.

	if ( nice_bool_option( '_custom_typography' ) && $font_top_bar = nice_get_option( '_font_top_bar' ) ) {
		$nice_top_bar_height = empty( $font_top_bar['font-size'] ) ? $nice_top_bar_height : $font_top_bar['font-size'];
	}

	/**
	 * @hook nice_top_bar_height
	 *
	 * Hook in here to modify the height of the top bar.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_top_bar_height', $nice_top_bar_height );
}
endif;

if ( ! function_exists( 'nice_top_bar_full_height' ) ) :
/**
 * Obtain the full height of the top bar, padding included.
 *
 * @since 1.0.3
 */
function nice_top_bar_full_height() {
	$height = intval( nice_top_bar_height() + ( nice_get_option( '_top_bar_padding' ) * 2 ) );

	/**
	 * @hook nice_top_bar_full_height
	 *
	 * Hook in here to modify the full_height of the top bar.
	 *
	 * @since 1.0.3
	 */
	return apply_filters( 'nice_top_bar_full_height', $height );
}
endif;

if ( ! function_exists( 'nice_header_max_height' ) ) :
/**
 * Try to obtain the maximum height of the header.
 *
 * @since  1.0.0
 *
 * @param  array $args
 *
 * @return int
 */
function nice_header_max_height( array $args = array() ) {
	if ( ! nice_header_display() ) {
		return null;
	}

	if ( nice_header_centered() ) {
		$font_menu  = nice_get_option( 'nice_font_menu' );
		$height     = intval( nice_get_option( '_logo_padding_top' ) ) + intval( nice_get_option( '_logo_padding_bottom' ) ) + 10;
		$height    += intval( $font_menu['font-size'] ) + 30;
	} else {
		$height  = ( intval( nice_get_option( 'nice_logo_padding' ) ) * 2 );
	}

	if ( nice_bool( nice_get_option( 'nice_text_title' ) ) ) {
		$title = nice_get_option( 'nice_font_site_title' );
		$unit  = ! empty( $title['unit'] ) ? $title['unit'] : 'px';

		$base_font_size = 32;

		switch ( $unit ) {
			case 'em':
				$font_size = ( $base_font_size * intval( $title['font-size'] ) );
				break;
			case '%':
				$font_size = ( $base_font_size / 100 ) * intval( $title['font-size'] );
				break;
			default:
				$font_size = intval( $title['font-size'] );
				break;
		}

		$height += $font_size;

		if ( nice_bool( nice_get_option( 'nice_text_tagline' ) ) ) {
			$tagline = nice_get_option( 'nice_font_tagline' );
			$height += intval( $tagline['font-size'] );
		}

	} elseif ( ! nice_bool_option( '_hide_logo' ) ) {
		$height += intval( nice_get_option( 'nice_logo_height' ) );
	}

	return $height;
}
endif;

if ( ! function_exists( 'nice_header_max_height_mobile' ) ) :
/**
 * Try to obtain the maximum height of the header for mobile viewports.
 *
 * This function operates with a static value to avoid re-processing it each
 * time is called, unless arguments are provided.
 *
 * @since  1.0.0
 *
 * @param  array $args
 *
 * @return int
 */
function nice_header_max_height_mobile( array $args = array() ) {
	// Contemplar header centered.
	$use_custom_settings = true;

	if ( ! $use_custom_settings ) {
		return nice_header_max_height( $args );
	}

	if ( ! nice_header_display() ) {
		return null;
	}

	$height  = ( nice_int_option( '_logo_mobile_padding' ) * 2 );

	if ( nice_bool( nice_get_option( 'nice_text_title' ) ) ) {
		$title = nice_get_option( 'nice_font_site_title' );
		$unit  = ! empty( $title['unit'] ) ? $title['unit'] : 'px';

		$base_font_size = 32;

		switch ( $unit ) {
			case 'em':
				$font_size = ( $base_font_size * intval( $title['font-size'] ) );
				break;
			case '%':
				$font_size = ( $base_font_size / 100 ) * intval( $title['font-size'] );
				break;
			default:
				$font_size = intval( $title['font-size'] );
				break;
		}

		$height += $font_size;

		if ( nice_bool( nice_get_option( 'nice_text_tagline' ) ) ) {
			$tagline = nice_get_option( 'nice_font_tagline' );
			$height += intval( $tagline['font-size'] );
		}

	} elseif ( ! nice_bool_option( '_hide_logo' ) ) {
		$height += nice_int_option( '_logo_mobile_height' );
	}

	return $height;
}
endif;

if ( ! function_exists( 'nice_header_min_height' ) ) :
/**
 * Try to obtain the minimum height of the header.
 *
 * This function operates with a static value to avoid re-processing it each
 * time is called.
 *
 * @since  1.0.0
 *
 * @return int
 */
function nice_header_min_height() {
	static $height = null;

	// Return previously saved value if we have one.
	if ( ! is_null( $height ) ) {
		return $height;
	}

	if ( ! nice_header_horizontal() && ! nice_overlay_navigation() ) {
		return null;
	}

	if ( nice_header_centered() ) {
		$font_menu = nice_get_option( 'nice_font_menu' );
		$height    = ( intval( nice_get_option( 'nice_logo_padding' ) ) + 10 ) / 2;
		$height   += intval( $font_menu['font-size'] ) + 30;
	} else {
		$height = apply_filters( 'nice_logo_min_padding', 20 ) * 2;
	}

	if ( nice_bool( nice_get_option( 'nice_text_title' ) ) ) {
		$title = nice_get_option( 'nice_font_site_title' );
		$unit  = ! empty( $title['unit'] ) ? $title['unit'] : 'px';

		$base_font_size = 32;

		switch ( $unit ) {
			case 'em':
				$font_size = ( $base_font_size * intval( $title['font-size'] ) );
				break;
			case '%':
				$font_size = ( $base_font_size / 100 ) * intval( $title['font-size'] );
				break;
			default:
				$font_size = intval( $title['font-size'] );
				break;
		}

		$height += $font_size;

		if ( nice_bool( nice_get_option( 'nice_text_tagline' ) ) ) {
			$tagline = nice_get_option( 'nice_font_tagline' );
			$height += intval( $tagline['font-size'] );
		}

	} else {
		$height += ( intval( nice_get_option( 'nice_logo_height' ) ) ) * 0.9;
	}

	return $height;
}
endif;

if ( ! function_exists( 'nice_header_fixed' ) ) :
/**
 * Check if the header should be fixed to the top of the page.
 *
 * @since 1.0.3
 */
function nice_header_fixed() {
	$header_fixed = ! nice_header_vertical() && nice_bool_option( '_header_fixed' );

	return apply_filters( 'nice_header_fixed', $header_fixed );
}
endif;

if ( ! function_exists( 'nice_header_fixed_mobile' ) ) :
/**
 * Check if the mobile header should be fixed to the top of the page.
 *
 * @since 1.0.3
 */
function nice_header_fixed_mobile() {
	$header_fixed_mobile = nice_bool_option( '_header_fixed_mobile' );

	/**
	 * @hook nice_header_fixed_mobile
	 *
	 * Hook in here to redefine if the mobile fixed header should be active.
	 *
	 * @since 1.0.3
	 */
	return apply_filters( 'nice_header_fixed_mobile', $header_fixed_mobile );
}
endif;

if ( ! function_exists( 'nice_header_transparent' ) ) :
/**
 * Whether the header should be transparent.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function nice_header_transparent() {
	$transparent_header = nice_bool( nice_get_option( 'nice_header_background_transparent' ) );

	if ( is_page() || is_single() ) {
		$_post_header_transparent = get_post_meta( get_the_ID(), '_post_header_background_transparent', true );

		if ( ! empty( $_post_header_transparent ) ) {
			$transparent_header = nice_bool( $_post_header_transparent );
		} else {
			$post_type                 = get_post_type();
			$_post_header_transparent = nice_get_option( "_{$post_type}_header_background_transparent" );

			if ( ! empty( $_post_header_transparent ) ) {
				$transparent_header = nice_bool( $_post_header_transparent );
			}
		}
	}

	$transparent_header = $transparent_header && ! nice_header_vertical();

	/**
	 * @hook nice_header_transparent
	 *
	 * Hook in here to redefine if the header should be transparent.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_header_transparent', $transparent_header );
}
endif;

if ( ! function_exists( 'nice_header_transparent_mobile' ) ) :
/**
 * Whether the mobile header should be transparent.
 *
 * @since 1.0.3
 *
 * @return bool
 */
function nice_header_transparent_mobile() {
	$transparent_header = nice_bool_option( '_header_background_transparent_mobile' );

	if ( is_page() || is_single() ) {
		$_post_header_transparent = get_post_meta( get_the_ID(), '_post_header_background_transparent_mobile', true );

		if ( ! empty( $_post_header_transparent ) ) {
			$transparent_header = nice_bool( $_post_header_transparent );
		} else {
			$post_type                = get_post_type();
			$_post_header_transparent = nice_get_option( "_{$post_type}_header_background_transparent_mobile" );

			if ( ! empty( $_post_header_transparent ) ) {
				$transparent_header = nice_bool( $_post_header_transparent );
			}
		}
	}

	/**
	 * @hook nice_header_transparent_mobile
	 *
	 * Hook in here to redefine if the mobile header should be transparent.
	 *
	 * @since 1.0.3
	 */
	return apply_filters( 'nice_header_transparent_mobile', $transparent_header );
}
endif;

if ( ! function_exists( 'nice_header_navigation_style' ) ) :
/**
 * Return the navigation style
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_header_navigation_style() {
	$nice_navigation_style = nice_get_option( 'nice_header_navigation' );

	if ( is_page() || is_single() ) {
		$_post_navigation_style = get_post_meta( get_the_ID(), '_post_navigation_style', true );

		if ( ! empty( $_post_navigation_style ) ) {
			$nice_navigation_style = $_post_navigation_style;
		} else {
			$post_type              = get_post_type();
			$_post_navigation_style = nice_get_option( "_{$post_type}_navigation_style" );

			if ( ! empty( $_post_navigation_style ) ) {
				$nice_navigation_style = $_post_navigation_style;
			}
		}
	}

	return $nice_navigation_style;

}
endif;

if ( ! function_exists( 'nice_header_skin' ) ) :
/**
 * Return the navigation content menu skin
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_header_skin() {
	$nice_header_skin  = nice_get_option( 'nice_header_skin', 'light' );
	$single            = is_page() || is_single();
	$transparent_fixed = nice_header_transparent() && nice_bool_option( '_header_fixed' );

	if ( $single && ! $transparent_fixed ) {
		$_post_header_skin = get_post_meta( get_the_ID(), '_post_header_skin', true );

		if ( ! empty( $_post_header_skin ) ) {
			$nice_header_skin = $_post_header_skin;
		} else {
			$post_type              = get_post_type();
			$_post_header_skin = nice_get_option( "_{$post_type}_header_skin" );

			if ( ! empty( $_post_header_skin ) ) {
				$nice_header_skin = $_post_header_skin;
			}
		}
	}

	return $nice_header_skin;
}
endif;

if ( ! function_exists( 'nice_transparent_header_skin' ) ) :
/**
 * Obtain current skin for transparent header.
 *
 * @since  1.0.0
 *
 * @return mixed|string|null
 */
function nice_transparent_header_skin() {
	if ( ! nice_header_transparent() && ! nice_header_transparent_mobile() ) {
		return null;
	}

	if ( is_page() || is_single() ) {
		$skin = get_post_meta( get_the_ID(), '_post_header_skin', true );
	}

	if ( empty( $skin ) ) {
		$skin = nice_get_option( '_header_background_transparent_skin', 'dark' );
	}

	return $skin;
}
endif;

if ( ! function_exists( 'nice_header_background_color' ) ) :
/**
 * Return the header background color.
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_header_background_color() {
	if ( is_page() || is_single() ) {
		$_post_background_color = get_post_meta( get_the_ID(), '_post_header_background_color', true );

		if ( ! empty( $_post_background_color ) ) {
			$background_color = $_post_background_color;
		} else {
			$post_type              = get_post_type();
			$_post_background_color = nice_get_option( "_{$post_type}_header_background_color" );

			if ( ! empty( $_post_background_color ) ) {
				$background_color = $_post_background_color;
			}
		}
	}

	if ( empty( $background_color ) ) {
		$background_color = nice_get_option( '_header_background_color' );
	}

	return $background_color;
}
endif;

if ( ! function_exists( 'nice_header_navigation_submenu_skin' ) ) :
/**
 * Return the navigation content submenu skin
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_header_navigation_submenu_skin() {
	$nice_navigation_submenu_skin = nice_get_option( 'nice_header_submenu_skin', 'dark' );

	if ( is_page() || is_single() ) {
		$_post_navigation_submenu_skin = get_post_meta( get_the_ID(), '_post_navigation_submenu_skin', true );

		if ( ! empty( $_post_navigation_submenu_skin ) ) {
			$nice_navigation_submenu_skin = $_post_navigation_submenu_skin;
		} else {
			$post_type                     = get_post_type();
			$_post_navigation_submenu_skin = nice_get_option( "_{$post_type}_navigation_submenu_skin" );

			if ( ! empty( $_post_navigation_submenu_skin ) ) {
				$nice_navigation_submenu_skin = $_post_navigation_submenu_skin;
			}
		}
	}

	return $nice_navigation_submenu_skin;
}
endif;

if ( ! function_exists( 'nice_header_centered' ) ) :
/**
 * Whether the header contents should be centered.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function nice_header_centered() {
	$centered_header = nice_header_horizontal() && ( 'logo-centered' === nice_get_option( 'nice_header_top_navigation_alignment' ) );

	/**
	 * @hook nice_header_centered
	 *
	 * Hook in here to redefine if the header should be centered.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_header_centered', $centered_header );
}
endif;

if ( ! function_exists( 'nice_header_horizontal' ) ) :
/**
 * Whether the header should be displayed horizontally.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function nice_header_horizontal() {
	$horizontal_header = ( in_array( nice_header_navigation_style(), array( 'default', 'overlay' ) ) );

	/**
	 * @hook nice_header_horizontal
	 *
	 * Hook in here to redefine if the header should be horizontal.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_header_horizontal', $horizontal_header );
}
endif;

if ( ! function_exists( 'nice_header_vertical' ) ) :
/**
 * Whether the header should be displayed vertically.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function nice_header_vertical() {
	$vertical_header = ( 'vertical' === nice_header_navigation_style() );

	/**
	 * @hook nice_header_vertical
	 *
	 * Hook in here to redefine if the header should be vertical.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_header_vertical', $vertical_header );
}
endif;

if ( ! function_exists( 'nice_header_display_search' ) ) :
/**
 * Return whether to display or not the search icon in the header.
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_header_display_search() {
	$nice_display_search = nice_get_option( 'nice_header_display_search' );

	if ( is_page() || is_single() ) {
		$_post_display_search = get_post_meta( get_the_ID(), '_post_header_display_search', true );

		if ( '' !== $_post_display_search ) {
			$nice_display_search = $_post_display_search;
		} else {
			$post_type            = get_post_type();
			$_post_display_search = nice_get_option( "_{$post_type}_header_display_search" );

			if ( '' !== $_post_display_search ) {
				$nice_display_search = $_post_display_search;
			}
		}
	}

	/**
	 * @hook nice_header_display_search
	 *
	 * Hook in here to redefine if search funcionality should be displayed in
	 * header.
	 *
	 * @since 1.0.0
	 */
	return nice_bool( apply_filters( 'nice_header_display_search', $nice_display_search ) );
}
endif;

if ( ! function_exists( 'nice_overlay_navigation' ) ) :
/**
 * Check if we should display an overlay navigation.
 *
 * @since  1.0.0
 *
 * @return bool
 */
function nice_overlay_navigation() {
	$overlay_navigation = ( 'overlay' === nice_header_navigation_style() );

	/**
	 * @hook nice_overlay_navigation
	 *
	 * Hook in here to redefine if the overlay navigation should be implemented.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_overlay_navigation', $overlay_navigation );
}
endif;

if ( ! function_exists( 'nice_boxed_layout' ) ) :
/**
 * Whether the general layout should be boxed.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function nice_boxed_layout() {
	$centered_header = ( 'boxed' === nice_get_option( 'nice_layout_type' ) );

	/**
	 * @hook nice_boxed_layout
	 *
	 * Check if the layout should be boxed or not.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_boxed_layout', $centered_header );
}
endif;

if ( ! function_exists( 'nice_heading_background_color' ) ) :
/**
 * Return the heading background color
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_heading_background_color() {
	$nice_heading_background_color = nice_get_option( 'nice_heading_background_color' );

	// @TODO: check the CF names here, not intuitive
	if ( is_page() || is_single() ) {
		$_post_background_color = get_post_meta( get_the_ID(), '_post_background_color', true );

		if ( ! empty( $_post_background_color ) ) {
			$nice_heading_background_color = $_post_background_color;
		} else {
			$post_type              = get_post_type();
			$_post_background_color = nice_get_option( "_{$post_type}_background_color" );

			if ( ! empty( $_post_background_color ) ) {
				$nice_heading_background_color = $_post_background_color;
			}
		}
	}

	return $nice_heading_background_color;
}
endif;

if ( ! function_exists( 'nice_post_navigation_skin' ) ) :
/**
 * Return the navigation skin
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_post_navigation_skin() {
	$nice_post_navigation_skin = nice_get_option( '_post_navigation_skin', 'light' );

	if ( is_page() || is_single() ) {
		$post_navigation_skin = get_post_meta( get_the_ID(), '_post_navigation_skin', true );

		if ( ! empty( $post_navigation_skin ) && $post_navigation_skin !== '' ) {
			$nice_post_navigation_skin = $post_navigation_skin;
		} else {
			$post_type            = get_post_type();
			$post_navigation_skin = nice_get_option( "_{$post_type}_navigation_skin" );

			if ( ! empty( $post_navigation_skin ) && $post_navigation_skin !== '' ) {
				$nice_post_navigation_skin = $post_navigation_skin;
			}
		}
	}

	return $nice_post_navigation_skin;
}
endif;

if ( ! function_exists( 'nice_post_navigation_class' ) ) :
/**
 * Print out classes for post navigation.
 *
 * @since  1.0.0
 *
 * @param  array  $class
 * @param  bool   $echo
 *
 * @return string
 */
function nice_post_navigation_class( array $class = array(), $echo = true ) {
	$classes = array(
		'pre-footer-navigation',
	);

	if ( $skin = nice_post_navigation_skin() ) {
		$classes[] = $skin;
	}

	if ( $bg_color = nice_post_navigation_background_color() ) {
		$classes[] = nice_theme_color_background_class( $bg_color );
	}

	if ( ! empty( $class ) ) {
		$classes = array_merge( $classes, $class );
	}

	return nice_css_classes( $classes, $echo );
}
endif;

if ( ! function_exists( 'nice_post_navigation_background_color' ) ) :
/**
 * Return the post navigation background color.
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_post_navigation_background_color() {
	if ( is_page() || is_single() ) {
		$_post_navigation_background_color = get_post_meta( get_the_ID(), '_post_navigation_background_color', true );

		if ( ! empty( $_post_navigation_background_color ) ) {
			$background_color = $_post_navigation_background_color;
		} else {
			$post_type            = get_post_type();
			$_post_navigation_background_color = nice_get_option( "_{$post_type}_navigation_background_color" );

			if ( ! empty( $_post_navigation_background_color ) ) {
				$background_color = $_post_navigation_background_color;
			}
		}
	}

	if ( empty( $background_color ) ) {
		$background_color = nice_get_option( '_post_navigation_background_color' );
	}

	return $background_color;
}
endif;

if ( ! function_exists( 'nice_footer_skin' ) ) :
/**
 * Return the footer skin
 *
 * @since 1.0.0
 *
 * @return string
 */
function nice_footer_skin() {
	$nice_footer_skin = nice_get_option( 'nice_footer_skin', 'dark' );

	if ( is_page() || is_single() ) {
		$post_footer_skin = get_post_meta( get_the_ID(), '_post_footer_skin', true );

		if ( ! empty( $post_footer_skin ) && $post_footer_skin !== '' ) {
			$nice_footer_skin = $post_footer_skin;
		} else {
			$post_type         = get_post_type();
			$post_footer_skin = nice_get_option( "_{$post_type}_footer_skin" );

			if ( ! empty( $post_footer_skin ) && $post_footer_skin !== '' ) {
				$nice_footer_skin = $post_footer_skin;
			}
		}
	}

	return $nice_footer_skin;
}
endif;

if ( ! function_exists( 'nice_content_skin' ) ) :
/**
 * Return the content skin
 *
 * @since 1.1
 *
 * @return string
 */
function nice_content_skin() {
	$nice_content_skin = nice_get_option( '_content_skin' );

	if ( is_page() || is_single() ) {
		$_content_skin = get_post_meta( get_the_ID(), '_post_content_skin', true );

		if ( ! empty( $_content_skin ) && $_content_skin !== '' ) {
			$nice_content_skin = $_content_skin;
		} else {
			$post_type     = get_post_type();
			$_content_skin = nice_get_option( "_{$post_type}_content_skin" );

			if ( ! empty( $_content_skin ) && $_content_skin !== '' ) {
				$nice_content_skin = $_content_skin;
			}
		}
	}

	return $nice_content_skin;
}
endif;

if ( ! function_exists( 'nice_page_footer_meta_tpl' ) ) :
add_filter( 'nice_post_footer_meta_template', 'nice_page_footer_meta_tpl' );
/**
 * Footer meta template for pages.
 *
 * @since 1.0.0
 *
 * @param  string $template
 *
 * @return string
 */
function nice_page_footer_meta_tpl( $template = '' ) {
	if ( is_page() ) {
		$template = '[post_comments]';
	}

	return $template;
}
endif;

if ( ! function_exists( 'nice_attachments_from_gallery' ) ) :
/**
 * Return IDs of attachments from gallery.
 *
 * @since 1.0.0
 */
function nice_attachments_from_gallery() {
	global $post;

	$pattern = get_shortcode_regex();
	$ids     = array();

	// Find the "gallery" shortcode and puts the image ids in an associative array at $matches[3].
	if ( preg_match_all( '/' . $pattern . '/s', $post->post_content, $matches ) ) {
		$count = count( $matches[3] ); // In case there is more than one gallery in the post.

		for ( $i = 0; $i < $count; $i ++ ) {
			$atts = shortcode_parse_atts( $matches[3][ $i ] );

			if ( isset( $atts['ids'] ) ) {
				$attachment_ids = explode( ',', $atts['ids'] );
				$ids = array_merge( $ids, $attachment_ids );
			}
		}
	}

	if ( ! empty( $ids ) ) {
		$ids = array_flip( $ids );
	}

	/**
	 * @hook nice_attachments_from_gallery
	 *
	 * Hook in here to modify the list of attachment IDs for the current post.
	 *
	 * @since 1.0.0
	 */
	$ids = apply_filters( 'nice_attachments_from_gallery', $ids );

	return $ids;
}
endif;


if ( ! function_exists( 'nice_content_without_gallery' ) ) :
add_filter( 'the_content', 'nice_content_without_gallery' );
/**
 * Remove gallery shortcodes from content and returns the content.
 *
 * @since 1.0.0
 *
 * @param  string $content
 * @return string
 */
function nice_content_without_gallery( $content ) {
	if ( has_post_format( 'gallery', get_the_ID() ) ) {
		$content = preg_replace( '/\[gallery[^\]]*\]/', '', $content );
	}

	return $content;
}
endif;

if ( ! function_exists( 'nice_post_gallery' ) ) :
add_filter( 'post_gallery', 'nice_post_gallery', 10, 2 );
/**
 * Hook the built in gallery shortcode and return the formatted content
 *
 * @since 1.0.0
 *
 * @param  string $output
 * @param  array  $attr
 *
 * @return string
 */
function nice_post_gallery( $output, $attr ) {
	if ( ! isset( $attr['ids'] ) ) {
		return '';
	}

	$post = get_post();

	// We're trusting author input, so let's at least make sure it looks like a valid orderby statement
	if ( isset( $attr['orderby'] ) ) {
		$attr['orderby'] = sanitize_sql_orderby( $attr['orderby'] );
		if ( ! $attr['orderby'] ) {
			unset( $attr['orderby'] );
		}
	}

	// Attempt to normalize the size of the image.
	if ( isset( $attr['columns'] ) ) {
		$attr['columns'] = intval( $attr['columns'] );

		if ( 6 < $attr['columns'] ) {
			$size = 'thumbnail';
		} elseif ( 1 === $attr['columns'] ) {
			$size = 'large';
		}
	}

	$attr = shortcode_atts( array(
		'ids'        => $attr['ids'],
		'order'      => 'ASC',
		'orderby'    => 'menu_order ID',
		'id'         => $post->ID,
		'columns'    => 3,
		'size'       => isset( $size ) ? $size : 'medium',
		'include'    => '',
		'exclude'    => '',
	), $attr );

	$output = nice_gallery( array(
			'columns'  => $attr['columns'],
			'ids'      => $attr['ids'],
			'size'     => $attr['size'],
			'echo'     => false,
		)
	);

	return $output;
}
endif;

if ( ! function_exists( 'nice_get_comments_number_text' ) ) :
/**
 * Display the language string for the number of comments the current post has.
 *
 * @since  1.0.0
 *
 * @param  string $zero Optional. Text for no comments. Default false.
 * @param  string $one  Optional. Text for one comment. Default false.
 * @param  string $more Optional. Text for more than one comment. Default false.
 *
 * @return string
 */
function nice_get_comments_number_text( $zero = '', $one = '', $more = '' ) {
	$number = intval( get_comments_number() );

	if ( 1 < $number ) {
		$output = str_replace( '%', number_format_i18n( $number ), ( false === $more ) ? esc_html__( '% Comments', 'smart' ) : $more );
	} elseif ( 0 === $number ) {
		$output = ( false === $zero ) ? esc_html__( 'No Comments', 'smart' ) : $zero;
	} else { // must be one
		$output = ( false === $one ) ? esc_html__( '1 Comment', 'smart' ) : $one;
	}

	/**
	 * @hook nice_comments_number
	 *
	 * Filter the comments count for display.
	 *
	 * @sine 1.0.0
	 */
	return apply_filters( 'nice_comments_number', $output, $number );
}
endif;
