<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains functions to manage navigation menus.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_nav_menu_walker' ) ) :
add_action( 'init', 'nice_nav_menu_walker' );
/**
 * Initialize custom navigation walker for later usage.
 *
 * @since  1.0.0
 *
 * @return mixed|null|Nice_Walker_Nav_Menu
 */
function nice_nav_menu_walker() {
	// Return early if we're using the overlay navigation.
	if ( nice_overlay_navigation() ) {
		return null;
	}

	static $walker = null;

	if ( ! did_action( 'init' ) ) {
		wp_die( sprintf( esc_html__( 'Sorry, you\'re not allowed to call %s before the %s action', 'smart' ), '<code>' . __FUNCTION__ . '</code>', '<code>init</code>' ) );
	}

	if ( is_null( $walker ) ) {
		$walker = new Nice_Walker_Nav_Menu;
		$walker->transient_duration = ( 60 * 60 * 24 ); // One day.
		$walker->init();
	}

	return $walker;
}
endif;

if ( ! function_exists( 'nice_top_nav_menu_css_class' ) ) :
add_filter( 'nav_menu_css_class', 'nice_top_nav_menu_css_class', 10, 4 );
/**
 * Modify CSS classes of a menu item.
 *
 * @since  1.0.0
 *
 * @param  array    $classes
 * @param  WP_Post  $item
 * @param  stdClass $args
 * @param  int      $depth
 *
 * @return array
 */
// @codingStandardsIgnoreStart
function nice_top_nav_menu_css_class( array $classes, $item, stdClass $args, $depth ) {
// @codingStandardsIgnoreEnd
	if ( empty( $args->menu_id ) || 'top-nav' !== $args->menu_id ) {
		return $classes;
	}

	if ( 1 === $depth && false !== ( $key = array_search( 'menu-item-has-children', $classes ) ) ) {
		unset( $classes[ $key ] );
	}

	return $classes;
}
endif;

if ( ! function_exists( 'nice_custom_navigation_menu_admin' ) ) :
add_action( 'init', 'nice_custom_navigation_menu_admin' );
/**
 * Initialize custom navigation menu options in admin side.
 *
 * @since 1.0.0
 */
function nice_custom_navigation_menu_admin() {
	$custom_navigation_menu = new Nice_Custom_Navigation_Menu( array(
			'menu_ids'           => array( 'main-nav' ),
			'transient_duration' => ( 60 * 60 * 24 ), // One day.
		)
	);

	$custom_navigation_menu->init();
}
endif;

if ( ! function_exists( 'nice_navigation_menu' ) ) :
add_action( 'nice_navigation_menu', 'nice_navigation_menu' );
/**
 * Print main navigation menu.
 *
 * @since 1.0.0
 */
function nice_navigation_menu() {
	/**
	 * @hook nice_before_navigation_menu
	 *
	 * Hook here to add HTML elements right before the menu is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_before_navigation_menu' );

	// Obtain required classes.
	if ( nice_overlay_navigation() ) {
		$menu_class = 'overlay';
		$nav_class  = 'overlay';

		if ( ( $nice_overlay_background_color = nice_get_option( 'nice_header_overlay_background_color' ) ) && ( 1 === intval( nice_get_option( 'nice_header_overlay_background_color_opacity' ) ) ) ) {
			$nav_class .= ' ' . nice_theme_color_background_class( $nice_overlay_background_color );
		}
	} else {
		$menu_class = 'default';
		$nav_class  = 'default';

		if ( nice_header_horizontal() ) {
			$alignment = nice_get_option( 'nice_header_top_navigation_alignment', 'navigation-right' );

			$menu_class .= ' ' . $alignment;
			$nav_class  .= ' ' . $alignment;
		}

		$menu_class .= nice_bool( nice_get_option( 'nice_header_menu_hide_arrows', false ) ) ? ' no-arrows' : '';
	}

	$menu_class .= nice_bool( nice_get_option( 'nice_header_menu_hide_separators', false ) ) ? ' no-separators' : '';

	$menu_class .= nice_bool( nice_get_option( 'nice_header_menu_remove_border', false ) ) ? ' sub-menu-no-border' : '';

	$args = array(
		'menu'            => '',
		'container'       => 'nav',
		'container_class' => $nav_class,
		'container_id'    => 'navigation',
		'menu_class'      => $menu_class,
		'menu_id'         => 'main-nav',
		'echo'            => true,
		'fallback_cb'     => 'nice_navigation_menu_fallback', /** Callable. @see nice_navigation_menu_fallback() */
		'before'          => '',
		'after'           => '',
		'link_before'     => '',
		'link_after'      => '',
		'depth'           => 0,
		'theme_location'  => 'navigation-menu',
	);

	// Set custom menu walker when not using overlay navigation.
	if ( ! nice_overlay_navigation() ) {
		$args['walker'] = nice_nav_menu_walker();
	}

	/**
	 * @hook nice_navigation_menu_args
	 *
	 * Hook in here to modify the default arguments to create the main
	 * navigation menu.
	 *
	 * @since 1.0.0
	 */
	$args = apply_filters( 'nice_navigation_menu_args', $args );

	// Print menu using given arguments.
	wp_nav_menu( $args );

	/**
	 * @hook nice_after_navigation_menu
	 *
	 * Hook here to add HTML elements right after the menu is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_after_navigation_menu' );
}
endif;

if ( ! function_exists( 'nice_navigation_menu_fallback' ) ) :
/**
 * Generate fallback navigation menu. Useful when we don't have any items to
 * show, but still need some markup to be printed.
 *
 * @since 1.0.0
 *
 * @param array $args
 */
function nice_navigation_menu_fallback( array $args = array() ) {
	$navigation_style     = nice_header_navigation_style();
	$navigation_alignment = nice_get_option( '_header_top_navigation_alignment' );
	$centered_inline_logo = 'default' === $navigation_style && 'logo-centered-inline' === $navigation_alignment;
	$has_top_nav          = nice_bool_option( '_top_bar' ) && has_nav_menu( 'top-bar-menu' );

	// Return early if we don't need a top navigation or a centered logo.
	if ( ! $has_top_nav && ! $centered_inline_logo ) {
		return;
	}

	// Print fallback navigation.
	?>
	<<?php echo esc_attr( $args['container'] ); ?> id="<?php echo esc_attr( $args['container_id'] ); ?>" <?php nice_css_classes( explode( ' ', $args['container_class'] ) ); ?>>
	<ul id="<?php echo esc_attr( $args['menu_id'] ); ?>" <?php nice_css_classes( explode( ' ', $args['menu_class'] ) ); ?>>
		<?php
			// Try to print the centered inline logo in case we need it.
			if ( $centered_inline_logo ) :
				$walker = nice_nav_menu_walker();

				echo $walker->logo_nav_menu(); // WPCS: XSS ok.
			endif;
		?>
	</ul>
	</<?php echo esc_attr( $args['container'] ); ?>>
	<?php
}
endif;

if ( ! function_exists( 'nice_navigation_menu_extras_wrapper_start' ) ) :
add_action( 'nice_navigation_menu', 'nice_navigation_menu_extras_wrapper_start', 15 );
/**
 * Open wrapper tag for navigation menu.
 *
 * @since 1.0.0
 */
function nice_navigation_menu_extras_wrapper_start() {
	$classes = array();

	if ( nice_header_display_search() ) {
		$classes[] = 'has-search-form';
	}

	if ( nice_header_display_social() ) {
		$classes[] = 'has-header-social';
	}

	if ( empty( $classes ) ) {
		$classes[] = 'no-extras';
	}

	$classes = apply_filters( 'nice_navigation_menu_extras_class', $classes );
?>
	<div id="navigation-extras" class="<?php echo esc_attr( join( ' ', $classes ) ); ?>">
		<div id="navigation-extras-inner">
<?php
}
endif;

if ( ! function_exists( 'nice_navigation_menu_extras_wrapper_end' ) ) :
add_action( 'nice_navigation_menu', 'nice_navigation_menu_extras_wrapper_end', 40 );
/**
 * Close wrapper tag for navigation menu.
 *
 * @since 1.0.0
 */
function nice_navigation_menu_extras_wrapper_end() {
?>
		</div>
	</div>
<?php
}
endif;

if ( ! function_exists( 'nice_navigation_menu_additional_html' ) ) :
add_action( 'nice_navigation_menu', 'nice_navigation_menu_additional_html', 35 );
/**
 * Add toggle and close buttons in main navigation menu.
 *
 * @since 1.0.0
 *
 * @param string $html
 */
function nice_navigation_menu_additional_html( $html = '' ) {
	if ( ! has_nav_menu( 'navigation-menu' ) && ! nice_bool_option( '_top_bar' ) && has_nav_menu( 'top-bar-menu' ) ) {
		return;
	}

	// Use navigation toggle button for smaller window widths.
	$html .= '
	<div id="toggle-nav-container">
		<a href="javascript:void(0);" class="toggle-nav" data-focus="#navigation"><i class="toggle-bar"></i></a>
	</div>';

	// Add close button if using overlay navigation.
	if ( nice_overlay_navigation() ) {
		$html .= '<a href="javascript:void(0);" id="close-nav-container" class="close">' . esc_html__( 'Close', 'smart' ) . '</a>';
	}

	echo $html; // WPCS: XSS ok.
}
endif;

if ( ! function_exists( 'nice_top_bar' ) ) :
add_action( 'nice_top_bar', 'nice_top_bar' );
/**
 * Display top bar.
 *
 * @since 1.0.0
 */
function nice_top_bar() {
	// Return early if the top bar is not active.
	if ( ! nice_bool_option( '_top_bar' ) ) {
		return;
	}

	$classes = array(
		nice_get_option( '_top_bar_skin', 'dark' ),
	);

	if ( $left_content = nice_top_bar_content_left() ) {
		$classes[] = 'has-left-content';
	}

	if ( $right_content = nice_top_bar_content_right() ) {
		$classes[] = 'has-right-content';
	}

	if ( $background_class = nice_theme_color_background_class( nice_get_option( '_top_bar_background_color' ) ) ) {
		$classes[] = $background_class;
	}

	if ( nice_bool_option( '_top_bar_fixed' ) ) {
		$classes[] = 'fixed';
	}

	$inner_class = nice_bool_option( '_header_full_width' ) ? 'full-width' : 'col-full';

	?>
	<div id="top-bar" class="<?php echo esc_attr( join( ' ', $classes ) ); ?>">
		<div id="top-bar-inner" class="<?php echo esc_attr( $inner_class ); ?>">
			<?php if ( $left_content ) : ?>
				<div id="top-bar-left">
					<div id="top-bar-left-inner">
						<?php echo $left_content; // WPCS: XSS ok. ?>
					</div>
				</div>
			<?php endif; ?>

			<?php do_action( 'nice_top_bar_menu' ); ?>

			<?php if ( $right_content ) : ?>
				<div id="top-bar-right">
					<div id="top-bar-right-inner">
						<?php echo $right_content; // WPCS: XSS ok. ?>
					</div>
				</div>
			<?php endif; ?>
		</div>
	</div>
	<?php
}
endif;

if ( ! function_exists( 'nice_top_bar_shortcodes' ) ) :
/**
 * Obtain a list of all the top bar specific shortcodes.
 *
 * @since  1.0.0
 *
 * @return array
 */
function nice_top_bar_shortcodes() {
	static $shortcodes = null;

	if ( is_null( $shortcodes ) ) {
		$shortcodes = apply_filters( 'nice_top_bar_shortcodes', array() );
	}

	return $shortcodes;
}
endif;

if ( ! function_exists( 'nice_top_bar_do_shortcode' ) ) :
/**
 * Process top bar specific shortcodes using a given content.
 *
 * @since  1.0.0
 *
 * @param  string $content
 *
 * @return string
 */
function nice_top_bar_do_shortcode( $content ) {
	$shortcodes = nice_top_bar_shortcodes();

	if ( ! empty( $shortcodes ) && is_array( $shortcodes ) ) {
		preg_match_all( '@\[([^<>&/\[\]\x00-\x20=]++)@', $content, $matches );
		$tagnames = array_intersect( array_keys( $shortcodes ), $matches[1] );

		$pattern = get_shortcode_regex( $tagnames );
		$content = preg_replace_callback( "/$pattern/", 'nice_top_bar_shortcode_tag', $content );
	}

	return $content;
}
endif;

if ( ! function_exists( 'nice_top_bar_shortcode_tag' ) ) :
/**
 * Custom shortcode parser for top bar.
 *
 * @since  1.0.0
 *
 * @see    do_shortcode_tag()
 *
 * @param  string $m
 *
 * @return string
 */
function nice_top_bar_shortcode_tag( $m ) {
	$shortcode_tags = nice_top_bar_shortcodes();

	// allow [[foo]] syntax for escaping a tag
	if ( '[' === $m[1] && ']' === $m[6] ) {
		return substr( $m[0], 1, -1 );
	}

	$tag = $m[2];
	$attr = shortcode_parse_atts( $m[3] );

	if ( ! is_callable( $shortcode_tags[ $tag ] ) ) {
		/* translators: %s: shortcode tag */
		$message = sprintf( esc_html__( 'Attempting to parse a shortcode without a valid callback: %s', 'smart' ), $tag );
		_nice_doing_it_wrong( __FUNCTION__, $message, '1.0.0' );
		return $m[0];
	}

	if ( is_string( $attr ) ) {
		$attr = array();
	}

	if ( isset( $m[5] ) ) {
		// enclosing tag - extra parameter
		return $m[1] . call_user_func( $shortcode_tags[ $tag ], $attr, $m[5], $tag ) . $m[6];
	} else {
		// self-closing tag
		return $m[1] . call_user_func( $shortcode_tags[ $tag ], $attr, null,  $tag ) . $m[6];
	}
}
endif;

if ( ! function_exists( 'nice_top_bar_content_left' ) ) :
/**
 * Obtain content for top bar left side.
 *
 * @since 1.0.0
 */
function nice_top_bar_content_left() {
	$content = nice_get_option( '_top_bar_content_left' );

	$content = nice_top_bar_do_shortcode( $content );

	return $content;
}
endif;

if ( ! function_exists( 'nice_top_bar_content_right' ) ) :
/**
 * Obtain content for top bar left side.
 *
 * @since 1.0.0
 */
function nice_top_bar_content_right() {
	$content = nice_get_option( '_top_bar_content_right' );

	$content = nice_top_bar_do_shortcode( $content );

	return $content;
}
endif;

if ( ! function_exists( 'nice_top_bar_menu' ) ) :
add_action( 'nice_top_bar_menu', 'nice_top_bar_menu' );
/**
 * Print top bar menu.
 *
 * @since 1.0.0
 */
function nice_top_bar_menu() {
	// Return early if the top bar is not active.
	if ( ! nice_bool_option( '_top_bar' ) ) {
		return;
	}

	/**
	 * @hook nice_before_top_bar_menu
	 *
	 * Hook here to add HTML elements right before the menu is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_before_top_bar_menu' );

	$args = array(
		'menu'            => '',
		'container'       => 'nav',
		'container_class' => '',
		'container_id'    => 'top-bar-navigation',
		'menu_class'      => '',
		'menu_id'         => 'top-nav',
		'echo'            => true,
		'fallback_cb'     => '',
		'before'          => '',
		'after'           => '',
		'link_before'     => '',
		'link_after'      => '',
		'depth'           => 2,
		'theme_location'  => 'top-bar-menu',
	);

	/**
	 * @hook nice_top_bar_menu_args
	 *
	 * Hook in here to modify the default arguments to create the top bar
	 * navigation menu.
	 *
	 * @since 1.0.0
	 */
	$args = apply_filters( 'nice_top_bar_menu_args', $args );

	// Print menu using given arguments.
	wp_nav_menu( $args );

	/**
	 * @hook nice_after_top_bar_menu
	 *
	 * Hook here to add HTML elements right after the menu is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_after_top_bar_menu' );
}
endif;
