<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains functions to manage the heading layout and its different
 * options.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_heading' ) ) :
add_action( 'nice_heading', 'nice_heading' );
/**
 * Print the heading block.
 *
 * @since  1.0.0
 *
 * @return string
 */
function nice_heading() {
	/**
	 * @hook nice_before_heading
	 *
	 * Hook here to add HTML elements right before the heading is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_before_heading' );
?>
	<!-- BEGIN #heading -->
	<div id="heading" role="heading" <?php nice_heading_class(); ?>>

		<?php
			/**
			 * @hook nice_heading_before_title
			 *
			 * Hook here to add HTML elements right before the heading title is printed.
			 *
			 * @since 1.0.0
			 *
			 * Hooked here:
			 * @see nice_page_title_start() - 10
			 * @see nice_single_post_meta_category_page_title() - 10
			 */
			do_action( 'nice_heading_before_title' );
		?>

		<?php
			/**
			 * @hook nice_heading_title
			 *
			 * The heading title is printed here. Hook to add or remove HTML
			 * elements from the title.
			 *
			 * @since 1.0.0
			 *
			 * Hooked here:
			 * @see nice_page_title() - 10
			 * @see nice_team_integration_single_member_position() - 10
			 * @see nice_woocommerce_integration_product_title() - 10
			 */
			do_action( 'nice_heading_title' );
		?>

		<?php
			/**
			 * @hook nice_heading_after_title
			 *
			 * Hook here to add HTML elements right after the heading title is printed.
			 *
			 * @since 1.0.0
			 *
			 * Hooked here:
			 * @see nice_page_title_end() - 10
			 * @see nice_page_title_scroll_down_arrow() - 15
			 */
			do_action( 'nice_heading_after_title' );
		?>

	</div><!-- END #heading -->
<?php
	/**
	 * @hook nice_after_heading
	 *
	 * Hook here to add HTML elements right after the heading is printed.
	 *
	 * @since 1.0.0
	 */
	do_action( 'nice_after_heading' );
}
endif;

if ( ! function_exists( 'nice_heading_display' ) ) :
/**
 * Check if the heading should be displayed.
 *
 * @since  1.0.0
 *
 * @return bool
 */
function nice_heading_display() {
	static $post_heading_display = null;

	if ( is_null( $post_heading_display ) ) {
		$post_heading_display = true;

		if ( ( is_page() || is_single() ) ) {
			$post_heading_display = get_post_meta( get_the_ID(), '_post_heading', true );
			$post_heading_display = ( '' !== $post_heading_display ) ? nice_bool( $post_heading_display ) : $post_heading_display;

			if ( '' === $post_heading_display ) {
				$post_type = get_post_type();

				$post_heading_display = nice_bool_option( "_{$post_type}_heading" );
			}
		}
	}

	/**
	 * @hook nice_heading_display
	 *
	 * Hook in here to redefine if the heading should be displayed.
	 *
	 * @since 1.1
	 */
	return apply_filters( 'nice_heading_display', $post_heading_display );
}
endif;

if ( ! function_exists( 'nice_heading_remove' ) ) :
add_action( 'wp', 'nice_heading_remove' );
/**
 * Remove heading area if set to not be displayed.
 *
 * @since 1.1
 */
function nice_heading_remove() {
	if ( nice_heading_display() && ! nice_is_homepage() ) {
		return;
	}

	remove_action( 'nice_heading', 'nice_heading' );
}
endif;

if ( ! function_exists( 'nice_heading_width' ) ) :
/**
 * Return the current width type for the heading element.
 *
 * @since 1.1
 *
 * @return string
 */
function nice_heading_width() {
	static $heading_width = null;

	if ( is_null( $heading_width ) ) {
		$heading_width = '';

		// Try to obtain heading width for single post object.
		if ( is_page() || is_single() ) {
			$heading_width = get_post_meta( get_the_ID(), '_post_title_width', true );

			// Try to obtain from theme settings for the current post type.
			if ( ! $heading_width ) {
				$heading_width = nice_get_option( get_post_type() . '_title_width' );
			}
		}
	}

	/**
	 * @hook nice_heading_width
	 *
	 * Hook in here to modify the width type for the heading element.
	 *
	 * @since 1.1
	 */
	return apply_filters( 'nice_heading_width', $heading_width );
}
endif;

if ( ! function_exists( 'nice_heading_width_value' ) ) :
/**
 * Return the current width of the heading.
 *
 * @since 1.1
 *
 * @return string
 */
function nice_heading_width_value() {
	static $heading_width = null;

	// Try to obtain heading width for single post object.
	if ( is_null( $heading_width ) && ( is_page() || is_single() ) ) {
		$heading_width       = '';
		$_post_heading_width = get_post_meta( get_the_ID(), '_post_title_width', true );

		// Try to obtain from post meta.
		switch ( $_post_heading_width ) {
			case 'full':
				$heading_width = '100%';
				break;
			case 'limit':
				$heading_width = get_post_meta( get_the_ID(), '_post_title_width_value', true );
				break;
			default:
				break;
		}

		// Try to obtain from theme settings for the current post type.
		if ( ! $heading_width ) {
			$post_type = get_post_type();

			switch ( nice_get_option( $post_type . '_title_width' ) ) {
				case 'full':
					$heading_width = '100%';
					break;
				case 'limit':
					$heading_width = nice_get_option( $post_type . '_title_width_value' );
					break;
				default:
					break;
			}
		}
	}

	// We may have obtained a value/unit pair, so in that case we need to parse it as a single string.
	if ( is_array( $heading_width ) ) {
		$heading_width = join( '', $heading_width );
	}

	/**
	 * @hook nice_heading_width_value
	 *
	 * Hook in here to modify the width of the heading.
	 *
	 * @since 1.1
	 */
	return apply_filters( 'nice_heading_width_value', $heading_width );
}
endif;
