<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains functions to manage the general layout according to options and custom fields.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_body_class' ) ) :
add_filter( 'body_class', 'nice_body_class' );
/**
 * Add custom classes to default body class.
 *
 * @since  1.0.0
 *
 * @param  array $classes List of body classes.
 * @return array
 */
function nice_body_class( $classes ) {
	/**
	 * Return `custom-background` class if we have a background selected and it
	 * hasn't been applied previously.
	 *
	 * @note: This should be removed when we develop full compatibility with
	 *        both custom backgrounds and Customizer.
	 *
	 * @since 1.0.0
	 */
	if ( nice_get_option( 'nice_background_image' ) && ! get_background_image() ) {
		$classes[] = 'custom-background';
	}

	/**
	 * Add custom classes when we're using a slider in the homepage.
	 *
	 * @since 1.0
	 */
	if ( nice_is_homepage() && post_type_exists( 'slide' ) && nice_bool( nice_get_option( 'nice_slider' ) ) ) {
		$has_slides = get_posts( array( 'post_type' => 'slide', 'showposts' => 1 ) );

		if ( ! empty ( $has_slides ) ) {
			$classes[] = 'has-slider has-slider-menu';
		}
	}

	/**
	 * Add custom page template class for versions of WordPress that don't
	 * support it (< 4.1).
	 *
	 * @since 1.0.0
	 */
	if ( nice_is_homepage() && ! in_array( 'page-template-home', $classes, true ) ) {
		$classes[] = 'page-template-home';
	}

	/**
	 * Add class for navigation type.
	 *
	 * @since 1.0
	 */
	if ( nice_header_vertical() ) {
		$classes[] = 'vertical-navigation';
		$classes[] = 'navigation-' . nice_get_option( 'nice_header_vertical_navigation_position', 'left' );

		if ( nice_bool( nice_get_option( 'nice_header_vertical_hidden', 'false' ) ) ) {
			$classes[] = 'navigation-initially-hidden';
		}

		if ( 'custom' === nice_get_option( 'nice_header_vertical_content_width' ) ) {
			$classes[] = 'content-custom-width';
		}
	} else {
		$classes[] = 'horizontal-navigation';
	}

	/**
	 * Add class for overlay menu.
	 *
	 * @since 1.0.0
	 */
	if ( nice_overlay_navigation() ) {
		$classes[] = 'has-overlay-menu';
	}

	/**
	 * Add class for overlay skin.
	 *
	 * @since 1.0.0
	 */
	if ( nice_overlay_navigation() || nice_header_display_search() ) {
		$classes[] = 'overlay-skin-' . nice_get_option( 'nice_header_overlay_skin', 'dark' );
	}

	/**
	 * Add class for boxed layout.
	 *
	 * @since 1.0
	 */
	if ( nice_boxed_layout() ) {
		$classes[] = 'boxed-layout';

		/**
		 * Add class for background color.
		 *
		 * @since 1.0
		 */
		if ( $background_color = nice_get_option( 'nice_background_color' ) ) {
			$classes[] = nice_theme_color_background_class( $background_color );
		}
	}

	/**
	 * Add class for content skin.
	 *
	 * @since 1.0
	 */
	$content_skin = nice_content_skin();

	if ( ! $content_skin || is_archive() || is_home() ) {
		$content_skin = nice_get_option( 'nice_content_skin' );
	}

	if ( in_array( $content_skin, array( 'light', 'dark' ), true ) ) {
		$classes[] = $content_skin . '-skin';
		$classes[] = $content_skin;
	}

	return $classes;
}
endif;

if ( ! function_exists( 'nice_body_data' ) ) :
/**
 * Print data attributes for body element.
 *
 * @since 1.0.0
 *
 * @param array $data
 */
function nice_body_data( array $data = null ) {
	$data = nice_body_get_data( $data );

	if ( is_array( $data ) && ! empty( $data ) ) {
		$data_parts = array();

		foreach ( $data as $key => $value ) {
			$data_parts[] = 'data-' . $key . '="' . str_replace( '_', '-', esc_attr( $value ) ) . '"';
		}
	}

	if ( ! empty( $data_parts ) ) {
		echo join( ' ', $data_parts ); // WPCS: XSS Ok.
	}
}
endif;

if ( ! function_exists( 'nice_body_get_data' ) ) :
/**
 * Obtain data attributes for body element.
 *
 * @since 1.0.0
 *
 * @param array  $data
 *
 * @return array
 */
function nice_body_get_data( array $data = null ) {
	$default_data = array();

	if ( $nice_page_loader = nice_get_option( 'nice_page_loader' ) ) {
		$data['page-loader'] = sanitize_title( $nice_page_loader );
	}

	if ( $btn_color = nice_get_option( '_btn_color' ) ) {
		$data['btn-color'] = $btn_color;
	}

	if ( $btn_shape = nice_get_option( '_btn_shape' ) ) {
		$data['btn-shape'] = $btn_shape;
	}

	if ( nice_lazyload_images() ) {
		$data['use-lazyload'] = true;
	}

	if ( nice_header_display() ) {
		if ( $navigation = nice_header_navigation_style() ) {
			$data['navigation'] = $navigation;
		}
	}

	$data = wp_parse_args( $data, $default_data );

	if ( nice_header_display() ) {
		$data['has-header'] = true;
	}

	if ( nice_bool_option( '_top_bar' ) ) {
		$data['has-top-bar'] = true;

		if ( nice_bool_option( '_top_bar_fixed' ) ) {
			$data['top-bar-fixed'] = true;
		}
	}

	if ( nice_header_fixed_mobile() ) {
		$data['header-mobile-fixed'] = true;
	}

	if ( nice_header_transparent_mobile() ) {
		$data['header-mobile-transparent'] = true;
	}

	if ( ! nice_header_vertical() && nice_option_equals( '_site_content_width', 'limit' ) ) {
		$data['content-width'] = 'custom';
	}

	if ( $menu_hover_effect = nice_get_option( '_menu_hover_effect' ) ) {
		$data['menu-hover-effect'] = $menu_hover_effect;
	}

	/**
	 * @hook nice_body_get_data
	 *
	 * Hook in here to add or remove data attributes.
	 *
	 * @since 1.0.0
	 */
	return apply_filters( 'nice_body_data', $data );
}
endif;

if ( ! function_exists( 'nice_site_content_width' ) ) :
/**
 * Return the current width type for the content area.
 *
 * @since 1.1
 *
 * @return string
 */
function nice_site_content_width() {
	static $content_width = null;

	if ( ! nice_header_vertical() && is_null( $content_width ) ) {
		$content_width = '';

		/**
		 * Define a function to clean up the "default" value if that's what
		 * we're getting. However, this shouldn't happen outside of our own
		 * development environment.
		 *
		 * @param  string $content_width
		 *
		 * @return string
		 */
		$parse = function( $content_width ) {
			return ( 'default' === $content_width ) ? '' : $content_width;
		};

		// Try to obtain content width for single post object.
		if ( is_page() || is_single() ) {
			$content_width = $parse( get_post_meta( get_the_ID(), '_post_content_width', true ) );

			// Try to obtain content width from theme settings for the current post type.
			if ( ! $content_width ) {
				$content_width = $parse( nice_get_option( '_' . get_post_type() . '_content_width' ) );
			}
		}

		// Try to obtain from general theme setting.
		if ( ! $content_width ) {
			$content_width = $parse( nice_get_option( '_site_content_width' ) );
		}
	}

	/**
	 * @hook nice_header_width
	 *
	 * Hook in here to modify the width type for the content area.
	 *
	 * @since 1.1
	 */
	return apply_filters( 'nice_site_content_width', $content_width );
}
endif;

if ( ! function_exists( 'nice_site_content_width_value' ) ) :
/**
 * Return the current width of the content area.
 *
 * @since 1.1
 *
 * @return string
 */
function nice_site_content_width_value() {
	static $content_width = null;

	// Try to obtain header width for single post object.
	if ( is_null( $content_width ) ) {
		$content_width   = '';

		if ( ( is_page() || is_single() ) ) {
			$post_width_type = get_post_meta( get_the_ID(), '_post_content_width', true );

			// Try to obtain from post meta.
			switch ( $post_width_type ) {
				case 'full':
					$content_width = '100%';
					break;
				case 'limit':
				case 'custom':
					$content_width = get_post_meta( get_the_ID(), '_post_content_width_value', true );
					break;
				default:
					break;
			}

			// Try to obtain from theme settings for the current post type.
			if ( ! $content_width ) {
				$post_type = get_post_type();

				switch ( nice_get_option( $post_type . '_content_width' ) ) {
					case 'full':
						$content_width = '100%';
						break;
					case 'limit':
						$content_width = nice_get_option( $post_type . '_content_width_value' );
						break;
					default:
						break;
				}
			}
		}

		// Try to obtain from general theme setting.
		if ( ! $content_width ) {
			switch ( nice_get_option( '_site_content_width' ) ) {
				case 'full':
					$content_width = '100%';
					break;
				case 'limit':
					$content_width = nice_get_option( '_site_content_width_value' );
					break;
				case 'custom': // This one is a longshot for development purposes, since this option was not included in any released version.
					$content_width = nice_int_option( '_site_content_custom_width' ) . 'px';
					break;
				default:
					break;
			}
		}
	}

	// We may have obtained a value/unit pair, so in that case we need to parse it as a single string.
	if ( is_array( $content_width ) ) {
		$content_width = join( '', $content_width );
	}

	/**
	 * @hook nice_header_width_value
	 *
	 * Hook in here to modify the width of the header.
	 *
	 * @since 1.1
	 */
	return apply_filters( 'nice_site_content_width_value', $content_width );
}
endif;

if ( ! function_exists( 'nice_post_content_styling' ) ) :
add_filter( 'nice_inline_styles', 'nice_post_content_styling', 20 );
/**
 * Add Custom Styling for the content
 *
 * @since 1.0.0
 *
 * @param string $inline_styles
 *
 * @return string
 */
function nice_post_content_styling( $inline_styles ) {
	$output = '';

	if ( $max_width = nice_site_content_width_value() ) {
		$output = '#container { width: 100%; max-width: ' . strval( $max_width ) . '; }' . "\n";
	}

	if ( ! empty( $output ) ) {
		$output = strip_tags( '/* Content Area */' . "\n\n" . $output );
		$inline_styles .= "\n" . $output;
	}

	return $inline_styles;
}
endif;
