<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains functions to manage the custom styles for the theme.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'nice_responsive_breakpoints' ) ) :
/**
 * Obtain responsive breakpoint sizes.
 *
 * These breakpoints should describe the minimum size for each kind of device.
 * 
 * IMPORTANT: All values here should match the ones defined in
 * `sass/variables/_structure.scss`. Otherwise responsive layouts may break.
 *
 * @since 1.0.0
 * 
 * @return array
 */
function nice_responsive_breakpoints() {
	return array(
		'desktop'   => 1024,
		'tablet'    => 768,
		'mobile-lg' => 425,
		'mobile-md' => 375,
		'mobile-sm' => 320,
	);
}
endif;

if ( ! function_exists( 'nice_custom_styling' ) ) :
add_action( 'nice_register_styles', 'nice_custom_styling', 10 );
/**
 * Add Custom Styling.
 *
 * @since 1.0.0
 */
function nice_custom_styling() {
	wp_add_inline_style( 'nice-styles', nice_inline_styles() );
}
endif;

if ( ! function_exists( 'nice_inline_styles' ) ) :
/**
 * Obtain inline styles as generated by following theme options.
 *
 * @since 1.0.0
 */
function nice_inline_styles() {
	global $is_IE, $is_safari;

	// Obtain all breakpoints.
	$breakpoints = nice_responsive_breakpoints();

	$header_display     = nice_header_display();
	$transparent_header = nice_header_transparent();
	$fixed_header       = nice_bool( nice_get_option( 'nice_header_fixed' ) );

	// Initialize HTML output.
	$output = '';

	/**
	 * Background styles for boxed layout.
	 */
	if ( nice_boxed_layout() ) {
		$background_styles = '';

		// Apply background image.
		if ( $background_image = nice_get_option( 'nice_background_image' ) ) {
			$background_styles .= 'body { background-image : url(' . $background_image . '); }' . "\n";

			// Apply background image position.
			if ( $background_image_position = nice_get_option( 'nice_background_image_position' ) ) {
				$background_styles .= 'body { background-position : ' . $background_image_position . '; }' . "\n";
			}

			// Apply background repeat.
			if ( $background_image_repeat = nice_get_option( 'nice_background_image_repeat' ) ) {
				$background_styles .= 'body { background-repeat : ' . $background_image_repeat . '; }' . "\n";
			}

			// Apply background attachment.
			if ( $background_image_attachment = nice_get_option( 'nice_background_image_attachment' ) ) {
				$background_styles .= 'body { background-attachment : ' . $background_image_attachment . '; }' . "\n";
			}
		}

		// Add to output.
		if ( $background_styles ) {
			$output .= '/* Body background. */' . "\n";
			$output .= $background_styles;
		}
	}

	/**
	 * Accent color.
	 */
	if ( $nice_accent_color = nice_get_option( 'nice_accent_color' ) ) {
		$output .= '/* Accent color. */' . "\n";

		if ( is_rtl() ) {
			$output .= '.entry blockquote { border-right-color:' . $nice_accent_color. '; }' . "\n";
		} else {
			$output .= '.entry blockquote { border-left-color:' . $nice_accent_color. '; }' . "\n";
		}

		$output .= '.wp-pagenavi span.current, span.required { color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '#navigation ul li a .bar, #navigation ul li a .bar:before, #navigation ul li a .bar:after, #navigation ul li a .bar, .nice-contact input[type="submit"]:hover, #commentform .button:hover, .single .featured-image a, .post .featured-image a, .nice-gallery .thumb a, #tabs .inside .tags a:hover{ background-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '.navbar ul.nav:not(.toggled) > li > ul.sub-menu, hr.nice-separator.nice-separator-accent-color { border-color: ' . $nice_accent_color . '}' . "\n";
		$output .= '#heading { background-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '.blog-masonry .post-footer-meta section a:hover, .blog-masonry .post-footer-meta section a:hover i { color: ' . $nice_accent_color . '; }' . "\n";
		$output .= 'a:hover, a:visited:hover, a:focus, a:active { color: ' . $nice_accent_color . ' }' . "\n";
		$output .= '.grid .item .item-content .entry-meta:not(.entry-contact) a:hover i { color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '#header.horizontal #main-nav.default > .menu-item > .sub-menu { border-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '#header.vertical.left #main-nav.default > .menu-item > .sub-menu { border-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '#header.vertical.right #main-nav.default > .menu-item > .sub-menu { border-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '#main-nav.default > .menu-item:not(.megamenu) > .sub-menu .sub-menu { border-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '.posts-loader.standard-pagination .page.active { background-color: ' . $nice_accent_color . '; }' . "\n";
		$output .= '.btn-accent-color, input[type="submit"] { background-color: ' . $nice_accent_color . '; border-color: ' . $nice_accent_color . ' }' . "\n";

		$output .= '::selection { background: ' . $nice_accent_color . ' }' . "\n";
		$output .= '::-moz-selection { background: ' . $nice_accent_color . ' }' . "\n";
	}

	/**
	 * Page Loader
	 */
	$output .= '/* Page Loader. */' . "\n";
	if ( $nice_page_loader = nice_get_option( 'nice_page_loader' ) ) {
		if ( 'top_bar' === $nice_page_loader ) {
			if ( $nice_page_loader_color = nice_get_option( 'nice_page_loader_color' ) ) {
				$output .= '.pace .pace-progress { background:' . nice_get_option( $nice_page_loader_color ) . ' }' . "\n";
			} elseif ( nice_get_option_default( 'nice_accent_color' ) !== ( $accent_color = nice_get_option( 'nice_accent_color' ) ) ) {
				$output .= '.pace .pace-progress { background:' . $accent_color . ' }' . "\n";
			}
		} elseif ( 'full_page' === $nice_page_loader ) {
			if ( $nice_page_loader_background_color_id = nice_get_option( 'nice_page_loader_full_page_background_color' ) ) {
				$nice_page_loader_background_color = nice_theme_color_value( $nice_page_loader_background_color_id );
				$output .= 'body[data-page-loader="full-page"] .nice-page-loader-full { background-color: ' . esc_attr( $nice_page_loader_background_color ) . ' }' . "\n";
			}

			if ( $nice_page_loader_spinner_color_id = nice_get_option( '_page_loader_full_page_loader_color' ) ) {
				$nice_page_loader_spinner_color = nice_theme_color_value( $nice_page_loader_spinner_color_id );
				$output .= 'body[data-page-loader="full-page"] .nice-page-loader-full .path { stroke: ' . esc_attr( $nice_page_loader_spinner_color ) . ' }' . "\n";
				$output .= 'body[data-page-loader="full-page"] .nice-page-loader-full .loader { background-color: ' . esc_attr( $nice_page_loader_spinner_color ) . ' }' . "\n";
			}
		}
	} else {
		$output .= '.pace .pace-progress { background: transparent !important; }' . "\n";
	}

	if ( $top_bar_bg_color = nice_get_option( '_top_bar_background_color' ) ) {
		$output .= '#top-nav .menu-item .sub-menu { background-color: ' . nice_theme_color_value( $top_bar_bg_color ) . ' !important; }' . "\n";
	}

	/**
	 * Header
	 */

	if ( $header_display ) {
		/**
		 * Inline styles for header width.
		 */
		if ( $nice_header_width = nice_header_width_value() ) {
			$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { #header-inner { max-width: ' . esc_attr( $nice_header_width ) . '; } }' . "\n";
		}

		/**
		 * Inline styles for top bar.
		 */
		if ( nice_bool_option( '_top_bar' ) ) {
			$top_bar_padding     = nice_get_option( '_top_bar_padding', 0 );
			$top_bar_height      = nice_top_bar_height();
			$top_bar_full_height = nice_top_bar_full_height();
			$top_bar_fixed       = nice_bool_option( '_top_bar_fixed' );
			$header_vertical     = nice_header_vertical();

			if ( $nice_header_width ) {
				if ( nice_boxed_layout() ) {
					$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.boxed-layout #top-bar-inner { max-width: ' . esc_attr( $nice_header_width ) . '; } }' . "\n";
				} else {
					$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { #top-bar-inner { max-width: ' . esc_attr( $nice_header_width ) . '; } }' . "\n";
				}
			}

			$output .= '#top-bar { height: ' . intval( $top_bar_height ) . 'px }' . "\n";
			$output .= '#top-bar-inner { height: ' . intval( $top_bar_full_height ) . 'px }' . "\n";

			$output .= '#top-bar { padding-top: ' . intval( $top_bar_padding ) . 'px; padding-bottom: ' . intval( $top_bar_padding ) . 'px; }' . "\n";
			$output .= '#top-bar-inner { margin-top: ' . ( 0 - intval( $top_bar_padding ) ) . 'px; margin-bottom: ' . ( 0 - intval( $top_bar_padding ) ) . 'px; }' . "\n";

			// Specific styles for horizontal header.
			if ( $top_bar_fixed && ! $header_vertical ) {
				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.horizontal-navigation[data-top-bar-fixed="1"] #wrapper { margin-top: ' . intval( $top_bar_full_height ) . 'px } }' . "\n";
			} elseif( ! $header_vertical ) {
				if ( is_user_logged_in() && nice_boxed_layout() ) {
					$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.admin-bar #top-bar:not(.fixed) + #header:not(.vertical) { top: ' . ( intval( $top_bar_full_height ) + 32 ). 'px } }' . "\n";
				}

				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { #top-bar:not(.fixed) + #header:not(.vertical) { top: ' . intval( $top_bar_full_height ) . 'px } }' . "\n";
			}

			// Specific styles for vertical header.
			if ( $header_vertical && $top_bar_fixed ) {
				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body { margin-top: ' . intval( $top_bar_full_height ) . 'px; } }' . "\n";
				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.vertical-navigation #top-bar.fixed { width: calc(100% - ' . intval( nice_get_option( '_header_vertical_width' ) ) . 'px) } }' . "\n";
			}
		}


		// Initialize header background styles.
		$header_background_styles = '';

		// Header background image.
		if ( $nice_header_background_image = nice_get_option( 'nice_header_background_image' ) ) {
			$header_background_styles .= '#header { background-image : url(' . $nice_header_background_image . '); }' . "\n";
		}

		// Header background position.
		if ( $nice_header_background_image_position = nice_get_option( 'nice_header_background_image_position' ) ) {
			$header_background_styles .= '#header { background-position : ' . $nice_header_background_image_position. '; }' . "\n";
		}

		// Background-repeat.
		if ( $nice_header_background_image_repeat = nice_get_option( 'nice_header_background_image_repeat' ) ) {
			$header_background_styles .= '#header { background-repeat : ' . $nice_header_background_image_repeat. '; }' . "\n";
		}

		// Background color.
		if ( $nice_header_background_color = nice_get_option( 'nice_header_background_color' ) ) {
			$nice_header_background_color_code = nice_get_option( $nice_header_background_color );

			if ( $transparent_header ) {
				$header_background_styles .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header { background-color: ' . $nice_header_background_color_code . '; } }' . "\n";
			}
		}

		// Background color (Dark).
		if ( $nice_header_background_color_dark = nice_get_option( 'nice_header_background_color_dark' ) ) {
			$nice_header_background_color_dark_code = nice_get_option( $nice_header_background_color_dark );

			$output .= '#header.header-dark { background-color: ' . $nice_header_background_color_dark_code . ' !important; }' . "\n";

			if ( $transparent_header ) {
				$header_background_styles .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header.header-dark { background-color: ' . $nice_header_background_color_dark_code . '; } }' . "\n";
			}
		}

		// Add to output.
		if ( $header_background_styles ) {
			$output .= '/* Header background. */' . "\n";
			$output .= $header_background_styles;
		}

		if ( $nice_logo_height = nice_get_option( 'nice_logo_height' ) ) {
			$output .= '/* Logo height. */' . "\n";
			$output .= '#logo .img-logo { height: ' . $nice_logo_height . 'px; }' . "\n";

			if ( ( nice_header_horizontal() && nice_bool( nice_get_option( 'nice_header_shrink' ) ) ) || nice_development_mode() ) {
				$output .= '#header.horizontal.shrunk #logo .img-logo { height: ' . ( $nice_logo_height * 0.9 ) . 'px; }' . "\n";
			}
		}

		/**
		 * Header padding.
		 */
		$nice_header_padding = nice_get_option( 'nice_logo_padding' );

		if ( nice_header_horizontal() && ! nice_header_centered() ) {
			$output .= '/* Header padding. */' . "\n";
			$output .= '#header #logo { padding: ' . $nice_header_padding . 'px 0 }' . "\n";
		}

		/**
		 * Mobile logo styles.
		 */
		$logo_mobile                     = nice_get_option( '_logo_mobile' );
		$logo_mobile_dark                = nice_get_option( '_logo_mobile_dark' );
		$logo_mobile_custom_settings     = nice_bool_option( '_logo_mobile_custom_settings' );
		$logo_mobile_apply_custom_styles = $logo_mobile_custom_settings || $logo_mobile || $logo_mobile_dark;

		if ( $logo_mobile_apply_custom_styles ) {
			$logo_mobile_padding = nice_get_option( '_logo_mobile_padding' );

			if ( $logo_mobile_height = nice_get_option( '_logo_mobile_height' ) ) {
				$output .= '/* Header mobile height. */' . "\n";
				$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header #logo .img-logo { height: ' . ( intval( nice_get_option( '_logo_mobile_height' ) ) ) . 'px; } }' . "\n";
			}

			$output .= '/* Header mobile padding. */' . "\n";
			$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header #logo { padding: ' . ( intval( $logo_mobile_padding ) ) . 'px 0 } }';
		}

		/**
		 * Top padding for first element after header.
		 */
		if ( ( nice_header_horizontal() || nice_overlay_navigation() ) && ! nice_bool_option( '_header_hide_until_scroll' ) ) {
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #heading { padding-top: ' . ( 30 + nice_header_max_height() ) . 'px; } }' . "\n";

			if ( nice_is_homepage() ) {
				$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #homepage-slider.flexslider.position-1 .slide .slide-info { padding-top: ' . ( 30 + nice_header_max_height() ) . 'px; } }' . "\n";
				$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { .home-block.position-1:not(.flexslider) > .inner { padding-top: ' . nice_header_max_height() . 'px; } }' . "\n";
			} else {
				$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header + #container { padding-top: ' . ( 30 + nice_header_max_height() ) . 'px; } }' . "\n";
			}
		}

		if ( nice_bool_option( '_top_bar' ) ) {
			$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { body[data-has-top-bar] { margin-top: ' . intval( nice_top_bar_full_height() ) . 'px; } }' . "\n";

			if ( nice_header_fixed_mobile() ) {
				$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { body[data-header-mobile-fixed][data-has-top-bar] { margin-top: ' . intval( nice_header_max_height() + nice_top_bar_full_height() ) . 'px; } }' . "\n";
			}

			if ( nice_header_transparent_mobile() ) {
				$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { body[data-header-mobile-transparent][data-has-top-bar] { margin-top: ' . intval( nice_top_bar_full_height() ) . 'px; } }' . "\n";
			}
		}

		if ( nice_header_fixed_mobile() ) {
			if ( is_user_logged_in() ) {
				$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { body.admin-bar #top-bar + #header.fixed-mobile { top: ' . intval( nice_top_bar_full_height() + 32 ) . 'px; } }' . "\n";
				$output .= '@media all and (max-width: 782px) { body.admin-bar #top-bar + #header.fixed-mobile { top: ' . intval( nice_top_bar_full_height() + 46 ) . 'px; } }' . "\n";
			}

			$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #top-bar + #header.fixed-mobile { top: ' . intval( nice_top_bar_full_height() ) . 'px; } }' . "\n";

			if ( ! nice_header_transparent_mobile() ) {
				$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { body[data-header-mobile-fixed] { margin-top: ' . nice_header_max_height_mobile() . 'px; } }' . "\n";
			}
		}

		if ( nice_header_transparent_mobile() ) {
			$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header.transparent-mobile + #heading, #header.transparent-mobile + #container { padding-top: ' . ( 30 + nice_header_max_height_mobile() ) . 'px; } }' . "\n";
			$output .= '@media all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { #header.transparent-mobile:not(.fixed-mobile) + #heading, #header.transparent-mobile:not(.fixed-mobile) + #container { margin-top: -' . nice_header_max_height_mobile() . 'px; } }' . "\n";
		}

		/**
		 * Horizontal non-centered menu.
		 */
		if ( nice_header_horizontal() && ! nice_header_centered() ) {
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.horizontal.align-default #logo { padding: ' . $nice_header_padding . 'px 0 } }' . "\n";

			if ( $is_safari ) {
				$output .= 'body.safari #header #navigation-extras #navigation-extras-inner { height: ' . nice_header_max_height( array( 'ignore_top_bar' => true ) ) . 'px }' . "\n";

				if ( nice_bool( nice_get_option( 'nice_header_shrink' ) ) ) {
					$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.safari #header.shrunk #navigation-extras #navigation-extras-inner { height: ' . nice_header_min_height() . 'px } }' . "\n";
				}
			}

			if ( $is_IE ) {
				$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default { height: ' . nice_header_max_height() . 'px } }' . "\n";
				$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default #navigation #main-nav > li > a { height: ' . nice_header_max_height() . 'px; line-height: ' . nice_header_max_height() . 'px } }' . "\n";
				$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default #navigation-extras #navigation-extras-inner > div { height: ' . nice_header_max_height() . 'px } }' . "\n";

				$output .= '@media only all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { html.ie.ie9 #header { height: ' . nice_header_max_height() . 'px } }' . "\n";
				$output .= '@media only all and (max-width: ' . ( $breakpoints['desktop'] - 1 ) . 'px) { html.ie.ie9 #header #navigation-extras #navigation-extras-inner > div { height: ' . nice_header_max_height() . 'px } }' . "\n";

				if ( nice_bool( nice_get_option( 'nice_header_shrink' ) ) ) {
					$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default.shrunk { height: ' . nice_header_min_height() . 'px } }' . "\n";
					$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default.shrunk #navigation #main-nav > li > a { height: ' . nice_header_min_height() . 'px; line-height: ' . nice_header_min_height() . 'px } }' . "\n";
					$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { html.ie.ie9 #header.horizontal.align-default.shrunk #navigation-extras #navigation-extras-inner > div { height: ' . nice_header_min_height() . 'px } }' . "\n";
				}
			}
		}

		/**
		 * Horizontal centered menu.
		 */
		if ( nice_header_horizontal() && nice_header_centered() ) {
			$nice_header_padding_top    = intval( nice_get_option( '_logo_padding_top', 40 ) );
			$nice_header_padding_bottom = intval( nice_get_option( '_logo_padding_bottom', 10 ) );

			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.horizontal.align-center #logo { padding: ' . $nice_header_padding_top . 'px 0 ' . $nice_header_padding_bottom . 'px } }' . "\n";
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.shrunk.horizontal.align-center #logo { padding: ' . ( $nice_header_padding_top / 2 ) . 'px 0 ' . ( $nice_header_padding_bottom / 2 ) . 'px } }' . "\n";
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.horizontal.align-center #navigation-extras > div > a { margin: ' . $nice_header_padding_top . 'px 0 ' . $nice_header_padding_bottom . 'px } }' . "\n";
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.shrunk.horizontal.align-center #navigation-extras > div > a { margin: ' . ( $nice_header_padding_top / 2 ) . 'px 0 ' . ( $nice_header_padding_bottom / 2 ) . 'px } }' . "\n";
		}

		/**
		 * Horizontal header layout.
		 */
		if ( ! nice_header_vertical() && nice_site_content_width() ) {
			$nice_site_width = nice_site_content_width_value() ?: '';

			if ( nice_boxed_layout() ) {
				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.boxed-layout #wrapper, body.boxed-layout #header { max-width: ' . esc_attr(  $nice_site_width ) . '; } }' . "\n";
				if ( nice_bool_option( '_top_bar' ) && empty( $nice_header_width ) ) {
					$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.boxed-layout #top-bar { max-width: ' . esc_attr( $nice_site_width ) . '; } }' . "\n";
				}
			} else {
				$output .= '@media all and (min-width: ' . intval( $breakpoints['desktop'] ) . 'px) { body.horizontal-navigation #container { max-width: ' . esc_attr( $nice_site_width ) . '; } }' . "\n";
			}
		}

		/**
		 * Vertical menu.
		 */
		if ( nice_header_vertical() ) {
			$nice_header_vertical_width    = nice_get_option( 'nice_header_vertical_width' );
			$nice_header_vertical_position = nice_get_option( 'nice_header_vertical_navigation_position', 'left' );
			$nice_header_vertical_hidden   = nice_get_option( 'nice_header_vertical_hidden', 'false' );

			if ( ! nice_bool( $nice_header_vertical_hidden ) ) {
				/**
				 * Manage inline styles for default menu behavior.
				 */
				if ( 'left' === $nice_header_vertical_position ) {
					$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.vertical-navigation.navigation-left { padding-left: ' . $nice_header_vertical_width . 'px } }' . "\n";
				} elseif ( 'right' === $nice_header_vertical_position ) {
					$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.vertical-navigation.navigation-right { padding-right: ' . $nice_header_vertical_width . 'px } }' . "\n";
				}
			} else {
				/**
				 * Manage inline styles for hidden menu.
				 */
				if ( 'left' === $nice_header_vertical_position ) {
					$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical.left.initially-hidden { margin-left: ' . ( 0 - ( $nice_header_vertical_width - 60 ) ) . 'px } }' . "\n";
				} elseif ( 'right' === $nice_header_vertical_position ) {
					$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical.right.initially-hidden { margin-right: ' . ( 0 - ( $nice_header_vertical_width - 60 ) ) . 'px } }' . "\n";
				}
			}

			if ( 'custom' === nice_get_option( 'nice_header_vertical_content_width' ) ) {
				$nice_header_vertical_content_custom_width = nice_get_option( 'nice_header_vertical_content_custom_width' );
				$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.vertical-navigation .col-full { max-width: ' . intval( $nice_header_vertical_content_custom_width ) . 'px } }' . "\n";

				if ( nice_boxed_layout() ) {
					$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.vertical-navigation #wrapper { max-width: ' . intval( $nice_header_vertical_content_custom_width ) . 'px } }' . "\n";
				}
			}

			// Manage width of vertical menu.
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical, #header.vertical #header-inner { width: ' . $nice_header_vertical_width . 'px } }' . "\n";
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical #logo { padding-top: ' . $nice_header_padding . 'px } }' . "\n";

			if ( $is_IE ) {
				$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.ie #header.vertical #navigation-extras { width: ' . $nice_header_vertical_width . 'px } } ' . "\n";
				$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { body.ie.navigation-initially-hidden.vertical-navigation.navigation-left #toggle-nav-container { left: ' . ( $nice_header_vertical_width - 40 ). 'px } } ' . "\n";
			}

			if ( nice_bool( nice_get_option( 'nice_logo_padding_remove_horizontal' ) ) ) {
				$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical #logo { padding-left: 0; padding-right: 0; } }' . "\n";
			}
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header.vertical #main-nav.default > .menu-item.megamenu > .sub-menu { width: calc( 100vw - ' . $nice_header_vertical_width . 'px ) } }' . "\n";
		}

		/**
		 * Transition for fixed and transparent header.
		 */
		if ( $transparent_header || $fixed_header ) {
			$transition_duration = nice_get_option( 'nice_header_transition_duration' );
			$transition_duration = $transition_duration ? $transition_duration / 1000 : 0;
			$output .= '/* Transition for fixed and transparent header. */' . "\n";
			$output .= '@media all and (min-width: ' . $breakpoints['desktop'] . 'px) { #logo, #logo img { -webkit-transition-duration: ' . $transition_duration .'s; -moz-transition-duration: ' . $transition_duration .'s; -ms-transition-duration: ' . $transition_duration .'s; -o-transition-duration: ' . $transition_duration .'s; transition-duration: ' . $transition_duration .'s; } }' . "\n";
		}
	}

	/**
	 * Heading (title bar).
	 */
	if ( nice_heading_display() && nice_heading_width() ) {
		$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { #heading .col-full { max-width: ' . esc_attr( nice_heading_width_value() ) . '; } }' . "\n";
	}

	/**
	 * Overlay color.
	 */
	if ( 'custom' === nice_get_option( 'nice_header_overlay_skin' ) ) {
		$output .= '/* Overlay color. */' . "\n";

		if ( ( $header_background_color = nice_get_option( 'nice_header_overlay_background_color' ) ) && ( 1 > ( $header_background_color_opacity = floatval( nice_get_option( 'nice_header_overlay_background_color_opacity' ) ) ) ) ) {
			$output .= 'body > .overlay { background-color: ' . nice_color_hex2rgba( nice_get_option( $header_background_color ), $header_background_color_opacity ) . ' !important; }' . "\n";
		}

		$nice_overlay_accent_color = nice_get_option( 'nice_header_overlay_accent_color' );

		$output .= '.overlay .close:before, .overlay .close:after { background-color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#main-nav.overlay .menu-item a, #main-nav.overlay .menu-item i:before, #header-search-container input.input-text, #header-search-container.overlay form label:after { color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#header-search-container input.input-text::-webkit-input-placeholder { color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#header-search-container input.input-text:-moz-input-placeholder { color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#header-search-container input.input-text::-moz-input-placeholder { color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#header-search-container input.input-text:-ms-input-placeholder { color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
		$output .= '#header-search-container input.input-text { border-color: ' . $nice_overlay_accent_color . ' !important; }' . "\n";
	}

	/**
	 * Light skin.
	 */

	if ( $light_skin_bg_color = nice_get_option( '_light_skin_background_color' ) ) {
		$output .= '.light #wrapper, #container .light { background-color: ' . nice_theme_color_value( $light_skin_bg_color ) . ' !important; }' . "\n";
	}

	if ( $light_skin_text_color = nice_get_option( '_light_skin_text_color' ) ) {
		$output .= '.light, .light #wrapper, .light .entry blockquote  { color: ' . nice_theme_color_value( $light_skin_text_color ) . ' !important; }' . "\n";
	}

	if ( $light_skin_link_color = nice_get_option( '_light_skin_link_color' ) ) {
		$output .= '.light #content a:not(.button), .light #container a, #container .light a { color: ' . nice_theme_color_value( $light_skin_link_color ) . '; }' . "\n";
	}

	if ( $light_skin_heading_color = nice_get_option( '_light_skin_heading_color' ) ) {
		$output .= '.light #container h1, .light #container h2, .light #container h3, .light #container h4, .light #container h5, .light #container h6 { color: ' . nice_theme_color_value( $light_skin_heading_color ) . '; }' . "\n";
		$output .= '.light #container .light h1, .light #container .light h2, .light #container .light h3, .light #container .light h4, #container .light h5, #container .light h6 { color: ' . nice_theme_color_value( $light_skin_heading_color ) . '; }' . "\n";
	}

	if ( $light_skin_nav_text_color = nice_get_option( '_light_skin_nav_text_color' ) ) {
		$output .= '.header-light #header-inner #main-nav > .menu-item > a, .header-light #header-inner a { color: ' . nice_theme_color_value( $light_skin_nav_text_color ) . ' !important; }' . "\n";
		$output .= '.header-light #header-inner a svg, .header-light #header-inner a svg path { fill: ' . nice_theme_color_value( $light_skin_nav_text_color ) . ' !important; }' . "\n";
		$output .= '.header-light #toggle-nav-container a.toggle-nav:before, .header-light #toggle-nav-container a.toggle-nav:after, .header-light #toggle-nav-container a.toggle-nav i.toggle-bar { background-color: ' . nice_theme_color_value( $light_skin_nav_text_color ) . ' !important; }';
	}

	if ( $light_skin_subnav_text_color = nice_get_option( '_light_skin_subnav_text_color' ) ) {
		$output .= '.submenu-light #main-nav > .menu-item > .sub-menu .menu-item a { color: ' . nice_theme_color_value( $light_skin_subnav_text_color ) . ' !important; }' . "\n";
	}

	if ( $light_skin_subnav_background_color = nice_get_option( '_light_skin_subnav_background_color' ) ) {
		$output .= '.submenu-light #main-nav.default > .menu-item .sub-menu { background-color: ' . nice_theme_color_value( $light_skin_subnav_background_color ) . ' !important; }' . "\n";
	}

	if ( $light_skin_footer_background_color = nice_get_option( '_light_skin_footer_background_color' ) ) {
		$output .= '#footer.site-footer.light { background: ' . nice_theme_color_value( $light_skin_footer_background_color ) . ' !important; }' . "\n";
	}

	if ( $light_skin_footer_extended_background_color = nice_get_option( '_light_skin_footer_extended_background_color' ) ) {
		$output .= '#footer.site-footer.light #extended-footer { background: ' . nice_theme_color_value( $light_skin_footer_extended_background_color ) . ' !important; }' . "\n";
	}

	/**
	 * Dark skin.
	 */

	if ( $dark_skin_bg_color = nice_get_option( '_dark_skin_background_color' ) ) {
		$output .= '.dark #wrapper, #container .dark { background-color: ' . nice_theme_color_value( $dark_skin_bg_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_text_color = nice_get_option( '_dark_skin_text_color' ) ) {
		$output .= '.dark, .dark #wrapper, .dark .entry blockquote { color: ' . nice_theme_color_value( $dark_skin_text_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_link_color = nice_get_option( '_dark_skin_link_color' ) ) {
		$output .= '.dark #content a:not(.button), .dark #container a, #container .dark a { color: ' . nice_theme_color_value( $dark_skin_link_color ) . ' !important; }' . "\n";
		$output .= '.light #content .dark a:not(.button), .light #container .dark a, .light #container .dark a { color: ' . nice_theme_color_value( $dark_skin_link_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_heading_color = nice_get_option( '_dark_skin_heading_color' ) ) {
		$output .= '.dark #container h1, .dark #container h2, .dark #container h3, .dark #container h4, .dark #container h5, .dark #container h6 { color: ' . nice_theme_color_value( $dark_skin_heading_color ) . ' !important; }' . "\n";
		$output .= '#container .dark h1, #container .dark h2, #container .dark h3, #container .dark h4, #container .dark h5, #container .dark h6 { color: ' . nice_theme_color_value( $dark_skin_heading_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_nav_text_color = nice_get_option( '_dark_skin_nav_text_color' ) ) {
		$output .= '.header-dark #header-inner #main-nav > .menu-item > a, .header-dark #header-inner a { color: ' . nice_theme_color_value( $dark_skin_nav_text_color ) . ' !important; }' . "\n";
		$output .= '.header-dark #header-inner a svg, .header-dark #header-inner a svg path { fill: ' . nice_theme_color_value( $dark_skin_nav_text_color ) . ' !important; }' . "\n";
		$output .= '.header-dark #toggle-nav-container a.toggle-nav:before, .header-dark #toggle-nav-container a.toggle-nav:after, .header-dark #toggle-nav-container a.toggle-nav i.toggle-bar { background-color: ' . nice_theme_color_value( $dark_skin_nav_text_color ) . ' !important; }';
	}

	if ( $dark_skin_subnav_text_color = nice_get_option( '_dark_skin_subnav_text_color' ) ) {
		$output .= '.submenu-dark #main-nav > .menu-item > .sub-menu .menu-item a { color: ' . nice_theme_color_value( $dark_skin_subnav_text_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_subnav_background_color = nice_get_option( '_dark_skin_subnav_background_color' ) ) {
		$output .= '.submenu-dark #main-nav.default > .menu-item .sub-menu { background-color: ' . nice_theme_color_value( $dark_skin_subnav_background_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_footer_background_color = nice_get_option( '_dark_skin_footer_background_color' ) ) {
		$output .= '#footer.site-footer.dark { background: ' . nice_theme_color_value( $dark_skin_footer_background_color ) . ' !important; }' . "\n";
	}

	if ( $dark_skin_footer_extended_background_color = nice_get_option( '_dark_skin_footer_extended_background_color' ) ) {
		$output .= '#footer.site-footer.dark #extended-footer { background: ' . nice_theme_color_value( $dark_skin_footer_extended_background_color ) . ' !important; }' . "\n";
	}

	/**
	 * Navigation Highlight Color.
	 */

	if ( $menu_highlight_color = nice_get_option( '_menu_highlight_color' ) ) {
		$output .= '@media only all and (min-width: ' . $breakpoints['desktop'] . 'px) { #header-inner #main-nav > .menu-item.current-menu-item > a, #header-inner #main-nav > .menu-item.current_page_item > a, #header-inner #main-nav > .menu-item > a:hover, #header-inner a:hover { color: ' . nice_theme_color_value( $menu_highlight_color ) . ' !important; } }';
		$output .= '#header-inner a:hover svg, #header-inner a:hover svg path { fill: ' . nice_theme_color_value( $menu_highlight_color ) . ' !important; }' . "\n";

		$output .= '#toggle-nav-container a.toggle-nav:hover:before, #toggle-nav-container a.toggle-nav:hover:after, #toggle-nav-container a.toggle-nav:hover i.toggle-bar { background-color: ' . nice_theme_color_value( $menu_highlight_color ) . ' !important; }';
	}

	/**
	 * Inline styles for footer width.
	 */
	if ( nice_footer_display() && 'limit' === nice_footer_width() ) {
		$output .= '#footer .col-full { max-width: ' . esc_attr( nice_footer_width_value() ) . ' }' . "\n";
	}

	/**
	 * @hook nice_inline_styles
	 *
	 * Hook here to add inline styles.
	 *
	 * @since 1.0.0
	 */
	$output = apply_filters( 'nice_inline_styles', $output );

	return $output;
}
endif;

if ( ! function_exists( 'nice_custom_styling_colors' ) ) :
add_filter( 'nice_inline_styles', 'nice_custom_styling_colors' );
/**
 * Add Custom Styling for the different colors.
 *
 * @since  1.0.0
 *
 * @param  string $inline_styles
 *
 * @return string
 */
function nice_custom_styling_colors( $inline_styles ) {
	// Initialize HTML output.
	$output = '';

	$nice_colors = nice_theme_colors();

	/**
	 * Default button color.
	 */
	$btn_color_id = nice_get_option( '_btn_color' );

	if ( $btn_color = nice_theme_color_value( $btn_color_id ) ) {
		$btn_body_class = 'body[data-btn-color="' . str_replace( '_', '-', $btn_color_id ) . '"]';

		$output .= '/* Default button color. */' . "\n";
		$output .= $btn_body_class . ' .btn:not(.btn-outline):not(.btn-cart), ' . $btn_body_class . ' .button:not(.btn-outline):not(.btn-cart), ' . $btn_body_class . ' input[type="submit"]:not(.btn-outline), ' . $btn_body_class . ' input[type="button"]:not(.btn-outline) { background-color: ' . $btn_color . ' !important; border-color: ' . $btn_color . ' !important; }' . "\n";
		$output .= $btn_body_class . ' .btn-default.btn-outline:not(.btn-cart), ' . $btn_body_class . ' .btn-outline.button:not(.btn-cart), ' . $btn_body_class . ' input.btn-outline[type="submit"], ' . $btn_body_class . ' input.btn-outline[type="button"] { color: ' . $btn_color . ' !important; border-color: ' . $btn_color . ' !important; }' . "\n";
		$output .= $btn_body_class . ' .btn-default:hover:not(.btn-outline):not(.btn-disable-hover):not(.btn-cart), ' . $btn_body_class . ' .button:hover:not(.btn-outline):not(.btn-disable-hover):not(.btn-cart), ' . $btn_body_class . ' input[type="submit"]:hover:not(.btn-outline):not(.btn-disable-hover), ' . $btn_body_class . ' input[type="button"]:hover:not(.btn-outline):not(.btn-disable-hover) { background-color: ' . nice_color_darken( $btn_color, .05 ). ' !important; border-color: ' . nice_color_darken( $btn_color, .05 ). ' !important; }' . "\n";
		$output .= $btn_body_class . ' .btn-default.btn-outline:hover:not(.btn-disable-hover):not(.btn-cart), ' . $btn_body_class . ' .btn-outline.button:hover:not(.btn-disable-hover):not(.btn-cart), ' . $btn_body_class . ' input.btn-outline[type="submit"]:hover:not(.btn-disable-hover), ' . $btn_body_class . ' input.btn-outline[type="button"]:hover:not(.btn-disable-hover) { border-color: ' . $btn_color . ' !important; background-color: ' . $btn_color . ' !important; }' . "\n";
	}

	foreach ( $nice_colors as $color ) {
		$color_hex        = $color['value'];
		$color_css_id     = $color['css_id'];
		$color_rgba       = ( false === strpos( $color_hex, 'rgba', 0 ) ) ? nice_color_hex2rgba( $color_hex, '0.25' ) : $color_hex;
		$btn_css_class    = '.' . nice_theme_color_button_class( $color_css_id );
		$txt_css_class    = '.' . nice_theme_color_text_class( $color_css_id );
		$bg_css_class     = '.' . nice_theme_color_background_class( $color_css_id );
		$border_css_class = '.' . nice_theme_color_border_class( $color_css_id );
		$pills_css_class  = '.' . nice_theme_color_pills_class( $color_css_id );

		$btn_color_attr = 'color: #fff !important;';
		$btn_hover_color_attr = 'color: #fff !important;';
		// Prevent having white text on white button.
		if ( '.btn-white-color' === $btn_css_class ) {
			$btn_color_attr = '';
			$btn_hover_color_attr = '';
		}

		$output .= '/* ' . $color['name'] . '. */' . "\n";
		$output .= $btn_css_class . ', ' . $btn_css_class . '.btn-outline:hover, body[data-btn-color] [class]' . $btn_css_class . ':not(.btn-outline), body[data-btn-color] [class]' . $btn_css_class . '.btn-outline:hover, body[data-btn-color] input[type]' . $btn_css_class . ', body[data-btn-color] input[type]' . $btn_css_class . '.btn-outline:hover { ' . $btn_color_attr . ' background: ' . $color_hex . ' !important; border-color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $btn_css_class . '.btn-outline, body[data-btn-color] [class]' . $btn_css_class . '.btn-outline, body[data-btn-color] input[type]' . $btn_css_class . '.btn-outline  { color: ' . $color_hex . ' !important; border-color: ' . $color_hex . ' !important; background: transparent !important; }' . "\n";
		$output .= $btn_css_class . ':not(.btn-outline):not(.btn-disable-hover):hover, body[data-btn-color] [class]' . $btn_css_class . ':not(.btn-outline):not(.btn-disable-hover):hover, body[data-btn-color] input[type]' . $btn_css_class . ':not(.btn-outline):not(.btn-disable-hover):hover { ' . $btn_hover_color_attr . ' background-color: ' . nice_color_darken( $color_hex, .05 ) . ' !important; border-color: ' . nice_color_darken( $color_hex, .05 ) . ' !important; }' . "\n";
		$output .= $btn_css_class . '.btn-outline:not(.btn-disable-hover):hover, body[data-btn-color] [class]' . $btn_css_class . '.btn-outline:not(.btn-disable-hover):hover, body[data-btn-color] input[type]' . $btn_css_class . '.btn-outline:not(.btn-disable-hover):hover { background-color: ' . $color_hex . ' !important; border-color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $txt_css_class . ' { color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $bg_css_class . ' { background-color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $bg_css_class . '.bottom_gradient { background: linear-gradient(' . $color_rgba . ', ' . $color_hex . ') !important; }' . "\n";
		$output .= $bg_css_class . '.top_gradient { background: linear-gradient(' . $color_hex . ', ' . $color_rgba . ' ) !important; }' . "\n";
		$output .= $border_css_class . ' { border-color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $pills_css_class . ' a { background-color: ' . $color_hex . ' !important; }' . "\n";
		$output .= $pills_css_class . '.pills-outline a { background-color: transparent !important; border-color: ' . $color_hex . ' !important; color: ' . $color_hex . ' !important; }' . "\n";

		// Home Slider
		$slider_btn_container = '#homepage-slider.flexslider.direction-nav-btn-solid.direction-nav-' . $color_css_id;
		$slider_btn           = $slider_btn_container . ' ul.flex-direction-nav .flex-nav-prev a';
		$slider_btn          .= ', ' . $slider_btn_container . ' ul.flex-direction-nav .flex-nav-next a';

		$output .= $slider_btn . ' { background: ' . $color_hex . ' !important; }' . "\n\n";
	}

	if ( ! empty( $output ) ) {
		$inline_styles .= "\n" . $output;
	}

	return $inline_styles;
}
endif;

if ( ! function_exists( 'nice_tag_cloud_remove_inline_styles' ) ) :
add_filter( 'wp_tag_cloud', 'nice_tag_cloud_remove_inline_styles' );
/**
 * Remove tag cloud inline styles.
 *
 * @param  string $tags
 *
 * @return string
 */
function nice_tag_cloud_remove_inline_styles( $tags ) {
	return preg_replace( "/style=\'[^\']*\'/", '', $tags );
}
endif;
