<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains a class to create the Adspace Widget.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Adspace_Widget' ) ) :
/**
 * Nice_Adspace_Widget
 *
 * This widget shows information about an author.
 *
 * @link http://codex.wordpress.org/Widgets_API
 *
 * @since 1.0.0
 */
class Nice_Adspace_Widget extends WP_Widget {
	/**
	 * Default values for widget instance.
	 *
	 * @since 1.0.0
	 * @var   array
	 */
	private $instance_defaults = array(
		'title'      => array( '', 'sanitize_text_field' ),
		'code'       => array( '', 'esc_html' ),
		'image'      => array( '', 'sanitize_text_field' ),
		'href'       => array( '', 'esc_url_raw' ),
		'alt'        => array( '', 'sanitize_text_field' ),
		'title_text' => array( '', 'sanitize_text_field' ),
	);

	/**
	 * Helper object to manage widget cache.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Cache
	 */
	private $cache;

	/**
	 * Helper object to manage common processes.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Normalizer
	 */
	private $normalizer;

	/**
	 * Initialize widget properties.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->alt_option_name = 'nice_ad';
		$this->normalizer      = new Nice_Widget_Normalizer( $this->instance_defaults );
		$this->cache           = new Nice_Widget_Cache( 'widget_nice_ad' );

		parent::__construct( 'nice_ad', esc_html__( '(NiceThemes) Adspace', 'smart' ), array(
				'description' => 'With this widget you can add any type of Ad as a widget.',
			)
		);

		add_action( 'save_post',    array( $this->cache, 'flush' ) );
		add_action( 'deleted_post', array( $this->cache, 'flush' ) );
		add_action( 'switch_theme', array( $this->cache, 'flush' ) );
	}

	/**
	 * Display the widget in the front-end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 * @param array $instance
	 */
	public function widget( $args, $instance ) {
		// If widget cache is found, print it and return early.
		if ( $cache = $this->cache->get() ) {
			echo $cache;

			return;
		}

		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		ob_start();

		extract( $args );
		extract( $instance );

		echo $before_widget;

		if ( $title ) {
			echo $before_title . $title . $after_title;
		}

		if ( $code ) {
			echo html_entity_decode( $code );
		} else { ?>
			<a href="<?php echo esc_url( $href ); ?>" title="<?php echo esc_attr( $title_text ); ?>">
				<img src="<?php echo esc_url( $image ); ?>" alt="<?php echo esc_attr( $alt ); ?>" />
			</a>
		<?php }

		echo $after_widget;

		$this->cache->set( ob_get_flush() );
	}

	/**
	 * Flush widget cache when updating settings.
	 *
	 * @since  1.0.0
	 *
	 * @param  array $new_instance
	 * @param  array $old_instance
	 * @return array
	 */
	function update( $new_instance, $old_instance ) {
		$this->cache->flush();

		// Normalize instance.
		$new_instance = $this->normalizer->normalize_instance( $new_instance );

		if ( ! current_user_can( 'unfiltered_html' ) ) {
			$new_instance['code'] = $old_instance['code'];
		}

		return $new_instance;
	}

	/**
	 * Display widget form in "Appearance > Widgets".
	 *
	 * @since  1.0.0
	 *
	 * @param  array $instance
	 * @return string|void
	 */
	function form( $instance ) {
		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		extract( $instance );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title (optional):', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo esc_attr( $title ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"/>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'code' ) ); ?>"><?php esc_html_e( 'Code:', 'smart' ); ?></label>
			<textarea name="<?php echo esc_attr( $this->get_field_name( 'code' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'code' ) ); ?>"><?php echo $code; ?></textarea>
		</p>
		<p>
			<strong><?php esc_html_e( 'Else, you can use an image', 'smart' ); ?></strong>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'image' ) ); ?>"><?php esc_html_e( 'Image URL:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'image' ) ); ?>" value="<?php echo esc_url( $image ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'image' ) ); ?>"/>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'alt' ) ); ?>"><?php esc_html_e( 'Image Alt text:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'alt' ) ); ?>" value="<?php echo esc_attr( $alt ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'alt' ) ); ?>"/>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'href' ) ); ?>"><?php esc_html_e( 'Link URL:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'href' ) ); ?>" value="<?php echo esc_url( $href ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'href' ) ); ?>"/>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title_text' ) ); ?>"><?php esc_html_e( 'Link Title text:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'title_text' ) ); ?>" value="<?php echo esc_attr( $title_text ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title_text' ) ); ?>"/>
		</p>
		<?php
	}
}
endif;
