<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains a class to create the Post Author widget.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Blog_Author_Widget' ) ) :
/**
 * Nice_Blog_Author_Widget
 *
 * This widget shows information about an author.
 *
 * @link http://codex.wordpress.org/Widgets_API
 *
 * @since 1.0.0
 */
class Nice_Blog_Author_Widget extends WP_Widget {
	/**
	 * Default values for widget instance.
	 *
	 * @since 1.0.0
	 * @var   array
	 */
	private $instance_defaults = array(
		'title'          => array( '', 'sanitize_text_field' ),
		'bio'            => array( '', 'sanitize_text_field' ),
		'twitter'        => array( '', 'sanitize_text_field' ),
		'custom_email'   => array( '', 'sanitize_email' ),
		'avatar_size'    => array( 0, 'sanitize_text_field' ),
		'avatar_align'   => array( '', 'sanitize_text_field' ),
		'read_more_text' => array( '', 'sanitize_text_field' ),
		'read_more_url'  => array( '', 'esc_url_raw' ),
		'page'           => array( '', 'sanitize_text_field' ),
	);

	/**
	 * Helper object to manage widget cache.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Cache
	 */
	private $cache;

	/**
	 * Helper object to manage common processes.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Normalizer
	 */
	private $normalizer;

	/**
	 * How often the tweets are refreshed (in seconds). Default is five minutes.
	 *
	 * @since 1.0.0
	 * @var   int
	 */
	public $refresh = 300;

	/**
	 * Initialize widget properties.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->refresh         = apply_filters( 'nice_widget_blog_author_refresh', $this->refresh );
		$this->alt_option_name = 'nice_blog_author';
		$this->normalizer      = new Nice_Widget_Normalizer( $this->instance_defaults );
		$this->cache           = new Nice_Widget_Cache( 'widget_nice_blog_author' );

		parent::__construct( 'nice_blog_author', esc_html__( '(NiceThemes) Blog Author', 'smart' ), array(
				'description' => esc_html__( 'This widget includes Blog Author Info.', 'smart' ),
			)
		);

		add_action( 'save_post',    array( $this->cache, 'flush' ) );
		add_action( 'deleted_post', array( $this->cache, 'flush' ) );
		add_action( 'switch_theme', array( $this->cache, 'flush' ) );
	}

	/**
	 * Display the widget in the front-end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 * @param array $instance
	 */
	public function widget( $args, $instance ) {
		// If widget cache is found, print it and return early.
		if ( $cache = $this->cache->get() ) {
			echo $cache;

			return;
		}

		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		ob_start();

		extract( $args );
		extract( $instance );

		if ( ! $avatar_size ) {
			$avatar_size = 50;
		}

		if ( ! $avatar_align ) {
			$avatar_align = 'left';
		}

		if ( ( 'home' === $page && is_home() ) || ( 'single' === $page && is_single() ) || 'all' === $page ) {
			echo $before_widget;

			if ( $title ) {
				echo $before_title . $title . $after_title;
			}

			?>

			<?php if ( $custom_email ) : ?>
				<div class="<?php echo esc_attr( $avatar_align ); ?> thumb">
					<?php echo get_avatar( $custom_email, $size = $avatar_size ); ?>
				</div>
			<?php endif; ?>

			<div class="bio">
				<p>
					<?php echo esc_html( $bio ); ?>
				</p>
			</div>

			<?php if ( $read_more_url ) : ?>
				<div class="read-more">
					<p>
						<a href="<?php echo esc_url( $read_more_url ); ?>"><?php echo esc_html( $read_more_text ); ?></a>
					</p>
				</div>
			<?php endif; ?>

			<?php if ( $twitter ) : ?>
				<div class="twitter">
					<a href="https://twitter.com/<?php echo esc_attr( $twitter ); ?>" class="twitter-follow-button" data-show-count="false"><?php printf( esc_html__( 'Follow %s', 'smart' ), esc_attr( $twitter ) ); ?></a>
					<script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+'://platform.twitter.com/widgets.js';fjs.parentNode.insertBefore(js,fjs);}}(document, 'script', 'twitter-wjs');</script>
				</div>
			<?php endif; ?>

			<?php
			echo $after_widget;
		}

		$this->cache->set( ob_get_flush() );
	}

	/**
	 * Flush widget cache when updating settings.
	 *
	 * @since  1.0.0
	 *
	 * @param  array $new_instance
	 * @param  array $old_instance
	 * @return array
	 */
	public function update( $new_instance, $old_instance ) {
		$this->cache->flush();

		// Normalize instance.
		$new_instance = $this->normalizer->normalize_instance( $new_instance );

		return $new_instance;
	}

	/**
	 * Display widget form in "Appearance > Widgets".
	 *
	 * @since  1.0.0
	 *
	 * @param  array $instance
	 * @return string|void
	 */
	function form( $instance ) {
		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		extract( $instance );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo esc_attr( $title ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'bio' ) ); ?>"><?php esc_html_e( 'Bio:', 'smart' ); ?></label>
			<textarea name="<?php echo esc_attr( $this->get_field_name( 'bio' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'bio' ) ); ?>"><?php echo esc_html( $bio ); ?></textarea>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'twitter' ) ); ?>"><?php printf( esc_html__( '%s Username:', 'smart' ), sprintf( '<a href="%1$s">%2$s</a>', 'http://twitter.com/', esc_html__( 'Twitter', 'smart' ) ) ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'twitter' ) ); ?>" value="<?php echo esc_attr( $twitter ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'twitter' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'custom_email' ) ); ?>"><?php printf( esc_html__( '%s E-mail:', 'smart' ), sprintf( '<a href="%1$s">%2$s</a>', 'http://gravatar.com/', esc_html__( 'Gravatar', 'smart' ) ) ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'custom_email' ) ); ?>" value="<?php echo esc_attr( $custom_email ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'custom_email' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'avatar_size' ) ); ?>"><?php esc_html_e( 'Gravatar Size:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'avatar_size' ) ); ?>" value="<?php echo esc_attr( $avatar_size ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'avatar_size' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'avatar_align' ) ); ?>"><?php esc_html_e( 'Gravatar Alignment:', 'smart' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'avatar_align' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'avatar_align' ) ); ?>">
				<option value="left" <?php selected( $avatar_align, 'left' ); ?>><?php esc_html_e( 'Left', 'smart' ); ?></option>
				<option value="right" <?php selected( $avatar_align, 'right' ); ?>><?php esc_html_e( 'Right', 'smart' ); ?></option>
			</select>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'read_more_text' ) ); ?>"><?php esc_html_e( 'Read More Text (optional):', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'read_more_text' ) ); ?>" value="<?php echo esc_attr( $read_more_text ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'read_more_text' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'read_more_url' ) ); ?>"><?php esc_html_e( 'Read More URL (optional):', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'read_more_url' ) ); ?>" value="<?php echo esc_url( $read_more_url ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'read_more_url' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'page' ) ); ?>"><?php esc_html_e( 'Visible Pages:', 'smart' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'page' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'page' ) ); ?>">
				<option value="all" <?php selected( $page, 'all' ); ?>><?php esc_html_e( 'All', 'smart' ); ?></option>
				<option value="home" <?php selected( $page, 'home' ); ?>><?php esc_html_e( 'Home only', 'smart' ); ?></option>
				<option value="single" <?php selected( $page, 'single' ); ?>><?php esc_html_e( 'Single only', 'smart' ); ?></option>
			</select>
		</p>
		<?php
	}
}
endif;
