<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains a class to create a Flickr widget.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Flickr_Widget' ) ) :
/**
 * Nice_Flickr_Widget
 *
 * This Flickr widget shows photos from a Flickr ID.
 *
 * @link http://codex.wordpress.org/Widgets_API
 *
 * @since 1.0.0
 */
class Nice_Flickr_Widget extends WP_Widget {
	/**
	 * Default values for widget instance.
	 *
	 * @since 1.0.0
	 * @var   array
	 */
	private $instance_defaults = array(
		'id'      => array( '', '' ),
		'number'  => array( 5, 'absint' ),
		'type'    => array( 'user', 'sanitize_key' ),
		'sorting' => array( 'latest', 'sanitize_key' ),
		'size'    => array( 'm', 'sanitize_key' ),
	);

	/**
	 * Helper object to manage widget cache.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Cache
	 */
	private $cache;

	/**
	 * Helper object to manage common processes.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Normalizer
	 */
	private $normalizer;

	/**
	 * How often widget cache is refreshed (in seconds). Default is five minutes.
	 *
	 * @since 1.0.0
	 * @var   int
	 */
	public $refresh = 300;

	/**
	 * Initialize widget properties.
	 *
	 * @since 1.0.0
	 */
	function __construct() {
		$this->refresh         = apply_filters( 'nice_widget_flickr_refresh', $this->refresh );
		$this->alt_option_name = 'nice_flickr';
		$this->normalizer      = new Nice_Widget_Normalizer( $this->instance_defaults );
		$this->cache           = new Nice_Widget_Cache( 'widget_nice_flickr' );

		parent::__construct( 'nice_flickr', esc_html__( '(NiceThemes) Flickr', 'smart' ), array(
				'description' => 'This Flickr widget shows photos from a Flickr ID.',
			)
		);

		add_action( 'save_post',    array( $this->cache, 'flush' ) );
		add_action( 'deleted_post', array( $this->cache, 'flush' ) );
		add_action( 'switch_theme', array( $this->cache, 'flush' ) );
	}

	/**
	 * Display the widget in the front-end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 * @param array $instance
	 */
	function widget( $args, $instance ) {
		// If widget cache is found, print it and return early.
		if ( $cache = $this->cache->get() ) {
			echo $cache;

			return;
		}

		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		ob_start();

		extract( $args );
		extract( $instance );

		?>
		<?php echo $before_widget; ?>
		<?php echo $before_title; ?>

		<?php printf( esc_html__( 'Photos on %s', 'smart' ), '<span>flick<span>r</span></span>' ); ?>

		<?php echo $after_title; ?>

		<div class="wrap <?php echo esc_attr( $size ); ?>">
			<script type="text/javascript" src="http://www.flickr.com/badge_code_v2.gne?count=<?php echo esc_attr( $number ); ?>&amp;display=<?php echo esc_attr( $sorting ); ?>&amp;&amp;layout=x&amp;source=<?php echo esc_attr( $type ); ?>&amp;<?php echo esc_attr( $type ); ?>=<?php echo esc_attr( $id ); ?>&amp;size=<?php echo esc_attr( $size ); ?>"></script>
		</div>

		<?php echo $after_widget; ?>
		<?php

		$this->cache->set( ob_get_flush() );
	}

	/**
	 * Flush widget cache when updating settings.
	 *
	 * @since  1.0.0
	 *
	 * @param  array $new_instance
	 * @param  array $old_instance
	 * @return array
	 */
	function update( $new_instance, $old_instance ) {
		$this->cache->flush();

		// Normalize instance.
		$new_instance = $this->normalizer->normalize_instance( $new_instance );

		return $new_instance;
	}

	/**
	 * Display widget form in "Appearance > Widgets".
	 *
	 * @since  1.0.0
	 *
	 * @param  array $instance
	 * @return string|void
	 */
	function form( $instance ) {
		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		extract( $instance );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'id' ) ); ?>"><?php printf( esc_attr__( 'Flickr ID (%s):', 'smart' ), sprintf( '<a href="%1$s">%2$s</a>', 'http://www.idgettr.com', esc_attr__( 'idGettr', 'smart' ) ) ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'id' ) ); ?>" value="<?php echo esc_attr( $id ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'id' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'number' ) ); ?>"><?php esc_html_e( 'Number of images:', 'smart' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'number' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'number' ) ); ?>">
				<?php for ( $i = 1; $i <= 10; $i += 1 ) { ?>
					<option value="<?php echo esc_attr( $i ); ?>" <?php if ( $number === $i ) {
						echo "selected='selected'";
					} ?>><?php echo esc_html( $i ); ?></option>
				<?php } ?>
			</select>
		</p>
		<p>
			<strong><?php esc_html_e( 'Type:', 'smart' ); ?></label></strong><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'type' ) ); ?>" value="user" id="<?php echo esc_attr( $this->get_field_id( 'type' ) ); ?>-user" <?php checked( $type, 'user' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'type' ) ); ?>-user"><?php esc_html_e( 'User', 'smart' ); ?></label><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'type' ) ); ?>" value="group" id="<?php echo esc_attr( $this->get_field_id( 'type' ) ); ?>-group" <?php checked( $type, 'group' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'type' ) ); ?>-group"><?php esc_html_e( 'Group', 'smart' ); ?></label>
		</p>
		<p>
			<strong><?php esc_html_e( 'Sorting:', 'smart' ); ?></label></strong><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'sorting' ) ); ?>" value="latest" id="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>-latest" <?php checked( $sorting, 'latest' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>-latest"><?php esc_html_e( 'Latest', 'smart' ); ?></label><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'sorting' ) ); ?>" value="random" id="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>-random" <?php checked( $sorting, 'random' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>-random"><?php esc_html_e( 'Random', 'smart' ); ?></label>
		</p>
		<p>
			<strong><?php esc_html_e( 'Size:', 'smart' ); ?></label></strong><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'size' ) ); ?>" value="s" id="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-s" <?php checked( $size, 's' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-s"><?php esc_html_e( 'Square', 'smart' ); ?></label><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'size' ) ); ?>" value="m" id="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-m" <?php checked( $size, 'm' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-m"><?php esc_html_e( 'Medium', 'smart' ); ?></label><br />
			<input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'size' ) ); ?>" value="t" id="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-t" <?php checked( $size, 't' ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>-t"><?php esc_html_e( 'Thumbnail', 'smart' ); ?></label>
		</p>
	<?php
	}
}
endif;
