<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains a class to create the Tabs widget.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Tabs_Widget' ) ) :
/**
 * Class Nice_Tabs_Widget
 *
 * Tabs Widget. It contains the Popular posts, Recent Posts, Recent comments and a Tag cloud.
 *
 * @since 1.0.0
 */
class Nice_Tabs_Widget extends WP_Widget {
	/**
	 * Default values for widget instance.
	 *
	 * @since 1.0.0
	 * @var   array
	 */
	private $instance_defaults = array(
		'limit'      => array( 5, 'absint' ),
		'thumb_size' => array( 45, 'absint' ),
		'first'      => array( 'popular', 'sanitize_text_field' ),
		'popular'    => array( '', 'sanitize_text_field' ),
		'recent'     => array( '', 'sanitize_text_field' ),
		'comments'   => array( '', 'sanitize_text_field' ),
		'tags'       => array( '', 'sanitize_text_field' ),
		'days'       => array( '', 'absint' ),
	);

	/**
	 * Helper object to manage widget cache.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Cache
	 */
	private $cache;

	/**
	 * Helper object to manage common processes.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Normalizer
	 */
	private $normalizer;

	/**
	 * How often the widget cache is refreshed (in seconds). Default is five minutes.
	 *
	 * @since 1.0.0
	 * @var   int
	 */
	public $refresh = 300;

	/**
	 * JavasCript file source.
	 *
	 * @since 1.0.0
	 * @var   string
	 */
	private $js_source = '';

	/**
	 * Initialize widget.
	 *
	 * @since 1.0.0
	 */
	function __construct() {
		$this->refresh         = apply_filters( 'nice_widget_tabs_refresh', $this->refresh );
		$this->alt_option_name = 'nice_tabs';
		$this->normalizer      = new Nice_Widget_Normalizer( $this->instance_defaults );
		$this->cache           = new Nice_Widget_Cache( 'widget_nice_tabs' );
		$this->js_source       = nice_get_file_uri( 'includes/assets/js/nice-tabs-widget.js', true );

		parent::__construct( 'nice_tabs', esc_html__( '(NiceThemes) Tabs', 'smart' ),
			array(
				'classname'   => 'widget_nice_tabs',
				'description' => esc_html__( 'Tabs Widget. It contains the Popular posts, Recent Posts, Recent comments and a Tag cloud.', 'smart' ),
			),
			array(
				'width'   => 250,
				'height'  => 350,
				'id_base' => 'nice_tabs',
			)
		);

		add_filter( 'nice_async_scripts', array( $this, 'async_scripts' ) );
		add_action( 'nice_register_scripts', array( $this, 'register_scripts' ), 0 );

		add_action( 'save_post',    array( $this->cache, 'flush' ) );
		add_action( 'deleted_post', array( $this->cache, 'flush' ) );
		add_action( 'switch_theme', array( $this->cache, 'flush' ) );
	}

	/**
	 * Register asynchronous scripts for this widget.
	 *
	 * @since 1.0.0
	 *
	 * @param array $scripts
	 *
	 * @return array
	 */
	public function async_scripts( array $scripts = array() ) {
		$scripts = array_merge( $scripts, array(
				'nice-tabs-widget-scripts',
			)
		);

		return $scripts;
	}

	/**
	 * Register scripts for this widget.
	 *
	 * @since 1.0.0
	 */
	public function register_scripts() {
		if ( wp_script_is( 'nice-tabs-widget-scripts', 'registered' ) ) {
			return;
		}

		wp_register_script( 'nice-tabs-widget-scripts', $this->js_source, array( 'jquery' ), false, true );
	}

	/**
	 * Register scripts for this widget.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {
		if ( wp_script_is( 'nice-tabs-widget-scripts', 'enqueued' ) ) {
			return;
		}

		wp_enqueue_script( 'nice-tabs-widget-scripts' );
	}

	/**
	 * Display the widget in the front end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Current properties of the widget.
	 */
	function widget( $args, $instance ) {
		$this->enqueue_scripts();

		// If widget cache is found, print it and return early.
		if ( $cache = $this->cache->get() ) {
			echo $cache;

			return;
		}

		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		ob_start();

		extract( $args );
		extract( $instance );

		?>
		<?php echo $before_widget; ?>

		<div id="tabs-<?php echo esc_attr( $widget_id ); ?>" class="nice-tabs-container">
			<ul class="nice-tabs">
				<?php if ( 'recent' === $first && 'on' !== $recent ) : ?>
					<li class="recent">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-recent"><?php esc_html_e( 'Recent', 'smart' ); ?></a>
					</li>
				<?php elseif ( 'comments' === $first && 'on' !== $recent ) : ?>
					<li class="comments">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-comments"><?php esc_html_e( 'Comments', 'smart' ); ?></a>
					</li>
				<?php elseif ( 'tags' === $first && 'on' !== $tags ) : ?>
					<li class="tags">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-tags"><?php esc_html_e( 'Tags', 'smart' ); ?></a>
					</li>
				<?php endif; ?>

				<?php if ( 'on' !== $popular ) : ?>
					<li class="popular">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-popular"><?php esc_html_e( 'Popular', 'smart' ); ?></a>
					</li>
				<?php endif; ?>

				<?php if ( 'recent' !== $first && 'on' !== $recent ) : ?>
					<li class="recent">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-recent"><?php esc_html_e( 'Recent', 'smart' ); ?></a>
					</li>
				<?php endif; ?>

				<?php if ( 'comments' !== $first && 'on' !== $comments ) : ?>
					<li class="comments">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-comments"><?php esc_html_e( 'Comments', 'smart' ); ?></a>
					</li>
				<?php endif; ?>

				<?php if ( 'tags' !== $first && 'on' !== $tags ) : ?>
					<li class="tags">
						<a href="#<?php echo esc_attr( $widget_id ); ?>-tab-tags"><?php esc_html_e( 'Tags', 'smart' ); ?></a>
					</li>
				<?php endif; ?>
			</ul>

			<div class="boxes box inside">
				<?php if ( 'recent' === $first && 'on' !== $recent ) : ?>
					<ul id="<?php echo esc_attr( $widget_id ); ?>-tab-recent" class="list">
						<?php self::tab_recent( $limit, $thumb_size ); ?>
					</ul>
				<?php elseif ( 'comments' === $first && 'on' !== $comments ) : ?>
					<ul id="<?php echo esc_attr( $widget_id ); ?>-tab-comments" class="list">
						<?php self::tab_comments( $limit, $thumb_size ); ?>
					</ul>
				<?php elseif ( 'tags' === $first && 'on' !== $tags ) : ?>
					<div id="<?php echo esc_attr( $widget_id ); ?>-tab-tags" class="list">
						<?php wp_tag_cloud( array( 'smallest' => 12, 'largest' => 20 ) ); ?>
					</div>
				<?php endif; ?>

				<?php if ( 'on' !== $popular ) : ?>
					<ul id="<?php echo esc_attr( $widget_id ); ?>-tab-popular" class="list">
						<?php self::tab_popular( $limit, $thumb_size, $days ); ?>
					</ul>
				<?php endif; ?>

				<?php if ( 'recent' !== $first && 'on' !== $recent ) : ?>
					<ul id="<?php echo esc_attr( $widget_id ); ?>-tab-recent" class="list">
						<?php self::tab_recent( $limit, $thumb_size ); ?>
					</ul>
				<?php endif; ?>

				<?php if ( 'comments' !== $first && 'on' !== $comments ) : ?>
					<ul id="<?php echo esc_attr( $widget_id ); ?>-tab-comments" class="list">
						<?php self::tab_comments( $limit, $thumb_size ); ?>
					</ul>
				<?php endif; ?>

				<?php if ( 'tags' !== $first && 'on' !== $tags ) : ?>
					<div id="<?php echo esc_attr( $widget_id ); ?>-tab-tags" class="list tags">
						<?php wp_tag_cloud( 'smallest=12&largest=20' ); ?>
					</div>
				<?php endif; ?>
				<!-- END .boxes -->
			</div>

			<!-- END niceTabs -->
		</div>

		<?php echo $after_widget; ?>
		<?php

		$this->cache->set( ob_get_flush() );
	}

	/**
	 * Flush the widget cache when the widget is updated.
	 *
	 * @since 1.0.0
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 *
	 * @return array
	 */
	function update( $new_instance, $old_instance ) {
		$this->cache->flush();

		// Normalize instance.
		$new_instance = $this->normalizer->normalize_instance( $new_instance );

		return $new_instance;
	}

	/**
	 * Display widget form in "Appearance > Widgets".
	 *
	 * @since  1.0
	 *
	 * @param  array       $instance
	 * @return string|void
	 */
	function form( $instance ) {
		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		extract( $instance );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'limit' ) ); ?>"><?php esc_attr_e( 'Number of items to list:', 'smart' ); ?>
				<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'limit' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'limit' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['limit'] ); ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'thumb_size' ) ); ?>"><?php esc_attr_e( 'Thumbnail Size, in pixels (0 to disable):', 'smart' ); ?>
				<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'thumb_size' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'thumb_size' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['thumb_size'] ); ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'first' ) ); ?>"><?php esc_attr_e( 'First Visible Tab:', 'smart' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'first' ) ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'first' ) ); ?>">
				<option value="popular" <?php selected( $instance['first'], 'popular' ); ?>><?php esc_attr_e( 'Popular', 'smart' ); ?></option>
				<option value="comments" <?php selected( $instance['first'], 'comments' ); ?>><?php esc_attr_e( 'Comments', 'smart' ); ?></option>
				<option value="recent" <?php selected( $instance['first'], 'recent' ); ?>><?php esc_attr_e( 'Recent', 'smart' ); ?></option>
				<option value="tags" <?php selected( $instance['first'], 'tags' ); ?>><?php esc_attr_e( 'Tags', 'smart' ); ?></option>
			</select>
		</p>
		<p><strong><?php esc_attr_e( 'Hide Tabs:', 'smart' ); ?></strong></p>

		<div style="width:50%; float:left;">
			<p>
				<input id="<?php echo esc_attr( $this->get_field_id( 'popular' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'popular' ) ); ?>" type="checkbox" <?php checked( $instance['popular'], 'on' ); ?> />
				<label for="<?php echo esc_attr( $this->get_field_id( 'popular' ) ); ?>"><?php esc_attr_e( 'Popular', 'smart' ); ?></label>
			</p>

			<p>
				<input id="<?php echo esc_attr( $this->get_field_id( 'recent' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'recent' ) ); ?>" type="checkbox" <?php checked( $instance['recent'], 'on' ); ?> />
				<label for="<?php echo esc_attr( $this->get_field_id( 'recent' ) ); ?>"><?php esc_attr_e( 'Recent', 'smart' ); ?></label>
			</p>
		</div>
		<div style="width:50%; float:left;">
			<p>
				<input id="<?php echo esc_attr( $this->get_field_id( 'comments' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'comments' ) ); ?>" type="checkbox" <?php checked( $instance['comments'], 'on' ); ?> />
				<label for="<?php echo esc_attr( $this->get_field_id( 'comments' ) ); ?>"><?php esc_attr_e( 'Comments', 'smart' ); ?></label>
			</p>

			<p>
				<input id="<?php echo esc_attr( $this->get_field_id( 'tags' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'tags' ) ); ?>" type="checkbox" <?php checked( $instance['tags'], 'on' ); ?> />
				<label for="<?php echo esc_attr( $this->get_field_id( 'tags' ) ); ?>"><?php esc_attr_e( 'Tags', 'smart' ); ?></label>
			</p>
		</div>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'days' ) ); ?>"><?php esc_attr_e( 'Popular limit (days):', 'smart' ); ?>
				<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'days' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'days' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['days'] ); ?>" />
			</label>
		</p>
		<?php
	}

	/**
	 * Display popular posts.
	 *
	 * @since 1.0.0
	 *
	 * @param int  $posts
	 * @param int  $size
	 * @param null $days
	 */
	private static function tab_popular( $posts = 5, $size = 45, $days = null ) {
		global $post;

		if ( $days ) {
			global $popular_days;
			$popular_days = $days;

			// Register the filtering function
			add_filter( 'posts_where', array( __CLASS__, 'where_filter' ) );
		}

		$posts = get_posts( array(
			'post_type'           => 'post',
			'post_status'         => 'publish',
			'no_found_rows'       => true,
			'suppress_filters'    => false,
			'ignore_sticky_posts' => 1,
			'orderby'             => 'comment_count',
			'numberposts'         => $posts,
		) );
		?>

		<?php foreach ( $posts as $post ) : setup_postdata( $post ); ?>
			<li class="popular-item">
				<?php if ( $size ) : ?>
					<?php nice_image( array(
						'size'   => '',
						'width'  => $size,
						'height' => $size,
						'class'  => 'thumbnail',
						'single' => true,
					) ); ?>
				<?php endif; ?>
				<a href="<?php the_permalink() ?>" title="<?php echo esc_attr( get_the_title() ? get_the_title() : get_the_ID() ); ?>">
					<?php if ( get_the_title() ) : ?>
						<?php the_title(); ?>
					<?php else : ?>
						<?php the_ID(); ?>
					<?php endif; ?>
				</a>
				<span class="meta"><?php the_time( get_option( 'date_format' ) ); ?></span>
			</li>
		<?php endforeach; ?>

		<?php
		wp_reset_postdata();

		if ( has_filter( 'posts_where', array( __CLASS__, 'where_filter' ) ) ) {
			remove_filter( 'posts_where', array( __CLASS__, 'where_filter' ) );
		}
	}

	/**
	 * Display recent posts.
	 *
	 * @since 1.0.0
	 *
	 * @param int $posts
	 * @param int $size
	 */
	private static function tab_recent( $posts = 5, $size = 45 ) {
		global $post;
		$args = array(
			'no_found_rows'       => true,
			'ignore_sticky_posts' => 1,
			'posts_per_page'      => $posts,
			'orderby'             => 'post_date',
			'order'               => 'desc',
		);
		$posts = get_posts( $args );

		?>
		<?php foreach ( $posts as $post ) : setup_postdata( $post );  ?>
			<li class="recent-item">
				<?php if ( 0 < $size ) : ?>
					<?php nice_image( array(
						'size'   => '',
						'width'  => $size,
						'height' => $size,
						'class'  => 'thumbnail',
						'single' => true,
					) ); ?>
				<?php endif; ?>
				<a href="<?php the_permalink() ?>" title="<?php echo esc_attr( get_the_title() ? get_the_title() : get_the_ID() ); ?>">
					<?php if ( get_the_title() ) : ?>
						<?php the_title(); ?>
					<?php else : ?>
						<?php the_ID(); ?>
					<?php endif; ?>
				</a>
				<span class="meta"><?php the_time( get_option( 'date_format' ) ); ?></span>
			</li>
		<?php endforeach; ?>
		<?php

		wp_reset_postdata();
	}

	/**
	 * Display comments.
	 *
	 * @since 1.0.0
	 *
	 * @param int $posts
	 * @param int $size
	 */
	private static function tab_comments( $posts = 5, $size = 45 ) {
		$comments = get_comments( array( 'status' => 'approve', 'number' => $posts ) );

		?>
		<?php if ( ! empty( $comments ) ) : ?>
			<?php foreach ( $comments as $comment ) : $post = get_post( $comment->comment_post_ID ); ?>
				<li class="recent-comments">
					<?php if ( $size ) : ?>
						<?php echo get_avatar( $comment, $size ); ?>
					<?php endif; ?>
					<a href="<?php echo esc_url( get_comment_link( $comment->comment_ID ) ); ?>" title="<?php echo wp_filter_nohtml_kses( $comment->comment_author ); ?> <?php esc_html_e( 'on', 'smart' ); ?> <?php echo esc_html( $post->post_title ); ?>"><?php echo wp_filter_nohtml_kses( $comment->comment_author ); ?></a>: <?php echo stripslashes( substr( wp_filter_nohtml_kses( $comment->comment_content ), 0, 50 ) ); ?>&hellip;
				</li>
			<?php endforeach; ?>
		<?php endif; ?>

		<?php
	}

	/**
	 * Create a new filtering function that will add our where clause to the query.
	 *
	 * @since  1.0.0
	 *
	 * @return string
	 */
	public static function where_filter() {
		global $popular_days;

		$filter .= " AND post_type = 'post' AND post_status = 'publish' AND post_date >= '" . date( 'Y-m-d', strtotime( '-' . $popular_days . ' days' ) ) . "'";

		return $filter;
	}
}
endif;
