<?php
/**
 * Smart by NiceThemes.
 *
 * This file contains a class to create the Twitter widget.
 *
 * @package   Smart
 * @author    NiceThemes <hello@nicethemes.com>
 * @license   GPL-2.0+
 * @link      http://nicethemes.com/product/smart
 * @copyright 2017 NiceThemes
 * @since     1.0.0
 */
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Nice_Twitter_Widget' ) ) :
/**
 * Class Nice_Twitter_Widget
 *
 * This widget displays a Twitter feed.
 *
 * @since 1.0.0
 */
class Nice_Twitter_Widget extends WP_Widget {
	/**
	 * Default values for widget instance.
	 *
	 * @since 1.0.0
	 * @var   array
	 */
	private $instance_defaults = array(
		'title'               => array( '', 'sanitize_text_field' ),
		'limit'               => array( 5, 'absint' ),
		'consumer_key'        => array( '' ),
		'consumer_secret'     => array( '' ),
		'access_token'        => array( '' ),
		'access_token_secret' => array( '' ),
		'include_retweets'    => array( false, 'nice_bool' ),
		'exclude_replies'     => array( false, 'nice_bool' ),
		'username'            => array( '', 'sanitize_key' ),
	);

	/**
	 * Helper object to manage widget cache.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Cache
	 */
	private $cache;

	/**
	 * Helper object to manage common processes.
	 *
	 * @since 1.0.0
	 * @var   Nice_Widget_Normalizer
	 */
	private $normalizer;

	/**
	* How often the widget cache is refreshed (in seconds). Default is five minutes.
	*
	* @since 1.0.0
	* @var   int
	*/
	public $refresh = 300;

	/**
	 * Initialize widget.
	 *
	 * @since 1.0.0
	 */
	function __construct() {
		$this->refresh         = apply_filters( 'nice_widget_twitter_refresh', $this->refresh );
		$this->alt_option_name = 'nice_twitter';
		$this->normalizer      = new Nice_Widget_Normalizer( $this->instance_defaults );
		$this->cache           = new Nice_Widget_Cache( 'widget_nice_twitter', $this->refresh );

		parent::__construct( 'nice_twitter', esc_html__( '(NiceThemes) Twitter', 'smart' ), array(
				'description' => esc_html__( 'Add your Twitter feed with this widget.', 'smart' ),
			)
		);

		add_action( 'switch_theme', array( $this->cache, 'flush' ) );
	}

	/**
	 * Display the widget in the front end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Current properties of the widget.
	 */
	function widget( $args, $instance ) {
		// If widget cache is found, print it and return early.
		if ( $cache = $this->cache->get() ) {
			echo $cache;

			return;
		}

		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		ob_start();

		// Get tweets.
		$tweets = $this->get_tweets( $instance );

		// Display error , save cache and return if there are no tweets.
		if ( ! $tweets ) {
			if ( current_user_can( 'edit_plugins' ) ) {
				echo '<p class="error">' . esc_html__( 'No tweets found. Please make sure your Twitter widget settings are correct.', 'smart' ) . '</p>';
			}

			$this->cache->set( ob_get_flush() );

			return;
		}

		?>

		<?php echo $args['before_widget']; ?>

		<?php if ( $instance['title'] ) : ?>
			<?php echo $args['before_title'] . esc_html( $instance['title'] ) . $args['after_title']; ?>
		<?php endif; ?>

		<div class="back">
			<ul id="twitter_update_list_<?php echo esc_attr( $args['widget_id'] ); ?>" class="<?php echo ( $instance['limit'] > 1 ) ? 'tweet-list' : 'one-tweet'; ?>">
				<?php foreach ( $tweets as $tweet ) : ?>
					<?php
						// Set the date and time format.
						$datetime_format = apply_filters( 'nice_twitter_datetime_format', 'l M j \- g:ia' );

						// Get the date and time posted as a nice string */
						$posted_since = apply_filters( 'nice_twitter_posted_since', date_i18n( $datetime_format , strtotime( $tweet->created_at ) ) );

						// Filter for linking dates to the tweet itself.
						$link_date = apply_filters( 'nice_twitter_link_date_to_tweet', true );
					?>

					<li>
						<div class="tweet">
							<?php echo $this->format_tweet( $tweet->text ); ?><br />
						</div>

						<?php if ( $link_date ) : ?>
							<small class="muted"><a href="https://twitter.com/<?php echo esc_attr( $tweet->user->screen_name ); ?>/status/<?php echo esc_attr( $tweet->id_str ); ?>"><?php echo esc_html( $posted_since ); ?></a></small>
						<?php endif; ?>
					</li>
				<?php endforeach; ?>

			</ul>
			<p class="tw-follow">
				<?php printf( esc_html__( 'Follow %s on Twitter', 'smart' ), '<a href="https://twitter.com/' . esc_attr( $instance['username'] ) . '" target="_blank">@' . esc_html( $instance['username'] ) . '</a>' ); ?>
			</p>
		</div>

		<?php echo $args['after_widget']; ?>
		<?php

		$this->cache->set( ob_get_flush() );
	}

	/**
	 * Flush the widget cache when the widget is updated.
	 *
	 * @since 1.0.0
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 *
	 * @return array
	 */
	function update( $new_instance, $old_instance ) {
		delete_transient( 'nice_twitter_widget_' . $this->id );

		$this->cache->flush();

		// Normalize instance.
		$new_instance = $this->normalizer->normalize_instance( $new_instance );

		return $new_instance;
	}

	/**
	 * Display widget form in "Appearance > Widgets".
	 *
	 * @since  1.0
	 *
	 * @param  array       $instance
	 * @return string|void
	 */
	function form( $instance ) {
		// Normalize instance.
		$instance = $this->normalizer->normalize_instance( $instance );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title', 'smart' ); ?>&nbsp;<small>(<?php esc_html_e( 'optional', 'smart' ); ?>)</small></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"  value="<?php echo esc_attr( $instance['title'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"/>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'consumer_key' ) ); ?>"><?php esc_html_e( 'Consumer Key', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'consumer_key' ) ); ?>"  value="<?php echo esc_attr( $instance['consumer_key'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'consumer_key' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'consumer_secret' ) ); ?>"><?php esc_html_e( 'Consumer Secret', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'consumer_secret' ) ); ?>"  value="<?php echo esc_attr( $instance['consumer_secret'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'consumer_secret' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'access_token' ) ); ?>"><?php esc_html_e( 'Access Token', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'access_token' ) ); ?>"  value="<?php echo esc_attr( $instance['access_token'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'access_token' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'access_token_secret' ) ); ?>"><?php esc_html_e('Access Token Secret', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'access_token_secret' ) ); ?>"  value="<?php echo esc_attr( $instance['access_token_secret'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'access_token_secret' ) ); ?>" />
		</p>
		<p>
			<input id="<?php echo esc_attr( $this->get_field_id( 'include_retweets' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'include_retweets' ) ); ?>" type="checkbox" value="true" <?php checked( $instance['include_retweets'], true ); ?> />
			<label for="<?php echo esc_attr( $this->get_field_id( 'include_retweets' ) ); ?>">&nbsp;<?php esc_html_e( 'Include Retweets', 'smart' ); ?></label>
		</p>
		<p>
			<input id="<?php echo esc_attr( $this->get_field_id( 'exclude_replies' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'exclude_replies' ) ); ?>" type="checkbox" value="true" <?php checked( $instance['exclude_replies'], true ); ?> />
			<label for="<?php echo esc_attr( $this->get_field_id( 'exclude_replies' ) ); ?>">&nbsp;<?php esc_html_e( 'Exclude Replies', 'smart' ); ?></label>
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>"><?php esc_html_e( 'Username', 'smart' ); ?>&nbsp;<small>(<?php esc_html_e( 'without @', 'smart' );?>)</small></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'username' ) ); ?>"  value="<?php echo esc_attr( $instance['username'] ); ?>" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'limit' ) ); ?>"><?php esc_html_e( 'Number of tweets to show:', 'smart' ); ?></label>
			<input type="text" name="<?php echo esc_attr( $this->get_field_name( 'limit' ) ); ?>"  value="<?php echo esc_attr( $instance['limit'] ); ?>" class="" size="3" id="<?php echo esc_attr( $this->get_field_id('limit') ); ?>" /><br />
			<small>(<?php esc_html_e( 'by default, 1', 'smart' );?>)</small>
		</p>
		<?php
	}

	/**
	 * Obtain tweets.
	 *
	 * @since  1.0.0
	 *
	 * @param  array $instance
	 * @return array
	 */
	public function get_tweets( $instance ) {
		if ( $tweets = get_transient( 'nice_twitter_widget_' . $this->id ) ) {
			return $tweets;
		}

		$instance = $this->normalizer->normalize_instance( $instance );

		// Merge arguments with defaults.
		$args = apply_filters( 'nice_twitter_args', array(
			'screen_name'		=> isset( $instance['username'] ) ? $instance['username'] : '',
			'count'				=> isset( $instance['limit'] ) ? $instance['limit'] : '',
			'include_rts'		=> isset( $instance['include_retweets'] ),
			'exclude_replies'	=> isset( $instance['exclude_replies'] ),
		) );

		// Require the twitter auth class.
		if ( ! class_exists( 'TwitterOAuth' ) ) {
			nice_loader( 'includes/widgets/widget-twitter/twitteroauth/twitteroauth.php' );
		}

		if ( isset( $instance['consumer_key'] ) ) {
			// Get Twitter connection.
			$twitter_connection = new TwitterOAuth(
				$instance['consumer_key'],
				$instance['consumer_secret'],
				$instance['access_token'],
				$instance['access_token_secret']
			);

			// Get tweets.
			$tweets = $twitter_connection->get( 'statuses/user_timeline', $args );
			set_transient( 'nice_twitter_widget_' . $this->id, $tweets, 60 * 15 );
		}

		// Bail if failed.
		if ( empty( $tweets ) || isset( $tweets->errors ) ) {
			return null;
		}

		return $tweets;
	}

	/**
	 * Formats tweet text to add URLs and hashtags
	 *
	 * @since 1.0.0
	 *
	 * @param  string $text
	 * @return string
	 */
	public function format_tweet( $text ) {
		$text = preg_replace( "#(^|[\n ])([\w]+?://[\w]+[^ \"\n\r\t< ]*)#", "\\1<a href=\"\\2\" target=\"_blank\">\\2</a>", $text );
		$text = preg_replace( "#(^|[\n ])((www|ftp)\.[^ \"\t\n\r< ]*)#", "\\1<a href=\"http://\\2\" target=\"_blank\">\\2</a>", $text );
		$text = preg_replace( '/@(\w+)/', "<a href=\"http://www.twitter.com/\\1\" target=\"_blank\">@\\1</a>", $text );
		$text = preg_replace( '/#(\w+)/', "<a href=\"http://twitter.com/search?q=%23\\1&src=hash\" target=\"_blank\">#\\1</a>", $text );

		return $text;
	}
}
endif;
